/* valainterfaceregisterfunction.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */
#define VALA_FREE_CHECKED(o,f) ((o) == NULL ? NULL : ((o) = (f (o), NULL)))
#define VALA_FREE_UNCHECKED(o,f) ((o) = (f (o), NULL))

#include "valainterfaceregisterfunction.h"

struct _ValaInterfaceRegisterFunctionPrivate {
	ValaInterface* _interface_reference;
};
#define VALA_INTERFACE_REGISTER_FUNCTION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_INTERFACE_REGISTER_FUNCTION, ValaInterfaceRegisterFunctionPrivate))
enum  {
	VALA_INTERFACE_REGISTER_FUNCTION_DUMMY_PROPERTY,
	VALA_INTERFACE_REGISTER_FUNCTION_INTERFACE_REFERENCE,
};
static ValaDataType* vala_interface_register_function_real_get_type_declaration (ValaTypeRegisterFunction* base);
static char* vala_interface_register_function_real_get_type_struct_name (ValaTypeRegisterFunction* base);
static char* vala_interface_register_function_real_get_class_init_func_name (ValaTypeRegisterFunction* base);
static char* vala_interface_register_function_real_get_instance_struct_size (ValaTypeRegisterFunction* base);
static char* vala_interface_register_function_real_get_instance_init_func_name (ValaTypeRegisterFunction* base);
static char* vala_interface_register_function_real_get_parent_type_name (ValaTypeRegisterFunction* base);
static ValaCCodeFragment* vala_interface_register_function_real_get_type_interface_init_statements (ValaTypeRegisterFunction* base);
static gpointer vala_interface_register_function_parent_class = NULL;
static void vala_interface_register_function_dispose (GObject * obj);


ValaInterfaceRegisterFunction* vala_interface_register_function_new (ValaInterface* iface)
{
	g_return_val_if_fail (VALA_IS_INTERFACE (iface), NULL);
	GParameter * __params = g_new0 (GParameter, 1);
	GParameter * __params_it = __params;
	(__params_it->name = "interface-reference", g_value_init (&__params_it->value, VALA_TYPE_INTERFACE), g_value_set_object (&__params_it->value, iface), __params_it++);
	ValaInterfaceRegisterFunction * self = g_object_newv (VALA_TYPE_INTERFACE_REGISTER_FUNCTION, __params_it - __params, __params);
	return self;
}


static ValaDataType* vala_interface_register_function_real_get_type_declaration (ValaTypeRegisterFunction* base)
{
	ValaInterfaceRegisterFunction * self = VALA_INTERFACE_REGISTER_FUNCTION (base);
	return VALA_DATA_TYPE (vala_interface_register_function_get_interface_reference (self));
}


static char* vala_interface_register_function_real_get_type_struct_name (ValaTypeRegisterFunction* base)
{
	ValaInterfaceRegisterFunction * self = VALA_INTERFACE_REGISTER_FUNCTION (base);
	const char* __temp1 = NULL;
	return (__temp1 = vala_interface_get_type_cname (vala_interface_register_function_get_interface_reference (self)), (__temp1 == NULL ? NULL : g_strdup (__temp1)));
}


static char* vala_interface_register_function_real_get_class_init_func_name (ValaTypeRegisterFunction* base)
{
	ValaInterfaceRegisterFunction * self = VALA_INTERFACE_REGISTER_FUNCTION (base);
	return g_strdup ("NULL");
}


static char* vala_interface_register_function_real_get_instance_struct_size (ValaTypeRegisterFunction* base)
{
	ValaInterfaceRegisterFunction * self = VALA_INTERFACE_REGISTER_FUNCTION (base);
	return g_strdup ("0");
}


static char* vala_interface_register_function_real_get_instance_init_func_name (ValaTypeRegisterFunction* base)
{
	ValaInterfaceRegisterFunction * self = VALA_INTERFACE_REGISTER_FUNCTION (base);
	return g_strdup ("NULL");
}


static char* vala_interface_register_function_real_get_parent_type_name (ValaTypeRegisterFunction* base)
{
	ValaInterfaceRegisterFunction * self = VALA_INTERFACE_REGISTER_FUNCTION (base);
	return g_strdup ("G_TYPE_INTERFACE");
}


static ValaCCodeFragment* vala_interface_register_function_real_get_type_interface_init_statements (ValaTypeRegisterFunction* base)
{
	ValaInterfaceRegisterFunction * self = VALA_INTERFACE_REGISTER_FUNCTION (base);
	ValaCCodeFragment* frag = g_object_new (VALA_TYPE_CCODE_FRAGMENT, NULL);
	return frag;
	(frag == NULL ? NULL : (frag = (g_object_unref (frag), NULL)));
}


ValaInterface* vala_interface_register_function_get_interface_reference (ValaInterfaceRegisterFunction* self)
{
	g_return_val_if_fail (VALA_IS_INTERFACE_REGISTER_FUNCTION (self), NULL);
	return self->priv->_interface_reference;
}


void vala_interface_register_function_set_interface_reference (ValaInterfaceRegisterFunction* self, ValaInterface* value)
{
	g_return_if_fail (VALA_IS_INTERFACE_REGISTER_FUNCTION (self));
	ValaInterface* __temp9 = NULL;
	self->priv->_interface_reference = (__temp9 = g_object_ref (value), (self->priv->_interface_reference == NULL ? NULL : (self->priv->_interface_reference = (g_object_unref (self->priv->_interface_reference), NULL))), __temp9);
}


static void vala_interface_register_function_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaInterfaceRegisterFunction * self = VALA_INTERFACE_REGISTER_FUNCTION (object);
	switch (property_id) {
		case VALA_INTERFACE_REGISTER_FUNCTION_INTERFACE_REFERENCE:
		g_value_set_object (value, vala_interface_register_function_get_interface_reference (self));
		break;
	}
}


static void vala_interface_register_function_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaInterfaceRegisterFunction * self = VALA_INTERFACE_REGISTER_FUNCTION (object);
	switch (property_id) {
		case VALA_INTERFACE_REGISTER_FUNCTION_INTERFACE_REFERENCE:
		vala_interface_register_function_set_interface_reference (self, g_value_get_object (value));
		break;
	}
}


static void vala_interface_register_function_class_init (ValaInterfaceRegisterFunctionClass * klass)
{
	vala_interface_register_function_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValaInterfaceRegisterFunctionPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_interface_register_function_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_interface_register_function_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_interface_register_function_dispose;
	VALA_TYPE_REGISTER_FUNCTION_CLASS (klass)->get_type_declaration = vala_interface_register_function_real_get_type_declaration;
	VALA_TYPE_REGISTER_FUNCTION_CLASS (klass)->get_type_struct_name = vala_interface_register_function_real_get_type_struct_name;
	VALA_TYPE_REGISTER_FUNCTION_CLASS (klass)->get_class_init_func_name = vala_interface_register_function_real_get_class_init_func_name;
	VALA_TYPE_REGISTER_FUNCTION_CLASS (klass)->get_instance_struct_size = vala_interface_register_function_real_get_instance_struct_size;
	VALA_TYPE_REGISTER_FUNCTION_CLASS (klass)->get_instance_init_func_name = vala_interface_register_function_real_get_instance_init_func_name;
	VALA_TYPE_REGISTER_FUNCTION_CLASS (klass)->get_parent_type_name = vala_interface_register_function_real_get_parent_type_name;
	VALA_TYPE_REGISTER_FUNCTION_CLASS (klass)->get_type_interface_init_statements = vala_interface_register_function_real_get_type_interface_init_statements;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_INTERFACE_REGISTER_FUNCTION_INTERFACE_REFERENCE, g_param_spec_object ("interface-reference", "foo", "bar", VALA_TYPE_INTERFACE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
}


static void vala_interface_register_function_init (ValaInterfaceRegisterFunction * self)
{
	self->priv = VALA_INTERFACE_REGISTER_FUNCTION_GET_PRIVATE (self);
}


static void vala_interface_register_function_dispose (GObject * obj)
{
	ValaInterfaceRegisterFunction * self = VALA_INTERFACE_REGISTER_FUNCTION (obj);
	(self->priv->_interface_reference == NULL ? NULL : (self->priv->_interface_reference = (g_object_unref (self->priv->_interface_reference), NULL)));
	ValaInterfaceRegisterFunctionClass * klass;
	GObjectClass * parent_class;
	klass = VALA_INTERFACE_REGISTER_FUNCTION_CLASS (g_type_class_peek (VALA_TYPE_INTERFACE_REGISTER_FUNCTION));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_interface_register_function_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaInterfaceRegisterFunctionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_interface_register_function_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaInterfaceRegisterFunction), 0, (GInstanceInitFunc) vala_interface_register_function_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_TYPE_REGISTER_FUNCTION, "ValaInterfaceRegisterFunction", &g_define_type_info, 0);
	}
	return g_define_type_id;
}




