/* valatyperegisterfunction.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */
#define VALA_FREE_CHECKED(o,f) ((o) == NULL ? NULL : ((o) = (f (o), NULL)))
#define VALA_FREE_UNCHECKED(o,f) ((o) = (f (o), NULL))

#include "valatyperegisterfunction.h"
#include <ccode/valaccodeblock.h>
#include <ccode/valaccodedeclaration.h>
#include <ccode/valaccodevariabledeclarator.h>
#include <ccode/valaccodeconstant.h>
#include <ccode/valaccodeexpression.h>
#include <ccode/valaccodedeclarator.h>
#include <ccode/valaccodemodifiers.h>
#include <ccode/valaccodenode.h>
#include <ccode/valaccodefunctioncall.h>
#include <ccode/valaccodeidentifier.h>
#include <ccode/valaccodebinaryexpression.h>
#include <ccode/valaccodeexpressionstatement.h>
#include <ccode/valaccodeassignment.h>
#include <ccode/valaccodeifstatement.h>
#include <ccode/valaccodestatement.h>
#include <ccode/valaccodereturnstatement.h>
#include <vala/valatyperegisterfunction.h>

struct _ValaTypeRegisterFunctionPrivate {
};
#define VALA_TYPE_REGISTER_FUNCTION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_TYPE_REGISTER_FUNCTION, ValaTypeRegisterFunctionPrivate))
enum  {
	VALA_TYPE_REGISTER_FUNCTION_DUMMY_PROPERTY,
};
static char* vala_type_register_function_real_get_type_flags (ValaTypeRegisterFunction* self);
static gpointer vala_type_register_function_parent_class = NULL;


/**
 * Constructs the C function from the specified type.
 */
void vala_type_register_function_init_from_type (ValaTypeRegisterFunction* self)
{
	g_return_if_fail (VALA_IS_TYPE_REGISTER_FUNCTION (self));
	char* __temp1 = NULL;
	char* __temp0 = NULL;
	vala_ccode_function_set_name (VALA_CCODE_FUNCTION (self), (__temp1 = g_strdup_printf ("%s_get_type", (__temp0 = vala_data_type_get_lower_case_cname (vala_type_register_function_get_type_declaration (self), NULL)))));
	(__temp1 == NULL ? NULL : (__temp1 = (g_free (__temp1), NULL)));
	(__temp0 == NULL ? NULL : (__temp0 = (g_free (__temp0), NULL)));
	vala_ccode_function_set_return_type (VALA_CCODE_FUNCTION (self), "GType");
	ValaCCodeBlock* type_block = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
	ValaCCodeDeclaration* cdecl = vala_ccode_declaration_new ("GType");
	ValaCCodeVariableDeclarator* __temp2 = NULL;
	vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR ((__temp2 = vala_ccode_variable_declarator_new_with_initializer ("g_define_type_id", VALA_CCODE_EXPRESSION (vala_ccode_constant_new ("0"))))));
	(__temp2 == NULL ? NULL : (__temp2 = (g_object_unref (__temp2), NULL)));
	vala_ccode_declaration_set_modifiers (cdecl, VALA_CCODE_MODIFIERS_STATIC);
	vala_ccode_block_add_statement (type_block, VALA_CCODE_NODE (cdecl));
	ValaCCodeFunctionCall* cond = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("G_UNLIKELY")));
	ValaCCodeBinaryExpression* __temp3 = NULL;
	vala_ccode_function_call_add_argument (cond, VALA_CCODE_EXPRESSION ((__temp3 = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_EQUALITY, VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("g_define_type_id")), VALA_CCODE_EXPRESSION (vala_ccode_constant_new ("0"))))));
	(__temp3 == NULL ? NULL : (__temp3 = (g_object_unref (__temp3), NULL)));
	ValaCCodeBlock* type_init = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
	ValaCCodeDeclaration* ctypedecl = vala_ccode_declaration_new ("const GTypeInfo");
	vala_ccode_declaration_set_modifiers (ctypedecl, VALA_CCODE_MODIFIERS_STATIC);
	ValaCCodeVariableDeclarator* __temp8 = NULL;
	char* __temp7 = NULL;
	char* __temp6 = NULL;
	char* __temp5 = NULL;
	char* __temp4 = NULL;
	vala_ccode_declaration_add_declarator (ctypedecl, VALA_CCODE_DECLARATOR ((__temp8 = vala_ccode_variable_declarator_new_with_initializer ("g_define_type_info", VALA_CCODE_EXPRESSION (vala_ccode_constant_new (g_strdup_printf ("{ sizeof (%s), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) %s, (GClassFinalizeFunc) NULL, NULL, %s, 0, (GInstanceInitFunc) %s }", (__temp4 = vala_type_register_function_get_type_struct_name (self)), (__temp5 = vala_type_register_function_get_class_init_func_name (self)), (__temp6 = vala_type_register_function_get_instance_struct_size (self)), (__temp7 = vala_type_register_function_get_instance_init_func_name (self)))))))));
	(__temp8 == NULL ? NULL : (__temp8 = (g_object_unref (__temp8), NULL)));
	(__temp7 == NULL ? NULL : (__temp7 = (g_free (__temp7), NULL)));
	(__temp6 == NULL ? NULL : (__temp6 = (g_free (__temp6), NULL)));
	(__temp5 == NULL ? NULL : (__temp5 = (g_free (__temp5), NULL)));
	(__temp4 == NULL ? NULL : (__temp4 = (g_free (__temp4), NULL)));
	vala_ccode_block_add_statement (type_init, VALA_CCODE_NODE (ctypedecl));
	ValaCCodeFunctionCall* reg_call = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("g_type_register_static")));
	ValaCCodeIdentifier* __temp9 = NULL;
	vala_ccode_function_call_add_argument (reg_call, VALA_CCODE_EXPRESSION ((__temp9 = vala_ccode_identifier_new (vala_type_register_function_get_parent_type_name (self)))));
	(__temp9 == NULL ? NULL : (__temp9 = (g_object_unref (__temp9), NULL)));
	ValaCCodeConstant* __temp10 = NULL;
	vala_ccode_function_call_add_argument (reg_call, VALA_CCODE_EXPRESSION ((__temp10 = vala_ccode_constant_new (g_strdup_printf ("\"%s\"", vala_data_type_get_cname (vala_type_register_function_get_type_declaration (self), FALSE))))));
	(__temp10 == NULL ? NULL : (__temp10 = (g_object_unref (__temp10), NULL)));
	ValaCCodeIdentifier* __temp11 = NULL;
	vala_ccode_function_call_add_argument (reg_call, VALA_CCODE_EXPRESSION ((__temp11 = vala_ccode_identifier_new ("&g_define_type_info"))));
	(__temp11 == NULL ? NULL : (__temp11 = (g_object_unref (__temp11), NULL)));
	ValaCCodeConstant* __temp12 = NULL;
	vala_ccode_function_call_add_argument (reg_call, VALA_CCODE_EXPRESSION ((__temp12 = vala_ccode_constant_new (vala_type_register_function_get_type_flags (self)))));
	(__temp12 == NULL ? NULL : (__temp12 = (g_object_unref (__temp12), NULL)));
	ValaCCodeExpressionStatement* __temp13 = NULL;
	vala_ccode_block_add_statement (type_init, VALA_CCODE_NODE ((__temp13 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("g_define_type_id")), VALA_CCODE_EXPRESSION (reg_call), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))))));
	(__temp13 == NULL ? NULL : (__temp13 = (g_object_unref (__temp13), NULL)));
	ValaCCodeFragment* __temp14 = NULL;
	vala_ccode_block_add_statement (type_init, VALA_CCODE_NODE ((__temp14 = vala_type_register_function_get_type_interface_init_statements (self))));
	(__temp14 == NULL ? NULL : (__temp14 = (g_object_unref (__temp14), NULL)));
	ValaCCodeIfStatement* cif = vala_ccode_if_statement_new (VALA_CCODE_EXPRESSION (cond), VALA_CCODE_STATEMENT (type_init), NULL);
	vala_ccode_block_add_statement (type_block, VALA_CCODE_NODE (cif));
	ValaCCodeReturnStatement* __temp15 = NULL;
	vala_ccode_block_add_statement (type_block, VALA_CCODE_NODE ((__temp15 = vala_ccode_return_statement_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("g_define_type_id"))))));
	(__temp15 == NULL ? NULL : (__temp15 = (g_object_unref (__temp15), NULL)));
	vala_ccode_function_set_block (VALA_CCODE_FUNCTION (self), type_block);
	(type_block == NULL ? NULL : (type_block = (g_object_unref (type_block), NULL)));
	(cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL)));
	(cond == NULL ? NULL : (cond = (g_object_unref (cond), NULL)));
	(type_init == NULL ? NULL : (type_init = (g_object_unref (type_init), NULL)));
	(ctypedecl == NULL ? NULL : (ctypedecl = (g_object_unref (ctypedecl), NULL)));
	(reg_call == NULL ? NULL : (reg_call = (g_object_unref (reg_call), NULL)));
	(cif == NULL ? NULL : (cif = (g_object_unref (cif), NULL)));
}


ValaDataType* vala_type_register_function_get_type_declaration (ValaTypeRegisterFunction* self)
{
	return VALA_TYPE_REGISTER_FUNCTION_GET_CLASS (self)->get_type_declaration (self);
}


char* vala_type_register_function_get_type_struct_name (ValaTypeRegisterFunction* self)
{
	return VALA_TYPE_REGISTER_FUNCTION_GET_CLASS (self)->get_type_struct_name (self);
}


char* vala_type_register_function_get_class_init_func_name (ValaTypeRegisterFunction* self)
{
	return VALA_TYPE_REGISTER_FUNCTION_GET_CLASS (self)->get_class_init_func_name (self);
}


char* vala_type_register_function_get_instance_struct_size (ValaTypeRegisterFunction* self)
{
	return VALA_TYPE_REGISTER_FUNCTION_GET_CLASS (self)->get_instance_struct_size (self);
}


char* vala_type_register_function_get_instance_init_func_name (ValaTypeRegisterFunction* self)
{
	return VALA_TYPE_REGISTER_FUNCTION_GET_CLASS (self)->get_instance_init_func_name (self);
}


char* vala_type_register_function_get_parent_type_name (ValaTypeRegisterFunction* self)
{
	return VALA_TYPE_REGISTER_FUNCTION_GET_CLASS (self)->get_parent_type_name (self);
}


/**
 * Returns the set of type flags to be applied when registering.
 *
 * @return type flags
 */
static char* vala_type_register_function_real_get_type_flags (ValaTypeRegisterFunction* self)
{
	g_return_val_if_fail (VALA_IS_TYPE_REGISTER_FUNCTION (self), NULL);
	return "0";
}


char* vala_type_register_function_get_type_flags (ValaTypeRegisterFunction* self)
{
	return VALA_TYPE_REGISTER_FUNCTION_GET_CLASS (self)->get_type_flags (self);
}


ValaCCodeFragment* vala_type_register_function_get_type_interface_init_statements (ValaTypeRegisterFunction* self)
{
	return VALA_TYPE_REGISTER_FUNCTION_GET_CLASS (self)->get_type_interface_init_statements (self);
}


/**
 * Returns the declaration for this type register function in C code.
 *
 * @return C function declaration
 */
ValaCCodeFunction* vala_type_register_function_get_declaration (ValaTypeRegisterFunction* self)
{
	g_return_val_if_fail (VALA_IS_TYPE_REGISTER_FUNCTION (self), NULL);
	return vala_ccode_function_new (vala_ccode_function_get_name (VALA_CCODE_FUNCTION (self)), vala_ccode_function_get_return_type (VALA_CCODE_FUNCTION (self)));
}


static void vala_type_register_function_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaTypeRegisterFunction * self = VALA_TYPE_REGISTER_FUNCTION (object);
	switch (property_id) {
	}
}


static void vala_type_register_function_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaTypeRegisterFunction * self = VALA_TYPE_REGISTER_FUNCTION (object);
	switch (property_id) {
	}
}


static void vala_type_register_function_class_init (ValaTypeRegisterFunctionClass * klass)
{
	vala_type_register_function_parent_class = g_type_class_peek_parent (klass);
	G_OBJECT_CLASS (klass)->get_property = vala_type_register_function_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_type_register_function_set_property;
	VALA_TYPE_REGISTER_FUNCTION_CLASS (klass)->get_type_flags = vala_type_register_function_real_get_type_flags;
}


static void vala_type_register_function_init (ValaTypeRegisterFunction * self)
{
}


GType vala_type_register_function_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaTypeRegisterFunctionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_type_register_function_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaTypeRegisterFunction), 0, (GInstanceInitFunc) vala_type_register_function_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_CCODE_FUNCTION, "ValaTypeRegisterFunction", &g_define_type_info, G_TYPE_FLAG_ABSTRACT);
	}
	return g_define_type_id;
}




