/* valacodegenerator.vala
 *
 * Copyright (C) 2006  Jürg Billeter, Raffaele Sandrini
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 *	Raffaele Sandrini <rasa@gmx.ch>
 */
#define VALA_FREE_CHECKED(o,f) ((o) == NULL ? NULL : ((o) = (f (o), NULL)))
#define VALA_FREE_UNCHECKED(o,f) ((o) = (f (o), NULL))

#include "valacodegenerator.h"
#include <ccode/valaccodestruct.h>
#include <ccode/valaccodeenum.h>
#include <ccode/valaccodefunction.h>
#include <ccode/valaccodenode.h>
#include <ccode/valaccodetypedefinition.h>
#include <ccode/valaccodevariabledeclarator.h>
#include <ccode/valaccodedeclarator.h>
#include <ccode/valaccodemacroreplacement.h>
#include <ccode/valaccodecomment.h>
#include <ccode/valaccodewriter.h>
#include <ccode/valaccodeoncesection.h>
#include <ccode/valaccodenewline.h>
#include <ccode/valaccodeformalparameter.h>
#include <ccode/valaccodemodifiers.h>
#include <ccode/valaccodefunctioncall.h>
#include <ccode/valaccodedeclaration.h>
#include <ccode/valaccodeconstant.h>
#include <ccode/valaccodeassignment.h>
#include <ccode/valaccodeexpressionstatement.h>
#include <ccode/valaccodememberaccess.h>
#include <ccode/valaccodeswitchstatement.h>
#include <ccode/valaccodecasestatement.h>
#include <ccode/valaccodebreakstatement.h>
#include <ccode/valaccodefunctiondeclarator.h>
#include <ccode/valaccodebinaryexpression.h>
#include <ccode/valaccodereturnstatement.h>
#include <ccode/valaccodecastexpression.h>
#include <ccode/valaccodeifstatement.h>
#include <ccode/valaccodeconditionalexpression.h>
#include <ccode/valaccodeemptystatement.h>
#include <ccode/valaccodeinitializerlist.h>
#include <ccode/valaccodeparenthesizedexpression.h>
#include <ccode/valaccodedostatement.h>
#include <ccode/valaccodewhilestatement.h>
#include <ccode/valaccodeforstatement.h>
#include <ccode/valaccodeelementaccess.h>
#include <ccode/valaccodecontinuestatement.h>
#include <ccode/valaccodeunaryexpression.h>
#include <vala/valasourcefilecycle.h>
#include <vala/valanamespace.h>
#include <vala/valasourcereference.h>
#include <vala/valaclassregisterfunction.h>
#include <vala/valatyperegisterfunction.h>
#include <vala/valainterfaceregisterfunction.h>
#include <vala/valamemberaccessibility.h>
#include <vala/valalockable.h>
#include <vala/valaarray.h>
#include <vala/valaflags.h>
#include <vala/valareport.h>
#include <vala/valalocalvariabledeclaration.h>
#include <vala/valainstancecast.h>
#include <vala/valaswitchsection.h>
#include <vala/valaswitchlabel.h>
#include <vala/valainvokable.h>

struct _ValaCodeGeneratorPrivate {
	gboolean _memory_management;
	ValaCodeContext* context;
	ValaSymbol* root_symbol;
	ValaSymbol* current_symbol;
	ValaSymbol* current_type_symbol;
	ValaClass* current_class;
	ValaTypeReference* current_return_type;
	ValaCCodeFragment* header_begin;
	ValaCCodeFragment* header_type_declaration;
	ValaCCodeFragment* header_type_definition;
	ValaCCodeFragment* header_type_member_declaration;
	ValaCCodeFragment* source_begin;
	ValaCCodeFragment* source_include_directives;
	ValaCCodeFragment* source_type_member_declaration;
	ValaCCodeFragment* source_signal_marshaller_declaration;
	ValaCCodeFragment* source_type_member_definition;
	ValaCCodeFragment* instance_init_fragment;
	ValaCCodeFragment* instance_dispose_fragment;
	ValaCCodeFragment* source_signal_marshaller_definition;
	ValaCCodeStruct* instance_struct;
	ValaCCodeStruct* type_struct;
	ValaCCodeStruct* instance_priv_struct;
	ValaCCodeEnum* prop_enum;
	ValaCCodeEnum* cenum;
	ValaCCodeFunction* function;
	ValaCCodeBlock* block;
	GList* temp_vars;
	GList* temp_ref_vars;
	GHashTable* user_marshal_list;
	GHashTable* predefined_marshal_list;
	int next_temp_var_id;
	ValaTypeReference* bool_type;
	ValaTypeReference* char_type;
	ValaTypeReference* unichar_type;
	ValaTypeReference* short_type;
	ValaTypeReference* ushort_type;
	ValaTypeReference* int_type;
	ValaTypeReference* uint_type;
	ValaTypeReference* long_type;
	ValaTypeReference* ulong_type;
	ValaTypeReference* int64_type;
	ValaTypeReference* uint64_type;
	ValaTypeReference* string_type;
	ValaTypeReference* float_type;
	ValaTypeReference* double_type;
	ValaDataType* list_type;
	ValaDataType* slist_type;
	ValaTypeReference* mutex_type;
};
#define VALA_CODE_GENERATOR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_CODE_GENERATOR, ValaCodeGeneratorPrivate))
enum  {
	VALA_CODE_GENERATOR_DUMMY_PROPERTY,
	VALA_CODE_GENERATOR_MEMORY_MANAGEMENT,
};
static ValaCCodeIncludeDirective* vala_code_generator_get_internal_include (ValaCodeGenerator* self, char* filename);
static void vala_code_generator_real_visit_begin_source_file (ValaCodeVisitor* base, ValaSourceFile* source_file);
static char* vala_code_generator_get_define_for_filename (char* filename);
static void vala_code_generator_real_visit_end_source_file (ValaCodeVisitor* base, ValaSourceFile* source_file);
static void vala_code_generator_real_visit_begin_class (ValaCodeVisitor* base, ValaClass* cl);
static void vala_code_generator_real_visit_end_class (ValaCodeVisitor* base, ValaClass* cl);
static void vala_code_generator_add_class_init_function (ValaCodeGenerator* self, ValaClass* cl);
static void vala_code_generator_add_interface_init_function (ValaCodeGenerator* self, ValaClass* cl, ValaInterface* iface);
static void vala_code_generator_add_instance_init_function (ValaCodeGenerator* self, ValaClass* cl);
static void vala_code_generator_add_dispose_function (ValaCodeGenerator* self, ValaClass* cl);
static ValaCCodeIdentifier* vala_code_generator_get_value_setter_function (ValaCodeGenerator* self, ValaTypeReference* type_reference);
static void vala_code_generator_add_get_property_function (ValaCodeGenerator* self, ValaClass* cl);
static void vala_code_generator_add_set_property_function (ValaCodeGenerator* self, ValaClass* cl);
static void vala_code_generator_real_visit_begin_struct (ValaCodeVisitor* base, ValaStruct* st);
static void vala_code_generator_real_visit_begin_interface (ValaCodeVisitor* base, ValaInterface* iface);
static void vala_code_generator_real_visit_end_interface (ValaCodeVisitor* base, ValaInterface* iface);
static void vala_code_generator_real_visit_begin_enum (ValaCodeVisitor* base, ValaEnum* en);
static void vala_code_generator_real_visit_enum_value (ValaCodeVisitor* base, ValaEnumValue* ev);
static void vala_code_generator_real_visit_end_callback (ValaCodeVisitor* base, ValaCallback* cb);
static void vala_code_generator_real_visit_member (ValaCodeVisitor* base, ValaMember* m);
static void vala_code_generator_real_visit_constant (ValaCodeVisitor* base, ValaConstant* c);
static void vala_code_generator_real_visit_field (ValaCodeVisitor* base, ValaField* f);
static void vala_code_generator_real_visit_begin_method (ValaCodeVisitor* base, ValaMethod* m);
static ValaCCodeStatement* vala_code_generator_create_method_type_check_statement (ValaCodeGenerator* self, ValaMethod* m, ValaDataType* t, gboolean non_null, char* var_name);
static ValaCCodeStatement* vala_code_generator_create_property_type_check_statement (ValaCodeGenerator* self, ValaProperty* prop, gboolean getter, ValaDataType* t, gboolean non_null, char* var_name);
static ValaCCodeStatement* vala_code_generator_create_type_check_statement (ValaCodeGenerator* self, ValaCodeNode* method_node, ValaDataType* ret_type, ValaDataType* t, gboolean non_null, char* var_name);
static ValaDataType* vala_code_generator_find_parent_type (ValaCodeGenerator* self, ValaCodeNode* node);
static char* vala_code_generator_get_array_length_cname (ValaCodeGenerator* self, char* array_cname, int dim);
static void vala_code_generator_real_visit_end_method (ValaCodeVisitor* base, ValaMethod* m);
static void vala_code_generator_real_visit_formal_parameter (ValaCodeVisitor* base, ValaFormalParameter* p);
static void vala_code_generator_real_visit_end_property (ValaCodeVisitor* base, ValaProperty* prop);
static void vala_code_generator_real_visit_begin_property_accessor (ValaCodeVisitor* base, ValaPropertyAccessor* acc);
static void vala_code_generator_real_visit_end_property_accessor (ValaCodeVisitor* base, ValaPropertyAccessor* acc);
static char* vala_code_generator_get_marshaller_type_name (ValaCodeGenerator* self, ValaDataType* t);
static char* vala_code_generator_get_signal_marshaller_function (ValaCodeGenerator* self, ValaSignal* sig, char* prefix);
static char* vala_code_generator_get_value_type_name_from_datatype (ValaCodeGenerator* self, ValaDataType* t);
static char* vala_code_generator_get_signal_signature (ValaCodeGenerator* self, ValaSignal* sig);
static void vala_code_generator_real_visit_end_signal (ValaCodeVisitor* base, ValaSignal* sig);
static void vala_code_generator_real_visit_end_constructor (ValaCodeVisitor* base, ValaConstructor* c);
static void vala_code_generator_real_visit_begin_block (ValaCodeVisitor* base, ValaBlock* b);
static void vala_code_generator_add_object_creation (ValaCodeGenerator* self, ValaCCodeBlock* b);
static void vala_code_generator_real_visit_end_block (ValaCodeVisitor* base, ValaBlock* b);
static void vala_code_generator_real_visit_empty_statement (ValaCodeVisitor* base, ValaEmptyStatement* stmt);
static void vala_code_generator_real_visit_declaration_statement (ValaCodeVisitor* base, ValaDeclarationStatement* stmt);
static void vala_code_generator_real_visit_variable_declarator (ValaCodeVisitor* base, ValaVariableDeclarator* decl);
static void vala_code_generator_real_visit_end_initializer_list (ValaCodeVisitor* base, ValaInitializerList* list);
static ValaVariableDeclarator* vala_code_generator_get_temp_variable_declarator (ValaCodeGenerator* self, ValaTypeReference* type, gboolean takes_ownership);
static ValaCCodeExpression* vala_code_generator_get_unref_expression (ValaCodeGenerator* self, ValaCCodeExpression* cvar, ValaTypeReference* type);
static void vala_code_generator_real_visit_end_full_expression (ValaCodeVisitor* base, ValaExpression* expr);
static void vala_code_generator_append_temp_decl (ValaCodeGenerator* self, ValaCCodeFragment* cfrag, GList* temp_vars);
static void vala_code_generator_real_visit_expression_statement (ValaCodeVisitor* base, ValaExpressionStatement* stmt);
static void vala_code_generator_create_temp_decl (ValaCodeGenerator* self, ValaStatement* stmt, GList* temp_vars);
static void vala_code_generator_real_visit_if_statement (ValaCodeVisitor* base, ValaIfStatement* stmt);
static void vala_code_generator_real_visit_switch_statement (ValaCodeVisitor* base, ValaSwitchStatement* stmt);
static void vala_code_generator_real_visit_while_statement (ValaCodeVisitor* base, ValaWhileStatement* stmt);
static void vala_code_generator_real_visit_for_statement (ValaCodeVisitor* base, ValaForStatement* stmt);
static void vala_code_generator_real_visit_end_foreach_statement (ValaCodeVisitor* base, ValaForeachStatement* stmt);
static void vala_code_generator_real_visit_break_statement (ValaCodeVisitor* base, ValaBreakStatement* stmt);
static void vala_code_generator_real_visit_continue_statement (ValaCodeVisitor* base, ValaContinueStatement* stmt);
static void vala_code_generator_append_local_free (ValaCodeGenerator* self, ValaSymbol* sym, ValaCCodeFragment* cfrag, gboolean stop_at_loop);
static void vala_code_generator_create_local_free (ValaCodeGenerator* self, ValaStatement* stmt);
static gboolean vala_code_generator_append_local_free_expr (ValaCodeGenerator* self, ValaSymbol* sym, ValaCCodeCommaExpression* ccomma, gboolean stop_at_loop);
static void vala_code_generator_create_local_free_expr (ValaCodeGenerator* self, ValaExpression* expr);
static void vala_code_generator_real_visit_begin_return_statement (ValaCodeVisitor* base, ValaReturnStatement* stmt);
static void vala_code_generator_real_visit_end_return_statement (ValaCodeVisitor* base, ValaReturnStatement* stmt);
static char* vala_code_generator_get_symbol_lock_name (ValaCodeGenerator* self, ValaSymbol* sym);
static void vala_code_generator_real_visit_lock_statement (ValaCodeVisitor* base, ValaLockStatement* stmt);
static void vala_code_generator_real_visit_end_array_creation_expression (ValaCodeVisitor* base, ValaArrayCreationExpression* expr);
static void vala_code_generator_real_visit_boolean_literal (ValaCodeVisitor* base, ValaBooleanLiteral* expr);
static void vala_code_generator_real_visit_character_literal (ValaCodeVisitor* base, ValaCharacterLiteral* expr);
static void vala_code_generator_real_visit_integer_literal (ValaCodeVisitor* base, ValaIntegerLiteral* expr);
static void vala_code_generator_real_visit_real_literal (ValaCodeVisitor* base, ValaRealLiteral* expr);
static void vala_code_generator_real_visit_string_literal (ValaCodeVisitor* base, ValaStringLiteral* expr);
static void vala_code_generator_real_visit_null_literal (ValaCodeVisitor* base, ValaNullLiteral* expr);
static void vala_code_generator_real_visit_literal_expression (ValaCodeVisitor* base, ValaLiteralExpression* expr);
static void vala_code_generator_process_cmember (ValaCodeGenerator* self, ValaMemberAccess* expr, ValaCCodeExpression* pub_inst, ValaDataType* base_type);
static void vala_code_generator_real_visit_parenthesized_expression (ValaCodeVisitor* base, ValaParenthesizedExpression* expr);
static void vala_code_generator_real_visit_member_access (ValaCodeVisitor* base, ValaMemberAccess* expr);
static ValaCCodeExpression* vala_code_generator_get_array_length_cexpression (ValaCodeGenerator* self, ValaExpression* array_expr, int dim);
static void vala_code_generator_real_visit_end_invocation_expression (ValaCodeVisitor* base, ValaInvocationExpression* expr);
static void vala_code_generator_real_visit_element_access (ValaCodeVisitor* base, ValaElementAccess* expr);
static void vala_code_generator_real_visit_base_access (ValaCodeVisitor* base, ValaBaseAccess* expr);
static void vala_code_generator_real_visit_postfix_expression (ValaCodeVisitor* base, ValaPostfixExpression* expr);
static ValaCCodeExpression* vala_code_generator_get_ref_expression (ValaCodeGenerator* self, ValaExpression* expr);
static void vala_code_generator_visit_expression (ValaCodeGenerator* self, ValaExpression* expr);
static void vala_code_generator_real_visit_end_object_creation_expression (ValaCodeVisitor* base, ValaObjectCreationExpression* expr);
static void vala_code_generator_real_visit_typeof_expression (ValaCodeVisitor* base, ValaTypeofExpression* expr);
static void vala_code_generator_real_visit_unary_expression (ValaCodeVisitor* base, ValaUnaryExpression* expr);
static void vala_code_generator_real_visit_cast_expression (ValaCodeVisitor* base, ValaCastExpression* expr);
static void vala_code_generator_real_visit_binary_expression (ValaCodeVisitor* base, ValaBinaryExpression* expr);
static void vala_code_generator_real_visit_type_check (ValaCodeVisitor* base, ValaTypeCheck* expr);
static void vala_code_generator_real_visit_conditional_expression (ValaCodeVisitor* base, ValaConditionalExpression* expr);
static void vala_code_generator_real_visit_end_lambda_expression (ValaCodeVisitor* base, ValaLambdaExpression* l);
static void vala_code_generator_real_visit_end_assignment (ValaCodeVisitor* base, ValaAssignment* a);
static gpointer vala_code_generator_parent_class = NULL;
static void vala_code_generator_dispose (GObject * obj);


ValaCodeGenerator* vala_code_generator_new (gboolean manage_memory)
{
	GParameter * __params = g_new0 (GParameter, 1);
	GParameter * __params_it = __params;
	(__params_it->name = "memory-management", g_value_init (&__params_it->value, G_TYPE_BOOLEAN), g_value_set_boolean (&__params_it->value, manage_memory), __params_it++);
	ValaCodeGenerator * self = g_object_newv (VALA_TYPE_CODE_GENERATOR, __params_it - __params, __params);
	GHashTable* __temp0 = NULL;
	self->priv->predefined_marshal_list = (__temp0 = g_hash_table_new (g_str_hash, g_str_equal), (self->priv->predefined_marshal_list == NULL ? NULL : (self->priv->predefined_marshal_list = (g_hash_table_unref (self->priv->predefined_marshal_list), NULL))), __temp0);
	g_hash_table_insert (self->priv->predefined_marshal_list, g_strdup ("VOID:VOID"), GINT_TO_POINTER (TRUE));
	g_hash_table_insert (self->priv->predefined_marshal_list, g_strdup ("VOID:BOOLEAN"), GINT_TO_POINTER (TRUE));
	g_hash_table_insert (self->priv->predefined_marshal_list, g_strdup ("VOID:CHAR"), GINT_TO_POINTER (TRUE));
	g_hash_table_insert (self->priv->predefined_marshal_list, g_strdup ("VOID:UCHAR"), GINT_TO_POINTER (TRUE));
	g_hash_table_insert (self->priv->predefined_marshal_list, g_strdup ("VOID:INT"), GINT_TO_POINTER (TRUE));
	g_hash_table_insert (self->priv->predefined_marshal_list, g_strdup ("VOID:UINT"), GINT_TO_POINTER (TRUE));
	g_hash_table_insert (self->priv->predefined_marshal_list, g_strdup ("VOID:LONG"), GINT_TO_POINTER (TRUE));
	g_hash_table_insert (self->priv->predefined_marshal_list, g_strdup ("VOID:ULONG"), GINT_TO_POINTER (TRUE));
	g_hash_table_insert (self->priv->predefined_marshal_list, g_strdup ("VOID:ENUM"), GINT_TO_POINTER (TRUE));
	g_hash_table_insert (self->priv->predefined_marshal_list, g_strdup ("VOID:FLAGS"), GINT_TO_POINTER (TRUE));
	g_hash_table_insert (self->priv->predefined_marshal_list, g_strdup ("VOID:FLOAT"), GINT_TO_POINTER (TRUE));
	g_hash_table_insert (self->priv->predefined_marshal_list, g_strdup ("VOID:DOUBLE"), GINT_TO_POINTER (TRUE));
	g_hash_table_insert (self->priv->predefined_marshal_list, g_strdup ("VOID:STRING"), GINT_TO_POINTER (TRUE));
	g_hash_table_insert (self->priv->predefined_marshal_list, g_strdup ("VOID:POINTER"), GINT_TO_POINTER (TRUE));
	g_hash_table_insert (self->priv->predefined_marshal_list, g_strdup ("VOID:OBJECT"), GINT_TO_POINTER (TRUE));
	g_hash_table_insert (self->priv->predefined_marshal_list, g_strdup ("STRING:OBJECT,POINTER"), GINT_TO_POINTER (TRUE));
	g_hash_table_insert (self->priv->predefined_marshal_list, g_strdup ("VOID:UINT,POINTER"), GINT_TO_POINTER (TRUE));
	g_hash_table_insert (self->priv->predefined_marshal_list, g_strdup ("BOOLEAN:FLAGS"), GINT_TO_POINTER (TRUE));
	return self;
}


/**
 * Generate and emit C code for the specified code context.
 *
 * @param context a code context
 */
void vala_code_generator_emit (ValaCodeGenerator* self, ValaCodeContext* context)
{
	g_return_if_fail (VALA_IS_CODE_GENERATOR (self));
	g_return_if_fail (VALA_IS_CODE_CONTEXT (context));
	ValaCodeContext* __temp1 = NULL;
	self->priv->context = (__temp1 = g_object_ref (context), (self->priv->context == NULL ? NULL : (self->priv->context = (g_object_unref (self->priv->context), NULL))), __temp1);
	vala_code_context_find_header_cycles (context);
	ValaSymbol* __temp2 = NULL;
	self->priv->root_symbol = (__temp2 = g_object_ref (vala_code_context_get_root (context)), (self->priv->root_symbol == NULL ? NULL : (self->priv->root_symbol = (g_object_unref (self->priv->root_symbol), NULL))), __temp2);
	ValaTypeReference* __temp3 = NULL;
	self->priv->bool_type = (__temp3 = vala_type_reference_new (), (self->priv->bool_type == NULL ? NULL : (self->priv->bool_type = (g_object_unref (self->priv->bool_type), NULL))), __temp3);
	vala_type_reference_set_data_type (self->priv->bool_type, VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "bool"))));
	ValaTypeReference* __temp4 = NULL;
	self->priv->char_type = (__temp4 = vala_type_reference_new (), (self->priv->char_type == NULL ? NULL : (self->priv->char_type = (g_object_unref (self->priv->char_type), NULL))), __temp4);
	vala_type_reference_set_data_type (self->priv->char_type, VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "char"))));
	ValaTypeReference* __temp5 = NULL;
	self->priv->unichar_type = (__temp5 = vala_type_reference_new (), (self->priv->unichar_type == NULL ? NULL : (self->priv->unichar_type = (g_object_unref (self->priv->unichar_type), NULL))), __temp5);
	vala_type_reference_set_data_type (self->priv->unichar_type, VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "unichar"))));
	ValaTypeReference* __temp6 = NULL;
	self->priv->short_type = (__temp6 = vala_type_reference_new (), (self->priv->short_type == NULL ? NULL : (self->priv->short_type = (g_object_unref (self->priv->short_type), NULL))), __temp6);
	vala_type_reference_set_data_type (self->priv->short_type, VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "short"))));
	ValaTypeReference* __temp7 = NULL;
	self->priv->ushort_type = (__temp7 = vala_type_reference_new (), (self->priv->ushort_type == NULL ? NULL : (self->priv->ushort_type = (g_object_unref (self->priv->ushort_type), NULL))), __temp7);
	vala_type_reference_set_data_type (self->priv->ushort_type, VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "ushort"))));
	ValaTypeReference* __temp8 = NULL;
	self->priv->int_type = (__temp8 = vala_type_reference_new (), (self->priv->int_type == NULL ? NULL : (self->priv->int_type = (g_object_unref (self->priv->int_type), NULL))), __temp8);
	vala_type_reference_set_data_type (self->priv->int_type, VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "int"))));
	ValaTypeReference* __temp9 = NULL;
	self->priv->uint_type = (__temp9 = vala_type_reference_new (), (self->priv->uint_type == NULL ? NULL : (self->priv->uint_type = (g_object_unref (self->priv->uint_type), NULL))), __temp9);
	vala_type_reference_set_data_type (self->priv->uint_type, VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "uint"))));
	ValaTypeReference* __temp10 = NULL;
	self->priv->long_type = (__temp10 = vala_type_reference_new (), (self->priv->long_type == NULL ? NULL : (self->priv->long_type = (g_object_unref (self->priv->long_type), NULL))), __temp10);
	vala_type_reference_set_data_type (self->priv->long_type, VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "long"))));
	ValaTypeReference* __temp11 = NULL;
	self->priv->ulong_type = (__temp11 = vala_type_reference_new (), (self->priv->ulong_type == NULL ? NULL : (self->priv->ulong_type = (g_object_unref (self->priv->ulong_type), NULL))), __temp11);
	vala_type_reference_set_data_type (self->priv->ulong_type, VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "ulong"))));
	ValaTypeReference* __temp12 = NULL;
	self->priv->int64_type = (__temp12 = vala_type_reference_new (), (self->priv->int64_type == NULL ? NULL : (self->priv->int64_type = (g_object_unref (self->priv->int64_type), NULL))), __temp12);
	vala_type_reference_set_data_type (self->priv->int64_type, VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "int64"))));
	ValaTypeReference* __temp13 = NULL;
	self->priv->uint64_type = (__temp13 = vala_type_reference_new (), (self->priv->uint64_type == NULL ? NULL : (self->priv->uint64_type = (g_object_unref (self->priv->uint64_type), NULL))), __temp13);
	vala_type_reference_set_data_type (self->priv->uint64_type, VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "uint64"))));
	ValaTypeReference* __temp14 = NULL;
	self->priv->float_type = (__temp14 = vala_type_reference_new (), (self->priv->float_type == NULL ? NULL : (self->priv->float_type = (g_object_unref (self->priv->float_type), NULL))), __temp14);
	vala_type_reference_set_data_type (self->priv->float_type, VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "float"))));
	ValaTypeReference* __temp15 = NULL;
	self->priv->double_type = (__temp15 = vala_type_reference_new (), (self->priv->double_type == NULL ? NULL : (self->priv->double_type = (g_object_unref (self->priv->double_type), NULL))), __temp15);
	vala_type_reference_set_data_type (self->priv->double_type, VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "double"))));
	ValaTypeReference* __temp16 = NULL;
	self->priv->string_type = (__temp16 = vala_type_reference_new (), (self->priv->string_type == NULL ? NULL : (self->priv->string_type = (g_object_unref (self->priv->string_type), NULL))), __temp16);
	vala_type_reference_set_data_type (self->priv->string_type, VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "string"))));
	ValaSymbol* glib_ns = vala_symbol_lookup (self->priv->root_symbol, "GLib");
	ValaDataType* __temp18 = NULL;
	ValaDataType* __temp17 = NULL;
	self->priv->list_type = (__temp18 = (__temp17 = VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_lookup (glib_ns, "List"))), (__temp17 == NULL ? NULL : g_object_ref (__temp17))), (self->priv->list_type == NULL ? NULL : (self->priv->list_type = (g_object_unref (self->priv->list_type), NULL))), __temp18);
	ValaDataType* __temp20 = NULL;
	ValaDataType* __temp19 = NULL;
	self->priv->slist_type = (__temp20 = (__temp19 = VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_lookup (glib_ns, "SList"))), (__temp19 == NULL ? NULL : g_object_ref (__temp19))), (self->priv->slist_type == NULL ? NULL : (self->priv->slist_type = (g_object_unref (self->priv->slist_type), NULL))), __temp20);
	ValaTypeReference* __temp21 = NULL;
	self->priv->mutex_type = (__temp21 = vala_type_reference_new (), (self->priv->mutex_type == NULL ? NULL : (self->priv->mutex_type = (g_object_unref (self->priv->mutex_type), NULL))), __temp21);
	vala_type_reference_set_data_type (self->priv->mutex_type, VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_lookup (glib_ns, "Mutex"))));
	/* we're only interested in non-pkg source files */
	GList* source_files = vala_code_context_get_source_files (context);
	{
		GList* __temp22 = NULL;
		__temp22 = source_files;
		GList* file_it;
		for (file_it = __temp22; file_it != NULL; file_it = file_it->next) {
			ValaSourceFile* file = file_it->data;
			{
				if (!vala_source_file_get_pkg (file)) {
					vala_source_file_accept (file, VALA_CODE_VISITOR (self));
				}
			}
		}
	}
	(source_files == NULL ? NULL : (source_files = (g_list_free (source_files), NULL)));
}


static ValaCCodeIncludeDirective* vala_code_generator_get_internal_include (ValaCodeGenerator* self, char* filename)
{
	g_return_val_if_fail (VALA_IS_CODE_GENERATOR (self), NULL);
	g_return_val_if_fail (filename != NULL, NULL);
	return vala_ccode_include_directive_new (filename, vala_code_context_get_library (self->priv->context) == NULL);
}


static void vala_code_generator_real_visit_begin_source_file (ValaCodeVisitor* base, ValaSourceFile* source_file)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_SOURCE_FILE (source_file));
	ValaCCodeFragment* __temp24 = NULL;
	self->priv->header_begin = (__temp24 = g_object_new (VALA_TYPE_CCODE_FRAGMENT, NULL), (self->priv->header_begin == NULL ? NULL : (self->priv->header_begin = (g_object_unref (self->priv->header_begin), NULL))), __temp24);
	ValaCCodeFragment* __temp25 = NULL;
	self->priv->header_type_declaration = (__temp25 = g_object_new (VALA_TYPE_CCODE_FRAGMENT, NULL), (self->priv->header_type_declaration == NULL ? NULL : (self->priv->header_type_declaration = (g_object_unref (self->priv->header_type_declaration), NULL))), __temp25);
	ValaCCodeFragment* __temp26 = NULL;
	self->priv->header_type_definition = (__temp26 = g_object_new (VALA_TYPE_CCODE_FRAGMENT, NULL), (self->priv->header_type_definition == NULL ? NULL : (self->priv->header_type_definition = (g_object_unref (self->priv->header_type_definition), NULL))), __temp26);
	ValaCCodeFragment* __temp27 = NULL;
	self->priv->header_type_member_declaration = (__temp27 = g_object_new (VALA_TYPE_CCODE_FRAGMENT, NULL), (self->priv->header_type_member_declaration == NULL ? NULL : (self->priv->header_type_member_declaration = (g_object_unref (self->priv->header_type_member_declaration), NULL))), __temp27);
	ValaCCodeFragment* __temp28 = NULL;
	self->priv->source_begin = (__temp28 = g_object_new (VALA_TYPE_CCODE_FRAGMENT, NULL), (self->priv->source_begin == NULL ? NULL : (self->priv->source_begin = (g_object_unref (self->priv->source_begin), NULL))), __temp28);
	ValaCCodeFragment* __temp29 = NULL;
	self->priv->source_include_directives = (__temp29 = g_object_new (VALA_TYPE_CCODE_FRAGMENT, NULL), (self->priv->source_include_directives == NULL ? NULL : (self->priv->source_include_directives = (g_object_unref (self->priv->source_include_directives), NULL))), __temp29);
	ValaCCodeFragment* __temp30 = NULL;
	self->priv->source_type_member_declaration = (__temp30 = g_object_new (VALA_TYPE_CCODE_FRAGMENT, NULL), (self->priv->source_type_member_declaration == NULL ? NULL : (self->priv->source_type_member_declaration = (g_object_unref (self->priv->source_type_member_declaration), NULL))), __temp30);
	ValaCCodeFragment* __temp31 = NULL;
	self->priv->source_type_member_definition = (__temp31 = g_object_new (VALA_TYPE_CCODE_FRAGMENT, NULL), (self->priv->source_type_member_definition == NULL ? NULL : (self->priv->source_type_member_definition = (g_object_unref (self->priv->source_type_member_definition), NULL))), __temp31);
	ValaCCodeFragment* __temp32 = NULL;
	self->priv->source_signal_marshaller_definition = (__temp32 = g_object_new (VALA_TYPE_CCODE_FRAGMENT, NULL), (self->priv->source_signal_marshaller_definition == NULL ? NULL : (self->priv->source_signal_marshaller_definition = (g_object_unref (self->priv->source_signal_marshaller_definition), NULL))), __temp32);
	ValaCCodeFragment* __temp33 = NULL;
	self->priv->source_signal_marshaller_declaration = (__temp33 = g_object_new (VALA_TYPE_CCODE_FRAGMENT, NULL), (self->priv->source_signal_marshaller_declaration == NULL ? NULL : (self->priv->source_signal_marshaller_declaration = (g_object_unref (self->priv->source_signal_marshaller_declaration), NULL))), __temp33);
	GHashTable* __temp34 = NULL;
	self->priv->user_marshal_list = (__temp34 = g_hash_table_new (g_str_hash, g_str_equal), (self->priv->user_marshal_list == NULL ? NULL : (self->priv->user_marshal_list = (g_hash_table_unref (self->priv->user_marshal_list), NULL))), __temp34);
	self->priv->next_temp_var_id = 0;
	ValaCCodeIncludeDirective* __temp35 = NULL;
	vala_ccode_fragment_append (self->priv->header_begin, VALA_CCODE_NODE ((__temp35 = vala_ccode_include_directive_new ("glib.h", FALSE))));
	(__temp35 == NULL ? NULL : (__temp35 = (g_object_unref (__temp35), NULL)));
	ValaCCodeIncludeDirective* __temp36 = NULL;
	vala_ccode_fragment_append (self->priv->header_begin, VALA_CCODE_NODE ((__temp36 = vala_ccode_include_directive_new ("glib-object.h", FALSE))));
	(__temp36 == NULL ? NULL : (__temp36 = (g_object_unref (__temp36), NULL)));
	ValaCCodeIncludeDirective* __temp37 = NULL;
	vala_ccode_fragment_append (self->priv->source_include_directives, VALA_CCODE_NODE ((__temp37 = vala_ccode_include_directive_new (vala_source_file_get_cheader_filename (source_file), TRUE))));
	(__temp37 == NULL ? NULL : (__temp37 = (g_object_unref (__temp37), NULL)));
	GList* used_includes = NULL;
	used_includes = g_list_append (used_includes, "glib.h");
	used_includes = g_list_append (used_includes, "glib-object.h");
	used_includes = g_list_append (used_includes, vala_source_file_get_cheader_filename (source_file));
	{
		GList* __temp39 = NULL;
		__temp39 = vala_source_file_get_header_external_includes (source_file);
		GList* filename1_it;
		for (filename1_it = __temp39; filename1_it != NULL; filename1_it = filename1_it->next) {
			char* filename1 = filename1_it->data;
			{
				if (g_list_find_custom (used_includes, filename1, (GCompareFunc) strcmp) == NULL) {
					ValaCCodeIncludeDirective* __temp38 = NULL;
					vala_ccode_fragment_append (self->priv->header_begin, VALA_CCODE_NODE ((__temp38 = vala_ccode_include_directive_new (filename1, FALSE))));
					(__temp38 == NULL ? NULL : (__temp38 = (g_object_unref (__temp38), NULL)));
					used_includes = g_list_append (used_includes, filename1);
				}
			}
		}
	}
	{
		GList* __temp41 = NULL;
		__temp41 = vala_source_file_get_header_internal_includes (source_file);
		GList* filename2_it;
		for (filename2_it = __temp41; filename2_it != NULL; filename2_it = filename2_it->next) {
			char* filename2 = filename2_it->data;
			{
				if (g_list_find_custom (used_includes, filename2, (GCompareFunc) strcmp) == NULL) {
					ValaCCodeIncludeDirective* __temp40 = NULL;
					vala_ccode_fragment_append (self->priv->header_begin, VALA_CCODE_NODE ((__temp40 = vala_code_generator_get_internal_include (self, filename2))));
					(__temp40 == NULL ? NULL : (__temp40 = (g_object_unref (__temp40), NULL)));
					used_includes = g_list_append (used_includes, filename2);
				}
			}
		}
	}
	{
		GList* __temp43 = NULL;
		__temp43 = vala_source_file_get_source_external_includes (source_file);
		GList* filename3_it;
		for (filename3_it = __temp43; filename3_it != NULL; filename3_it = filename3_it->next) {
			char* filename3 = filename3_it->data;
			{
				if (g_list_find_custom (used_includes, filename3, (GCompareFunc) strcmp) == NULL) {
					ValaCCodeIncludeDirective* __temp42 = NULL;
					vala_ccode_fragment_append (self->priv->source_include_directives, VALA_CCODE_NODE ((__temp42 = vala_ccode_include_directive_new (filename3, FALSE))));
					(__temp42 == NULL ? NULL : (__temp42 = (g_object_unref (__temp42), NULL)));
					used_includes = g_list_append (used_includes, filename3);
				}
			}
		}
	}
	{
		GList* __temp45 = NULL;
		__temp45 = vala_source_file_get_source_internal_includes (source_file);
		GList* filename4_it;
		for (filename4_it = __temp45; filename4_it != NULL; filename4_it = filename4_it->next) {
			char* filename4 = filename4_it->data;
			{
				if (g_list_find_custom (used_includes, filename4, (GCompareFunc) strcmp) == NULL) {
					ValaCCodeIncludeDirective* __temp44 = NULL;
					vala_ccode_fragment_append (self->priv->source_include_directives, VALA_CCODE_NODE ((__temp44 = vala_code_generator_get_internal_include (self, filename4))));
					(__temp44 == NULL ? NULL : (__temp44 = (g_object_unref (__temp44), NULL)));
					used_includes = g_list_append (used_includes, filename4);
				}
			}
		}
	}
	if (vala_source_file_get_is_cycle_head (source_file)) {
		{
			GList* __temp52 = NULL;
			__temp52 = vala_source_file_get_cycle (source_file)->files;
			GList* cycle_file_it;
			for (cycle_file_it = __temp52; cycle_file_it != NULL; cycle_file_it = cycle_file_it->next) {
				ValaSourceFile* cycle_file = cycle_file_it->data;
				{
					GList* namespaces = vala_source_file_get_namespaces (cycle_file);
					{
						GList* __temp51 = NULL;
						__temp51 = namespaces;
						GList* ns_it;
						for (ns_it = __temp51; ns_it != NULL; ns_it = ns_it->next) {
							ValaNamespace* ns = ns_it->data;
							{
								GList* structs = vala_namespace_get_structs (ns);
								{
									GList* __temp47 = NULL;
									__temp47 = structs;
									GList* st_it;
									for (st_it = __temp47; st_it != NULL; st_it = st_it->next) {
										ValaStruct* st = st_it->data;
										{
											ValaCCodeTypeDefinition* __temp46 = NULL;
											vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp46 = vala_ccode_type_definition_new (g_strdup_printf ("struct _%s", vala_data_type_get_cname (VALA_DATA_TYPE (st), FALSE)), VALA_CCODE_DECLARATOR (vala_ccode_variable_declarator_new (vala_data_type_get_cname (VALA_DATA_TYPE (st), FALSE)))))));
											(__temp46 == NULL ? NULL : (__temp46 = (g_object_unref (__temp46), NULL)));
										}
									}
								}
								GList* classes = vala_namespace_get_classes (ns);
								{
									GList* __temp50 = NULL;
									__temp50 = classes;
									GList* cl_it;
									for (cl_it = __temp50; cl_it != NULL; cl_it = cl_it->next) {
										ValaClass* cl = cl_it->data;
										{
											ValaCCodeTypeDefinition* __temp48 = NULL;
											vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp48 = vala_ccode_type_definition_new (g_strdup_printf ("struct _%s", vala_data_type_get_cname (VALA_DATA_TYPE (cl), FALSE)), VALA_CCODE_DECLARATOR (vala_ccode_variable_declarator_new (vala_data_type_get_cname (VALA_DATA_TYPE (cl), FALSE)))))));
											(__temp48 == NULL ? NULL : (__temp48 = (g_object_unref (__temp48), NULL)));
											ValaCCodeTypeDefinition* __temp49 = NULL;
											vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp49 = vala_ccode_type_definition_new (g_strdup_printf ("struct _%sClass", vala_data_type_get_cname (VALA_DATA_TYPE (cl), FALSE)), VALA_CCODE_DECLARATOR (vala_ccode_variable_declarator_new (g_strdup_printf ("%sClass", vala_data_type_get_cname (VALA_DATA_TYPE (cl), FALSE))))))));
											(__temp49 == NULL ? NULL : (__temp49 = (g_object_unref (__temp49), NULL)));
										}
									}
								}
								(structs == NULL ? NULL : (structs = (g_list_free (structs), NULL)));
								(classes == NULL ? NULL : (classes = (g_list_free (classes), NULL)));
							}
						}
					}
					(namespaces == NULL ? NULL : (namespaces = (g_list_free (namespaces), NULL)));
				}
			}
		}
	}
	/* generate hardcoded "well-known" macros */
	ValaCCodeMacroReplacement* __temp53 = NULL;
	vala_ccode_fragment_append (self->priv->source_begin, VALA_CCODE_NODE ((__temp53 = vala_ccode_macro_replacement_new ("VALA_FREE_CHECKED(o,f)", "((o) == NULL ? NULL : ((o) = (f (o), NULL)))"))));
	(__temp53 == NULL ? NULL : (__temp53 = (g_object_unref (__temp53), NULL)));
	ValaCCodeMacroReplacement* __temp54 = NULL;
	vala_ccode_fragment_append (self->priv->source_begin, VALA_CCODE_NODE ((__temp54 = vala_ccode_macro_replacement_new ("VALA_FREE_UNCHECKED(o,f)", "((o) = (f (o), NULL))"))));
	(__temp54 == NULL ? NULL : (__temp54 = (g_object_unref (__temp54), NULL)));
	(used_includes == NULL ? NULL : (used_includes = (g_list_free (used_includes), NULL)));
}


static char* vala_code_generator_get_define_for_filename (char* filename)
{
	g_return_val_if_fail (filename != NULL, NULL);
	GString* define = g_string_new ("__");
	char* i = filename;
	while (g_utf8_strlen (i, -1) > 0) {
		gunichar c = g_utf8_get_char (i);
		if (g_unichar_isalnum (c) && c < 0x80) {
			g_string_append_unichar (define, g_unichar_toupper (c));
		} else {
			g_string_append_c (define, '_');
		}
		i = g_utf8_next_char (i);
	}
	g_string_append (define, "__");
	const char* __temp55 = NULL;
	char* __temp56 = NULL;
	return (__temp56 = (__temp55 = define->str, (__temp55 == NULL ? NULL : g_strdup (__temp55))), (define == NULL ? NULL : (define = (g_string_free (define, TRUE), NULL))), __temp56);
	(define == NULL ? NULL : (define = (g_string_free (define, TRUE), NULL)));
}


static void vala_code_generator_real_visit_end_source_file (ValaCodeVisitor* base, ValaSourceFile* source_file)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_SOURCE_FILE (source_file));
	char* header_define = vala_code_generator_get_define_for_filename (vala_source_file_get_cheader_filename (source_file));
	ValaCCodeComment* comment = NULL;
	if (vala_source_file_get_comment (source_file) != NULL) {
		ValaCCodeComment* __temp57 = NULL;
		comment = (__temp57 = vala_ccode_comment_new (vala_source_file_get_comment (source_file)), (comment == NULL ? NULL : (comment = (g_object_unref (comment), NULL))), __temp57);
	}
	ValaCCodeWriter* writer = vala_ccode_writer_new (vala_source_file_get_cheader_filename (source_file));
	if (comment != NULL) {
		vala_ccode_node_write (VALA_CCODE_NODE (comment), writer);
	}
	vala_ccode_writer_write_newline (writer);
	ValaCCodeOnceSection* once = vala_ccode_once_section_new (header_define);
	ValaCCodeNewline* __temp58 = NULL;
	vala_ccode_fragment_append (VALA_CCODE_FRAGMENT (once), VALA_CCODE_NODE ((__temp58 = g_object_new (VALA_TYPE_CCODE_NEWLINE, NULL))));
	(__temp58 == NULL ? NULL : (__temp58 = (g_object_unref (__temp58), NULL)));
	vala_ccode_fragment_append (VALA_CCODE_FRAGMENT (once), VALA_CCODE_NODE (self->priv->header_begin));
	ValaCCodeNewline* __temp59 = NULL;
	vala_ccode_fragment_append (VALA_CCODE_FRAGMENT (once), VALA_CCODE_NODE ((__temp59 = g_object_new (VALA_TYPE_CCODE_NEWLINE, NULL))));
	(__temp59 == NULL ? NULL : (__temp59 = (g_object_unref (__temp59), NULL)));
	ValaCCodeIdentifier* __temp60 = NULL;
	vala_ccode_fragment_append (VALA_CCODE_FRAGMENT (once), VALA_CCODE_NODE ((__temp60 = vala_ccode_identifier_new ("G_BEGIN_DECLS"))));
	(__temp60 == NULL ? NULL : (__temp60 = (g_object_unref (__temp60), NULL)));
	ValaCCodeNewline* __temp61 = NULL;
	vala_ccode_fragment_append (VALA_CCODE_FRAGMENT (once), VALA_CCODE_NODE ((__temp61 = g_object_new (VALA_TYPE_CCODE_NEWLINE, NULL))));
	(__temp61 == NULL ? NULL : (__temp61 = (g_object_unref (__temp61), NULL)));
	ValaCCodeNewline* __temp62 = NULL;
	vala_ccode_fragment_append (VALA_CCODE_FRAGMENT (once), VALA_CCODE_NODE ((__temp62 = g_object_new (VALA_TYPE_CCODE_NEWLINE, NULL))));
	(__temp62 == NULL ? NULL : (__temp62 = (g_object_unref (__temp62), NULL)));
	vala_ccode_fragment_append (VALA_CCODE_FRAGMENT (once), VALA_CCODE_NODE (self->priv->header_type_declaration));
	ValaCCodeNewline* __temp63 = NULL;
	vala_ccode_fragment_append (VALA_CCODE_FRAGMENT (once), VALA_CCODE_NODE ((__temp63 = g_object_new (VALA_TYPE_CCODE_NEWLINE, NULL))));
	(__temp63 == NULL ? NULL : (__temp63 = (g_object_unref (__temp63), NULL)));
	vala_ccode_fragment_append (VALA_CCODE_FRAGMENT (once), VALA_CCODE_NODE (self->priv->header_type_definition));
	ValaCCodeNewline* __temp64 = NULL;
	vala_ccode_fragment_append (VALA_CCODE_FRAGMENT (once), VALA_CCODE_NODE ((__temp64 = g_object_new (VALA_TYPE_CCODE_NEWLINE, NULL))));
	(__temp64 == NULL ? NULL : (__temp64 = (g_object_unref (__temp64), NULL)));
	vala_ccode_fragment_append (VALA_CCODE_FRAGMENT (once), VALA_CCODE_NODE (self->priv->header_type_member_declaration));
	ValaCCodeNewline* __temp65 = NULL;
	vala_ccode_fragment_append (VALA_CCODE_FRAGMENT (once), VALA_CCODE_NODE ((__temp65 = g_object_new (VALA_TYPE_CCODE_NEWLINE, NULL))));
	(__temp65 == NULL ? NULL : (__temp65 = (g_object_unref (__temp65), NULL)));
	ValaCCodeIdentifier* __temp66 = NULL;
	vala_ccode_fragment_append (VALA_CCODE_FRAGMENT (once), VALA_CCODE_NODE ((__temp66 = vala_ccode_identifier_new ("G_END_DECLS"))));
	(__temp66 == NULL ? NULL : (__temp66 = (g_object_unref (__temp66), NULL)));
	ValaCCodeNewline* __temp67 = NULL;
	vala_ccode_fragment_append (VALA_CCODE_FRAGMENT (once), VALA_CCODE_NODE ((__temp67 = g_object_new (VALA_TYPE_CCODE_NEWLINE, NULL))));
	(__temp67 == NULL ? NULL : (__temp67 = (g_object_unref (__temp67), NULL)));
	ValaCCodeNewline* __temp68 = NULL;
	vala_ccode_fragment_append (VALA_CCODE_FRAGMENT (once), VALA_CCODE_NODE ((__temp68 = g_object_new (VALA_TYPE_CCODE_NEWLINE, NULL))));
	(__temp68 == NULL ? NULL : (__temp68 = (g_object_unref (__temp68), NULL)));
	vala_ccode_node_write (VALA_CCODE_NODE (once), writer);
	vala_ccode_writer_close (writer);
	ValaCCodeWriter* __temp69 = NULL;
	writer = (__temp69 = vala_ccode_writer_new (vala_source_file_get_csource_filename (source_file)), (writer == NULL ? NULL : (writer = (g_object_unref (writer), NULL))), __temp69);
	if (comment != NULL) {
		vala_ccode_node_write (VALA_CCODE_NODE (comment), writer);
	}
	vala_ccode_node_write (VALA_CCODE_NODE (self->priv->source_begin), writer);
	vala_ccode_writer_write_newline (writer);
	vala_ccode_node_write (VALA_CCODE_NODE (self->priv->source_include_directives), writer);
	vala_ccode_writer_write_newline (writer);
	vala_ccode_node_write (VALA_CCODE_NODE (self->priv->source_type_member_declaration), writer);
	vala_ccode_writer_write_newline (writer);
	vala_ccode_node_write (VALA_CCODE_NODE (self->priv->source_signal_marshaller_declaration), writer);
	vala_ccode_writer_write_newline (writer);
	vala_ccode_node_write (VALA_CCODE_NODE (self->priv->source_type_member_definition), writer);
	vala_ccode_writer_write_newline (writer);
	vala_ccode_node_write (VALA_CCODE_NODE (self->priv->source_signal_marshaller_definition), writer);
	vala_ccode_writer_write_newline (writer);
	vala_ccode_writer_close (writer);
	ValaCCodeFragment* __temp70 = NULL;
	self->priv->header_begin = (__temp70 = NULL, (self->priv->header_begin == NULL ? NULL : (self->priv->header_begin = (g_object_unref (self->priv->header_begin), NULL))), __temp70);
	ValaCCodeFragment* __temp71 = NULL;
	self->priv->header_type_declaration = (__temp71 = NULL, (self->priv->header_type_declaration == NULL ? NULL : (self->priv->header_type_declaration = (g_object_unref (self->priv->header_type_declaration), NULL))), __temp71);
	ValaCCodeFragment* __temp72 = NULL;
	self->priv->header_type_definition = (__temp72 = NULL, (self->priv->header_type_definition == NULL ? NULL : (self->priv->header_type_definition = (g_object_unref (self->priv->header_type_definition), NULL))), __temp72);
	ValaCCodeFragment* __temp73 = NULL;
	self->priv->header_type_member_declaration = (__temp73 = NULL, (self->priv->header_type_member_declaration == NULL ? NULL : (self->priv->header_type_member_declaration = (g_object_unref (self->priv->header_type_member_declaration), NULL))), __temp73);
	ValaCCodeFragment* __temp74 = NULL;
	self->priv->source_begin = (__temp74 = NULL, (self->priv->source_begin == NULL ? NULL : (self->priv->source_begin = (g_object_unref (self->priv->source_begin), NULL))), __temp74);
	ValaCCodeFragment* __temp75 = NULL;
	self->priv->source_include_directives = (__temp75 = NULL, (self->priv->source_include_directives == NULL ? NULL : (self->priv->source_include_directives = (g_object_unref (self->priv->source_include_directives), NULL))), __temp75);
	ValaCCodeFragment* __temp76 = NULL;
	self->priv->source_type_member_declaration = (__temp76 = NULL, (self->priv->source_type_member_declaration == NULL ? NULL : (self->priv->source_type_member_declaration = (g_object_unref (self->priv->source_type_member_declaration), NULL))), __temp76);
	ValaCCodeFragment* __temp77 = NULL;
	self->priv->source_type_member_definition = (__temp77 = NULL, (self->priv->source_type_member_definition == NULL ? NULL : (self->priv->source_type_member_definition = (g_object_unref (self->priv->source_type_member_definition), NULL))), __temp77);
	ValaCCodeFragment* __temp78 = NULL;
	self->priv->source_signal_marshaller_definition = (__temp78 = NULL, (self->priv->source_signal_marshaller_definition == NULL ? NULL : (self->priv->source_signal_marshaller_definition = (g_object_unref (self->priv->source_signal_marshaller_definition), NULL))), __temp78);
	ValaCCodeFragment* __temp79 = NULL;
	self->priv->source_signal_marshaller_declaration = (__temp79 = NULL, (self->priv->source_signal_marshaller_declaration == NULL ? NULL : (self->priv->source_signal_marshaller_declaration = (g_object_unref (self->priv->source_signal_marshaller_declaration), NULL))), __temp79);
	(header_define == NULL ? NULL : (header_define = (g_free (header_define), NULL)));
	(comment == NULL ? NULL : (comment = (g_object_unref (comment), NULL)));
	(writer == NULL ? NULL : (writer = (g_object_unref (writer), NULL)));
	(once == NULL ? NULL : (once = (g_object_unref (once), NULL)));
}


static void vala_code_generator_real_visit_begin_class (ValaCodeVisitor* base, ValaClass* cl)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_CLASS (cl));
	ValaSymbol* __temp81 = NULL;
	ValaSymbol* __temp80 = NULL;
	self->priv->current_symbol = (__temp81 = (__temp80 = vala_code_node_get_symbol (VALA_CODE_NODE (cl)), (__temp80 == NULL ? NULL : g_object_ref (__temp80))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp81);
	ValaSymbol* __temp83 = NULL;
	ValaSymbol* __temp82 = NULL;
	self->priv->current_type_symbol = (__temp83 = (__temp82 = vala_code_node_get_symbol (VALA_CODE_NODE (cl)), (__temp82 == NULL ? NULL : g_object_ref (__temp82))), (self->priv->current_type_symbol == NULL ? NULL : (self->priv->current_type_symbol = (g_object_unref (self->priv->current_type_symbol), NULL))), __temp83);
	ValaClass* __temp84 = NULL;
	self->priv->current_class = (__temp84 = g_object_ref (cl), (self->priv->current_class == NULL ? NULL : (self->priv->current_class = (g_object_unref (self->priv->current_class), NULL))), __temp84);
	ValaCCodeStruct* __temp85 = NULL;
	self->priv->instance_struct = (__temp85 = vala_ccode_struct_new (g_strdup_printf ("_%s", vala_data_type_get_cname (VALA_DATA_TYPE (cl), FALSE))), (self->priv->instance_struct == NULL ? NULL : (self->priv->instance_struct = (g_object_unref (self->priv->instance_struct), NULL))), __temp85);
	ValaCCodeStruct* __temp86 = NULL;
	self->priv->type_struct = (__temp86 = vala_ccode_struct_new (g_strdup_printf ("_%sClass", vala_data_type_get_cname (VALA_DATA_TYPE (cl), FALSE))), (self->priv->type_struct == NULL ? NULL : (self->priv->type_struct = (g_object_unref (self->priv->type_struct), NULL))), __temp86);
	ValaCCodeStruct* __temp87 = NULL;
	self->priv->instance_priv_struct = (__temp87 = vala_ccode_struct_new (g_strdup_printf ("_%sPrivate", vala_data_type_get_cname (VALA_DATA_TYPE (cl), FALSE))), (self->priv->instance_priv_struct == NULL ? NULL : (self->priv->instance_priv_struct = (g_object_unref (self->priv->instance_priv_struct), NULL))), __temp87);
	ValaCCodeEnum* __temp88 = NULL;
	self->priv->prop_enum = (__temp88 = vala_ccode_enum_new (NULL), (self->priv->prop_enum == NULL ? NULL : (self->priv->prop_enum = (g_object_unref (self->priv->prop_enum), NULL))), __temp88);
	char* __temp90 = NULL;
	char* __temp89 = NULL;
	vala_ccode_enum_add_value (self->priv->prop_enum, (__temp90 = g_strdup_printf ("%s_DUMMY_PROPERTY", (__temp89 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), NULL)))), NULL);
	(__temp90 == NULL ? NULL : (__temp90 = (g_free (__temp90), NULL)));
	(__temp89 == NULL ? NULL : (__temp89 = (g_free (__temp89), NULL)));
	ValaCCodeFragment* __temp91 = NULL;
	self->priv->instance_init_fragment = (__temp91 = g_object_new (VALA_TYPE_CCODE_FRAGMENT, NULL), (self->priv->instance_init_fragment == NULL ? NULL : (self->priv->instance_init_fragment = (g_object_unref (self->priv->instance_init_fragment), NULL))), __temp91);
	ValaCCodeFragment* __temp92 = NULL;
	self->priv->instance_dispose_fragment = (__temp92 = g_object_new (VALA_TYPE_CCODE_FRAGMENT, NULL), (self->priv->instance_dispose_fragment == NULL ? NULL : (self->priv->instance_dispose_fragment = (g_object_unref (self->priv->instance_dispose_fragment), NULL))), __temp92);
	ValaCCodeNewline* __temp93 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp93 = g_object_new (VALA_TYPE_CCODE_NEWLINE, NULL))));
	(__temp93 == NULL ? NULL : (__temp93 = (g_object_unref (__temp93), NULL)));
	char* __temp94 = NULL;
	char* __temp95 = NULL;
	char* macro = (__temp95 = g_strdup_printf ("(%s_get_type ())", (__temp94 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL))), (__temp94 == NULL ? NULL : (__temp94 = (g_free (__temp94), NULL))), __temp95);
	ValaCCodeMacroReplacement* __temp96 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp96 = vala_ccode_macro_replacement_new (vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), "TYPE_"), macro))));
	(__temp96 == NULL ? NULL : (__temp96 = (g_object_unref (__temp96), NULL)));
	char* __temp98 = NULL;
	char* __temp97 = NULL;
	macro = (__temp98 = g_strdup_printf ("(G_TYPE_CHECK_INSTANCE_CAST ((obj), %s, %s))", (__temp97 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), "TYPE_")), vala_data_type_get_cname (VALA_DATA_TYPE (cl), FALSE)), (macro == NULL ? NULL : (macro = (g_free (macro), NULL))), __temp98);
	(__temp97 == NULL ? NULL : (__temp97 = (g_free (__temp97), NULL)));
	ValaCCodeMacroReplacement* __temp100 = NULL;
	char* __temp99 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp100 = vala_ccode_macro_replacement_new (g_strdup_printf ("%s(obj)", (__temp99 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), NULL))), macro))));
	(__temp100 == NULL ? NULL : (__temp100 = (g_object_unref (__temp100), NULL)));
	(__temp99 == NULL ? NULL : (__temp99 = (g_free (__temp99), NULL)));
	char* __temp102 = NULL;
	char* __temp101 = NULL;
	macro = (__temp102 = g_strdup_printf ("(G_TYPE_CHECK_CLASS_CAST ((klass), %s, %sClass))", (__temp101 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), "TYPE_")), vala_data_type_get_cname (VALA_DATA_TYPE (cl), FALSE)), (macro == NULL ? NULL : (macro = (g_free (macro), NULL))), __temp102);
	(__temp101 == NULL ? NULL : (__temp101 = (g_free (__temp101), NULL)));
	ValaCCodeMacroReplacement* __temp104 = NULL;
	char* __temp103 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp104 = vala_ccode_macro_replacement_new (g_strdup_printf ("%s_CLASS(klass)", (__temp103 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), NULL))), macro))));
	(__temp104 == NULL ? NULL : (__temp104 = (g_object_unref (__temp104), NULL)));
	(__temp103 == NULL ? NULL : (__temp103 = (g_free (__temp103), NULL)));
	char* __temp106 = NULL;
	char* __temp105 = NULL;
	macro = (__temp106 = g_strdup_printf ("(G_TYPE_CHECK_INSTANCE_TYPE ((obj), %s))", (__temp105 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), "TYPE_"))), (macro == NULL ? NULL : (macro = (g_free (macro), NULL))), __temp106);
	(__temp105 == NULL ? NULL : (__temp105 = (g_free (__temp105), NULL)));
	ValaCCodeMacroReplacement* __temp108 = NULL;
	char* __temp107 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp108 = vala_ccode_macro_replacement_new (g_strdup_printf ("%s(obj)", (__temp107 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), "IS_"))), macro))));
	(__temp108 == NULL ? NULL : (__temp108 = (g_object_unref (__temp108), NULL)));
	(__temp107 == NULL ? NULL : (__temp107 = (g_free (__temp107), NULL)));
	char* __temp110 = NULL;
	char* __temp109 = NULL;
	macro = (__temp110 = g_strdup_printf ("(G_TYPE_CHECK_CLASS_TYPE ((klass), %s))", (__temp109 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), "TYPE_"))), (macro == NULL ? NULL : (macro = (g_free (macro), NULL))), __temp110);
	(__temp109 == NULL ? NULL : (__temp109 = (g_free (__temp109), NULL)));
	ValaCCodeMacroReplacement* __temp112 = NULL;
	char* __temp111 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp112 = vala_ccode_macro_replacement_new (g_strdup_printf ("%s_CLASS(klass)", (__temp111 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), "IS_"))), macro))));
	(__temp112 == NULL ? NULL : (__temp112 = (g_object_unref (__temp112), NULL)));
	(__temp111 == NULL ? NULL : (__temp111 = (g_free (__temp111), NULL)));
	char* __temp114 = NULL;
	char* __temp113 = NULL;
	macro = (__temp114 = g_strdup_printf ("(G_TYPE_INSTANCE_GET_CLASS ((obj), %s, %sClass))", (__temp113 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), "TYPE_")), vala_data_type_get_cname (VALA_DATA_TYPE (cl), FALSE)), (macro == NULL ? NULL : (macro = (g_free (macro), NULL))), __temp114);
	(__temp113 == NULL ? NULL : (__temp113 = (g_free (__temp113), NULL)));
	ValaCCodeMacroReplacement* __temp116 = NULL;
	char* __temp115 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp116 = vala_ccode_macro_replacement_new (g_strdup_printf ("%s_GET_CLASS(obj)", (__temp115 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), NULL))), macro))));
	(__temp116 == NULL ? NULL : (__temp116 = (g_object_unref (__temp116), NULL)));
	(__temp115 == NULL ? NULL : (__temp115 = (g_free (__temp115), NULL)));
	ValaCCodeNewline* __temp117 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp117 = g_object_new (VALA_TYPE_CCODE_NEWLINE, NULL))));
	(__temp117 == NULL ? NULL : (__temp117 = (g_object_unref (__temp117), NULL)));
	if (vala_source_file_get_cycle (vala_source_reference_get_file (vala_code_node_get_source_reference (VALA_CODE_NODE (cl)))) == NULL) {
		ValaCCodeTypeDefinition* __temp118 = NULL;
		vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp118 = vala_ccode_type_definition_new (g_strdup_printf ("struct %s", vala_ccode_struct_get_name (self->priv->instance_struct)), VALA_CCODE_DECLARATOR (vala_ccode_variable_declarator_new (vala_data_type_get_cname (VALA_DATA_TYPE (cl), FALSE)))))));
		(__temp118 == NULL ? NULL : (__temp118 = (g_object_unref (__temp118), NULL)));
		ValaCCodeTypeDefinition* __temp119 = NULL;
		vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp119 = vala_ccode_type_definition_new (g_strdup_printf ("struct %s", vala_ccode_struct_get_name (self->priv->type_struct)), VALA_CCODE_DECLARATOR (vala_ccode_variable_declarator_new (g_strdup_printf ("%sClass", vala_data_type_get_cname (VALA_DATA_TYPE (cl), FALSE))))))));
		(__temp119 == NULL ? NULL : (__temp119 = (g_object_unref (__temp119), NULL)));
	}
	ValaCCodeTypeDefinition* __temp120 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp120 = vala_ccode_type_definition_new (g_strdup_printf ("struct %s", vala_ccode_struct_get_name (self->priv->instance_priv_struct)), VALA_CCODE_DECLARATOR (vala_ccode_variable_declarator_new (g_strdup_printf ("%sPrivate", vala_data_type_get_cname (VALA_DATA_TYPE (cl), FALSE))))))));
	(__temp120 == NULL ? NULL : (__temp120 = (g_object_unref (__temp120), NULL)));
	vala_ccode_struct_add_field (self->priv->instance_struct, vala_data_type_get_cname (VALA_DATA_TYPE (vala_class_get_base_class (cl)), FALSE), "parent");
	char* __temp121 = NULL;
	vala_ccode_struct_add_field (self->priv->instance_struct, (__temp121 = g_strdup_printf ("%sPrivate *", vala_data_type_get_cname (VALA_DATA_TYPE (cl), FALSE))), "priv");
	(__temp121 == NULL ? NULL : (__temp121 = (g_free (__temp121), NULL)));
	char* __temp122 = NULL;
	vala_ccode_struct_add_field (self->priv->type_struct, (__temp122 = g_strdup_printf ("%sClass", vala_data_type_get_cname (VALA_DATA_TYPE (vala_class_get_base_class (cl)), FALSE))), "parent");
	(__temp122 == NULL ? NULL : (__temp122 = (g_free (__temp122), NULL)));
	if (vala_source_reference_get_comment (vala_code_node_get_source_reference (VALA_CODE_NODE (cl))) != NULL) {
		ValaCCodeComment* __temp123 = NULL;
		vala_ccode_fragment_append (self->priv->header_type_definition, VALA_CCODE_NODE ((__temp123 = vala_ccode_comment_new (vala_source_reference_get_comment (vala_code_node_get_source_reference (VALA_CODE_NODE (cl)))))));
		(__temp123 == NULL ? NULL : (__temp123 = (g_object_unref (__temp123), NULL)));
	}
	vala_ccode_fragment_append (self->priv->header_type_definition, VALA_CCODE_NODE (self->priv->instance_struct));
	vala_ccode_fragment_append (self->priv->header_type_definition, VALA_CCODE_NODE (self->priv->type_struct));
	vala_ccode_fragment_append (self->priv->source_type_member_declaration, VALA_CCODE_NODE (self->priv->instance_priv_struct));
	char* __temp125 = NULL;
	char* __temp124 = NULL;
	macro = (__temp125 = g_strdup_printf ("(G_TYPE_INSTANCE_GET_PRIVATE ((o), %s, %sPrivate))", (__temp124 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), "TYPE_")), vala_data_type_get_cname (VALA_DATA_TYPE (cl), FALSE)), (macro == NULL ? NULL : (macro = (g_free (macro), NULL))), __temp125);
	(__temp124 == NULL ? NULL : (__temp124 = (g_free (__temp124), NULL)));
	ValaCCodeMacroReplacement* __temp127 = NULL;
	char* __temp126 = NULL;
	vala_ccode_fragment_append (self->priv->source_type_member_declaration, VALA_CCODE_NODE ((__temp127 = vala_ccode_macro_replacement_new (g_strdup_printf ("%s_GET_PRIVATE(o)", (__temp126 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), NULL))), macro))));
	(__temp127 == NULL ? NULL : (__temp127 = (g_object_unref (__temp127), NULL)));
	(__temp126 == NULL ? NULL : (__temp126 = (g_free (__temp126), NULL)));
	vala_ccode_fragment_append (self->priv->source_type_member_declaration, VALA_CCODE_NODE (self->priv->prop_enum));
	(macro == NULL ? NULL : (macro = (g_free (macro), NULL)));
}


static void vala_code_generator_real_visit_end_class (ValaCodeVisitor* base, ValaClass* cl)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_CLASS (cl));
	vala_code_generator_add_get_property_function (self, cl);
	vala_code_generator_add_set_property_function (self, cl);
	vala_code_generator_add_class_init_function (self, cl);
	{
		GList* __temp128 = NULL;
		__temp128 = vala_class_get_base_types (cl);
		GList* base_type_it;
		for (base_type_it = __temp128; base_type_it != NULL; base_type_it = base_type_it->next) {
			ValaTypeReference* base_type = base_type_it->data;
			{
				if (VALA_IS_INTERFACE (vala_type_reference_get_data_type (base_type))) {
					vala_code_generator_add_interface_init_function (self, cl, VALA_INTERFACE (vala_type_reference_get_data_type (base_type)));
				}
			}
		}
		(__temp128 == NULL ? NULL : (__temp128 = (g_list_free (__temp128), NULL)));
	}
	vala_code_generator_add_instance_init_function (self, cl);
	GList* __temp129 = NULL;
	gboolean __temp130;
	if ((__temp130 = vala_code_generator_get_memory_management (self) && (__temp129 = vala_class_get_fields (cl)) != NULL, (__temp129 == NULL ? NULL : (__temp129 = (g_list_free (__temp129), NULL))), __temp130)) {
		vala_code_generator_add_dispose_function (self, cl);
	}
	ValaClassRegisterFunction* type_fun = vala_class_register_function_new (cl);
	vala_type_register_function_init_from_type (VALA_TYPE_REGISTER_FUNCTION (type_fun));
	ValaCCodeFunction* __temp131 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_member_declaration, VALA_CCODE_NODE ((__temp131 = vala_type_register_function_get_declaration (VALA_TYPE_REGISTER_FUNCTION (type_fun)))));
	(__temp131 == NULL ? NULL : (__temp131 = (g_object_unref (__temp131), NULL)));
	vala_ccode_fragment_append (self->priv->source_type_member_definition, VALA_CCODE_NODE (type_fun));
	ValaSymbol* __temp132 = NULL;
	self->priv->current_type_symbol = (__temp132 = NULL, (self->priv->current_type_symbol == NULL ? NULL : (self->priv->current_type_symbol = (g_object_unref (self->priv->current_type_symbol), NULL))), __temp132);
	ValaClass* __temp133 = NULL;
	self->priv->current_class = (__temp133 = NULL, (self->priv->current_class == NULL ? NULL : (self->priv->current_class = (g_object_unref (self->priv->current_class), NULL))), __temp133);
	(type_fun == NULL ? NULL : (type_fun = (g_object_unref (type_fun), NULL)));
}


static void vala_code_generator_add_class_init_function (ValaCodeGenerator* self, ValaClass* cl)
{
	g_return_if_fail (VALA_IS_CODE_GENERATOR (self));
	g_return_if_fail (VALA_IS_CLASS (cl));
	char* __temp134 = NULL;
	ValaCCodeFunction* __temp135 = NULL;
	ValaCCodeFunction* class_init = (__temp135 = vala_ccode_function_new (g_strdup_printf ("%s_class_init", (__temp134 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL))), "void"), (__temp134 == NULL ? NULL : (__temp134 = (g_free (__temp134), NULL))), __temp135);
	ValaCCodeFormalParameter* __temp136 = NULL;
	vala_ccode_function_add_parameter (class_init, (__temp136 = vala_ccode_formal_parameter_new ("klass", g_strdup_printf ("%sClass *", vala_data_type_get_cname (VALA_DATA_TYPE (cl), FALSE)))));
	(__temp136 == NULL ? NULL : (__temp136 = (g_object_unref (__temp136), NULL)));
	vala_ccode_function_set_modifiers (class_init, VALA_CCODE_MODIFIERS_STATIC);
	ValaCCodeBlock* init_block = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
	vala_ccode_function_set_block (class_init, init_block);
	ValaCCodeFunctionCall* ccall = NULL;
	/* save pointer to parent class */
	ValaCCodeDeclaration* parent_decl = vala_ccode_declaration_new ("gpointer");
	char* __temp137 = NULL;
	ValaCCodeVariableDeclarator* __temp138 = NULL;
	ValaCCodeVariableDeclarator* parent_var_decl = (__temp138 = vala_ccode_variable_declarator_new (g_strdup_printf ("%s_parent_class", (__temp137 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL)))), (__temp137 == NULL ? NULL : (__temp137 = (g_free (__temp137), NULL))), __temp138);
	ValaCCodeConstant* __temp139 = NULL;
	vala_ccode_variable_declarator_set_initializer (parent_var_decl, VALA_CCODE_EXPRESSION ((__temp139 = vala_ccode_constant_new ("NULL"))));
	(__temp139 == NULL ? NULL : (__temp139 = (g_object_unref (__temp139), NULL)));
	vala_ccode_declaration_add_declarator (parent_decl, VALA_CCODE_DECLARATOR (parent_var_decl));
	vala_ccode_declaration_set_modifiers (parent_decl, VALA_CCODE_MODIFIERS_STATIC);
	vala_ccode_fragment_append (self->priv->source_type_member_declaration, VALA_CCODE_NODE (parent_decl));
	ValaCCodeFunctionCall* __temp140 = NULL;
	ccall = (__temp140 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("g_type_class_peek_parent"))), (ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL))), __temp140);
	ValaCCodeIdentifier* __temp141 = NULL;
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp141 = vala_ccode_identifier_new ("klass"))));
	(__temp141 == NULL ? NULL : (__temp141 = (g_object_unref (__temp141), NULL)));
	char* __temp142 = NULL;
	ValaCCodeAssignment* __temp143 = NULL;
	ValaCCodeAssignment* parent_assignment = (__temp143 = vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (g_strdup_printf ("%s_parent_class", (__temp142 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL))))), VALA_CCODE_EXPRESSION (ccall), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE), (__temp142 == NULL ? NULL : (__temp142 = (g_free (__temp142), NULL))), __temp143);
	ValaCCodeExpressionStatement* __temp144 = NULL;
	vala_ccode_block_add_statement (init_block, VALA_CCODE_NODE ((__temp144 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (parent_assignment)))));
	(__temp144 == NULL ? NULL : (__temp144 = (g_object_unref (__temp144), NULL)));
	/* add struct for private fields */
	if (vala_class_get_has_private_fields (cl)) {
		ValaCCodeFunctionCall* __temp145 = NULL;
		ccall = (__temp145 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("g_type_class_add_private"))), (ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL))), __temp145);
		ValaCCodeIdentifier* __temp146 = NULL;
		vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp146 = vala_ccode_identifier_new ("klass"))));
		(__temp146 == NULL ? NULL : (__temp146 = (g_object_unref (__temp146), NULL)));
		ValaCCodeConstant* __temp147 = NULL;
		vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp147 = vala_ccode_constant_new (g_strdup_printf ("sizeof (%sPrivate)", vala_data_type_get_cname (VALA_DATA_TYPE (cl), FALSE))))));
		(__temp147 == NULL ? NULL : (__temp147 = (g_object_unref (__temp147), NULL)));
		ValaCCodeExpressionStatement* __temp148 = NULL;
		vala_ccode_block_add_statement (init_block, VALA_CCODE_NODE ((__temp148 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (ccall)))));
		(__temp148 == NULL ? NULL : (__temp148 = (g_object_unref (__temp148), NULL)));
	}
	/* set property handlers */
	ValaCCodeFunctionCall* __temp149 = NULL;
	ccall = (__temp149 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("G_OBJECT_CLASS"))), (ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL))), __temp149);
	ValaCCodeIdentifier* __temp150 = NULL;
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp150 = vala_ccode_identifier_new ("klass"))));
	(__temp150 == NULL ? NULL : (__temp150 = (g_object_unref (__temp150), NULL)));
	ValaCCodeExpressionStatement* __temp152 = NULL;
	char* __temp151 = NULL;
	vala_ccode_block_add_statement (init_block, VALA_CCODE_NODE ((__temp152 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_member_access_new_pointer (VALA_CCODE_EXPRESSION (ccall), "get_property")), VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (g_strdup_printf ("%s_get_property", (__temp151 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL))))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))))));
	(__temp152 == NULL ? NULL : (__temp152 = (g_object_unref (__temp152), NULL)));
	(__temp151 == NULL ? NULL : (__temp151 = (g_free (__temp151), NULL)));
	ValaCCodeExpressionStatement* __temp154 = NULL;
	char* __temp153 = NULL;
	vala_ccode_block_add_statement (init_block, VALA_CCODE_NODE ((__temp154 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_member_access_new_pointer (VALA_CCODE_EXPRESSION (ccall), "set_property")), VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (g_strdup_printf ("%s_set_property", (__temp153 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL))))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))))));
	(__temp154 == NULL ? NULL : (__temp154 = (g_object_unref (__temp154), NULL)));
	(__temp153 == NULL ? NULL : (__temp153 = (g_free (__temp153), NULL)));
	/* set constructor */
	if (vala_class_get_constructor (cl) != NULL) {
		ValaCCodeFunctionCall* ccast = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("G_OBJECT_CLASS")));
		ValaCCodeIdentifier* __temp155 = NULL;
		vala_ccode_function_call_add_argument (ccast, VALA_CCODE_EXPRESSION ((__temp155 = vala_ccode_identifier_new ("klass"))));
		(__temp155 == NULL ? NULL : (__temp155 = (g_object_unref (__temp155), NULL)));
		ValaCCodeExpressionStatement* __temp157 = NULL;
		char* __temp156 = NULL;
		vala_ccode_block_add_statement (init_block, VALA_CCODE_NODE ((__temp157 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_member_access_new_pointer (VALA_CCODE_EXPRESSION (ccast), "constructor")), VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (g_strdup_printf ("%s_constructor", (__temp156 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL))))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))))));
		(__temp157 == NULL ? NULL : (__temp157 = (g_object_unref (__temp157), NULL)));
		(__temp156 == NULL ? NULL : (__temp156 = (g_free (__temp156), NULL)));
		(ccast == NULL ? NULL : (ccast = (g_object_unref (ccast), NULL)));
	}
	/* set dispose function */
	GList* __temp158 = NULL;
	gboolean __temp159;
	if ((__temp159 = vala_code_generator_get_memory_management (self) && (__temp158 = vala_class_get_fields (cl)) != NULL, (__temp158 == NULL ? NULL : (__temp158 = (g_list_free (__temp158), NULL))), __temp159)) {
		ValaCCodeFunctionCall* ccast = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("G_OBJECT_CLASS")));
		ValaCCodeIdentifier* __temp160 = NULL;
		vala_ccode_function_call_add_argument (ccast, VALA_CCODE_EXPRESSION ((__temp160 = vala_ccode_identifier_new ("klass"))));
		(__temp160 == NULL ? NULL : (__temp160 = (g_object_unref (__temp160), NULL)));
		ValaCCodeExpressionStatement* __temp162 = NULL;
		char* __temp161 = NULL;
		vala_ccode_block_add_statement (init_block, VALA_CCODE_NODE ((__temp162 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_member_access_new_pointer (VALA_CCODE_EXPRESSION (ccast), "dispose")), VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (g_strdup_printf ("%s_dispose", (__temp161 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL))))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))))));
		(__temp162 == NULL ? NULL : (__temp162 = (g_object_unref (__temp162), NULL)));
		(__temp161 == NULL ? NULL : (__temp161 = (g_free (__temp161), NULL)));
		(ccast == NULL ? NULL : (ccast = (g_object_unref (ccast), NULL)));
	}
	/* connect overridden methods */
	GList* methods = vala_class_get_methods (cl);
	{
		GList* __temp167 = NULL;
		__temp167 = methods;
		GList* m_it;
		for (m_it = __temp167; m_it != NULL; m_it = m_it->next) {
			ValaMethod* m = m_it->data;
			{
				if (!vala_method_get_is_virtual (m) && !vala_method_get_overrides (m)) {
					continue;
				}
				ValaCodeNode* base_type = vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (vala_method_get_base_method (m)))));
				if (VALA_IS_INTERFACE (base_type)) {
					continue;
				}
				char* __temp163 = NULL;
				ValaCCodeFunctionCall* __temp164 = NULL;
				ValaCCodeFunctionCall* ccast = (__temp164 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (g_strdup_printf ("%s_CLASS", (__temp163 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE ((VALA_CLASS (base_type))), NULL)))))), (__temp163 == NULL ? NULL : (__temp163 = (g_free (__temp163), NULL))), __temp164);
				ValaCCodeIdentifier* __temp165 = NULL;
				vala_ccode_function_call_add_argument (ccast, VALA_CCODE_EXPRESSION ((__temp165 = vala_ccode_identifier_new ("klass"))));
				(__temp165 == NULL ? NULL : (__temp165 = (g_object_unref (__temp165), NULL)));
				ValaCCodeExpressionStatement* __temp166 = NULL;
				vala_ccode_block_add_statement (init_block, VALA_CCODE_NODE ((__temp166 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_member_access_new_pointer (VALA_CCODE_EXPRESSION (ccast), vala_method_get_name (m))), VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_method_get_real_cname (m))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))))));
				(__temp166 == NULL ? NULL : (__temp166 = (g_object_unref (__temp166), NULL)));
				(ccast == NULL ? NULL : (ccast = (g_object_unref (ccast), NULL)));
			}
		}
	}
	/* create properties */
	GList* props = vala_class_get_properties (cl);
	{
		GList* __temp209 = NULL;
		__temp209 = props;
		GList* prop_it;
		for (prop_it = __temp209; prop_it != NULL; prop_it = prop_it->next) {
			ValaProperty* prop = prop_it->data;
			{
				ValaCCodeFunctionCall* cinst = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("g_object_class_install_property")));
				vala_ccode_function_call_add_argument (cinst, VALA_CCODE_EXPRESSION (ccall));
				ValaCCodeConstant* __temp168 = NULL;
				vala_ccode_function_call_add_argument (cinst, VALA_CCODE_EXPRESSION ((__temp168 = vala_ccode_constant_new (vala_property_get_upper_case_cname (prop)))));
				(__temp168 == NULL ? NULL : (__temp168 = (g_object_unref (__temp168), NULL)));
				ValaCCodeFunctionCall* cspec = vala_ccode_function_call_new (NULL);
				ValaCCodeConstant* __temp169 = NULL;
				vala_ccode_function_call_add_argument (cspec, VALA_CCODE_EXPRESSION ((__temp169 = vala_property_get_canonical_cconstant (prop))));
				(__temp169 == NULL ? NULL : (__temp169 = (g_object_unref (__temp169), NULL)));
				ValaCCodeConstant* __temp170 = NULL;
				vala_ccode_function_call_add_argument (cspec, VALA_CCODE_EXPRESSION ((__temp170 = vala_ccode_constant_new ("\"foo\""))));
				(__temp170 == NULL ? NULL : (__temp170 = (g_object_unref (__temp170), NULL)));
				ValaCCodeConstant* __temp171 = NULL;
				vala_ccode_function_call_add_argument (cspec, VALA_CCODE_EXPRESSION ((__temp171 = vala_ccode_constant_new ("\"bar\""))));
				(__temp171 == NULL ? NULL : (__temp171 = (g_object_unref (__temp171), NULL)));
				if (VALA_IS_CLASS (vala_type_reference_get_data_type (vala_property_get_type_reference (prop)))) {
					ValaCCodeIdentifier* __temp172 = NULL;
					vala_ccode_function_call_set_call (cspec, VALA_CCODE_EXPRESSION ((__temp172 = vala_ccode_identifier_new ("g_param_spec_object"))));
					(__temp172 == NULL ? NULL : (__temp172 = (g_object_unref (__temp172), NULL)));
					ValaCCodeIdentifier* __temp173 = NULL;
					vala_ccode_function_call_add_argument (cspec, VALA_CCODE_EXPRESSION ((__temp173 = vala_ccode_identifier_new (vala_data_type_get_upper_case_cname (vala_type_reference_get_data_type (vala_property_get_type_reference (prop)), "TYPE_")))));
					(__temp173 == NULL ? NULL : (__temp173 = (g_object_unref (__temp173), NULL)));
				} else {
					if (vala_type_reference_get_data_type (vala_property_get_type_reference (prop)) == vala_type_reference_get_data_type (self->priv->string_type)) {
						ValaCCodeIdentifier* __temp174 = NULL;
						vala_ccode_function_call_set_call (cspec, VALA_CCODE_EXPRESSION ((__temp174 = vala_ccode_identifier_new ("g_param_spec_string"))));
						(__temp174 == NULL ? NULL : (__temp174 = (g_object_unref (__temp174), NULL)));
						ValaCCodeConstant* __temp175 = NULL;
						vala_ccode_function_call_add_argument (cspec, VALA_CCODE_EXPRESSION ((__temp175 = vala_ccode_constant_new ("NULL"))));
						(__temp175 == NULL ? NULL : (__temp175 = (g_object_unref (__temp175), NULL)));
					} else {
						if (vala_type_reference_get_data_type (vala_property_get_type_reference (prop)) == vala_type_reference_get_data_type (self->priv->int_type) || VALA_IS_ENUM (vala_type_reference_get_data_type (vala_property_get_type_reference (prop)))) {
							ValaCCodeIdentifier* __temp176 = NULL;
							vala_ccode_function_call_set_call (cspec, VALA_CCODE_EXPRESSION ((__temp176 = vala_ccode_identifier_new ("g_param_spec_int"))));
							(__temp176 == NULL ? NULL : (__temp176 = (g_object_unref (__temp176), NULL)));
							ValaCCodeConstant* __temp177 = NULL;
							vala_ccode_function_call_add_argument (cspec, VALA_CCODE_EXPRESSION ((__temp177 = vala_ccode_constant_new ("G_MININT"))));
							(__temp177 == NULL ? NULL : (__temp177 = (g_object_unref (__temp177), NULL)));
							ValaCCodeConstant* __temp178 = NULL;
							vala_ccode_function_call_add_argument (cspec, VALA_CCODE_EXPRESSION ((__temp178 = vala_ccode_constant_new ("G_MAXINT"))));
							(__temp178 == NULL ? NULL : (__temp178 = (g_object_unref (__temp178), NULL)));
							ValaCCodeConstant* __temp179 = NULL;
							vala_ccode_function_call_add_argument (cspec, VALA_CCODE_EXPRESSION ((__temp179 = vala_ccode_constant_new ("0"))));
							(__temp179 == NULL ? NULL : (__temp179 = (g_object_unref (__temp179), NULL)));
						} else {
							if (vala_type_reference_get_data_type (vala_property_get_type_reference (prop)) == vala_type_reference_get_data_type (self->priv->uint_type)) {
								ValaCCodeIdentifier* __temp180 = NULL;
								vala_ccode_function_call_set_call (cspec, VALA_CCODE_EXPRESSION ((__temp180 = vala_ccode_identifier_new ("g_param_spec_uint"))));
								(__temp180 == NULL ? NULL : (__temp180 = (g_object_unref (__temp180), NULL)));
								ValaCCodeConstant* __temp181 = NULL;
								vala_ccode_function_call_add_argument (cspec, VALA_CCODE_EXPRESSION ((__temp181 = vala_ccode_constant_new ("0"))));
								(__temp181 == NULL ? NULL : (__temp181 = (g_object_unref (__temp181), NULL)));
								ValaCCodeConstant* __temp182 = NULL;
								vala_ccode_function_call_add_argument (cspec, VALA_CCODE_EXPRESSION ((__temp182 = vala_ccode_constant_new ("G_MAXUINT"))));
								(__temp182 == NULL ? NULL : (__temp182 = (g_object_unref (__temp182), NULL)));
								ValaCCodeConstant* __temp183 = NULL;
								vala_ccode_function_call_add_argument (cspec, VALA_CCODE_EXPRESSION ((__temp183 = vala_ccode_constant_new ("0"))));
								(__temp183 == NULL ? NULL : (__temp183 = (g_object_unref (__temp183), NULL)));
							} else {
								if (vala_type_reference_get_data_type (vala_property_get_type_reference (prop)) == vala_type_reference_get_data_type (self->priv->long_type)) {
									ValaCCodeIdentifier* __temp184 = NULL;
									vala_ccode_function_call_set_call (cspec, VALA_CCODE_EXPRESSION ((__temp184 = vala_ccode_identifier_new ("g_param_spec_long"))));
									(__temp184 == NULL ? NULL : (__temp184 = (g_object_unref (__temp184), NULL)));
									ValaCCodeConstant* __temp185 = NULL;
									vala_ccode_function_call_add_argument (cspec, VALA_CCODE_EXPRESSION ((__temp185 = vala_ccode_constant_new ("G_MINLONG"))));
									(__temp185 == NULL ? NULL : (__temp185 = (g_object_unref (__temp185), NULL)));
									ValaCCodeConstant* __temp186 = NULL;
									vala_ccode_function_call_add_argument (cspec, VALA_CCODE_EXPRESSION ((__temp186 = vala_ccode_constant_new ("G_MAXLONG"))));
									(__temp186 == NULL ? NULL : (__temp186 = (g_object_unref (__temp186), NULL)));
									ValaCCodeConstant* __temp187 = NULL;
									vala_ccode_function_call_add_argument (cspec, VALA_CCODE_EXPRESSION ((__temp187 = vala_ccode_constant_new ("0"))));
									(__temp187 == NULL ? NULL : (__temp187 = (g_object_unref (__temp187), NULL)));
								} else {
									if (vala_type_reference_get_data_type (vala_property_get_type_reference (prop)) == vala_type_reference_get_data_type (self->priv->ulong_type)) {
										ValaCCodeIdentifier* __temp188 = NULL;
										vala_ccode_function_call_set_call (cspec, VALA_CCODE_EXPRESSION ((__temp188 = vala_ccode_identifier_new ("g_param_spec_ulong"))));
										(__temp188 == NULL ? NULL : (__temp188 = (g_object_unref (__temp188), NULL)));
										ValaCCodeConstant* __temp189 = NULL;
										vala_ccode_function_call_add_argument (cspec, VALA_CCODE_EXPRESSION ((__temp189 = vala_ccode_constant_new ("0"))));
										(__temp189 == NULL ? NULL : (__temp189 = (g_object_unref (__temp189), NULL)));
										ValaCCodeConstant* __temp190 = NULL;
										vala_ccode_function_call_add_argument (cspec, VALA_CCODE_EXPRESSION ((__temp190 = vala_ccode_constant_new ("G_MAXULONG"))));
										(__temp190 == NULL ? NULL : (__temp190 = (g_object_unref (__temp190), NULL)));
										ValaCCodeConstant* __temp191 = NULL;
										vala_ccode_function_call_add_argument (cspec, VALA_CCODE_EXPRESSION ((__temp191 = vala_ccode_constant_new ("0"))));
										(__temp191 == NULL ? NULL : (__temp191 = (g_object_unref (__temp191), NULL)));
									} else {
										if (vala_type_reference_get_data_type (vala_property_get_type_reference (prop)) == vala_type_reference_get_data_type (self->priv->bool_type)) {
											ValaCCodeIdentifier* __temp192 = NULL;
											vala_ccode_function_call_set_call (cspec, VALA_CCODE_EXPRESSION ((__temp192 = vala_ccode_identifier_new ("g_param_spec_boolean"))));
											(__temp192 == NULL ? NULL : (__temp192 = (g_object_unref (__temp192), NULL)));
											ValaCCodeConstant* __temp193 = NULL;
											vala_ccode_function_call_add_argument (cspec, VALA_CCODE_EXPRESSION ((__temp193 = vala_ccode_constant_new ("FALSE"))));
											(__temp193 == NULL ? NULL : (__temp193 = (g_object_unref (__temp193), NULL)));
										} else {
											if (vala_type_reference_get_data_type (vala_property_get_type_reference (prop)) == vala_type_reference_get_data_type (self->priv->float_type)) {
												ValaCCodeIdentifier* __temp194 = NULL;
												vala_ccode_function_call_set_call (cspec, VALA_CCODE_EXPRESSION ((__temp194 = vala_ccode_identifier_new ("g_param_spec_float"))));
												(__temp194 == NULL ? NULL : (__temp194 = (g_object_unref (__temp194), NULL)));
												ValaCCodeConstant* __temp195 = NULL;
												vala_ccode_function_call_add_argument (cspec, VALA_CCODE_EXPRESSION ((__temp195 = vala_ccode_constant_new ("-G_MAXFLOAT"))));
												(__temp195 == NULL ? NULL : (__temp195 = (g_object_unref (__temp195), NULL)));
												ValaCCodeConstant* __temp196 = NULL;
												vala_ccode_function_call_add_argument (cspec, VALA_CCODE_EXPRESSION ((__temp196 = vala_ccode_constant_new ("G_MAXFLOAT"))));
												(__temp196 == NULL ? NULL : (__temp196 = (g_object_unref (__temp196), NULL)));
												ValaCCodeConstant* __temp197 = NULL;
												vala_ccode_function_call_add_argument (cspec, VALA_CCODE_EXPRESSION ((__temp197 = vala_ccode_constant_new ("0"))));
												(__temp197 == NULL ? NULL : (__temp197 = (g_object_unref (__temp197), NULL)));
											} else {
												if (vala_type_reference_get_data_type (vala_property_get_type_reference (prop)) == vala_type_reference_get_data_type (self->priv->double_type)) {
													ValaCCodeIdentifier* __temp198 = NULL;
													vala_ccode_function_call_set_call (cspec, VALA_CCODE_EXPRESSION ((__temp198 = vala_ccode_identifier_new ("g_param_spec_double"))));
													(__temp198 == NULL ? NULL : (__temp198 = (g_object_unref (__temp198), NULL)));
													ValaCCodeConstant* __temp199 = NULL;
													vala_ccode_function_call_add_argument (cspec, VALA_CCODE_EXPRESSION ((__temp199 = vala_ccode_constant_new ("-G_MAXDOUBLE"))));
													(__temp199 == NULL ? NULL : (__temp199 = (g_object_unref (__temp199), NULL)));
													ValaCCodeConstant* __temp200 = NULL;
													vala_ccode_function_call_add_argument (cspec, VALA_CCODE_EXPRESSION ((__temp200 = vala_ccode_constant_new ("G_MAXDOUBLE"))));
													(__temp200 == NULL ? NULL : (__temp200 = (g_object_unref (__temp200), NULL)));
													ValaCCodeConstant* __temp201 = NULL;
													vala_ccode_function_call_add_argument (cspec, VALA_CCODE_EXPRESSION ((__temp201 = vala_ccode_constant_new ("0"))));
													(__temp201 == NULL ? NULL : (__temp201 = (g_object_unref (__temp201), NULL)));
												} else {
													ValaCCodeIdentifier* __temp202 = NULL;
													vala_ccode_function_call_set_call (cspec, VALA_CCODE_EXPRESSION ((__temp202 = vala_ccode_identifier_new ("g_param_spec_pointer"))));
													(__temp202 == NULL ? NULL : (__temp202 = (g_object_unref (__temp202), NULL)));
												}
											}
										}
									}
								}
							}
						}
					}
				}
				char* pflags = g_strdup ("G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB");
				if (vala_property_get_get_accessor (prop) != NULL) {
					char* __temp203 = NULL;
					pflags = (__temp203 = g_strdup_printf ("%s%s", pflags, " | G_PARAM_READABLE"), (pflags == NULL ? NULL : (pflags = (g_free (pflags), NULL))), __temp203);
				}
				if (vala_property_get_set_accessor (prop) != NULL) {
					char* __temp204 = NULL;
					pflags = (__temp204 = g_strdup_printf ("%s%s", pflags, " | G_PARAM_WRITABLE"), (pflags == NULL ? NULL : (pflags = (g_free (pflags), NULL))), __temp204);
					if (vala_property_accessor_get_construction (vala_property_get_set_accessor (prop))) {
						if (vala_property_accessor_get_writable (vala_property_get_set_accessor (prop))) {
							char* __temp205 = NULL;
							pflags = (__temp205 = g_strdup_printf ("%s%s", pflags, " | G_PARAM_CONSTRUCT"), (pflags == NULL ? NULL : (pflags = (g_free (pflags), NULL))), __temp205);
						} else {
							char* __temp206 = NULL;
							pflags = (__temp206 = g_strdup_printf ("%s%s", pflags, " | G_PARAM_CONSTRUCT_ONLY"), (pflags == NULL ? NULL : (pflags = (g_free (pflags), NULL))), __temp206);
						}
					}
				}
				ValaCCodeConstant* __temp207 = NULL;
				vala_ccode_function_call_add_argument (cspec, VALA_CCODE_EXPRESSION ((__temp207 = vala_ccode_constant_new (pflags))));
				(__temp207 == NULL ? NULL : (__temp207 = (g_object_unref (__temp207), NULL)));
				vala_ccode_function_call_add_argument (cinst, VALA_CCODE_EXPRESSION (cspec));
				ValaCCodeExpressionStatement* __temp208 = NULL;
				vala_ccode_block_add_statement (init_block, VALA_CCODE_NODE ((__temp208 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (cinst)))));
				(__temp208 == NULL ? NULL : (__temp208 = (g_object_unref (__temp208), NULL)));
				(cinst == NULL ? NULL : (cinst = (g_object_unref (cinst), NULL)));
				(cspec == NULL ? NULL : (cspec = (g_object_unref (cspec), NULL)));
				(pflags == NULL ? NULL : (pflags = (g_free (pflags), NULL)));
			}
		}
	}
	{
		GList* __temp222 = NULL;
		__temp222 = vala_class_get_signals (cl);
		GList* sig_it;
		for (sig_it = __temp222; sig_it != NULL; sig_it = sig_it->next) {
			ValaSignal* sig = sig_it->data;
			{
				/* create signals */
				ValaCCodeFunctionCall* csignew = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("g_signal_new")));
				ValaCCodeConstant* __temp210 = NULL;
				vala_ccode_function_call_add_argument (csignew, VALA_CCODE_EXPRESSION ((__temp210 = vala_ccode_constant_new (g_strdup_printf ("\"%s\"", vala_signal_get_name (sig))))));
				(__temp210 == NULL ? NULL : (__temp210 = (g_object_unref (__temp210), NULL)));
				ValaCCodeIdentifier* __temp211 = NULL;
				vala_ccode_function_call_add_argument (csignew, VALA_CCODE_EXPRESSION ((__temp211 = vala_ccode_identifier_new (vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), "TYPE_")))));
				(__temp211 == NULL ? NULL : (__temp211 = (g_object_unref (__temp211), NULL)));
				ValaCCodeConstant* __temp212 = NULL;
				vala_ccode_function_call_add_argument (csignew, VALA_CCODE_EXPRESSION ((__temp212 = vala_ccode_constant_new ("G_SIGNAL_RUN_LAST"))));
				(__temp212 == NULL ? NULL : (__temp212 = (g_object_unref (__temp212), NULL)));
				ValaCCodeConstant* __temp213 = NULL;
				vala_ccode_function_call_add_argument (csignew, VALA_CCODE_EXPRESSION ((__temp213 = vala_ccode_constant_new ("0"))));
				(__temp213 == NULL ? NULL : (__temp213 = (g_object_unref (__temp213), NULL)));
				ValaCCodeConstant* __temp214 = NULL;
				vala_ccode_function_call_add_argument (csignew, VALA_CCODE_EXPRESSION ((__temp214 = vala_ccode_constant_new ("NULL"))));
				(__temp214 == NULL ? NULL : (__temp214 = (g_object_unref (__temp214), NULL)));
				ValaCCodeConstant* __temp215 = NULL;
				vala_ccode_function_call_add_argument (csignew, VALA_CCODE_EXPRESSION ((__temp215 = vala_ccode_constant_new ("NULL"))));
				(__temp215 == NULL ? NULL : (__temp215 = (g_object_unref (__temp215), NULL)));
				/* TODO: generate marshallers */
				char* marshaller = vala_code_generator_get_signal_marshaller_function (self, sig, NULL);
				ValaCCodeIdentifier* marshal_arg = vala_ccode_identifier_new (marshaller);
				vala_ccode_function_call_add_argument (csignew, VALA_CCODE_EXPRESSION (marshal_arg));
				GList* params = vala_invokable_get_parameters (VALA_INVOKABLE (sig));
				unsigned int params_len = g_list_length (params);
				if (vala_type_reference_get_data_type (vala_signal_get_return_type (sig)) == NULL) {
					ValaCCodeConstant* __temp216 = NULL;
					vala_ccode_function_call_add_argument (csignew, VALA_CCODE_EXPRESSION ((__temp216 = vala_ccode_constant_new ("G_TYPE_NONE"))));
					(__temp216 == NULL ? NULL : (__temp216 = (g_object_unref (__temp216), NULL)));
				} else {
					ValaCCodeConstant* __temp217 = NULL;
					vala_ccode_function_call_add_argument (csignew, VALA_CCODE_EXPRESSION ((__temp217 = vala_ccode_constant_new (vala_data_type_get_type_id (vala_type_reference_get_data_type (vala_signal_get_return_type (sig)))))));
					(__temp217 == NULL ? NULL : (__temp217 = (g_object_unref (__temp217), NULL)));
				}
				ValaCCodeConstant* __temp218 = NULL;
				vala_ccode_function_call_add_argument (csignew, VALA_CCODE_EXPRESSION ((__temp218 = vala_ccode_constant_new (g_strdup_printf ("%d", params_len)))));
				(__temp218 == NULL ? NULL : (__temp218 = (g_object_unref (__temp218), NULL)));
				{
					GList* __temp220 = NULL;
					__temp220 = params;
					GList* param_it;
					for (param_it = __temp220; param_it != NULL; param_it = param_it->next) {
						ValaFormalParameter* param = param_it->data;
						{
							ValaCCodeConstant* __temp219 = NULL;
							vala_ccode_function_call_add_argument (csignew, VALA_CCODE_EXPRESSION ((__temp219 = vala_ccode_constant_new (vala_data_type_get_type_id (vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (param)))))));
							(__temp219 == NULL ? NULL : (__temp219 = (g_object_unref (__temp219), NULL)));
						}
					}
				}
				vala_ccode_identifier_set_name (marshal_arg, marshaller);
				ValaCCodeExpressionStatement* __temp221 = NULL;
				vala_ccode_block_add_statement (init_block, VALA_CCODE_NODE ((__temp221 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (csignew)))));
				(__temp221 == NULL ? NULL : (__temp221 = (g_object_unref (__temp221), NULL)));
				(csignew == NULL ? NULL : (csignew = (g_object_unref (csignew), NULL)));
				(marshaller == NULL ? NULL : (marshaller = (g_free (marshaller), NULL)));
				(marshal_arg == NULL ? NULL : (marshal_arg = (g_object_unref (marshal_arg), NULL)));
				(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
			}
		}
		(__temp222 == NULL ? NULL : (__temp222 = (g_list_free (__temp222), NULL)));
	}
	vala_ccode_fragment_append (self->priv->source_type_member_definition, VALA_CCODE_NODE (class_init));
	(class_init == NULL ? NULL : (class_init = (g_object_unref (class_init), NULL)));
	(init_block == NULL ? NULL : (init_block = (g_object_unref (init_block), NULL)));
	(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
	(parent_decl == NULL ? NULL : (parent_decl = (g_object_unref (parent_decl), NULL)));
	(parent_var_decl == NULL ? NULL : (parent_var_decl = (g_object_unref (parent_var_decl), NULL)));
	(parent_assignment == NULL ? NULL : (parent_assignment = (g_object_unref (parent_assignment), NULL)));
	(methods == NULL ? NULL : (methods = (g_list_free (methods), NULL)));
	(props == NULL ? NULL : (props = (g_list_free (props), NULL)));
}


static void vala_code_generator_add_interface_init_function (ValaCodeGenerator* self, ValaClass* cl, ValaInterface* iface)
{
	g_return_if_fail (VALA_IS_CODE_GENERATOR (self));
	g_return_if_fail (VALA_IS_CLASS (cl));
	g_return_if_fail (VALA_IS_INTERFACE (iface));
	char* __temp224 = NULL;
	char* __temp223 = NULL;
	ValaCCodeFunction* __temp225 = NULL;
	ValaCCodeFunction* iface_init = (__temp225 = vala_ccode_function_new (g_strdup_printf ("%s_%s_interface_init", (__temp223 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL)), (__temp224 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (iface), NULL))), "void"), (__temp224 == NULL ? NULL : (__temp224 = (g_free (__temp224), NULL))), (__temp223 == NULL ? NULL : (__temp223 = (g_free (__temp223), NULL))), __temp225);
	ValaCCodeFormalParameter* __temp226 = NULL;
	vala_ccode_function_add_parameter (iface_init, (__temp226 = vala_ccode_formal_parameter_new ("iface", g_strdup_printf ("%s *", vala_interface_get_type_cname (iface)))));
	(__temp226 == NULL ? NULL : (__temp226 = (g_object_unref (__temp226), NULL)));
	vala_ccode_function_set_modifiers (iface_init, VALA_CCODE_MODIFIERS_STATIC);
	ValaCCodeBlock* init_block = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
	vala_ccode_function_set_block (iface_init, init_block);
	ValaCCodeFunctionCall* ccall = NULL;
	GList* methods = vala_class_get_methods (cl);
	{
		GList* __temp228 = NULL;
		__temp228 = methods;
		GList* m_it;
		for (m_it = __temp228; m_it != NULL; m_it = m_it->next) {
			ValaMethod* m = m_it->data;
			{
				if (!vala_method_get_overrides (m)) {
					continue;
				}
				ValaCodeNode* base_type = vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (vala_method_get_base_method (m)))));
				if (base_type != VALA_CODE_NODE (iface)) {
					continue;
				}
				ValaCCodeIdentifier* ciface = vala_ccode_identifier_new ("iface");
				ValaCCodeExpressionStatement* __temp227 = NULL;
				vala_ccode_block_add_statement (init_block, VALA_CCODE_NODE ((__temp227 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_member_access_new_pointer (VALA_CCODE_EXPRESSION (ciface), vala_method_get_name (m))), VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_method_get_real_cname (m))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))))));
				(__temp227 == NULL ? NULL : (__temp227 = (g_object_unref (__temp227), NULL)));
				(ciface == NULL ? NULL : (ciface = (g_object_unref (ciface), NULL)));
			}
		}
	}
	vala_ccode_fragment_append (self->priv->source_type_member_definition, VALA_CCODE_NODE (iface_init));
	(iface_init == NULL ? NULL : (iface_init = (g_object_unref (iface_init), NULL)));
	(init_block == NULL ? NULL : (init_block = (g_object_unref (init_block), NULL)));
	(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
	(methods == NULL ? NULL : (methods = (g_list_free (methods), NULL)));
}


static void vala_code_generator_add_instance_init_function (ValaCodeGenerator* self, ValaClass* cl)
{
	g_return_if_fail (VALA_IS_CODE_GENERATOR (self));
	g_return_if_fail (VALA_IS_CLASS (cl));
	char* __temp229 = NULL;
	ValaCCodeFunction* __temp230 = NULL;
	ValaCCodeFunction* instance_init = (__temp230 = vala_ccode_function_new (g_strdup_printf ("%s_init", (__temp229 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL))), "void"), (__temp229 == NULL ? NULL : (__temp229 = (g_free (__temp229), NULL))), __temp230);
	ValaCCodeFormalParameter* __temp231 = NULL;
	vala_ccode_function_add_parameter (instance_init, (__temp231 = vala_ccode_formal_parameter_new ("self", g_strdup_printf ("%s *", vala_data_type_get_cname (VALA_DATA_TYPE (cl), FALSE)))));
	(__temp231 == NULL ? NULL : (__temp231 = (g_object_unref (__temp231), NULL)));
	vala_ccode_function_set_modifiers (instance_init, VALA_CCODE_MODIFIERS_STATIC);
	ValaCCodeBlock* init_block = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
	vala_ccode_function_set_block (instance_init, init_block);
	if (vala_class_get_has_private_fields (cl)) {
		char* __temp232 = NULL;
		ValaCCodeFunctionCall* __temp233 = NULL;
		ValaCCodeFunctionCall* ccall = (__temp233 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (g_strdup_printf ("%s_GET_PRIVATE", (__temp232 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), NULL)))))), (__temp232 == NULL ? NULL : (__temp232 = (g_free (__temp232), NULL))), __temp233);
		ValaCCodeIdentifier* __temp234 = NULL;
		vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp234 = vala_ccode_identifier_new ("self"))));
		(__temp234 == NULL ? NULL : (__temp234 = (g_object_unref (__temp234), NULL)));
		ValaCCodeExpressionStatement* __temp235 = NULL;
		vala_ccode_block_add_statement (init_block, VALA_CCODE_NODE ((__temp235 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_member_access_new_pointer (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("self")), "priv")), VALA_CCODE_EXPRESSION (ccall), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))))));
		(__temp235 == NULL ? NULL : (__temp235 = (g_object_unref (__temp235), NULL)));
		(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
	}
	vala_ccode_block_add_statement (init_block, VALA_CCODE_NODE (self->priv->instance_init_fragment));
	ValaSymbol* init_sym = vala_symbol_lookup (vala_code_node_get_symbol (VALA_CODE_NODE (cl)), "init");
	if (init_sym != NULL) {
		ValaMethod* init_fun = VALA_METHOD (vala_symbol_get_node (init_sym));
		vala_ccode_block_add_statement (init_block, vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_method_get_body (init_fun))));
	}
	vala_ccode_fragment_append (self->priv->source_type_member_definition, VALA_CCODE_NODE (instance_init));
	(instance_init == NULL ? NULL : (instance_init = (g_object_unref (instance_init), NULL)));
	(init_block == NULL ? NULL : (init_block = (g_object_unref (init_block), NULL)));
}


static void vala_code_generator_add_dispose_function (ValaCodeGenerator* self, ValaClass* cl)
{
	g_return_if_fail (VALA_IS_CODE_GENERATOR (self));
	g_return_if_fail (VALA_IS_CLASS (cl));
	ValaCCodeFunction* __temp237 = NULL;
	char* __temp236 = NULL;
	self->priv->function = (__temp237 = vala_ccode_function_new (g_strdup_printf ("%s_dispose", (__temp236 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL))), "void"), (self->priv->function == NULL ? NULL : (self->priv->function = (g_object_unref (self->priv->function), NULL))), __temp237);
	(__temp236 == NULL ? NULL : (__temp236 = (g_free (__temp236), NULL)));
	vala_ccode_function_set_modifiers (self->priv->function, VALA_CCODE_MODIFIERS_STATIC);
	ValaCCodeFormalParameter* __temp238 = NULL;
	vala_ccode_function_add_parameter (self->priv->function, (__temp238 = vala_ccode_formal_parameter_new ("obj", "GObject *")));
	(__temp238 == NULL ? NULL : (__temp238 = (g_object_unref (__temp238), NULL)));
	ValaCCodeFunction* __temp239 = NULL;
	vala_ccode_fragment_append (self->priv->source_type_member_declaration, VALA_CCODE_NODE ((__temp239 = vala_ccode_function_copy (self->priv->function))));
	(__temp239 == NULL ? NULL : (__temp239 = (g_object_unref (__temp239), NULL)));
	ValaCCodeBlock* cblock = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
	ValaCCodeFunctionCall* ccall = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), NULL))));
	ValaCCodeIdentifier* __temp240 = NULL;
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp240 = vala_ccode_identifier_new ("obj"))));
	(__temp240 == NULL ? NULL : (__temp240 = (g_object_unref (__temp240), NULL)));
	ValaCCodeDeclaration* cdecl = vala_ccode_declaration_new (g_strdup_printf ("%s *", vala_data_type_get_cname (VALA_DATA_TYPE (cl), FALSE)));
	ValaCCodeVariableDeclarator* __temp241 = NULL;
	vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR ((__temp241 = vala_ccode_variable_declarator_new_with_initializer ("self", VALA_CCODE_EXPRESSION (ccall)))));
	(__temp241 == NULL ? NULL : (__temp241 = (g_object_unref (__temp241), NULL)));
	vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE (cdecl));
	vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE (self->priv->instance_dispose_fragment));
	ValaCCodeDeclaration* __temp242 = NULL;
	cdecl = (__temp242 = vala_ccode_declaration_new (g_strdup_printf ("%sClass *", vala_data_type_get_cname (VALA_DATA_TYPE (cl), FALSE))), (cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL))), __temp242);
	ValaCCodeVariableDeclarator* __temp243 = NULL;
	vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR ((__temp243 = vala_ccode_variable_declarator_new ("klass"))));
	(__temp243 == NULL ? NULL : (__temp243 = (g_object_unref (__temp243), NULL)));
	vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE (cdecl));
	ValaCCodeDeclaration* __temp244 = NULL;
	cdecl = (__temp244 = vala_ccode_declaration_new ("GObjectClass *"), (cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL))), __temp244);
	ValaCCodeVariableDeclarator* __temp245 = NULL;
	vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR ((__temp245 = vala_ccode_variable_declarator_new ("parent_class"))));
	(__temp245 == NULL ? NULL : (__temp245 = (g_object_unref (__temp245), NULL)));
	vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE (cdecl));
	ValaCCodeFunctionCall* __temp246 = NULL;
	ccall = (__temp246 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("g_type_class_peek"))), (ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL))), __temp246);
	ValaCCodeIdentifier* __temp247 = NULL;
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp247 = vala_ccode_identifier_new (vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), "TYPE_")))));
	(__temp247 == NULL ? NULL : (__temp247 = (g_object_unref (__temp247), NULL)));
	char* __temp248 = NULL;
	ValaCCodeFunctionCall* __temp249 = NULL;
	ValaCCodeFunctionCall* ccast = (__temp249 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (g_strdup_printf ("%s_CLASS", (__temp248 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), NULL)))))), (__temp248 == NULL ? NULL : (__temp248 = (g_free (__temp248), NULL))), __temp249);
	vala_ccode_function_call_add_argument (ccast, VALA_CCODE_EXPRESSION (ccall));
	ValaCCodeExpressionStatement* __temp250 = NULL;
	vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE ((__temp250 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("klass")), VALA_CCODE_EXPRESSION (ccast), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))))));
	(__temp250 == NULL ? NULL : (__temp250 = (g_object_unref (__temp250), NULL)));
	ValaCCodeFunctionCall* __temp251 = NULL;
	ccall = (__temp251 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("g_type_class_peek_parent"))), (ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL))), __temp251);
	ValaCCodeIdentifier* __temp252 = NULL;
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp252 = vala_ccode_identifier_new ("klass"))));
	(__temp252 == NULL ? NULL : (__temp252 = (g_object_unref (__temp252), NULL)));
	ValaCCodeFunctionCall* __temp253 = NULL;
	ccast = (__temp253 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("G_OBJECT_CLASS"))), (ccast == NULL ? NULL : (ccast = (g_object_unref (ccast), NULL))), __temp253);
	vala_ccode_function_call_add_argument (ccast, VALA_CCODE_EXPRESSION (ccall));
	ValaCCodeExpressionStatement* __temp254 = NULL;
	vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE ((__temp254 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("parent_class")), VALA_CCODE_EXPRESSION (ccast), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))))));
	(__temp254 == NULL ? NULL : (__temp254 = (g_object_unref (__temp254), NULL)));
	ValaCCodeFunctionCall* __temp255 = NULL;
	ccall = (__temp255 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_member_access_new_pointer (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("parent_class")), "dispose"))), (ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL))), __temp255);
	ValaCCodeIdentifier* __temp256 = NULL;
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp256 = vala_ccode_identifier_new ("obj"))));
	(__temp256 == NULL ? NULL : (__temp256 = (g_object_unref (__temp256), NULL)));
	ValaCCodeExpressionStatement* __temp257 = NULL;
	vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE ((__temp257 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (ccall)))));
	(__temp257 == NULL ? NULL : (__temp257 = (g_object_unref (__temp257), NULL)));
	vala_ccode_function_set_block (self->priv->function, cblock);
	vala_ccode_fragment_append (self->priv->source_type_member_definition, VALA_CCODE_NODE (self->priv->function));
	(cblock == NULL ? NULL : (cblock = (g_object_unref (cblock), NULL)));
	(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
	(cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL)));
	(ccast == NULL ? NULL : (ccast = (g_object_unref (ccast), NULL)));
}


static ValaCCodeIdentifier* vala_code_generator_get_value_setter_function (ValaCodeGenerator* self, ValaTypeReference* type_reference)
{
	g_return_val_if_fail (VALA_IS_CODE_GENERATOR (self), NULL);
	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (type_reference), NULL);
	if (VALA_IS_CLASS (vala_type_reference_get_data_type (type_reference))) {
		return vala_ccode_identifier_new ("g_value_set_object");
	} else {
		if (vala_type_reference_get_data_type (type_reference) == vala_type_reference_get_data_type (self->priv->string_type)) {
			return vala_ccode_identifier_new ("g_value_set_string");
		} else {
			if (vala_type_reference_get_data_type (type_reference) == vala_type_reference_get_data_type (self->priv->int_type) || VALA_IS_ENUM (vala_type_reference_get_data_type (type_reference))) {
				return vala_ccode_identifier_new ("g_value_set_int");
			} else {
				if (vala_type_reference_get_data_type (type_reference) == vala_type_reference_get_data_type (self->priv->uint_type)) {
					return vala_ccode_identifier_new ("g_value_set_uint");
				} else {
					if (vala_type_reference_get_data_type (type_reference) == vala_type_reference_get_data_type (self->priv->long_type)) {
						return vala_ccode_identifier_new ("g_value_set_long");
					} else {
						if (vala_type_reference_get_data_type (type_reference) == vala_type_reference_get_data_type (self->priv->ulong_type)) {
							return vala_ccode_identifier_new ("g_value_set_ulong");
						} else {
							if (vala_type_reference_get_data_type (type_reference) == vala_type_reference_get_data_type (self->priv->bool_type)) {
								return vala_ccode_identifier_new ("g_value_set_boolean");
							} else {
								if (vala_type_reference_get_data_type (type_reference) == vala_type_reference_get_data_type (self->priv->float_type)) {
									return vala_ccode_identifier_new ("g_value_set_float");
								} else {
									if (vala_type_reference_get_data_type (type_reference) == vala_type_reference_get_data_type (self->priv->double_type)) {
										return vala_ccode_identifier_new ("g_value_set_double");
									} else {
										return vala_ccode_identifier_new ("g_value_set_pointer");
									}
								}
							}
						}
					}
				}
			}
		}
	}
}


static void vala_code_generator_add_get_property_function (ValaCodeGenerator* self, ValaClass* cl)
{
	g_return_if_fail (VALA_IS_CODE_GENERATOR (self));
	g_return_if_fail (VALA_IS_CLASS (cl));
	char* __temp268 = NULL;
	ValaCCodeFunction* __temp269 = NULL;
	ValaCCodeFunction* get_prop = (__temp269 = vala_ccode_function_new (g_strdup_printf ("%s_get_property", (__temp268 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL))), "void"), (__temp268 == NULL ? NULL : (__temp268 = (g_free (__temp268), NULL))), __temp269);
	vala_ccode_function_set_modifiers (get_prop, VALA_CCODE_MODIFIERS_STATIC);
	ValaCCodeFormalParameter* __temp270 = NULL;
	vala_ccode_function_add_parameter (get_prop, (__temp270 = vala_ccode_formal_parameter_new ("object", "GObject *")));
	(__temp270 == NULL ? NULL : (__temp270 = (g_object_unref (__temp270), NULL)));
	ValaCCodeFormalParameter* __temp271 = NULL;
	vala_ccode_function_add_parameter (get_prop, (__temp271 = vala_ccode_formal_parameter_new ("property_id", "guint")));
	(__temp271 == NULL ? NULL : (__temp271 = (g_object_unref (__temp271), NULL)));
	ValaCCodeFormalParameter* __temp272 = NULL;
	vala_ccode_function_add_parameter (get_prop, (__temp272 = vala_ccode_formal_parameter_new ("value", "GValue *")));
	(__temp272 == NULL ? NULL : (__temp272 = (g_object_unref (__temp272), NULL)));
	ValaCCodeFormalParameter* __temp273 = NULL;
	vala_ccode_function_add_parameter (get_prop, (__temp273 = vala_ccode_formal_parameter_new ("pspec", "GParamSpec *")));
	(__temp273 == NULL ? NULL : (__temp273 = (g_object_unref (__temp273), NULL)));
	ValaCCodeBlock* block = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
	ValaCCodeFunctionCall* ccall = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), NULL))));
	ValaCCodeIdentifier* __temp274 = NULL;
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp274 = vala_ccode_identifier_new ("object"))));
	(__temp274 == NULL ? NULL : (__temp274 = (g_object_unref (__temp274), NULL)));
	ValaCCodeDeclaration* cdecl = vala_ccode_declaration_new (g_strdup_printf ("%s *", vala_data_type_get_cname (VALA_DATA_TYPE (cl), FALSE)));
	ValaCCodeVariableDeclarator* __temp275 = NULL;
	vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR ((__temp275 = vala_ccode_variable_declarator_new_with_initializer ("self", VALA_CCODE_EXPRESSION (ccall)))));
	(__temp275 == NULL ? NULL : (__temp275 = (g_object_unref (__temp275), NULL)));
	vala_ccode_block_add_statement (block, VALA_CCODE_NODE (cdecl));
	ValaCCodeSwitchStatement* cswitch = vala_ccode_switch_statement_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("property_id")));
	GList* props = vala_class_get_properties (cl);
	{
		GList* __temp283 = NULL;
		__temp283 = props;
		GList* prop_it;
		for (prop_it = __temp283; prop_it != NULL; prop_it = prop_it->next) {
			ValaProperty* prop = prop_it->data;
			{
				if (vala_property_get_get_accessor (prop) == NULL) {
					continue;
				}
				ValaCCodeCaseStatement* ccase = vala_ccode_case_statement_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_property_get_upper_case_cname (prop))));
				char* __temp276 = NULL;
				ValaCCodeFunctionCall* __temp277 = NULL;
				ValaCCodeFunctionCall* ccall = (__temp277 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (g_strdup_printf ("%s_get_%s", (__temp276 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL)), vala_property_get_name (prop))))), (__temp276 == NULL ? NULL : (__temp276 = (g_free (__temp276), NULL))), __temp277);
				ValaCCodeIdentifier* __temp278 = NULL;
				vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp278 = vala_ccode_identifier_new ("self"))));
				(__temp278 == NULL ? NULL : (__temp278 = (g_object_unref (__temp278), NULL)));
				ValaCCodeFunctionCall* csetcall = vala_ccode_function_call_new (NULL);
				ValaCCodeIdentifier* __temp279 = NULL;
				vala_ccode_function_call_set_call (csetcall, VALA_CCODE_EXPRESSION ((__temp279 = vala_code_generator_get_value_setter_function (self, vala_property_get_type_reference (prop)))));
				(__temp279 == NULL ? NULL : (__temp279 = (g_object_unref (__temp279), NULL)));
				ValaCCodeIdentifier* __temp280 = NULL;
				vala_ccode_function_call_add_argument (csetcall, VALA_CCODE_EXPRESSION ((__temp280 = vala_ccode_identifier_new ("value"))));
				(__temp280 == NULL ? NULL : (__temp280 = (g_object_unref (__temp280), NULL)));
				vala_ccode_function_call_add_argument (csetcall, VALA_CCODE_EXPRESSION (ccall));
				ValaCCodeExpressionStatement* __temp281 = NULL;
				vala_ccode_case_statement_add_statement (ccase, VALA_CCODE_STATEMENT ((__temp281 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (csetcall)))));
				(__temp281 == NULL ? NULL : (__temp281 = (g_object_unref (__temp281), NULL)));
				ValaCCodeBreakStatement* __temp282 = NULL;
				vala_ccode_case_statement_add_statement (ccase, VALA_CCODE_STATEMENT ((__temp282 = g_object_new (VALA_TYPE_CCODE_BREAK_STATEMENT, NULL))));
				(__temp282 == NULL ? NULL : (__temp282 = (g_object_unref (__temp282), NULL)));
				vala_ccode_switch_statement_add_case (cswitch, ccase);
				(ccase == NULL ? NULL : (ccase = (g_object_unref (ccase), NULL)));
				(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
				(csetcall == NULL ? NULL : (csetcall = (g_object_unref (csetcall), NULL)));
			}
		}
	}
	vala_ccode_block_add_statement (block, VALA_CCODE_NODE (cswitch));
	vala_ccode_function_set_block (get_prop, block);
	vala_ccode_fragment_append (self->priv->source_type_member_definition, VALA_CCODE_NODE (get_prop));
	(get_prop == NULL ? NULL : (get_prop = (g_object_unref (get_prop), NULL)));
	(block == NULL ? NULL : (block = (g_object_unref (block), NULL)));
	(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
	(cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL)));
	(cswitch == NULL ? NULL : (cswitch = (g_object_unref (cswitch), NULL)));
	(props == NULL ? NULL : (props = (g_list_free (props), NULL)));
}


static void vala_code_generator_add_set_property_function (ValaCodeGenerator* self, ValaClass* cl)
{
	g_return_if_fail (VALA_IS_CODE_GENERATOR (self));
	g_return_if_fail (VALA_IS_CLASS (cl));
	char* __temp284 = NULL;
	ValaCCodeFunction* __temp285 = NULL;
	ValaCCodeFunction* set_prop = (__temp285 = vala_ccode_function_new (g_strdup_printf ("%s_set_property", (__temp284 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL))), "void"), (__temp284 == NULL ? NULL : (__temp284 = (g_free (__temp284), NULL))), __temp285);
	vala_ccode_function_set_modifiers (set_prop, VALA_CCODE_MODIFIERS_STATIC);
	ValaCCodeFormalParameter* __temp286 = NULL;
	vala_ccode_function_add_parameter (set_prop, (__temp286 = vala_ccode_formal_parameter_new ("object", "GObject *")));
	(__temp286 == NULL ? NULL : (__temp286 = (g_object_unref (__temp286), NULL)));
	ValaCCodeFormalParameter* __temp287 = NULL;
	vala_ccode_function_add_parameter (set_prop, (__temp287 = vala_ccode_formal_parameter_new ("property_id", "guint")));
	(__temp287 == NULL ? NULL : (__temp287 = (g_object_unref (__temp287), NULL)));
	ValaCCodeFormalParameter* __temp288 = NULL;
	vala_ccode_function_add_parameter (set_prop, (__temp288 = vala_ccode_formal_parameter_new ("value", "const GValue *")));
	(__temp288 == NULL ? NULL : (__temp288 = (g_object_unref (__temp288), NULL)));
	ValaCCodeFormalParameter* __temp289 = NULL;
	vala_ccode_function_add_parameter (set_prop, (__temp289 = vala_ccode_formal_parameter_new ("pspec", "GParamSpec *")));
	(__temp289 == NULL ? NULL : (__temp289 = (g_object_unref (__temp289), NULL)));
	ValaCCodeBlock* block = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
	ValaCCodeFunctionCall* ccall = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), NULL))));
	ValaCCodeIdentifier* __temp290 = NULL;
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp290 = vala_ccode_identifier_new ("object"))));
	(__temp290 == NULL ? NULL : (__temp290 = (g_object_unref (__temp290), NULL)));
	ValaCCodeDeclaration* cdecl = vala_ccode_declaration_new (g_strdup_printf ("%s *", vala_data_type_get_cname (VALA_DATA_TYPE (cl), FALSE)));
	ValaCCodeVariableDeclarator* __temp291 = NULL;
	vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR ((__temp291 = vala_ccode_variable_declarator_new_with_initializer ("self", VALA_CCODE_EXPRESSION (ccall)))));
	(__temp291 == NULL ? NULL : (__temp291 = (g_object_unref (__temp291), NULL)));
	vala_ccode_block_add_statement (block, VALA_CCODE_NODE (cdecl));
	ValaCCodeSwitchStatement* cswitch = vala_ccode_switch_statement_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("property_id")));
	GList* props = vala_class_get_properties (cl);
	{
		GList* __temp308 = NULL;
		__temp308 = props;
		GList* prop_it;
		for (prop_it = __temp308; prop_it != NULL; prop_it = prop_it->next) {
			ValaProperty* prop = prop_it->data;
			{
				if (vala_property_get_set_accessor (prop) == NULL) {
					continue;
				}
				ValaCCodeCaseStatement* ccase = vala_ccode_case_statement_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_property_get_upper_case_cname (prop))));
				char* __temp292 = NULL;
				ValaCCodeFunctionCall* __temp293 = NULL;
				ValaCCodeFunctionCall* ccall = (__temp293 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (g_strdup_printf ("%s_set_%s", (__temp292 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL)), vala_property_get_name (prop))))), (__temp292 == NULL ? NULL : (__temp292 = (g_free (__temp292), NULL))), __temp293);
				ValaCCodeIdentifier* __temp294 = NULL;
				vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp294 = vala_ccode_identifier_new ("self"))));
				(__temp294 == NULL ? NULL : (__temp294 = (g_object_unref (__temp294), NULL)));
				ValaCCodeFunctionCall* cgetcall = vala_ccode_function_call_new (NULL);
				if (VALA_IS_CLASS (vala_type_reference_get_data_type (vala_property_get_type_reference (prop)))) {
					ValaCCodeIdentifier* __temp295 = NULL;
					vala_ccode_function_call_set_call (cgetcall, VALA_CCODE_EXPRESSION ((__temp295 = vala_ccode_identifier_new ("g_value_get_object"))));
					(__temp295 == NULL ? NULL : (__temp295 = (g_object_unref (__temp295), NULL)));
				} else {
					if (g_utf8_collate (vala_type_reference_get_type_name (vala_property_get_type_reference (prop)), "string") == 0) {
						ValaCCodeIdentifier* __temp296 = NULL;
						vala_ccode_function_call_set_call (cgetcall, VALA_CCODE_EXPRESSION ((__temp296 = vala_ccode_identifier_new ("g_value_get_string"))));
						(__temp296 == NULL ? NULL : (__temp296 = (g_object_unref (__temp296), NULL)));
					} else {
						if (g_utf8_collate (vala_type_reference_get_type_name (vala_property_get_type_reference (prop)), "int") == 0 || VALA_IS_ENUM (vala_type_reference_get_data_type (vala_property_get_type_reference (prop)))) {
							ValaCCodeIdentifier* __temp297 = NULL;
							vala_ccode_function_call_set_call (cgetcall, VALA_CCODE_EXPRESSION ((__temp297 = vala_ccode_identifier_new ("g_value_get_int"))));
							(__temp297 == NULL ? NULL : (__temp297 = (g_object_unref (__temp297), NULL)));
						} else {
							if (g_utf8_collate (vala_type_reference_get_type_name (vala_property_get_type_reference (prop)), "uint") == 0) {
								ValaCCodeIdentifier* __temp298 = NULL;
								vala_ccode_function_call_set_call (cgetcall, VALA_CCODE_EXPRESSION ((__temp298 = vala_ccode_identifier_new ("g_value_get_uint"))));
								(__temp298 == NULL ? NULL : (__temp298 = (g_object_unref (__temp298), NULL)));
							} else {
								if (g_utf8_collate (vala_type_reference_get_type_name (vala_property_get_type_reference (prop)), "long") == 0) {
									ValaCCodeIdentifier* __temp299 = NULL;
									vala_ccode_function_call_set_call (cgetcall, VALA_CCODE_EXPRESSION ((__temp299 = vala_ccode_identifier_new ("g_value_get_long"))));
									(__temp299 == NULL ? NULL : (__temp299 = (g_object_unref (__temp299), NULL)));
								} else {
									if (g_utf8_collate (vala_type_reference_get_type_name (vala_property_get_type_reference (prop)), "ulong") == 0) {
										ValaCCodeIdentifier* __temp300 = NULL;
										vala_ccode_function_call_set_call (cgetcall, VALA_CCODE_EXPRESSION ((__temp300 = vala_ccode_identifier_new ("g_value_get_ulong"))));
										(__temp300 == NULL ? NULL : (__temp300 = (g_object_unref (__temp300), NULL)));
									} else {
										if (g_utf8_collate (vala_type_reference_get_type_name (vala_property_get_type_reference (prop)), "bool") == 0) {
											ValaCCodeIdentifier* __temp301 = NULL;
											vala_ccode_function_call_set_call (cgetcall, VALA_CCODE_EXPRESSION ((__temp301 = vala_ccode_identifier_new ("g_value_get_boolean"))));
											(__temp301 == NULL ? NULL : (__temp301 = (g_object_unref (__temp301), NULL)));
										} else {
											if (g_utf8_collate (vala_type_reference_get_type_name (vala_property_get_type_reference (prop)), "float") == 0) {
												ValaCCodeIdentifier* __temp302 = NULL;
												vala_ccode_function_call_set_call (cgetcall, VALA_CCODE_EXPRESSION ((__temp302 = vala_ccode_identifier_new ("g_value_get_float"))));
												(__temp302 == NULL ? NULL : (__temp302 = (g_object_unref (__temp302), NULL)));
											} else {
												if (g_utf8_collate (vala_type_reference_get_type_name (vala_property_get_type_reference (prop)), "double") == 0) {
													ValaCCodeIdentifier* __temp303 = NULL;
													vala_ccode_function_call_set_call (cgetcall, VALA_CCODE_EXPRESSION ((__temp303 = vala_ccode_identifier_new ("g_value_get_double"))));
													(__temp303 == NULL ? NULL : (__temp303 = (g_object_unref (__temp303), NULL)));
												} else {
													ValaCCodeIdentifier* __temp304 = NULL;
													vala_ccode_function_call_set_call (cgetcall, VALA_CCODE_EXPRESSION ((__temp304 = vala_ccode_identifier_new ("g_value_get_pointer"))));
													(__temp304 == NULL ? NULL : (__temp304 = (g_object_unref (__temp304), NULL)));
												}
											}
										}
									}
								}
							}
						}
					}
				}
				ValaCCodeIdentifier* __temp305 = NULL;
				vala_ccode_function_call_add_argument (cgetcall, VALA_CCODE_EXPRESSION ((__temp305 = vala_ccode_identifier_new ("value"))));
				(__temp305 == NULL ? NULL : (__temp305 = (g_object_unref (__temp305), NULL)));
				vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION (cgetcall));
				ValaCCodeExpressionStatement* __temp306 = NULL;
				vala_ccode_case_statement_add_statement (ccase, VALA_CCODE_STATEMENT ((__temp306 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (ccall)))));
				(__temp306 == NULL ? NULL : (__temp306 = (g_object_unref (__temp306), NULL)));
				ValaCCodeBreakStatement* __temp307 = NULL;
				vala_ccode_case_statement_add_statement (ccase, VALA_CCODE_STATEMENT ((__temp307 = g_object_new (VALA_TYPE_CCODE_BREAK_STATEMENT, NULL))));
				(__temp307 == NULL ? NULL : (__temp307 = (g_object_unref (__temp307), NULL)));
				vala_ccode_switch_statement_add_case (cswitch, ccase);
				(ccase == NULL ? NULL : (ccase = (g_object_unref (ccase), NULL)));
				(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
				(cgetcall == NULL ? NULL : (cgetcall = (g_object_unref (cgetcall), NULL)));
			}
		}
	}
	vala_ccode_block_add_statement (block, VALA_CCODE_NODE (cswitch));
	vala_ccode_function_set_block (set_prop, block);
	vala_ccode_fragment_append (self->priv->source_type_member_definition, VALA_CCODE_NODE (set_prop));
	(set_prop == NULL ? NULL : (set_prop = (g_object_unref (set_prop), NULL)));
	(block == NULL ? NULL : (block = (g_object_unref (block), NULL)));
	(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
	(cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL)));
	(cswitch == NULL ? NULL : (cswitch = (g_object_unref (cswitch), NULL)));
	(props == NULL ? NULL : (props = (g_list_free (props), NULL)));
}


static void vala_code_generator_real_visit_begin_struct (ValaCodeVisitor* base, ValaStruct* st)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_STRUCT (st));
	ValaCCodeStruct* __temp309 = NULL;
	self->priv->instance_struct = (__temp309 = vala_ccode_struct_new (g_strdup_printf ("_%s", vala_data_type_get_name (VALA_DATA_TYPE (st)))), (self->priv->instance_struct == NULL ? NULL : (self->priv->instance_struct = (g_object_unref (self->priv->instance_struct), NULL))), __temp309);
	if (vala_source_reference_get_comment (vala_code_node_get_source_reference (VALA_CODE_NODE (st))) != NULL) {
		ValaCCodeComment* __temp310 = NULL;
		vala_ccode_fragment_append (self->priv->header_type_definition, VALA_CCODE_NODE ((__temp310 = vala_ccode_comment_new (vala_source_reference_get_comment (vala_code_node_get_source_reference (VALA_CODE_NODE (st)))))));
		(__temp310 == NULL ? NULL : (__temp310 = (g_object_unref (__temp310), NULL)));
	}
	vala_ccode_fragment_append (self->priv->header_type_definition, VALA_CCODE_NODE (self->priv->instance_struct));
}


static void vala_code_generator_real_visit_begin_interface (ValaCodeVisitor* base, ValaInterface* iface)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_INTERFACE (iface));
	ValaSymbol* __temp312 = NULL;
	ValaSymbol* __temp311 = NULL;
	self->priv->current_symbol = (__temp312 = (__temp311 = vala_code_node_get_symbol (VALA_CODE_NODE (iface)), (__temp311 == NULL ? NULL : g_object_ref (__temp311))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp312);
	ValaSymbol* __temp314 = NULL;
	ValaSymbol* __temp313 = NULL;
	self->priv->current_type_symbol = (__temp314 = (__temp313 = vala_code_node_get_symbol (VALA_CODE_NODE (iface)), (__temp313 == NULL ? NULL : g_object_ref (__temp313))), (self->priv->current_type_symbol == NULL ? NULL : (self->priv->current_type_symbol = (g_object_unref (self->priv->current_type_symbol), NULL))), __temp314);
	ValaCCodeStruct* __temp315 = NULL;
	self->priv->type_struct = (__temp315 = vala_ccode_struct_new (g_strdup_printf ("_%s", vala_interface_get_type_cname (iface))), (self->priv->type_struct == NULL ? NULL : (self->priv->type_struct = (g_object_unref (self->priv->type_struct), NULL))), __temp315);
	ValaCCodeNewline* __temp316 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp316 = g_object_new (VALA_TYPE_CCODE_NEWLINE, NULL))));
	(__temp316 == NULL ? NULL : (__temp316 = (g_object_unref (__temp316), NULL)));
	char* __temp317 = NULL;
	char* __temp318 = NULL;
	char* macro = (__temp318 = g_strdup_printf ("(%s_get_type ())", (__temp317 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (iface), NULL))), (__temp317 == NULL ? NULL : (__temp317 = (g_free (__temp317), NULL))), __temp318);
	ValaCCodeMacroReplacement* __temp319 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp319 = vala_ccode_macro_replacement_new (vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (iface), "TYPE_"), macro))));
	(__temp319 == NULL ? NULL : (__temp319 = (g_object_unref (__temp319), NULL)));
	char* __temp321 = NULL;
	char* __temp320 = NULL;
	macro = (__temp321 = g_strdup_printf ("(G_TYPE_CHECK_INSTANCE_CAST ((obj), %s, %s))", (__temp320 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (iface), "TYPE_")), vala_data_type_get_cname (VALA_DATA_TYPE (iface), FALSE)), (macro == NULL ? NULL : (macro = (g_free (macro), NULL))), __temp321);
	(__temp320 == NULL ? NULL : (__temp320 = (g_free (__temp320), NULL)));
	ValaCCodeMacroReplacement* __temp323 = NULL;
	char* __temp322 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp323 = vala_ccode_macro_replacement_new (g_strdup_printf ("%s(obj)", (__temp322 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (iface), NULL))), macro))));
	(__temp323 == NULL ? NULL : (__temp323 = (g_object_unref (__temp323), NULL)));
	(__temp322 == NULL ? NULL : (__temp322 = (g_free (__temp322), NULL)));
	char* __temp325 = NULL;
	char* __temp324 = NULL;
	macro = (__temp325 = g_strdup_printf ("(G_TYPE_CHECK_INSTANCE_TYPE ((obj), %s))", (__temp324 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (iface), "TYPE_"))), (macro == NULL ? NULL : (macro = (g_free (macro), NULL))), __temp325);
	(__temp324 == NULL ? NULL : (__temp324 = (g_free (__temp324), NULL)));
	ValaCCodeMacroReplacement* __temp327 = NULL;
	char* __temp326 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp327 = vala_ccode_macro_replacement_new (g_strdup_printf ("%s(obj)", (__temp326 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (iface), "IS_"))), macro))));
	(__temp327 == NULL ? NULL : (__temp327 = (g_object_unref (__temp327), NULL)));
	(__temp326 == NULL ? NULL : (__temp326 = (g_free (__temp326), NULL)));
	char* __temp329 = NULL;
	char* __temp328 = NULL;
	macro = (__temp329 = g_strdup_printf ("(G_TYPE_INSTANCE_GET_INTERFACE ((obj), %s, %s))", (__temp328 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (iface), "TYPE_")), vala_interface_get_type_cname (iface)), (macro == NULL ? NULL : (macro = (g_free (macro), NULL))), __temp329);
	(__temp328 == NULL ? NULL : (__temp328 = (g_free (__temp328), NULL)));
	ValaCCodeMacroReplacement* __temp331 = NULL;
	char* __temp330 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp331 = vala_ccode_macro_replacement_new (g_strdup_printf ("%s_GET_INTERFACE(obj)", (__temp330 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (iface), NULL))), macro))));
	(__temp331 == NULL ? NULL : (__temp331 = (g_object_unref (__temp331), NULL)));
	(__temp330 == NULL ? NULL : (__temp330 = (g_free (__temp330), NULL)));
	ValaCCodeNewline* __temp332 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp332 = g_object_new (VALA_TYPE_CCODE_NEWLINE, NULL))));
	(__temp332 == NULL ? NULL : (__temp332 = (g_object_unref (__temp332), NULL)));
	if (vala_source_file_get_cycle (vala_source_reference_get_file (vala_code_node_get_source_reference (VALA_CODE_NODE (iface)))) == NULL) {
		ValaCCodeTypeDefinition* __temp333 = NULL;
		vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp333 = vala_ccode_type_definition_new (g_strdup_printf ("struct _%s", vala_data_type_get_cname (VALA_DATA_TYPE (iface), FALSE)), VALA_CCODE_DECLARATOR (vala_ccode_variable_declarator_new (vala_data_type_get_cname (VALA_DATA_TYPE (iface), FALSE)))))));
		(__temp333 == NULL ? NULL : (__temp333 = (g_object_unref (__temp333), NULL)));
		ValaCCodeTypeDefinition* __temp334 = NULL;
		vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp334 = vala_ccode_type_definition_new (g_strdup_printf ("struct %s", vala_ccode_struct_get_name (self->priv->type_struct)), VALA_CCODE_DECLARATOR (vala_ccode_variable_declarator_new (vala_interface_get_type_cname (iface)))))));
		(__temp334 == NULL ? NULL : (__temp334 = (g_object_unref (__temp334), NULL)));
	}
	vala_ccode_struct_add_field (self->priv->type_struct, "GTypeInterface", "parent");
	if (vala_source_reference_get_comment (vala_code_node_get_source_reference (VALA_CODE_NODE (iface))) != NULL) {
		ValaCCodeComment* __temp335 = NULL;
		vala_ccode_fragment_append (self->priv->header_type_definition, VALA_CCODE_NODE ((__temp335 = vala_ccode_comment_new (vala_source_reference_get_comment (vala_code_node_get_source_reference (VALA_CODE_NODE (iface)))))));
		(__temp335 == NULL ? NULL : (__temp335 = (g_object_unref (__temp335), NULL)));
	}
	vala_ccode_fragment_append (self->priv->header_type_definition, VALA_CCODE_NODE (self->priv->type_struct));
	(macro == NULL ? NULL : (macro = (g_free (macro), NULL)));
}


static void vala_code_generator_real_visit_end_interface (ValaCodeVisitor* base, ValaInterface* iface)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_INTERFACE (iface));
	ValaInterfaceRegisterFunction* type_fun = vala_interface_register_function_new (iface);
	vala_type_register_function_init_from_type (VALA_TYPE_REGISTER_FUNCTION (type_fun));
	ValaCCodeFunction* __temp336 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_member_declaration, VALA_CCODE_NODE ((__temp336 = vala_type_register_function_get_declaration (VALA_TYPE_REGISTER_FUNCTION (type_fun)))));
	(__temp336 == NULL ? NULL : (__temp336 = (g_object_unref (__temp336), NULL)));
	vala_ccode_fragment_append (self->priv->source_type_member_definition, VALA_CCODE_NODE (type_fun));
	ValaSymbol* __temp337 = NULL;
	self->priv->current_type_symbol = (__temp337 = NULL, (self->priv->current_type_symbol == NULL ? NULL : (self->priv->current_type_symbol = (g_object_unref (self->priv->current_type_symbol), NULL))), __temp337);
	(type_fun == NULL ? NULL : (type_fun = (g_object_unref (type_fun), NULL)));
}


static void vala_code_generator_real_visit_begin_enum (ValaCodeVisitor* base, ValaEnum* en)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_ENUM (en));
	ValaCCodeEnum* __temp338 = NULL;
	self->priv->cenum = (__temp338 = vala_ccode_enum_new (vala_data_type_get_cname (VALA_DATA_TYPE (en), FALSE)), (self->priv->cenum == NULL ? NULL : (self->priv->cenum = (g_object_unref (self->priv->cenum), NULL))), __temp338);
	if (vala_source_reference_get_comment (vala_code_node_get_source_reference (VALA_CODE_NODE (en))) != NULL) {
		ValaCCodeComment* __temp339 = NULL;
		vala_ccode_fragment_append (self->priv->header_type_definition, VALA_CCODE_NODE ((__temp339 = vala_ccode_comment_new (vala_source_reference_get_comment (vala_code_node_get_source_reference (VALA_CODE_NODE (en)))))));
		(__temp339 == NULL ? NULL : (__temp339 = (g_object_unref (__temp339), NULL)));
	}
	vala_ccode_fragment_append (self->priv->header_type_definition, VALA_CCODE_NODE (self->priv->cenum));
}


static void vala_code_generator_real_visit_enum_value (ValaCodeVisitor* base, ValaEnumValue* ev)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_ENUM_VALUE (ev));
	vala_ccode_enum_add_value (self->priv->cenum, vala_enum_value_get_cname (ev), NULL);
}


static void vala_code_generator_real_visit_end_callback (ValaCodeVisitor* base, ValaCallback* cb)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_CALLBACK (cb));
	ValaCCodeFunctionDeclarator* cfundecl = vala_ccode_function_declarator_new (vala_data_type_get_cname (VALA_DATA_TYPE (cb), FALSE));
	{
		GList* __temp340 = NULL;
		__temp340 = vala_callback_get_parameters (cb);
		GList* param_it;
		for (param_it = __temp340; param_it != NULL; param_it = param_it->next) {
			ValaFormalParameter* param = param_it->data;
			{
				vala_ccode_function_declarator_add_parameter (cfundecl, VALA_CCODE_FORMAL_PARAMETER (vala_code_node_get_ccodenode (VALA_CODE_NODE (param))));
			}
		}
		(__temp340 == NULL ? NULL : (__temp340 = (g_list_free (__temp340), NULL)));
	}
	ValaCCodeTypeDefinition* ctypedef = vala_ccode_type_definition_new (vala_type_reference_get_cname (vala_callback_get_return_type (cb), FALSE, FALSE), VALA_CCODE_DECLARATOR (cfundecl));
	if (VALA_DATA_TYPE (cb)->access != VALA_MEMBER_ACCESSIBILITY_PRIVATE) {
		vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE (ctypedef));
	} else {
		vala_ccode_fragment_append (self->priv->source_type_member_declaration, VALA_CCODE_NODE (ctypedef));
	}
	(cfundecl == NULL ? NULL : (cfundecl = (g_object_unref (cfundecl), NULL)));
	(ctypedef == NULL ? NULL : (ctypedef = (g_object_unref (ctypedef), NULL)));
}


static void vala_code_generator_real_visit_member (ValaCodeVisitor* base, ValaMember* m)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_MEMBER (m));
	/* stuff meant for all lockable members */
	if (VALA_IS_LOCKABLE (m) && vala_lockable_get_lock_used ((VALA_LOCKABLE (m)))) {
		char* __temp342 = NULL;
		char* __temp341 = NULL;
		vala_ccode_struct_add_field (self->priv->instance_priv_struct, (__temp341 = vala_type_reference_get_cname (self->priv->mutex_type, FALSE, FALSE)), (__temp342 = vala_code_generator_get_symbol_lock_name (self, vala_code_node_get_symbol (VALA_CODE_NODE (m)))));
		(__temp342 == NULL ? NULL : (__temp342 = (g_free (__temp342), NULL)));
		(__temp341 == NULL ? NULL : (__temp341 = (g_free (__temp341), NULL)));
		ValaCCodeExpressionStatement* __temp343 = NULL;
		vala_ccode_fragment_append (self->priv->instance_init_fragment, VALA_CCODE_NODE ((__temp343 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_member_access_new_pointer (VALA_CCODE_EXPRESSION (vala_ccode_member_access_new_pointer (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("self")), "priv")), vala_code_generator_get_symbol_lock_name (self, vala_code_node_get_symbol (VALA_CODE_NODE (m))))), VALA_CCODE_EXPRESSION (vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_method_get_cname (vala_struct_get_default_construction_method ((VALA_STRUCT (vala_type_reference_get_data_type (self->priv->mutex_type))))))))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))))));
		(__temp343 == NULL ? NULL : (__temp343 = (g_object_unref (__temp343), NULL)));
		ValaCCodeFunctionCall* fc = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("VALA_FREE_CHECKED")));
		ValaCCodeMemberAccess* __temp344 = NULL;
		vala_ccode_function_call_add_argument (fc, VALA_CCODE_EXPRESSION ((__temp344 = vala_ccode_member_access_new_pointer (VALA_CCODE_EXPRESSION (vala_ccode_member_access_new_pointer (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("self")), "priv")), vala_code_generator_get_symbol_lock_name (self, vala_code_node_get_symbol (VALA_CODE_NODE (m)))))));
		(__temp344 == NULL ? NULL : (__temp344 = (g_object_unref (__temp344), NULL)));
		ValaCCodeIdentifier* __temp345 = NULL;
		vala_ccode_function_call_add_argument (fc, VALA_CCODE_EXPRESSION ((__temp345 = vala_ccode_identifier_new (vala_data_type_get_free_function (vala_type_reference_get_data_type (self->priv->mutex_type))))));
		(__temp345 == NULL ? NULL : (__temp345 = (g_object_unref (__temp345), NULL)));
		ValaCCodeExpressionStatement* __temp346 = NULL;
		vala_ccode_fragment_append (self->priv->instance_dispose_fragment, VALA_CCODE_NODE ((__temp346 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (fc)))));
		(__temp346 == NULL ? NULL : (__temp346 = (g_object_unref (__temp346), NULL)));
		(fc == NULL ? NULL : (fc = (g_object_unref (fc), NULL)));
	}
}


static void vala_code_generator_real_visit_constant (ValaCodeVisitor* base, ValaConstant* c)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_CONSTANT (c));
	if (VALA_IS_DATA_TYPE (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (c)))))) {
		ValaDataType* t = VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (c)))));
		ValaCCodeDeclaration* cdecl = vala_ccode_declaration_new (vala_type_reference_get_const_cname (vala_constant_get_type_reference (c)));
		char* arr = "";
		if (VALA_IS_ARRAY (vala_type_reference_get_data_type (vala_constant_get_type_reference (c)))) {
			arr = "[]";
		}
		ValaCCodeVariableDeclarator* __temp347 = NULL;
		vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR ((__temp347 = vala_ccode_variable_declarator_new_with_initializer (g_strdup_printf ("%s%s", vala_constant_get_cname (c), arr), VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_constant_get_initializer (c))))))));
		(__temp347 == NULL ? NULL : (__temp347 = (g_object_unref (__temp347), NULL)));
		vala_ccode_declaration_set_modifiers (cdecl, VALA_CCODE_MODIFIERS_STATIC);
		if (c->access != VALA_MEMBER_ACCESSIBILITY_PRIVATE) {
			vala_ccode_fragment_append (self->priv->header_type_member_declaration, VALA_CCODE_NODE (cdecl));
		} else {
			vala_ccode_fragment_append (self->priv->source_type_member_declaration, VALA_CCODE_NODE (cdecl));
		}
		(cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL)));
	}
}


static void vala_code_generator_real_visit_field (ValaCodeVisitor* base, ValaField* f)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_FIELD (f));
	ValaCCodeExpression* lhs = NULL;
	ValaCCodeStruct* st = NULL;
	if (f->access != VALA_MEMBER_ACCESSIBILITY_PRIVATE) {
		st = self->priv->instance_struct;
		if (vala_field_get_instance (f)) {
			ValaCCodeExpression* __temp348 = NULL;
			lhs = (__temp348 = VALA_CCODE_EXPRESSION (vala_ccode_member_access_new_pointer (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("self")), vala_field_get_cname (f))), (lhs == NULL ? NULL : (lhs = (g_object_unref (lhs), NULL))), __temp348);
		}
	} else {
		if (f->access == VALA_MEMBER_ACCESSIBILITY_PRIVATE) {
			if (vala_field_get_instance (f)) {
				st = self->priv->instance_priv_struct;
				ValaCCodeExpression* __temp349 = NULL;
				lhs = (__temp349 = VALA_CCODE_EXPRESSION (vala_ccode_member_access_new_pointer (VALA_CCODE_EXPRESSION (vala_ccode_member_access_new_pointer (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("self")), "priv")), vala_field_get_cname (f))), (lhs == NULL ? NULL : (lhs = (g_object_unref (lhs), NULL))), __temp349);
			} else {
				if (VALA_IS_DATA_TYPE (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (f)))))) {
					ValaDataType* t = VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (f)))));
					ValaCCodeDeclaration* cdecl = vala_ccode_declaration_new (vala_type_reference_get_cname (vala_field_get_type_reference (f), FALSE, FALSE));
					ValaCCodeVariableDeclarator* var_decl = vala_ccode_variable_declarator_new (vala_field_get_cname (f));
					if (vala_field_get_initializer (f) != NULL) {
						vala_ccode_variable_declarator_set_initializer (var_decl, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_field_get_initializer (f)))));
					}
					vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR (var_decl));
					vala_ccode_declaration_set_modifiers (cdecl, VALA_CCODE_MODIFIERS_STATIC);
					vala_ccode_fragment_append (self->priv->source_type_member_declaration, VALA_CCODE_NODE (cdecl));
					(cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL)));
					(var_decl == NULL ? NULL : (var_decl = (g_object_unref (var_decl), NULL)));
				}
			}
		}
	}
	if (vala_field_get_instance (f)) {
		char* __temp350 = NULL;
		vala_ccode_struct_add_field (st, (__temp350 = vala_type_reference_get_cname (vala_field_get_type_reference (f), FALSE, FALSE)), vala_field_get_cname (f));
		(__temp350 == NULL ? NULL : (__temp350 = (g_free (__temp350), NULL)));
		if (VALA_IS_ARRAY (vala_type_reference_get_data_type (vala_field_get_type_reference (f))) && !vala_field_get_no_array_length (f)) {
			/* create fields to store array dimensions*/
			ValaArray* arr = VALA_ARRAY (vala_type_reference_get_data_type (vala_field_get_type_reference (f)));
			{
				int dim;
				for (dim = 1; dim <= vala_array_get_rank (arr); dim++) {
					ValaTypeReference* len_type = vala_type_reference_new ();
					vala_type_reference_set_data_type (len_type, vala_type_reference_get_data_type (self->priv->int_type));
					char* __temp352 = NULL;
					char* __temp351 = NULL;
					vala_ccode_struct_add_field (st, (__temp351 = vala_type_reference_get_cname (len_type, FALSE, FALSE)), (__temp352 = vala_code_generator_get_array_length_cname (self, vala_field_get_name (f), dim)));
					(__temp352 == NULL ? NULL : (__temp352 = (g_free (__temp352), NULL)));
					(__temp351 == NULL ? NULL : (__temp351 = (g_free (__temp351), NULL)));
					(len_type == NULL ? NULL : (len_type = (g_object_unref (len_type), NULL)));
				}
			}
		}
		if (vala_field_get_initializer (f) != NULL) {
			ValaCCodeExpressionStatement* __temp353 = NULL;
			vala_ccode_fragment_append (self->priv->instance_init_fragment, VALA_CCODE_NODE ((__temp353 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (vala_ccode_assignment_new (lhs, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_field_get_initializer (f)))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))))));
			(__temp353 == NULL ? NULL : (__temp353 = (g_object_unref (__temp353), NULL)));
			if (VALA_IS_ARRAY (vala_type_reference_get_data_type (vala_field_get_type_reference (f))) && !vala_field_get_no_array_length (f) && VALA_IS_ARRAY_CREATION_EXPRESSION (vala_field_get_initializer (f))) {
				ValaMemberAccess* ma = vala_member_access_new_simple (vala_field_get_name (f), NULL);
				vala_expression_set_symbol_reference (VALA_EXPRESSION (ma), vala_code_node_get_symbol (VALA_CODE_NODE (f)));
				ValaCCodeExpression* array_len_lhs = vala_code_generator_get_array_length_cexpression (self, VALA_EXPRESSION (ma), 1);
				GList* sizes = vala_array_creation_expression_get_sizes ((VALA_ARRAY_CREATION_EXPRESSION (vala_field_get_initializer (f))));
				ValaExpression* size = VALA_EXPRESSION (sizes->data);
				ValaCCodeExpressionStatement* __temp354 = NULL;
				vala_ccode_fragment_append (self->priv->instance_init_fragment, VALA_CCODE_NODE ((__temp354 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (vala_ccode_assignment_new (array_len_lhs, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (size))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))))));
				(__temp354 == NULL ? NULL : (__temp354 = (g_object_unref (__temp354), NULL)));
				(ma == NULL ? NULL : (ma = (g_object_unref (ma), NULL)));
				(array_len_lhs == NULL ? NULL : (array_len_lhs = (g_object_unref (array_len_lhs), NULL)));
				(sizes == NULL ? NULL : (sizes = (g_list_free (sizes), NULL)));
			}
		}
		if (vala_type_reference_get_takes_ownership (vala_field_get_type_reference (f))) {
			ValaCCodeExpressionStatement* __temp355 = NULL;
			vala_ccode_fragment_append (self->priv->instance_dispose_fragment, VALA_CCODE_NODE ((__temp355 = vala_ccode_expression_statement_new (vala_code_generator_get_unref_expression (self, lhs, vala_field_get_type_reference (f))))));
			(__temp355 == NULL ? NULL : (__temp355 = (g_object_unref (__temp355), NULL)));
		}
	}
	(lhs == NULL ? NULL : (lhs = (g_object_unref (lhs), NULL)));
}


static void vala_code_generator_real_visit_begin_method (ValaCodeVisitor* base, ValaMethod* m)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_METHOD (m));
	ValaSymbol* __temp357 = NULL;
	ValaSymbol* __temp356 = NULL;
	self->priv->current_symbol = (__temp357 = (__temp356 = vala_code_node_get_symbol (VALA_CODE_NODE (m)), (__temp356 == NULL ? NULL : g_object_ref (__temp356))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp357);
	ValaTypeReference* __temp359 = NULL;
	ValaTypeReference* __temp358 = NULL;
	self->priv->current_return_type = (__temp359 = (__temp358 = vala_method_get_return_type (m), (__temp358 == NULL ? NULL : g_object_ref (__temp358))), (self->priv->current_return_type == NULL ? NULL : (self->priv->current_return_type = (g_object_unref (self->priv->current_return_type), NULL))), __temp359);
}


static ValaCCodeStatement* vala_code_generator_create_method_type_check_statement (ValaCodeGenerator* self, ValaMethod* m, ValaDataType* t, gboolean non_null, char* var_name)
{
	g_return_val_if_fail (VALA_IS_CODE_GENERATOR (self), NULL);
	g_return_val_if_fail (VALA_IS_METHOD (m), NULL);
	g_return_val_if_fail (VALA_IS_DATA_TYPE (t), NULL);
	g_return_val_if_fail (var_name != NULL, NULL);
	return vala_code_generator_create_type_check_statement (self, VALA_CODE_NODE (m), vala_type_reference_get_data_type (vala_method_get_return_type (m)), t, non_null, var_name);
}


static ValaCCodeStatement* vala_code_generator_create_property_type_check_statement (ValaCodeGenerator* self, ValaProperty* prop, gboolean getter, ValaDataType* t, gboolean non_null, char* var_name)
{
	g_return_val_if_fail (VALA_IS_CODE_GENERATOR (self), NULL);
	g_return_val_if_fail (VALA_IS_PROPERTY (prop), NULL);
	g_return_val_if_fail (VALA_IS_DATA_TYPE (t), NULL);
	g_return_val_if_fail (var_name != NULL, NULL);
	if (getter) {
		return vala_code_generator_create_type_check_statement (self, VALA_CODE_NODE (prop), vala_type_reference_get_data_type (vala_property_get_type_reference (prop)), t, non_null, var_name);
	} else {
		return vala_code_generator_create_type_check_statement (self, VALA_CODE_NODE (prop), NULL, t, non_null, var_name);
	}
}


static ValaCCodeStatement* vala_code_generator_create_type_check_statement (ValaCodeGenerator* self, ValaCodeNode* method_node, ValaDataType* ret_type, ValaDataType* t, gboolean non_null, char* var_name)
{
	g_return_val_if_fail (VALA_IS_CODE_GENERATOR (self), NULL);
	g_return_val_if_fail (VALA_IS_CODE_NODE (method_node), NULL);
	g_return_val_if_fail (ret_type == NULL || VALA_IS_DATA_TYPE (ret_type), NULL);
	g_return_val_if_fail (VALA_IS_DATA_TYPE (t), NULL);
	g_return_val_if_fail (var_name != NULL, NULL);
	ValaCCodeFunctionCall* ccheck = vala_ccode_function_call_new (NULL);
	if (VALA_IS_CLASS (t) || VALA_IS_INTERFACE (t)) {
		ValaCCodeFunctionCall* ctype_check = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_data_type_get_upper_case_cname (t, "IS_"))));
		ValaCCodeIdentifier* __temp363 = NULL;
		vala_ccode_function_call_add_argument (ctype_check, VALA_CCODE_EXPRESSION ((__temp363 = vala_ccode_identifier_new (var_name))));
		(__temp363 == NULL ? NULL : (__temp363 = (g_object_unref (__temp363), NULL)));
		ValaCCodeFunctionCall* __temp364 = NULL;
		ValaCCodeExpression* cexpr = VALA_CCODE_EXPRESSION ((__temp364 = ctype_check, (__temp364 == NULL ? NULL : g_object_ref (__temp364))));
		if (!non_null) {
			ValaCCodeBinaryExpression* cnull = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_EQUALITY, VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (var_name)), VALA_CCODE_EXPRESSION (vala_ccode_constant_new ("NULL")));
			ValaCCodeExpression* __temp365 = NULL;
			cexpr = (__temp365 = VALA_CCODE_EXPRESSION (vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_OR, VALA_CCODE_EXPRESSION (cnull), VALA_CCODE_EXPRESSION (ctype_check))), (cexpr == NULL ? NULL : (cexpr = (g_object_unref (cexpr), NULL))), __temp365);
			(cnull == NULL ? NULL : (cnull = (g_object_unref (cnull), NULL)));
		}
		vala_ccode_function_call_add_argument (ccheck, cexpr);
		(ctype_check == NULL ? NULL : (ctype_check = (g_object_unref (ctype_check), NULL)));
		(cexpr == NULL ? NULL : (cexpr = (g_object_unref (cexpr), NULL)));
	} else {
		if (!non_null) {
			gpointer __temp366;
			return VALA_CCODE_STATEMENT ((__temp366 = NULL, (ccheck == NULL ? NULL : (ccheck = (g_object_unref (ccheck), NULL))), __temp366));
		} else {
			ValaCCodeBinaryExpression* cnonnull = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_INEQUALITY, VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (var_name)), VALA_CCODE_EXPRESSION (vala_ccode_constant_new ("NULL")));
			vala_ccode_function_call_add_argument (ccheck, VALA_CCODE_EXPRESSION (cnonnull));
			(cnonnull == NULL ? NULL : (cnonnull = (g_object_unref (cnonnull), NULL)));
		}
	}
	if (ret_type == NULL) {
		/* void function */
		ValaCCodeIdentifier* __temp367 = NULL;
		vala_ccode_function_call_set_call (ccheck, VALA_CCODE_EXPRESSION ((__temp367 = vala_ccode_identifier_new ("g_return_if_fail"))));
		(__temp367 == NULL ? NULL : (__temp367 = (g_object_unref (__temp367), NULL)));
	} else {
		ValaCCodeIdentifier* __temp368 = NULL;
		vala_ccode_function_call_set_call (ccheck, VALA_CCODE_EXPRESSION ((__temp368 = vala_ccode_identifier_new ("g_return_val_if_fail"))));
		(__temp368 == NULL ? NULL : (__temp368 = (g_object_unref (__temp368), NULL)));
		if (vala_data_type_is_reference_type (ret_type)) {
			ValaCCodeConstant* __temp369 = NULL;
			vala_ccode_function_call_add_argument (ccheck, VALA_CCODE_EXPRESSION ((__temp369 = vala_ccode_constant_new ("NULL"))));
			(__temp369 == NULL ? NULL : (__temp369 = (g_object_unref (__temp369), NULL)));
		} else {
			if (ret_type == vala_type_reference_get_data_type (self->priv->bool_type)) {
				ValaCCodeConstant* __temp370 = NULL;
				vala_ccode_function_call_add_argument (ccheck, VALA_CCODE_EXPRESSION ((__temp370 = vala_ccode_constant_new ("FALSE"))));
				(__temp370 == NULL ? NULL : (__temp370 = (g_object_unref (__temp370), NULL)));
			} else {
				if (ret_type == vala_type_reference_get_data_type (self->priv->char_type) || ret_type == vala_type_reference_get_data_type (self->priv->unichar_type) || ret_type == vala_type_reference_get_data_type (self->priv->short_type) || ret_type == vala_type_reference_get_data_type (self->priv->ushort_type) || ret_type == vala_type_reference_get_data_type (self->priv->int_type) || ret_type == vala_type_reference_get_data_type (self->priv->uint_type) || ret_type == vala_type_reference_get_data_type (self->priv->long_type) || ret_type == vala_type_reference_get_data_type (self->priv->ulong_type) || ret_type == vala_type_reference_get_data_type (self->priv->int64_type) || ret_type == vala_type_reference_get_data_type (self->priv->uint64_type) || ret_type == vala_type_reference_get_data_type (self->priv->double_type) || ret_type == vala_type_reference_get_data_type (self->priv->float_type) || VALA_IS_ENUM (ret_type) || VALA_IS_FLAGS (ret_type)) {
					ValaCCodeConstant* __temp371 = NULL;
					vala_ccode_function_call_add_argument (ccheck, VALA_CCODE_EXPRESSION ((__temp371 = vala_ccode_constant_new ("0"))));
					(__temp371 == NULL ? NULL : (__temp371 = (g_object_unref (__temp371), NULL)));
				} else {
					vala_report_error (vala_code_node_get_source_reference (method_node), "not supported return type for runtime type checks");
					gpointer __temp372;
					return VALA_CCODE_STATEMENT ((__temp372 = NULL, (ccheck == NULL ? NULL : (ccheck = (g_object_unref (ccheck), NULL))), __temp372));
				}
			}
		}
	}
	ValaCCodeExpressionStatement* __temp373 = NULL;
	return VALA_CCODE_STATEMENT ((__temp373 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (ccheck)), (ccheck == NULL ? NULL : (ccheck = (g_object_unref (ccheck), NULL))), __temp373));
	(ccheck == NULL ? NULL : (ccheck = (g_object_unref (ccheck), NULL)));
}


static ValaDataType* vala_code_generator_find_parent_type (ValaCodeGenerator* self, ValaCodeNode* node)
{
	g_return_val_if_fail (VALA_IS_CODE_GENERATOR (self), NULL);
	g_return_val_if_fail (node == NULL || VALA_IS_CODE_NODE (node), NULL);
	ValaSymbol* sym = vala_code_node_get_symbol (node);
	while (sym != NULL) {
		if (VALA_IS_DATA_TYPE (vala_symbol_get_node (sym))) {
			return VALA_DATA_TYPE (vala_symbol_get_node (sym));
		}
		sym = vala_symbol_get_parent_symbol (sym);
	}
	return VALA_DATA_TYPE (NULL);
}


static char* vala_code_generator_get_array_length_cname (ValaCodeGenerator* self, char* array_cname, int dim)
{
	g_return_val_if_fail (VALA_IS_CODE_GENERATOR (self), NULL);
	g_return_val_if_fail (array_cname != NULL, NULL);
	return g_strdup_printf ("%s_length%d", array_cname, dim);
}


static void vala_code_generator_real_visit_end_method (ValaCodeVisitor* base, ValaMethod* m)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_METHOD (m));
	ValaSymbol* __temp378 = NULL;
	ValaSymbol* __temp377 = NULL;
	self->priv->current_symbol = (__temp378 = (__temp377 = vala_symbol_get_parent_symbol (self->priv->current_symbol), (__temp377 == NULL ? NULL : g_object_ref (__temp377))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp378);
	ValaTypeReference* __temp379 = NULL;
	self->priv->current_return_type = (__temp379 = NULL, (self->priv->current_return_type == NULL ? NULL : (self->priv->current_return_type = (g_object_unref (self->priv->current_return_type), NULL))), __temp379);
	if (vala_symbol_get_parent_symbol (self->priv->current_symbol) != NULL && VALA_IS_METHOD (vala_symbol_get_node (vala_symbol_get_parent_symbol (self->priv->current_symbol)))) {
		/* lambda expressions produce nested methods */
		ValaMethod* up_method = VALA_METHOD (vala_symbol_get_node (vala_symbol_get_parent_symbol (self->priv->current_symbol)));
		ValaTypeReference* __temp381 = NULL;
		ValaTypeReference* __temp380 = NULL;
		self->priv->current_return_type = (__temp381 = (__temp380 = vala_method_get_return_type (up_method), (__temp380 == NULL ? NULL : g_object_ref (__temp380))), (self->priv->current_return_type == NULL ? NULL : (self->priv->current_return_type = (g_object_unref (self->priv->current_return_type), NULL))), __temp381);
	}
	ValaCCodeFunction* __temp382 = NULL;
	self->priv->function = (__temp382 = vala_ccode_function_new (vala_method_get_real_cname (m), vala_type_reference_get_cname (vala_method_get_return_type (m), FALSE, FALSE)), (self->priv->function == NULL ? NULL : (self->priv->function = (g_object_unref (self->priv->function), NULL))), __temp382);
	ValaCCodeFunctionDeclarator* vdeclarator = NULL;
	ValaCCodeFormalParameter* instance_param = NULL;
	if (vala_method_get_instance (m)) {
		ValaTypeReference* this_type = vala_type_reference_new ();
		vala_type_reference_set_data_type (this_type, vala_code_generator_find_parent_type (self, VALA_CODE_NODE (m)));
		if (!vala_method_get_overrides (m)) {
			ValaCCodeFormalParameter* __temp383 = NULL;
			instance_param = (__temp383 = vala_ccode_formal_parameter_new ("self", vala_type_reference_get_cname (this_type, FALSE, FALSE)), (instance_param == NULL ? NULL : (instance_param = (g_object_unref (instance_param), NULL))), __temp383);
		} else {
			ValaTypeReference* base_type = vala_type_reference_new ();
			vala_type_reference_set_data_type (base_type, VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (vala_method_get_base_method (m)))))));
			ValaCCodeFormalParameter* __temp384 = NULL;
			instance_param = (__temp384 = vala_ccode_formal_parameter_new ("base", vala_type_reference_get_cname (base_type, FALSE, FALSE)), (instance_param == NULL ? NULL : (instance_param = (g_object_unref (instance_param), NULL))), __temp384);
			(base_type == NULL ? NULL : (base_type = (g_object_unref (base_type), NULL)));
		}
		if (!vala_method_get_instance_last (m)) {
			vala_ccode_function_add_parameter (self->priv->function, instance_param);
		}
		if (vala_method_get_is_abstract (m) || vala_method_get_is_virtual (m)) {
			ValaCCodeDeclaration* vdecl = vala_ccode_declaration_new (vala_type_reference_get_cname (vala_method_get_return_type (m), FALSE, FALSE));
			ValaCCodeFunctionDeclarator* __temp385 = NULL;
			vdeclarator = (__temp385 = vala_ccode_function_declarator_new (vala_method_get_name (m)), (vdeclarator == NULL ? NULL : (vdeclarator = (g_object_unref (vdeclarator), NULL))), __temp385);
			vala_ccode_declaration_add_declarator (vdecl, VALA_CCODE_DECLARATOR (vdeclarator));
			vala_ccode_struct_add_declaration (self->priv->type_struct, vdecl);
			vala_ccode_function_declarator_add_parameter (vdeclarator, instance_param);
			(vdecl == NULL ? NULL : (vdecl = (g_object_unref (vdecl), NULL)));
		}
		(this_type == NULL ? NULL : (this_type = (g_object_unref (this_type), NULL)));
	}
	GList* params = vala_invokable_get_parameters (VALA_INVOKABLE (m));
	{
		GList* __temp386 = NULL;
		__temp386 = params;
		GList* param_it;
		for (param_it = __temp386; param_it != NULL; param_it = param_it->next) {
			ValaFormalParameter* param = param_it->data;
			{
				if (!vala_formal_parameter_get_no_array_length (param) && VALA_IS_ARRAY (vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (param)))) {
					ValaArray* arr = VALA_ARRAY (vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (param)));
					char* length_ctype = "int";
					if (vala_type_reference_get_is_out (vala_formal_parameter_get_type_reference (param))) {
						length_ctype = "int*";
					}
					{
						int dim;
						for (dim = 1; dim <= vala_array_get_rank (arr); dim++) {
							ValaCCodeFormalParameter* cparam = vala_ccode_formal_parameter_new (vala_code_generator_get_array_length_cname (self, vala_formal_parameter_get_name (param), dim), length_ctype);
							vala_ccode_function_add_parameter (self->priv->function, cparam);
							if (vdeclarator != NULL) {
								vala_ccode_function_declarator_add_parameter (vdeclarator, cparam);
							}
							(cparam == NULL ? NULL : (cparam = (g_object_unref (cparam), NULL)));
						}
					}
				}
				vala_ccode_function_add_parameter (self->priv->function, VALA_CCODE_FORMAL_PARAMETER (vala_code_node_get_ccodenode (VALA_CODE_NODE (param))));
				if (vdeclarator != NULL) {
					vala_ccode_function_declarator_add_parameter (vdeclarator, VALA_CCODE_FORMAL_PARAMETER (vala_code_node_get_ccodenode (VALA_CODE_NODE (param))));
				}
			}
		}
	}
	if (vala_method_get_instance (m) && vala_method_get_instance_last (m)) {
		vala_ccode_function_add_parameter (self->priv->function, instance_param);
	}
	/* real function declaration and definition not needed
	 * for abstract methods */
	if (!vala_method_get_is_abstract (m)) {
		if (m->access != VALA_MEMBER_ACCESSIBILITY_PRIVATE && !(vala_method_get_is_virtual (m) || vala_method_get_overrides (m))) {
			/* public methods need function declaration in
			 * header file except virtual/overridden methods */
			ValaCCodeFunction* __temp387 = NULL;
			vala_ccode_fragment_append (self->priv->header_type_member_declaration, VALA_CCODE_NODE ((__temp387 = vala_ccode_function_copy (self->priv->function))));
			(__temp387 == NULL ? NULL : (__temp387 = (g_object_unref (__temp387), NULL)));
		} else {
			/* declare all other functions in source file to
			 * avoid dependency on order within source file */
			vala_ccode_function_set_modifiers (self->priv->function, vala_ccode_function_get_modifiers (self->priv->function) | VALA_CCODE_MODIFIERS_STATIC);
			ValaCCodeFunction* __temp388 = NULL;
			vala_ccode_fragment_append (self->priv->source_type_member_declaration, VALA_CCODE_NODE ((__temp388 = vala_ccode_function_copy (self->priv->function))));
			(__temp388 == NULL ? NULL : (__temp388 = (g_object_unref (__temp388), NULL)));
		}
		/* Methods imported from a plain C file don't
		 * have a body, e.g. Vala.Parser.parse_file () */
		if (vala_method_get_body (m) != NULL) {
			vala_ccode_function_set_block (self->priv->function, VALA_CCODE_BLOCK (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_method_get_body (m)))));
			ValaCCodeFragment* cinit = g_object_new (VALA_TYPE_CCODE_FRAGMENT, NULL);
			vala_ccode_block_prepend_statement (vala_ccode_function_get_block (self->priv->function), VALA_CCODE_NODE (cinit));
			if (VALA_IS_CLASS (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (m)))))) {
				ValaClass* cl = VALA_CLASS (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (m)))));
				if (vala_method_get_overrides (m)) {
					ValaCCodeFunctionCall* ccall = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), NULL))));
					ValaCCodeIdentifier* __temp389 = NULL;
					vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp389 = vala_ccode_identifier_new ("base"))));
					(__temp389 == NULL ? NULL : (__temp389 = (g_object_unref (__temp389), NULL)));
					ValaCCodeDeclaration* cdecl = vala_ccode_declaration_new (g_strdup_printf ("%s *", vala_data_type_get_cname (VALA_DATA_TYPE (cl), FALSE)));
					ValaCCodeVariableDeclarator* __temp390 = NULL;
					vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR ((__temp390 = vala_ccode_variable_declarator_new_with_initializer ("self", VALA_CCODE_EXPRESSION (ccall)))));
					(__temp390 == NULL ? NULL : (__temp390 = (g_object_unref (__temp390), NULL)));
					vala_ccode_fragment_append (cinit, VALA_CCODE_NODE (cdecl));
					(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
					(cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL)));
				} else {
					if (vala_method_get_instance (m)) {
						ValaCCodeStatement* __temp391 = NULL;
						vala_ccode_fragment_append (cinit, VALA_CCODE_NODE ((__temp391 = vala_code_generator_create_method_type_check_statement (self, m, VALA_DATA_TYPE (cl), TRUE, "self"))));
						(__temp391 == NULL ? NULL : (__temp391 = (g_object_unref (__temp391), NULL)));
					}
				}
			}
			{
				GList* __temp392 = NULL;
				__temp392 = vala_invokable_get_parameters (VALA_INVOKABLE (m));
				GList* param_it;
				for (param_it = __temp392; param_it != NULL; param_it = param_it->next) {
					ValaFormalParameter* param = param_it->data;
					{
						ValaDataType* t = vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (param));
						if (t != NULL && vala_data_type_is_reference_type (t) && !vala_type_reference_get_is_out (vala_formal_parameter_get_type_reference (param))) {
							ValaCCodeStatement* type_check = vala_code_generator_create_method_type_check_statement (self, m, t, vala_type_reference_get_non_null (vala_formal_parameter_get_type_reference (param)), vala_formal_parameter_get_name (param));
							if (type_check != NULL) {
								vala_ccode_fragment_append (cinit, VALA_CCODE_NODE (type_check));
							}
							(type_check == NULL ? NULL : (type_check = (g_object_unref (type_check), NULL)));
						}
					}
				}
				(__temp392 == NULL ? NULL : (__temp392 = (g_list_free (__temp392), NULL)));
			}
			if (vala_code_node_get_source_reference (VALA_CODE_NODE (m)) != NULL && vala_source_reference_get_comment (vala_code_node_get_source_reference (VALA_CODE_NODE (m))) != NULL) {
				ValaCCodeComment* __temp393 = NULL;
				vala_ccode_fragment_append (self->priv->source_type_member_definition, VALA_CCODE_NODE ((__temp393 = vala_ccode_comment_new (vala_source_reference_get_comment (vala_code_node_get_source_reference (VALA_CODE_NODE (m)))))));
				(__temp393 == NULL ? NULL : (__temp393 = (g_object_unref (__temp393), NULL)));
			}
			vala_ccode_fragment_append (self->priv->source_type_member_definition, VALA_CCODE_NODE (self->priv->function));
			if (vala_method_get_construction (m)) {
				/* declare construction parameter array*/
				ValaCCodeFunctionCall* cparamsinit = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("g_new0")));
				ValaCCodeIdentifier* __temp394 = NULL;
				vala_ccode_function_call_add_argument (cparamsinit, VALA_CCODE_EXPRESSION ((__temp394 = vala_ccode_identifier_new ("GParameter"))));
				(__temp394 == NULL ? NULL : (__temp394 = (g_object_unref (__temp394), NULL)));
				ValaCCodeConstant* __temp395 = NULL;
				vala_ccode_function_call_add_argument (cparamsinit, VALA_CCODE_EXPRESSION ((__temp395 = vala_ccode_constant_new (g_strdup_printf ("%i", vala_method_get_n_construction_params (m))))));
				(__temp395 == NULL ? NULL : (__temp395 = (g_object_unref (__temp395), NULL)));
				ValaCCodeDeclaration* cdecl = vala_ccode_declaration_new ("GParameter *");
				ValaCCodeVariableDeclarator* __temp396 = NULL;
				vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR ((__temp396 = vala_ccode_variable_declarator_new_with_initializer ("__params", VALA_CCODE_EXPRESSION (cparamsinit)))));
				(__temp396 == NULL ? NULL : (__temp396 = (g_object_unref (__temp396), NULL)));
				vala_ccode_fragment_append (cinit, VALA_CCODE_NODE (cdecl));
				ValaCCodeDeclaration* __temp397 = NULL;
				cdecl = (__temp397 = vala_ccode_declaration_new ("GParameter *"), (cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL))), __temp397);
				ValaCCodeVariableDeclarator* __temp398 = NULL;
				vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR ((__temp398 = vala_ccode_variable_declarator_new_with_initializer ("__params_it", VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("__params"))))));
				(__temp398 == NULL ? NULL : (__temp398 = (g_object_unref (__temp398), NULL)));
				vala_ccode_fragment_append (cinit, VALA_CCODE_NODE (cdecl));
				(cparamsinit == NULL ? NULL : (cparamsinit = (g_object_unref (cparamsinit), NULL)));
				(cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL)));
			}
			(cinit == NULL ? NULL : (cinit = (g_object_unref (cinit), NULL)));
		}
	}
	if (vala_method_get_is_abstract (m) || vala_method_get_is_virtual (m)) {
		ValaCCodeFunction* vfunc = vala_ccode_function_new (vala_method_get_cname (m), vala_type_reference_get_cname (vala_method_get_return_type (m), FALSE, FALSE));
		ValaTypeReference* this_type = vala_type_reference_new ();
		vala_type_reference_set_data_type (this_type, VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (m))))));
		ValaCCodeFormalParameter* cparam = vala_ccode_formal_parameter_new ("self", vala_type_reference_get_cname (this_type, FALSE, FALSE));
		vala_ccode_function_add_parameter (vfunc, cparam);
		ValaCCodeBlock* vblock = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
		ValaCCodeFunctionCall* vcast = NULL;
		if (VALA_IS_INTERFACE (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (m)))))) {
			ValaInterface* iface = VALA_INTERFACE (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (m)))));
			ValaCCodeFunctionCall* __temp400 = NULL;
			char* __temp399 = NULL;
			vcast = (__temp400 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (g_strdup_printf ("%s_GET_INTERFACE", (__temp399 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (iface), NULL)))))), (vcast == NULL ? NULL : (vcast = (g_object_unref (vcast), NULL))), __temp400);
			(__temp399 == NULL ? NULL : (__temp399 = (g_free (__temp399), NULL)));
		} else {
			ValaClass* cl = VALA_CLASS (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (m)))));
			ValaCCodeFunctionCall* __temp402 = NULL;
			char* __temp401 = NULL;
			vcast = (__temp402 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (g_strdup_printf ("%s_GET_CLASS", (__temp401 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), NULL)))))), (vcast == NULL ? NULL : (vcast = (g_object_unref (vcast), NULL))), __temp402);
			(__temp401 == NULL ? NULL : (__temp401 = (g_free (__temp401), NULL)));
		}
		ValaCCodeIdentifier* __temp403 = NULL;
		vala_ccode_function_call_add_argument (vcast, VALA_CCODE_EXPRESSION ((__temp403 = vala_ccode_identifier_new ("self"))));
		(__temp403 == NULL ? NULL : (__temp403 = (g_object_unref (__temp403), NULL)));
		ValaCCodeFunctionCall* vcall = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_member_access_new_pointer (VALA_CCODE_EXPRESSION (vcast), vala_method_get_name (m))));
		ValaCCodeIdentifier* __temp404 = NULL;
		vala_ccode_function_call_add_argument (vcall, VALA_CCODE_EXPRESSION ((__temp404 = vala_ccode_identifier_new ("self"))));
		(__temp404 == NULL ? NULL : (__temp404 = (g_object_unref (__temp404), NULL)));
		GList* params = vala_invokable_get_parameters (VALA_INVOKABLE (m));
		{
			GList* __temp406 = NULL;
			__temp406 = params;
			GList* param_it;
			for (param_it = __temp406; param_it != NULL; param_it = param_it->next) {
				ValaFormalParameter* param = param_it->data;
				{
					vala_ccode_function_add_parameter (vfunc, VALA_CCODE_FORMAL_PARAMETER (vala_code_node_get_ccodenode (VALA_CODE_NODE (param))));
					ValaCCodeIdentifier* __temp405 = NULL;
					vala_ccode_function_call_add_argument (vcall, VALA_CCODE_EXPRESSION ((__temp405 = vala_ccode_identifier_new (vala_formal_parameter_get_name (param)))));
					(__temp405 == NULL ? NULL : (__temp405 = (g_object_unref (__temp405), NULL)));
				}
			}
		}
		if (vala_type_reference_get_data_type (vala_method_get_return_type (m)) == NULL) {
			ValaCCodeExpressionStatement* __temp407 = NULL;
			vala_ccode_block_add_statement (vblock, VALA_CCODE_NODE ((__temp407 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (vcall)))));
			(__temp407 == NULL ? NULL : (__temp407 = (g_object_unref (__temp407), NULL)));
		} else {
			/* pass method return value */
			ValaCCodeReturnStatement* __temp408 = NULL;
			vala_ccode_block_add_statement (vblock, VALA_CCODE_NODE ((__temp408 = vala_ccode_return_statement_new (VALA_CCODE_EXPRESSION (vcall)))));
			(__temp408 == NULL ? NULL : (__temp408 = (g_object_unref (__temp408), NULL)));
		}
		ValaCCodeFunction* __temp409 = NULL;
		vala_ccode_fragment_append (self->priv->header_type_member_declaration, VALA_CCODE_NODE ((__temp409 = vala_ccode_function_copy (vfunc))));
		(__temp409 == NULL ? NULL : (__temp409 = (g_object_unref (__temp409), NULL)));
		vala_ccode_function_set_block (vfunc, vblock);
		vala_ccode_fragment_append (self->priv->source_type_member_definition, VALA_CCODE_NODE (vfunc));
		(vfunc == NULL ? NULL : (vfunc = (g_object_unref (vfunc), NULL)));
		(this_type == NULL ? NULL : (this_type = (g_object_unref (this_type), NULL)));
		(cparam == NULL ? NULL : (cparam = (g_object_unref (cparam), NULL)));
		(vblock == NULL ? NULL : (vblock = (g_object_unref (vblock), NULL)));
		(vcast == NULL ? NULL : (vcast = (g_object_unref (vcast), NULL)));
		(vcall == NULL ? NULL : (vcall = (g_object_unref (vcall), NULL)));
		(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
	}
	if (vala_method_get_construction (m)) {
		ValaCCodeReturnStatement* creturn = vala_ccode_return_statement_new (NULL);
		ValaCCodeIdentifier* __temp410 = NULL;
		vala_ccode_return_statement_set_return_expression (creturn, VALA_CCODE_EXPRESSION ((__temp410 = vala_ccode_identifier_new ("self"))));
		(__temp410 == NULL ? NULL : (__temp410 = (g_object_unref (__temp410), NULL)));
		vala_ccode_block_add_statement (vala_ccode_function_get_block (self->priv->function), VALA_CCODE_NODE (creturn));
		(creturn == NULL ? NULL : (creturn = (g_object_unref (creturn), NULL)));
	}
	if (vala_method_get_name (m) != NULL && g_utf8_collate (vala_method_get_name (m), "main") == 0) {
		ValaCCodeFunction* cmain = vala_ccode_function_new ("main", "int");
		ValaCCodeFormalParameter* __temp411 = NULL;
		vala_ccode_function_add_parameter (cmain, (__temp411 = vala_ccode_formal_parameter_new ("argc", "int")));
		(__temp411 == NULL ? NULL : (__temp411 = (g_object_unref (__temp411), NULL)));
		ValaCCodeFormalParameter* __temp412 = NULL;
		vala_ccode_function_add_parameter (cmain, (__temp412 = vala_ccode_formal_parameter_new ("argv", "char **")));
		(__temp412 == NULL ? NULL : (__temp412 = (g_object_unref (__temp412), NULL)));
		ValaCCodeBlock* main_block = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
		ValaCCodeExpressionStatement* __temp413 = NULL;
		vala_ccode_block_add_statement (main_block, VALA_CCODE_NODE ((__temp413 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("g_type_init"))))))));
		(__temp413 == NULL ? NULL : (__temp413 = (g_object_unref (__temp413), NULL)));
		ValaCCodeFunctionCall* main_call = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_ccode_function_get_name (self->priv->function))));
		ValaCCodeIdentifier* __temp414 = NULL;
		vala_ccode_function_call_add_argument (main_call, VALA_CCODE_EXPRESSION ((__temp414 = vala_ccode_identifier_new ("argc"))));
		(__temp414 == NULL ? NULL : (__temp414 = (g_object_unref (__temp414), NULL)));
		ValaCCodeIdentifier* __temp415 = NULL;
		vala_ccode_function_call_add_argument (main_call, VALA_CCODE_EXPRESSION ((__temp415 = vala_ccode_identifier_new ("argv"))));
		(__temp415 == NULL ? NULL : (__temp415 = (g_object_unref (__temp415), NULL)));
		ValaCCodeReturnStatement* __temp416 = NULL;
		vala_ccode_block_add_statement (main_block, VALA_CCODE_NODE ((__temp416 = vala_ccode_return_statement_new (VALA_CCODE_EXPRESSION (main_call)))));
		(__temp416 == NULL ? NULL : (__temp416 = (g_object_unref (__temp416), NULL)));
		vala_ccode_function_set_block (cmain, main_block);
		vala_ccode_fragment_append (self->priv->source_type_member_definition, VALA_CCODE_NODE (cmain));
		(cmain == NULL ? NULL : (cmain = (g_object_unref (cmain), NULL)));
		(main_block == NULL ? NULL : (main_block = (g_object_unref (main_block), NULL)));
		(main_call == NULL ? NULL : (main_call = (g_object_unref (main_call), NULL)));
	}
	(vdeclarator == NULL ? NULL : (vdeclarator = (g_object_unref (vdeclarator), NULL)));
	(instance_param == NULL ? NULL : (instance_param = (g_object_unref (instance_param), NULL)));
	(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
}


static void vala_code_generator_real_visit_formal_parameter (ValaCodeVisitor* base, ValaFormalParameter* p)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_FORMAL_PARAMETER (p));
	if (!vala_formal_parameter_get_ellipsis (p)) {
		ValaCCodeFormalParameter* __temp417 = NULL;
		vala_code_node_set_ccodenode (VALA_CODE_NODE (p), VALA_CCODE_NODE ((__temp417 = vala_ccode_formal_parameter_new (vala_formal_parameter_get_name (p), vala_type_reference_get_cname (vala_formal_parameter_get_type_reference (p), FALSE, FALSE)))));
		(__temp417 == NULL ? NULL : (__temp417 = (g_object_unref (__temp417), NULL)));
	}
}


static void vala_code_generator_real_visit_end_property (ValaCodeVisitor* base, ValaProperty* prop)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_PROPERTY (prop));
	char* __temp418 = NULL;
	vala_ccode_enum_add_value (self->priv->prop_enum, (__temp418 = vala_property_get_upper_case_cname (prop)), NULL);
	(__temp418 == NULL ? NULL : (__temp418 = (g_free (__temp418), NULL)));
}


static void vala_code_generator_real_visit_begin_property_accessor (ValaCodeVisitor* base, ValaPropertyAccessor* acc)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_PROPERTY_ACCESSOR (acc));
	ValaProperty* prop = VALA_PROPERTY (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (acc)))));
	if (vala_property_accessor_get_readable (acc)) {
		ValaTypeReference* __temp419 = NULL;
		self->priv->current_return_type = (__temp419 = g_object_ref (vala_property_get_type_reference (prop)), (self->priv->current_return_type == NULL ? NULL : (self->priv->current_return_type = (g_object_unref (self->priv->current_return_type), NULL))), __temp419);
	} else {
		/* void*/
		ValaTypeReference* __temp420 = NULL;
		self->priv->current_return_type = (__temp420 = vala_type_reference_new (), (self->priv->current_return_type == NULL ? NULL : (self->priv->current_return_type = (g_object_unref (self->priv->current_return_type), NULL))), __temp420);
	}
}


static void vala_code_generator_real_visit_end_property_accessor (ValaCodeVisitor* base, ValaPropertyAccessor* acc)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_PROPERTY_ACCESSOR (acc));
	ValaProperty* prop = VALA_PROPERTY (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (acc)))));
	ValaClass* cl = VALA_CLASS (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (prop)))));
	ValaTypeReference* __temp421 = NULL;
	self->priv->current_return_type = (__temp421 = NULL, (self->priv->current_return_type == NULL ? NULL : (self->priv->current_return_type = (g_object_unref (self->priv->current_return_type), NULL))), __temp421);
	if (vala_property_accessor_get_readable (acc)) {
		ValaCCodeFunction* __temp423 = NULL;
		char* __temp422 = NULL;
		self->priv->function = (__temp423 = vala_ccode_function_new (g_strdup_printf ("%s_get_%s", (__temp422 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL)), vala_property_get_name (prop)), vala_type_reference_get_cname (vala_property_get_type_reference (prop), FALSE, FALSE)), (self->priv->function == NULL ? NULL : (self->priv->function = (g_object_unref (self->priv->function), NULL))), __temp423);
		(__temp422 == NULL ? NULL : (__temp422 = (g_free (__temp422), NULL)));
	} else {
		ValaCCodeFunction* __temp425 = NULL;
		char* __temp424 = NULL;
		self->priv->function = (__temp425 = vala_ccode_function_new (g_strdup_printf ("%s_set_%s", (__temp424 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL)), vala_property_get_name (prop)), "void"), (self->priv->function == NULL ? NULL : (self->priv->function = (g_object_unref (self->priv->function), NULL))), __temp425);
		(__temp424 == NULL ? NULL : (__temp424 = (g_free (__temp424), NULL)));
	}
	ValaTypeReference* this_type = vala_type_reference_new ();
	vala_type_reference_set_data_type (this_type, VALA_DATA_TYPE (cl));
	ValaCCodeFormalParameter* cparam = vala_ccode_formal_parameter_new ("self", vala_type_reference_get_cname (this_type, FALSE, FALSE));
	vala_ccode_function_add_parameter (self->priv->function, cparam);
	if (vala_property_accessor_get_writable (acc) || vala_property_accessor_get_construction (acc)) {
		ValaCCodeFormalParameter* __temp426 = NULL;
		vala_ccode_function_add_parameter (self->priv->function, (__temp426 = vala_ccode_formal_parameter_new ("value", vala_type_reference_get_cname (vala_property_get_type_reference (prop), FALSE, TRUE))));
		(__temp426 == NULL ? NULL : (__temp426 = (g_object_unref (__temp426), NULL)));
	}
	ValaCCodeFunction* __temp427 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_member_declaration, VALA_CCODE_NODE ((__temp427 = vala_ccode_function_copy (self->priv->function))));
	(__temp427 == NULL ? NULL : (__temp427 = (g_object_unref (__temp427), NULL)));
	if (vala_property_accessor_get_body (acc) != NULL) {
		vala_ccode_function_set_block (self->priv->function, VALA_CCODE_BLOCK (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_property_accessor_get_body (acc)))));
		ValaCCodeStatement* __temp428 = NULL;
		vala_ccode_block_prepend_statement (vala_ccode_function_get_block (self->priv->function), VALA_CCODE_NODE ((__temp428 = vala_code_generator_create_property_type_check_statement (self, prop, vala_property_accessor_get_readable (acc), VALA_DATA_TYPE (cl), TRUE, "self"))));
		(__temp428 == NULL ? NULL : (__temp428 = (g_object_unref (__temp428), NULL)));
	}
	vala_ccode_fragment_append (self->priv->source_type_member_definition, VALA_CCODE_NODE (self->priv->function));
	(this_type == NULL ? NULL : (this_type = (g_object_unref (this_type), NULL)));
	(cparam == NULL ? NULL : (cparam = (g_object_unref (cparam), NULL)));
}


static char* vala_code_generator_get_marshaller_type_name (ValaCodeGenerator* self, ValaDataType* t)
{
	g_return_val_if_fail (VALA_IS_CODE_GENERATOR (self), NULL);
	g_return_val_if_fail (t == NULL || VALA_IS_DATA_TYPE (t), NULL);
	if (t == NULL) {
		return ("VOID");
	} else {
		return vala_data_type_get_marshaller_type_name (t);
	}
}


static char* vala_code_generator_get_signal_marshaller_function (ValaCodeGenerator* self, ValaSignal* sig, char* prefix)
{
	g_return_val_if_fail (VALA_IS_CODE_GENERATOR (self), NULL);
	g_return_val_if_fail (VALA_IS_SIGNAL (sig), NULL);
	char* signature = vala_code_generator_get_signal_signature (self, sig);
	char* ret = NULL;
	GList* params = vala_invokable_get_parameters (VALA_INVOKABLE (sig));
	if (prefix == NULL) {
		if (g_hash_table_lookup (self->priv->predefined_marshal_list, signature) != NULL) {
			prefix = "g_cclosure_marshal";
		} else {
			prefix = "g_cclosure_user_marshal";
		}
	}
	char* __temp431 = NULL;
	ret = (__temp431 = g_strdup_printf ("%s_%s_", prefix, vala_code_generator_get_marshaller_type_name (self, vala_type_reference_get_data_type (vala_signal_get_return_type (sig)))), (ret == NULL ? NULL : (ret = (g_free (ret), NULL))), __temp431);
	if (params == NULL) {
		char* __temp432 = NULL;
		ret = (__temp432 = g_strconcat (ret, "_VOID", NULL), (ret == NULL ? NULL : (ret = (g_free (ret), NULL))), __temp432);
	} else {
		{
			GList* __temp434 = NULL;
			__temp434 = params;
			GList* p_it;
			for (p_it = __temp434; p_it != NULL; p_it = p_it->next) {
				ValaFormalParameter* p = p_it->data;
				{
					char* __temp433 = NULL;
					ret = (__temp433 = g_strdup_printf ("%s_%s", ret, vala_code_generator_get_marshaller_type_name (self, vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (p)))), (ret == NULL ? NULL : (ret = (g_free (ret), NULL))), __temp433);
				}
			}
		}
	}
	char* __temp435 = NULL;
	return (__temp435 = ret, (signature == NULL ? NULL : (signature = (g_free (signature), NULL))), (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp435);
	(signature == NULL ? NULL : (signature = (g_free (signature), NULL)));
	(ret == NULL ? NULL : (ret = (g_free (ret), NULL)));
	(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
}


static char* vala_code_generator_get_value_type_name_from_datatype (ValaCodeGenerator* self, ValaDataType* t)
{
	g_return_val_if_fail (VALA_IS_CODE_GENERATOR (self), NULL);
	g_return_val_if_fail (t == NULL || VALA_IS_DATA_TYPE (t), NULL);
	if (t == NULL) {
		return "void";
	} else {
		if (VALA_IS_CLASS (t)) {
			return "GObject *";
		} else {
			if (VALA_IS_STRUCT (t)) {
				if (vala_data_type_is_reference_type (VALA_DATA_TYPE ((VALA_STRUCT (t))))) {
					return "gpointer";
				} else {
					return vala_data_type_get_cname (t, FALSE);
				}
			} else {
				if (VALA_IS_ENUM (t)) {
					return "gint";
				} else {
					if (VALA_IS_FLAGS (t)) {
						return "guint";
					} else {
						if (VALA_IS_ARRAY (t)) {
							return "gpointer";
						}
					}
				}
			}
		}
	}
	return NULL;
}


static char* vala_code_generator_get_signal_signature (ValaCodeGenerator* self, ValaSignal* sig)
{
	g_return_val_if_fail (VALA_IS_CODE_GENERATOR (self), NULL);
	g_return_val_if_fail (VALA_IS_SIGNAL (sig), NULL);
	char* signature = NULL;
	GList* params = vala_invokable_get_parameters (VALA_INVOKABLE (sig));
	char* __temp444 = NULL;
	signature = (__temp444 = g_strdup_printf ("%s:", vala_code_generator_get_marshaller_type_name (self, vala_type_reference_get_data_type (vala_signal_get_return_type (sig)))), (signature == NULL ? NULL : (signature = (g_free (signature), NULL))), __temp444);
	if (params == NULL) {
		char* __temp445 = NULL;
		signature = (__temp445 = g_strconcat (signature, "VOID", NULL), (signature == NULL ? NULL : (signature = (g_free (signature), NULL))), __temp445);
	} else {
		gboolean first = TRUE;
		{
			GList* __temp448 = NULL;
			__temp448 = params;
			GList* p_it;
			for (p_it = __temp448; p_it != NULL; p_it = p_it->next) {
				ValaFormalParameter* p = p_it->data;
				{
					if (first) {
						char* __temp446 = NULL;
						signature = (__temp446 = g_strconcat (signature, vala_code_generator_get_marshaller_type_name (self, vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (p))), NULL), (signature == NULL ? NULL : (signature = (g_free (signature), NULL))), __temp446);
						first = FALSE;
					} else {
						char* __temp447 = NULL;
						signature = (__temp447 = g_strdup_printf ("%s,%s", signature, vala_code_generator_get_marshaller_type_name (self, vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (p)))), (signature == NULL ? NULL : (signature = (g_free (signature), NULL))), __temp447);
					}
				}
			}
		}
	}
	char* __temp449 = NULL;
	return (__temp449 = signature, (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp449);
	(signature == NULL ? NULL : (signature = (g_free (signature), NULL)));
	(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
}


static void vala_code_generator_real_visit_end_signal (ValaCodeVisitor* base, ValaSignal* sig)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_SIGNAL (sig));
	char* signature = NULL;
	GList* params = vala_invokable_get_parameters (VALA_INVOKABLE (sig));
	int n_params;
	int i;
	/* check whether a signal with the same signature already exists for this source file (or predefined) */
	char* __temp450 = NULL;
	signature = (__temp450 = vala_code_generator_get_signal_signature (self, sig), (signature == NULL ? NULL : (signature = (g_free (signature), NULL))), __temp450);
	if (g_hash_table_lookup (self->priv->predefined_marshal_list, signature) != NULL || g_hash_table_lookup (self->priv->user_marshal_list, signature) != NULL) {
		(signature == NULL ? NULL : (signature = (g_free (signature), NULL)));
		(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
		return;
	}
	ValaCCodeFunction* signal_marshaller = vala_ccode_function_new (vala_code_generator_get_signal_marshaller_function (self, sig, NULL), "void");
	vala_ccode_function_set_modifiers (signal_marshaller, VALA_CCODE_MODIFIERS_STATIC);
	ValaCCodeFormalParameter* __temp451 = NULL;
	vala_ccode_function_add_parameter (signal_marshaller, (__temp451 = vala_ccode_formal_parameter_new ("closure", "GClosure *")));
	(__temp451 == NULL ? NULL : (__temp451 = (g_object_unref (__temp451), NULL)));
	ValaCCodeFormalParameter* __temp452 = NULL;
	vala_ccode_function_add_parameter (signal_marshaller, (__temp452 = vala_ccode_formal_parameter_new ("return_value", "GValue *")));
	(__temp452 == NULL ? NULL : (__temp452 = (g_object_unref (__temp452), NULL)));
	ValaCCodeFormalParameter* __temp453 = NULL;
	vala_ccode_function_add_parameter (signal_marshaller, (__temp453 = vala_ccode_formal_parameter_new ("n_param_values", "guint")));
	(__temp453 == NULL ? NULL : (__temp453 = (g_object_unref (__temp453), NULL)));
	ValaCCodeFormalParameter* __temp454 = NULL;
	vala_ccode_function_add_parameter (signal_marshaller, (__temp454 = vala_ccode_formal_parameter_new ("param_values", "const GValue *")));
	(__temp454 == NULL ? NULL : (__temp454 = (g_object_unref (__temp454), NULL)));
	ValaCCodeFormalParameter* __temp455 = NULL;
	vala_ccode_function_add_parameter (signal_marshaller, (__temp455 = vala_ccode_formal_parameter_new ("invocation_hint", "gpointer")));
	(__temp455 == NULL ? NULL : (__temp455 = (g_object_unref (__temp455), NULL)));
	ValaCCodeFormalParameter* __temp456 = NULL;
	vala_ccode_function_add_parameter (signal_marshaller, (__temp456 = vala_ccode_formal_parameter_new ("marshal_data", "gpointer")));
	(__temp456 == NULL ? NULL : (__temp456 = (g_object_unref (__temp456), NULL)));
	ValaCCodeFunction* __temp457 = NULL;
	vala_ccode_fragment_append (self->priv->source_signal_marshaller_declaration, VALA_CCODE_NODE ((__temp457 = vala_ccode_function_copy (signal_marshaller))));
	(__temp457 == NULL ? NULL : (__temp457 = (g_object_unref (__temp457), NULL)));
	ValaCCodeBlock* marshaller_body = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
	ValaCCodeFunctionDeclarator* callback_decl = vala_ccode_function_declarator_new (vala_code_generator_get_signal_marshaller_function (self, sig, "GMarshalFunc"));
	ValaCCodeFormalParameter* __temp458 = NULL;
	vala_ccode_function_declarator_add_parameter (callback_decl, (__temp458 = vala_ccode_formal_parameter_new ("data1", "gpointer")));
	(__temp458 == NULL ? NULL : (__temp458 = (g_object_unref (__temp458), NULL)));
	n_params = 1;
	{
		GList* __temp460 = NULL;
		__temp460 = params;
		GList* p_it;
		for (p_it = __temp460; p_it != NULL; p_it = p_it->next) {
			ValaFormalParameter* p = p_it->data;
			{
				ValaCCodeFormalParameter* __temp459 = NULL;
				vala_ccode_function_declarator_add_parameter (callback_decl, (__temp459 = vala_ccode_formal_parameter_new (g_strdup_printf ("arg_%d", n_params), vala_code_generator_get_value_type_name_from_datatype (self, vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (p))))));
				(__temp459 == NULL ? NULL : (__temp459 = (g_object_unref (__temp459), NULL)));
				n_params++;
			}
		}
	}
	ValaCCodeFormalParameter* __temp461 = NULL;
	vala_ccode_function_declarator_add_parameter (callback_decl, (__temp461 = vala_ccode_formal_parameter_new ("data2", "gpointer")));
	(__temp461 == NULL ? NULL : (__temp461 = (g_object_unref (__temp461), NULL)));
	ValaCCodeTypeDefinition* __temp462 = NULL;
	vala_ccode_block_add_statement (marshaller_body, VALA_CCODE_NODE ((__temp462 = vala_ccode_type_definition_new (vala_code_generator_get_value_type_name_from_datatype (self, vala_type_reference_get_data_type (vala_signal_get_return_type (sig))), VALA_CCODE_DECLARATOR (callback_decl)))));
	(__temp462 == NULL ? NULL : (__temp462 = (g_object_unref (__temp462), NULL)));
	ValaCCodeDeclaration* var_decl = vala_ccode_declaration_new (vala_code_generator_get_signal_marshaller_function (self, sig, "GMarshalFunc"));
	vala_ccode_declaration_set_modifiers (var_decl, VALA_CCODE_MODIFIERS_REGISTER);
	ValaCCodeVariableDeclarator* __temp463 = NULL;
	vala_ccode_declaration_add_declarator (var_decl, VALA_CCODE_DECLARATOR ((__temp463 = vala_ccode_variable_declarator_new ("callback"))));
	(__temp463 == NULL ? NULL : (__temp463 = (g_object_unref (__temp463), NULL)));
	vala_ccode_block_add_statement (marshaller_body, VALA_CCODE_NODE (var_decl));
	ValaCCodeDeclaration* __temp464 = NULL;
	var_decl = (__temp464 = vala_ccode_declaration_new ("GCClosure *"), (var_decl == NULL ? NULL : (var_decl = (g_object_unref (var_decl), NULL))), __temp464);
	vala_ccode_declaration_set_modifiers (var_decl, VALA_CCODE_MODIFIERS_REGISTER);
	ValaCCodeVariableDeclarator* __temp465 = NULL;
	vala_ccode_declaration_add_declarator (var_decl, VALA_CCODE_DECLARATOR ((__temp465 = vala_ccode_variable_declarator_new_with_initializer ("cc", VALA_CCODE_EXPRESSION (vala_ccode_cast_expression_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("closure")), "GCClosure *"))))));
	(__temp465 == NULL ? NULL : (__temp465 = (g_object_unref (__temp465), NULL)));
	vala_ccode_block_add_statement (marshaller_body, VALA_CCODE_NODE (var_decl));
	ValaCCodeDeclaration* __temp466 = NULL;
	var_decl = (__temp466 = vala_ccode_declaration_new ("gpointer"), (var_decl == NULL ? NULL : (var_decl = (g_object_unref (var_decl), NULL))), __temp466);
	vala_ccode_declaration_set_modifiers (var_decl, VALA_CCODE_MODIFIERS_REGISTER);
	ValaCCodeVariableDeclarator* __temp467 = NULL;
	vala_ccode_declaration_add_declarator (var_decl, VALA_CCODE_DECLARATOR ((__temp467 = vala_ccode_variable_declarator_new ("data1"))));
	(__temp467 == NULL ? NULL : (__temp467 = (g_object_unref (__temp467), NULL)));
	ValaCCodeVariableDeclarator* __temp468 = NULL;
	vala_ccode_declaration_add_declarator (var_decl, VALA_CCODE_DECLARATOR ((__temp468 = vala_ccode_variable_declarator_new ("data2"))));
	(__temp468 == NULL ? NULL : (__temp468 = (g_object_unref (__temp468), NULL)));
	vala_ccode_block_add_statement (marshaller_body, VALA_CCODE_NODE (var_decl));
	ValaCCodeFunctionCall* fc = NULL;
	if (vala_type_reference_get_data_type (vala_signal_get_return_type (sig)) != NULL) {
		ValaCCodeDeclaration* __temp469 = NULL;
		var_decl = (__temp469 = vala_ccode_declaration_new (vala_code_generator_get_value_type_name_from_datatype (self, vala_type_reference_get_data_type (vala_signal_get_return_type (sig)))), (var_decl == NULL ? NULL : (var_decl = (g_object_unref (var_decl), NULL))), __temp469);
		ValaCCodeVariableDeclarator* __temp470 = NULL;
		vala_ccode_declaration_add_declarator (var_decl, VALA_CCODE_DECLARATOR ((__temp470 = vala_ccode_variable_declarator_new ("v_return"))));
		(__temp470 == NULL ? NULL : (__temp470 = (g_object_unref (__temp470), NULL)));
		vala_ccode_block_add_statement (marshaller_body, VALA_CCODE_NODE (var_decl));
		ValaCCodeFunctionCall* __temp471 = NULL;
		fc = (__temp471 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("g_return_if_fail"))), (fc == NULL ? NULL : (fc = (g_object_unref (fc), NULL))), __temp471);
		ValaCCodeBinaryExpression* __temp472 = NULL;
		vala_ccode_function_call_add_argument (fc, VALA_CCODE_EXPRESSION ((__temp472 = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_INEQUALITY, VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("return_value")), VALA_CCODE_EXPRESSION (vala_ccode_constant_new ("NULL"))))));
		(__temp472 == NULL ? NULL : (__temp472 = (g_object_unref (__temp472), NULL)));
		ValaCCodeExpressionStatement* __temp473 = NULL;
		vala_ccode_block_add_statement (marshaller_body, VALA_CCODE_NODE ((__temp473 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (fc)))));
		(__temp473 == NULL ? NULL : (__temp473 = (g_object_unref (__temp473), NULL)));
	}
	ValaCCodeFunctionCall* __temp474 = NULL;
	fc = (__temp474 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("g_return_if_fail"))), (fc == NULL ? NULL : (fc = (g_object_unref (fc), NULL))), __temp474);
	ValaCCodeBinaryExpression* __temp475 = NULL;
	vala_ccode_function_call_add_argument (fc, VALA_CCODE_EXPRESSION ((__temp475 = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_EQUALITY, VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("n_param_values")), VALA_CCODE_EXPRESSION (vala_ccode_constant_new (g_strdup_printf ("%i", n_params)))))));
	(__temp475 == NULL ? NULL : (__temp475 = (g_object_unref (__temp475), NULL)));
	ValaCCodeExpressionStatement* __temp476 = NULL;
	vala_ccode_block_add_statement (marshaller_body, VALA_CCODE_NODE ((__temp476 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (fc)))));
	(__temp476 == NULL ? NULL : (__temp476 = (g_object_unref (__temp476), NULL)));
	ValaCCodeMemberAccess* data = vala_ccode_member_access_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("closure")), "data", TRUE);
	ValaCCodeMemberAccess* param = vala_ccode_member_access_new (VALA_CCODE_EXPRESSION (vala_ccode_member_access_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("param_values")), "data[0]", TRUE)), "v_pointer", FALSE);
	ValaCCodeFunctionCall* cond = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_constant_new ("G_CCLOSURE_SWAP_DATA")));
	ValaCCodeIdentifier* __temp477 = NULL;
	vala_ccode_function_call_add_argument (cond, VALA_CCODE_EXPRESSION ((__temp477 = vala_ccode_identifier_new ("closure"))));
	(__temp477 == NULL ? NULL : (__temp477 = (g_object_unref (__temp477), NULL)));
	ValaCCodeBlock* true_block = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
	ValaCCodeExpressionStatement* __temp478 = NULL;
	vala_ccode_block_add_statement (true_block, VALA_CCODE_NODE ((__temp478 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("data1")), VALA_CCODE_EXPRESSION (data), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))))));
	(__temp478 == NULL ? NULL : (__temp478 = (g_object_unref (__temp478), NULL)));
	ValaCCodeExpressionStatement* __temp479 = NULL;
	vala_ccode_block_add_statement (true_block, VALA_CCODE_NODE ((__temp479 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("data2")), VALA_CCODE_EXPRESSION (param), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))))));
	(__temp479 == NULL ? NULL : (__temp479 = (g_object_unref (__temp479), NULL)));
	ValaCCodeBlock* false_block = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
	ValaCCodeExpressionStatement* __temp480 = NULL;
	vala_ccode_block_add_statement (false_block, VALA_CCODE_NODE ((__temp480 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("data1")), VALA_CCODE_EXPRESSION (param), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))))));
	(__temp480 == NULL ? NULL : (__temp480 = (g_object_unref (__temp480), NULL)));
	ValaCCodeExpressionStatement* __temp481 = NULL;
	vala_ccode_block_add_statement (false_block, VALA_CCODE_NODE ((__temp481 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("data2")), VALA_CCODE_EXPRESSION (data), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))))));
	(__temp481 == NULL ? NULL : (__temp481 = (g_object_unref (__temp481), NULL)));
	ValaCCodeIfStatement* __temp482 = NULL;
	vala_ccode_block_add_statement (marshaller_body, VALA_CCODE_NODE ((__temp482 = vala_ccode_if_statement_new (VALA_CCODE_EXPRESSION (cond), VALA_CCODE_STATEMENT (true_block), VALA_CCODE_STATEMENT (false_block)))));
	(__temp482 == NULL ? NULL : (__temp482 = (g_object_unref (__temp482), NULL)));
	ValaCCodeAssignment* c_assign = vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("callback")), VALA_CCODE_EXPRESSION (vala_ccode_cast_expression_new (VALA_CCODE_EXPRESSION (vala_ccode_conditional_expression_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("marshal_data")), VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("marshal_data")), VALA_CCODE_EXPRESSION (vala_ccode_member_access_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("cc")), "callback", TRUE)))), vala_code_generator_get_signal_marshaller_function (self, sig, "GMarshalFunc"))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE);
	ValaCCodeExpressionStatement* __temp483 = NULL;
	vala_ccode_block_add_statement (marshaller_body, VALA_CCODE_NODE ((__temp483 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (c_assign)))));
	(__temp483 == NULL ? NULL : (__temp483 = (g_object_unref (__temp483), NULL)));
	ValaCCodeFunctionCall* __temp484 = NULL;
	fc = (__temp484 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("callback"))), (fc == NULL ? NULL : (fc = (g_object_unref (fc), NULL))), __temp484);
	ValaCCodeIdentifier* __temp485 = NULL;
	vala_ccode_function_call_add_argument (fc, VALA_CCODE_EXPRESSION ((__temp485 = vala_ccode_identifier_new ("data1"))));
	(__temp485 == NULL ? NULL : (__temp485 = (g_object_unref (__temp485), NULL)));
	i = 1;
	{
		GList* __temp487 = NULL;
		__temp487 = params;
		GList* p_it;
		for (p_it = __temp487; p_it != NULL; p_it = p_it->next) {
			ValaFormalParameter* p = p_it->data;
			{
				ValaCCodeFunctionCall* inner_fc = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_data_type_get_get_value_function (vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (p))))));
				ValaCCodeBinaryExpression* __temp486 = NULL;
				vala_ccode_function_call_add_argument (inner_fc, VALA_CCODE_EXPRESSION ((__temp486 = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_PLUS, VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("param_values")), VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (g_strdup_printf ("%i", i)))))));
				(__temp486 == NULL ? NULL : (__temp486 = (g_object_unref (__temp486), NULL)));
				vala_ccode_function_call_add_argument (fc, VALA_CCODE_EXPRESSION (inner_fc));
				i++;
				(inner_fc == NULL ? NULL : (inner_fc = (g_object_unref (inner_fc), NULL)));
			}
		}
	}
	ValaCCodeIdentifier* __temp488 = NULL;
	vala_ccode_function_call_add_argument (fc, VALA_CCODE_EXPRESSION ((__temp488 = vala_ccode_identifier_new ("data2"))));
	(__temp488 == NULL ? NULL : (__temp488 = (g_object_unref (__temp488), NULL)));
	if (vala_type_reference_get_data_type (vala_signal_get_return_type (sig)) != NULL) {
		ValaCCodeExpressionStatement* __temp489 = NULL;
		vala_ccode_block_add_statement (marshaller_body, VALA_CCODE_NODE ((__temp489 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("v_return")), VALA_CCODE_EXPRESSION (fc), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))))));
		(__temp489 == NULL ? NULL : (__temp489 = (g_object_unref (__temp489), NULL)));
		ValaCCodeFunctionCall* set_fc = NULL;
		if (VALA_IS_CLASS (vala_type_reference_get_data_type (vala_signal_get_return_type (sig)))) {
			ValaCCodeFunctionCall* __temp490 = NULL;
			set_fc = (__temp490 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("g_value_take_object"))), (set_fc == NULL ? NULL : (set_fc = (g_object_unref (set_fc), NULL))), __temp490);
		} else {
			if (vala_type_reference_get_data_type (vala_signal_get_return_type (sig)) == vala_type_reference_get_data_type (self->priv->string_type)) {
				ValaCCodeFunctionCall* __temp491 = NULL;
				set_fc = (__temp491 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("g_value_take_string"))), (set_fc == NULL ? NULL : (set_fc = (g_object_unref (set_fc), NULL))), __temp491);
			} else {
				ValaCCodeFunctionCall* __temp492 = NULL;
				set_fc = (__temp492 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_data_type_get_set_value_function (vala_type_reference_get_data_type (vala_signal_get_return_type (sig)))))), (set_fc == NULL ? NULL : (set_fc = (g_object_unref (set_fc), NULL))), __temp492);
			}
		}
		ValaCCodeIdentifier* __temp493 = NULL;
		vala_ccode_function_call_add_argument (set_fc, VALA_CCODE_EXPRESSION ((__temp493 = vala_ccode_identifier_new ("return_value"))));
		(__temp493 == NULL ? NULL : (__temp493 = (g_object_unref (__temp493), NULL)));
		ValaCCodeIdentifier* __temp494 = NULL;
		vala_ccode_function_call_add_argument (set_fc, VALA_CCODE_EXPRESSION ((__temp494 = vala_ccode_identifier_new ("v_return"))));
		(__temp494 == NULL ? NULL : (__temp494 = (g_object_unref (__temp494), NULL)));
		ValaCCodeExpressionStatement* __temp495 = NULL;
		vala_ccode_block_add_statement (marshaller_body, VALA_CCODE_NODE ((__temp495 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (set_fc)))));
		(__temp495 == NULL ? NULL : (__temp495 = (g_object_unref (__temp495), NULL)));
		(set_fc == NULL ? NULL : (set_fc = (g_object_unref (set_fc), NULL)));
	} else {
		ValaCCodeExpressionStatement* __temp496 = NULL;
		vala_ccode_block_add_statement (marshaller_body, VALA_CCODE_NODE ((__temp496 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (fc)))));
		(__temp496 == NULL ? NULL : (__temp496 = (g_object_unref (__temp496), NULL)));
	}
	vala_ccode_function_set_block (signal_marshaller, marshaller_body);
	vala_ccode_fragment_append (self->priv->source_signal_marshaller_definition, VALA_CCODE_NODE (signal_marshaller));
	const char* __temp497 = NULL;
	g_hash_table_insert (self->priv->user_marshal_list, (__temp497 = signature, (__temp497 == NULL ? NULL : g_strdup (__temp497))), GINT_TO_POINTER (TRUE));
	(signature == NULL ? NULL : (signature = (g_free (signature), NULL)));
	(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
	(signal_marshaller == NULL ? NULL : (signal_marshaller = (g_object_unref (signal_marshaller), NULL)));
	(marshaller_body == NULL ? NULL : (marshaller_body = (g_object_unref (marshaller_body), NULL)));
	(callback_decl == NULL ? NULL : (callback_decl = (g_object_unref (callback_decl), NULL)));
	(var_decl == NULL ? NULL : (var_decl = (g_object_unref (var_decl), NULL)));
	(fc == NULL ? NULL : (fc = (g_object_unref (fc), NULL)));
	(data == NULL ? NULL : (data = (g_object_unref (data), NULL)));
	(param == NULL ? NULL : (param = (g_object_unref (param), NULL)));
	(cond == NULL ? NULL : (cond = (g_object_unref (cond), NULL)));
	(true_block == NULL ? NULL : (true_block = (g_object_unref (true_block), NULL)));
	(false_block == NULL ? NULL : (false_block = (g_object_unref (false_block), NULL)));
	(c_assign == NULL ? NULL : (c_assign = (g_object_unref (c_assign), NULL)));
}


static void vala_code_generator_real_visit_end_constructor (ValaCodeVisitor* base, ValaConstructor* c)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_CONSTRUCTOR (c));
	ValaClass* cl = VALA_CLASS (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (c)))));
	ValaCCodeFunction* __temp499 = NULL;
	char* __temp498 = NULL;
	self->priv->function = (__temp499 = vala_ccode_function_new (g_strdup_printf ("%s_constructor", (__temp498 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL))), "GObject *"), (self->priv->function == NULL ? NULL : (self->priv->function = (g_object_unref (self->priv->function), NULL))), __temp499);
	(__temp498 == NULL ? NULL : (__temp498 = (g_free (__temp498), NULL)));
	vala_ccode_function_set_modifiers (self->priv->function, VALA_CCODE_MODIFIERS_STATIC);
	ValaCCodeFormalParameter* __temp500 = NULL;
	vala_ccode_function_add_parameter (self->priv->function, (__temp500 = vala_ccode_formal_parameter_new ("type", "GType")));
	(__temp500 == NULL ? NULL : (__temp500 = (g_object_unref (__temp500), NULL)));
	ValaCCodeFormalParameter* __temp501 = NULL;
	vala_ccode_function_add_parameter (self->priv->function, (__temp501 = vala_ccode_formal_parameter_new ("n_construct_properties", "guint")));
	(__temp501 == NULL ? NULL : (__temp501 = (g_object_unref (__temp501), NULL)));
	ValaCCodeFormalParameter* __temp502 = NULL;
	vala_ccode_function_add_parameter (self->priv->function, (__temp502 = vala_ccode_formal_parameter_new ("construct_properties", "GObjectConstructParam *")));
	(__temp502 == NULL ? NULL : (__temp502 = (g_object_unref (__temp502), NULL)));
	ValaCCodeFunction* __temp503 = NULL;
	vala_ccode_fragment_append (self->priv->source_type_member_declaration, VALA_CCODE_NODE ((__temp503 = vala_ccode_function_copy (self->priv->function))));
	(__temp503 == NULL ? NULL : (__temp503 = (g_object_unref (__temp503), NULL)));
	ValaCCodeBlock* cblock = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
	ValaCCodeDeclaration* cdecl = vala_ccode_declaration_new ("GObject *");
	ValaCCodeVariableDeclarator* __temp504 = NULL;
	vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR ((__temp504 = vala_ccode_variable_declarator_new ("obj"))));
	(__temp504 == NULL ? NULL : (__temp504 = (g_object_unref (__temp504), NULL)));
	vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE (cdecl));
	ValaCCodeDeclaration* __temp505 = NULL;
	cdecl = (__temp505 = vala_ccode_declaration_new (g_strdup_printf ("%sClass *", vala_data_type_get_cname (VALA_DATA_TYPE (cl), FALSE))), (cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL))), __temp505);
	ValaCCodeVariableDeclarator* __temp506 = NULL;
	vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR ((__temp506 = vala_ccode_variable_declarator_new ("klass"))));
	(__temp506 == NULL ? NULL : (__temp506 = (g_object_unref (__temp506), NULL)));
	vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE (cdecl));
	ValaCCodeDeclaration* __temp507 = NULL;
	cdecl = (__temp507 = vala_ccode_declaration_new ("GObjectClass *"), (cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL))), __temp507);
	ValaCCodeVariableDeclarator* __temp508 = NULL;
	vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR ((__temp508 = vala_ccode_variable_declarator_new ("parent_class"))));
	(__temp508 == NULL ? NULL : (__temp508 = (g_object_unref (__temp508), NULL)));
	vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE (cdecl));
	ValaCCodeFunctionCall* ccall = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("g_type_class_peek")));
	ValaCCodeIdentifier* __temp509 = NULL;
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp509 = vala_ccode_identifier_new (vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), "TYPE_")))));
	(__temp509 == NULL ? NULL : (__temp509 = (g_object_unref (__temp509), NULL)));
	char* __temp510 = NULL;
	ValaCCodeFunctionCall* __temp511 = NULL;
	ValaCCodeFunctionCall* ccast = (__temp511 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (g_strdup_printf ("%s_CLASS", (__temp510 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), NULL)))))), (__temp510 == NULL ? NULL : (__temp510 = (g_free (__temp510), NULL))), __temp511);
	vala_ccode_function_call_add_argument (ccast, VALA_CCODE_EXPRESSION (ccall));
	ValaCCodeExpressionStatement* __temp512 = NULL;
	vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE ((__temp512 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("klass")), VALA_CCODE_EXPRESSION (ccast), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))))));
	(__temp512 == NULL ? NULL : (__temp512 = (g_object_unref (__temp512), NULL)));
	ValaCCodeFunctionCall* __temp513 = NULL;
	ccall = (__temp513 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("g_type_class_peek_parent"))), (ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL))), __temp513);
	ValaCCodeIdentifier* __temp514 = NULL;
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp514 = vala_ccode_identifier_new ("klass"))));
	(__temp514 == NULL ? NULL : (__temp514 = (g_object_unref (__temp514), NULL)));
	ValaCCodeFunctionCall* __temp515 = NULL;
	ccast = (__temp515 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("G_OBJECT_CLASS"))), (ccast == NULL ? NULL : (ccast = (g_object_unref (ccast), NULL))), __temp515);
	vala_ccode_function_call_add_argument (ccast, VALA_CCODE_EXPRESSION (ccall));
	ValaCCodeExpressionStatement* __temp516 = NULL;
	vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE ((__temp516 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("parent_class")), VALA_CCODE_EXPRESSION (ccast), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))))));
	(__temp516 == NULL ? NULL : (__temp516 = (g_object_unref (__temp516), NULL)));
	ValaCCodeFunctionCall* __temp517 = NULL;
	ccall = (__temp517 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_member_access_new_pointer (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("parent_class")), "constructor"))), (ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL))), __temp517);
	ValaCCodeIdentifier* __temp518 = NULL;
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp518 = vala_ccode_identifier_new ("type"))));
	(__temp518 == NULL ? NULL : (__temp518 = (g_object_unref (__temp518), NULL)));
	ValaCCodeIdentifier* __temp519 = NULL;
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp519 = vala_ccode_identifier_new ("n_construct_properties"))));
	(__temp519 == NULL ? NULL : (__temp519 = (g_object_unref (__temp519), NULL)));
	ValaCCodeIdentifier* __temp520 = NULL;
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp520 = vala_ccode_identifier_new ("construct_properties"))));
	(__temp520 == NULL ? NULL : (__temp520 = (g_object_unref (__temp520), NULL)));
	ValaCCodeExpressionStatement* __temp521 = NULL;
	vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE ((__temp521 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("obj")), VALA_CCODE_EXPRESSION (ccall), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))))));
	(__temp521 == NULL ? NULL : (__temp521 = (g_object_unref (__temp521), NULL)));
	ValaCCodeFunctionCall* __temp522 = NULL;
	ccall = (__temp522 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), NULL)))), (ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL))), __temp522);
	ValaCCodeIdentifier* __temp523 = NULL;
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp523 = vala_ccode_identifier_new ("obj"))));
	(__temp523 == NULL ? NULL : (__temp523 = (g_object_unref (__temp523), NULL)));
	ValaCCodeDeclaration* __temp524 = NULL;
	cdecl = (__temp524 = vala_ccode_declaration_new (g_strdup_printf ("%s *", vala_data_type_get_cname (VALA_DATA_TYPE (cl), FALSE))), (cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL))), __temp524);
	ValaCCodeVariableDeclarator* __temp525 = NULL;
	vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR ((__temp525 = vala_ccode_variable_declarator_new_with_initializer ("self", VALA_CCODE_EXPRESSION (ccall)))));
	(__temp525 == NULL ? NULL : (__temp525 = (g_object_unref (__temp525), NULL)));
	vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE (cdecl));
	vala_ccode_block_add_statement (cblock, vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_constructor_get_body (c))));
	ValaCCodeReturnStatement* __temp526 = NULL;
	vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE ((__temp526 = vala_ccode_return_statement_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("obj"))))));
	(__temp526 == NULL ? NULL : (__temp526 = (g_object_unref (__temp526), NULL)));
	vala_ccode_function_set_block (self->priv->function, cblock);
	if (vala_source_reference_get_comment (vala_code_node_get_source_reference (VALA_CODE_NODE (c))) != NULL) {
		ValaCCodeComment* __temp527 = NULL;
		vala_ccode_fragment_append (self->priv->source_type_member_definition, VALA_CCODE_NODE ((__temp527 = vala_ccode_comment_new (vala_source_reference_get_comment (vala_code_node_get_source_reference (VALA_CODE_NODE (c)))))));
		(__temp527 == NULL ? NULL : (__temp527 = (g_object_unref (__temp527), NULL)));
	}
	vala_ccode_fragment_append (self->priv->source_type_member_definition, VALA_CCODE_NODE (self->priv->function));
	(cblock == NULL ? NULL : (cblock = (g_object_unref (cblock), NULL)));
	(cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL)));
	(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
	(ccast == NULL ? NULL : (ccast = (g_object_unref (ccast), NULL)));
}


static void vala_code_generator_real_visit_begin_block (ValaCodeVisitor* base, ValaBlock* b)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_BLOCK (b));
	ValaSymbol* __temp529 = NULL;
	ValaSymbol* __temp528 = NULL;
	self->priv->current_symbol = (__temp529 = (__temp528 = vala_code_node_get_symbol (VALA_CODE_NODE (b)), (__temp528 == NULL ? NULL : g_object_ref (__temp528))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp529);
}


static void vala_code_generator_add_object_creation (ValaCodeGenerator* self, ValaCCodeBlock* b)
{
	g_return_if_fail (VALA_IS_CODE_GENERATOR (self));
	g_return_if_fail (VALA_IS_CCODE_BLOCK (b));
	ValaClass* cl = VALA_CLASS (vala_symbol_get_node (self->priv->current_type_symbol));
	ValaCCodeFunctionCall* ccall = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("g_object_newv")));
	ValaCCodeConstant* __temp530 = NULL;
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp530 = vala_ccode_constant_new (vala_data_type_get_type_id (VALA_DATA_TYPE (cl))))));
	(__temp530 == NULL ? NULL : (__temp530 = (g_object_unref (__temp530), NULL)));
	ValaCCodeConstant* __temp531 = NULL;
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp531 = vala_ccode_constant_new ("__params_it - __params"))));
	(__temp531 == NULL ? NULL : (__temp531 = (g_object_unref (__temp531), NULL)));
	ValaCCodeConstant* __temp532 = NULL;
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp532 = vala_ccode_constant_new ("__params"))));
	(__temp532 == NULL ? NULL : (__temp532 = (g_object_unref (__temp532), NULL)));
	ValaCCodeVariableDeclarator* cdecl = vala_ccode_variable_declarator_new ("self");
	vala_ccode_variable_declarator_set_initializer (cdecl, VALA_CCODE_EXPRESSION (ccall));
	ValaCCodeDeclaration* cdeclaration = vala_ccode_declaration_new (g_strdup_printf ("%s *", vala_data_type_get_cname (VALA_DATA_TYPE (cl), FALSE)));
	vala_ccode_declaration_add_declarator (cdeclaration, VALA_CCODE_DECLARATOR (cdecl));
	vala_ccode_block_add_statement (b, VALA_CCODE_NODE (cdeclaration));
	(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
	(cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL)));
	(cdeclaration == NULL ? NULL : (cdeclaration = (g_object_unref (cdeclaration), NULL)));
}


static void vala_code_generator_real_visit_end_block (ValaCodeVisitor* base, ValaBlock* b)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_BLOCK (b));
	GList* local_vars = vala_block_get_local_variables (b);
	{
		GList* __temp533 = NULL;
		__temp533 = local_vars;
		GList* decl_it;
		for (decl_it = __temp533; decl_it != NULL; decl_it = decl_it->next) {
			ValaVariableDeclarator* decl = decl_it->data;
			{
				vala_symbol_set_active (vala_code_node_get_symbol (VALA_CODE_NODE (decl)), FALSE);
			}
		}
	}
	gboolean in_construction = vala_statement_get_construction (VALA_STATEMENT (b));
	ValaCCodeBlock* cblock = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
	{
		GList* __temp536 = NULL;
		__temp536 = vala_block_get_statements (b);
		GList* stmt_it;
		for (stmt_it = __temp536; stmt_it != NULL; stmt_it = stmt_it->next) {
			ValaStatement* stmt = stmt_it->data;
			{
				if (in_construction && !vala_statement_get_construction (stmt)) {
					/* construction part of construction method ends here*/
					vala_code_generator_add_object_creation (self, cblock);
					in_construction = FALSE;
				}
				ValaSourceReference* src = vala_code_node_get_source_reference (VALA_CODE_NODE (stmt));
				if (src != NULL && vala_source_reference_get_comment (src) != NULL) {
					ValaCCodeComment* __temp534 = NULL;
					vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE ((__temp534 = vala_ccode_comment_new (vala_source_reference_get_comment (src)))));
					(__temp534 == NULL ? NULL : (__temp534 = (g_object_unref (__temp534), NULL)));
				}
				if (VALA_IS_CCODE_FRAGMENT (vala_code_node_get_ccodenode (VALA_CODE_NODE (stmt)))) {
					{
						GList* __temp535 = NULL;
						__temp535 = vala_ccode_fragment_get_children ((VALA_CCODE_FRAGMENT (vala_code_node_get_ccodenode (VALA_CODE_NODE (stmt)))));
						GList* cstmt_it;
						for (cstmt_it = __temp535; cstmt_it != NULL; cstmt_it = cstmt_it->next) {
							ValaCCodeStatement* cstmt = cstmt_it->data;
							{
								vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE (cstmt));
							}
						}
						(__temp535 == NULL ? NULL : (__temp535 = (g_list_free (__temp535), NULL)));
					}
				} else {
					vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE (VALA_CCODE_STATEMENT (vala_code_node_get_ccodenode (VALA_CODE_NODE (stmt)))));
				}
			}
		}
		(__temp536 == NULL ? NULL : (__temp536 = (g_list_free (__temp536), NULL)));
	}
	if (in_construction) {
		/* construction method doesn't contain non-construction parts*/
		vala_code_generator_add_object_creation (self, cblock);
	}
	if (vala_code_generator_get_memory_management (self)) {
		{
			GList* __temp539 = NULL;
			__temp539 = local_vars;
			GList* decl_it;
			for (decl_it = __temp539; decl_it != NULL; decl_it = decl_it->next) {
				ValaVariableDeclarator* decl = decl_it->data;
				{
					if (vala_data_type_is_reference_type (vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl))) && vala_type_reference_get_takes_ownership (vala_variable_declarator_get_type_reference (decl))) {
						ValaCCodeExpressionStatement* __temp538 = NULL;
						ValaCCodeIdentifier* __temp537 = NULL;
						vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE ((__temp538 = vala_ccode_expression_statement_new (vala_code_generator_get_unref_expression (self, VALA_CCODE_EXPRESSION ((__temp537 = vala_ccode_identifier_new (vala_variable_declarator_get_name (decl)))), vala_variable_declarator_get_type_reference (decl))))));
						(__temp538 == NULL ? NULL : (__temp538 = (g_object_unref (__temp538), NULL)));
						(__temp537 == NULL ? NULL : (__temp537 = (g_object_unref (__temp537), NULL)));
					}
				}
			}
		}
	}
	vala_code_node_set_ccodenode (VALA_CODE_NODE (b), VALA_CCODE_NODE (cblock));
	ValaSymbol* __temp541 = NULL;
	ValaSymbol* __temp540 = NULL;
	self->priv->current_symbol = (__temp541 = (__temp540 = vala_symbol_get_parent_symbol (self->priv->current_symbol), (__temp540 == NULL ? NULL : g_object_ref (__temp540))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp541);
	(local_vars == NULL ? NULL : (local_vars = (g_list_free (local_vars), NULL)));
	(cblock == NULL ? NULL : (cblock = (g_object_unref (cblock), NULL)));
}


static void vala_code_generator_real_visit_empty_statement (ValaCodeVisitor* base, ValaEmptyStatement* stmt)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_EMPTY_STATEMENT (stmt));
	ValaCCodeEmptyStatement* __temp542 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (stmt), VALA_CCODE_NODE ((__temp542 = g_object_new (VALA_TYPE_CCODE_EMPTY_STATEMENT, NULL))));
	(__temp542 == NULL ? NULL : (__temp542 = (g_object_unref (__temp542), NULL)));
}


static void vala_code_generator_real_visit_declaration_statement (ValaCodeVisitor* base, ValaDeclarationStatement* stmt)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_DECLARATION_STATEMENT (stmt));
	/* split declaration statement as var declarators
	 * might have different types */
	ValaCCodeFragment* cfrag = g_object_new (VALA_TYPE_CCODE_FRAGMENT, NULL);
	{
		GList* __temp543 = NULL;
		__temp543 = vala_local_variable_declaration_get_variable_declarators (vala_declaration_statement_get_declaration (stmt));
		GList* decl_it;
		for (decl_it = __temp543; decl_it != NULL; decl_it = decl_it->next) {
			ValaVariableDeclarator* decl = decl_it->data;
			{
				ValaCCodeDeclaration* cdecl = vala_ccode_declaration_new (vala_type_reference_get_cname (vala_variable_declarator_get_type_reference (decl), FALSE, FALSE));
				vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR (VALA_CCODE_VARIABLE_DECLARATOR (vala_code_node_get_ccodenode (VALA_CODE_NODE (decl)))));
				vala_ccode_fragment_append (cfrag, VALA_CCODE_NODE (cdecl));
				(cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL)));
			}
		}
		(__temp543 == NULL ? NULL : (__temp543 = (g_list_free (__temp543), NULL)));
	}
	vala_code_node_set_ccodenode (VALA_CODE_NODE (stmt), VALA_CCODE_NODE (cfrag));
	{
		GList* __temp544 = NULL;
		__temp544 = vala_local_variable_declaration_get_variable_declarators (vala_declaration_statement_get_declaration (stmt));
		GList* decl_it;
		for (decl_it = __temp544; decl_it != NULL; decl_it = decl_it->next) {
			ValaVariableDeclarator* decl = decl_it->data;
			{
				if (vala_variable_declarator_get_initializer (decl) != NULL) {
					vala_code_generator_create_temp_decl (self, VALA_STATEMENT (stmt), vala_variable_declarator_get_initializer (decl)->temp_vars);
				}
			}
		}
		(__temp544 == NULL ? NULL : (__temp544 = (g_list_free (__temp544), NULL)));
	}
	vala_code_generator_create_temp_decl (self, VALA_STATEMENT (stmt), self->priv->temp_vars);
	GList* __temp545 = NULL;
	self->priv->temp_vars = (__temp545 = NULL, (self->priv->temp_vars == NULL ? NULL : (self->priv->temp_vars = (g_list_foreach (self->priv->temp_vars, (GFunc) g_object_unref, NULL), g_list_free (self->priv->temp_vars), NULL))), __temp545);
	(cfrag == NULL ? NULL : (cfrag = (g_object_unref (cfrag), NULL)));
}


static void vala_code_generator_real_visit_variable_declarator (ValaCodeVisitor* base, ValaVariableDeclarator* decl)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_VARIABLE_DECLARATOR (decl));
	if (VALA_IS_ARRAY (vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl)))) {
		/* create variables to store array dimensions*/
		ValaArray* arr = VALA_ARRAY (vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl)));
		{
			int dim;
			for (dim = 1; dim <= vala_array_get_rank (arr); dim++) {
				ValaVariableDeclarator* len_decl = vala_variable_declarator_new (vala_code_generator_get_array_length_cname (self, vala_variable_declarator_get_name (decl), dim), NULL, NULL);
				ValaTypeReference* __temp546 = NULL;
				vala_variable_declarator_set_type_reference (len_decl, (__temp546 = vala_type_reference_new ()));
				(__temp546 == NULL ? NULL : (__temp546 = (g_object_unref (__temp546), NULL)));
				vala_type_reference_set_data_type (vala_variable_declarator_get_type_reference (len_decl), vala_type_reference_get_data_type (self->priv->int_type));
				ValaVariableDeclarator* __temp547 = NULL;
				self->priv->temp_vars = g_list_prepend (self->priv->temp_vars, (__temp547 = len_decl, (__temp547 == NULL ? NULL : g_object_ref (__temp547))));
				(len_decl == NULL ? NULL : (len_decl = (g_object_unref (len_decl), NULL)));
			}
		}
	}
	ValaCCodeExpression* rhs = NULL;
	if (vala_variable_declarator_get_initializer (decl) != NULL) {
		ValaCCodeExpression* __temp549 = NULL;
		ValaCCodeExpression* __temp548 = NULL;
		rhs = (__temp549 = (__temp548 = VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_variable_declarator_get_initializer (decl)))), (__temp548 == NULL ? NULL : g_object_ref (__temp548))), (rhs == NULL ? NULL : (rhs = (g_object_unref (rhs), NULL))), __temp549);
		if (vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl)) != NULL && vala_type_reference_get_data_type (vala_expression_get_static_type (vala_variable_declarator_get_initializer (decl))) != NULL && vala_data_type_is_reference_type (vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl))) && vala_type_reference_get_data_type (vala_expression_get_static_type (vala_variable_declarator_get_initializer (decl))) != vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl))) {
			/* FIXME: use C cast if debugging disabled*/
			ValaCCodeExpression* __temp550 = NULL;
			rhs = (__temp550 = VALA_CCODE_EXPRESSION (vala_instance_cast_new (rhs, vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl)))), (rhs == NULL ? NULL : (rhs = (g_object_unref (rhs), NULL))), __temp550);
		}
		if (VALA_IS_ARRAY (vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl)))) {
			ValaCCodeCommaExpression* ccomma = g_object_new (VALA_TYPE_CCODE_COMMA_EXPRESSION, NULL);
			ValaVariableDeclarator* temp_decl = vala_code_generator_get_temp_variable_declarator (self, vala_variable_declarator_get_type_reference (decl), TRUE);
			ValaVariableDeclarator* __temp551 = NULL;
			self->priv->temp_vars = g_list_prepend (self->priv->temp_vars, (__temp551 = temp_decl, (__temp551 == NULL ? NULL : g_object_ref (__temp551))));
			ValaCCodeAssignment* __temp552 = NULL;
			vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION ((__temp552 = vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_variable_declarator_get_name (temp_decl))), rhs, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))));
			(__temp552 == NULL ? NULL : (__temp552 = (g_object_unref (__temp552), NULL)));
			ValaCCodeIdentifier* lhs_array_len = vala_ccode_identifier_new (vala_code_generator_get_array_length_cname (self, vala_variable_declarator_get_name (decl), 1));
			ValaCCodeExpression* rhs_array_len = vala_code_generator_get_array_length_cexpression (self, vala_variable_declarator_get_initializer (decl), 1);
			ValaCCodeAssignment* __temp553 = NULL;
			vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION ((__temp553 = vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (lhs_array_len), rhs_array_len, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))));
			(__temp553 == NULL ? NULL : (__temp553 = (g_object_unref (__temp553), NULL)));
			ValaCCodeIdentifier* __temp554 = NULL;
			vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION ((__temp554 = vala_ccode_identifier_new (vala_variable_declarator_get_name (temp_decl)))));
			(__temp554 == NULL ? NULL : (__temp554 = (g_object_unref (__temp554), NULL)));
			ValaCCodeExpression* __temp556 = NULL;
			ValaCCodeCommaExpression* __temp555 = NULL;
			rhs = (__temp556 = VALA_CCODE_EXPRESSION ((__temp555 = ccomma, (__temp555 == NULL ? NULL : g_object_ref (__temp555)))), (rhs == NULL ? NULL : (rhs = (g_object_unref (rhs), NULL))), __temp556);
			(ccomma == NULL ? NULL : (ccomma = (g_object_unref (ccomma), NULL)));
			(temp_decl == NULL ? NULL : (temp_decl = (g_object_unref (temp_decl), NULL)));
			(lhs_array_len == NULL ? NULL : (lhs_array_len = (g_object_unref (lhs_array_len), NULL)));
			(rhs_array_len == NULL ? NULL : (rhs_array_len = (g_object_unref (rhs_array_len), NULL)));
		}
	} else {
		if (vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl)) != NULL && vala_data_type_is_reference_type (vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl)))) {
			ValaCCodeExpression* __temp557 = NULL;
			rhs = (__temp557 = VALA_CCODE_EXPRESSION (vala_ccode_constant_new ("NULL")), (rhs == NULL ? NULL : (rhs = (g_object_unref (rhs), NULL))), __temp557);
		}
	}
	ValaCCodeVariableDeclarator* __temp558 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (decl), VALA_CCODE_NODE ((__temp558 = vala_ccode_variable_declarator_new_with_initializer (vala_variable_declarator_get_name (decl), rhs))));
	(__temp558 == NULL ? NULL : (__temp558 = (g_object_unref (__temp558), NULL)));
	vala_symbol_set_active (vala_code_node_get_symbol (VALA_CODE_NODE (decl)), TRUE);
	(rhs == NULL ? NULL : (rhs = (g_object_unref (rhs), NULL)));
}


static void vala_code_generator_real_visit_end_initializer_list (ValaCodeVisitor* base, ValaInitializerList* list)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_INITIALIZER_LIST (list));
	if (vala_expression_get_expected_type (VALA_EXPRESSION (list)) != NULL && VALA_IS_ARRAY (vala_type_reference_get_data_type (vala_expression_get_expected_type (VALA_EXPRESSION (list))))) {
	} else {
		/* TODO */
		ValaCCodeInitializerList* clist = g_object_new (VALA_TYPE_CCODE_INITIALIZER_LIST, NULL);
		{
			GList* __temp559 = NULL;
			__temp559 = vala_initializer_list_get_initializers (list);
			GList* expr_it;
			for (expr_it = __temp559; expr_it != NULL; expr_it = expr_it->next) {
				ValaExpression* expr = expr_it->data;
				{
					vala_ccode_initializer_list_append (clist, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (expr))));
				}
			}
			(__temp559 == NULL ? NULL : (__temp559 = (g_list_free (__temp559), NULL)));
		}
		vala_code_node_set_ccodenode (VALA_CODE_NODE (list), VALA_CCODE_NODE (clist));
		(clist == NULL ? NULL : (clist = (g_object_unref (clist), NULL)));
	}
}


static ValaVariableDeclarator* vala_code_generator_get_temp_variable_declarator (ValaCodeGenerator* self, ValaTypeReference* type, gboolean takes_ownership)
{
	g_return_val_if_fail (VALA_IS_CODE_GENERATOR (self), NULL);
	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (type), NULL);
	ValaVariableDeclarator* decl = vala_variable_declarator_new (g_strdup_printf ("__temp%d", self->priv->next_temp_var_id), NULL, NULL);
	ValaTypeReference* __temp560 = NULL;
	vala_variable_declarator_set_type_reference (decl, (__temp560 = vala_type_reference_copy (type)));
	(__temp560 == NULL ? NULL : (__temp560 = (g_object_unref (__temp560), NULL)));
	vala_type_reference_set_reference_to_value_type (vala_variable_declarator_get_type_reference (decl), FALSE);
	vala_type_reference_set_is_out (vala_variable_declarator_get_type_reference (decl), FALSE);
	vala_type_reference_set_takes_ownership (vala_variable_declarator_get_type_reference (decl), takes_ownership);
	self->priv->next_temp_var_id++;
	return decl;
	(decl == NULL ? NULL : (decl = (g_object_unref (decl), NULL)));
}


static ValaCCodeExpression* vala_code_generator_get_unref_expression (ValaCodeGenerator* self, ValaCCodeExpression* cvar, ValaTypeReference* type)
{
	g_return_val_if_fail (VALA_IS_CODE_GENERATOR (self), NULL);
	g_return_val_if_fail (VALA_IS_CCODE_EXPRESSION (cvar), NULL);
	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (type), NULL);
	/* (foo == NULL ? NULL : foo = (unref (foo), NULL)) 
	 can be simplified to
	 * foo = (unref (foo), NULL)
	 * if foo is of static type non-null
	 */
	ValaCCodeBinaryExpression* cisnull = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_EQUALITY, cvar, VALA_CCODE_EXPRESSION (vala_ccode_constant_new ("NULL")));
	char* unref_function = NULL;
	if (vala_data_type_is_reference_counting (vala_type_reference_get_data_type (type))) {
		unref_function = vala_data_type_get_unref_function (vala_type_reference_get_data_type (type));
	} else {
		unref_function = vala_data_type_get_free_function (vala_type_reference_get_data_type (type));
	}
	if (VALA_IS_ARRAY (vala_type_reference_get_data_type (type)) && g_utf8_collate (vala_data_type_get_name (vala_array_get_element_type ((VALA_ARRAY (vala_type_reference_get_data_type (type))))), "string") == 0) {
		unref_function = "g_strfreev";
	}
	ValaCCodeFunctionCall* ccall = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (unref_function)));
	vala_ccode_function_call_add_argument (ccall, cvar);
	/* set freed references to NULL to prevent further use */
	ValaCCodeCommaExpression* ccomma = g_object_new (VALA_TYPE_CCODE_COMMA_EXPRESSION, NULL);
	if (g_utf8_collate (unref_function, "g_list_free") == 0) {
		gboolean is_ref = FALSE;
		gboolean is_class = FALSE;
		GList* type_args = vala_type_reference_get_type_arguments (type);
		{
			GList* __temp562 = NULL;
			__temp562 = type_args;
			GList* type_arg_it;
			for (type_arg_it = __temp562; type_arg_it != NULL; type_arg_it = type_arg_it->next) {
				ValaTypeReference* type_arg = type_arg_it->data;
				{
					is_ref = vala_type_reference_get_takes_ownership (type_arg);
					is_class = VALA_IS_CLASS (vala_type_reference_get_data_type (type_arg));
				}
			}
		}
		if (is_ref) {
			ValaCCodeFunctionCall* cunrefcall = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("g_list_foreach")));
			vala_ccode_function_call_add_argument (cunrefcall, cvar);
			if (is_class) {
				ValaCCodeIdentifier* __temp563 = NULL;
				vala_ccode_function_call_add_argument (cunrefcall, VALA_CCODE_EXPRESSION ((__temp563 = vala_ccode_identifier_new ("(GFunc) g_object_unref"))));
				(__temp563 == NULL ? NULL : (__temp563 = (g_object_unref (__temp563), NULL)));
			} else {
				ValaCCodeIdentifier* __temp564 = NULL;
				vala_ccode_function_call_add_argument (cunrefcall, VALA_CCODE_EXPRESSION ((__temp564 = vala_ccode_identifier_new ("(GFunc) g_free"))));
				(__temp564 == NULL ? NULL : (__temp564 = (g_object_unref (__temp564), NULL)));
			}
			ValaCCodeConstant* __temp565 = NULL;
			vala_ccode_function_call_add_argument (cunrefcall, VALA_CCODE_EXPRESSION ((__temp565 = vala_ccode_constant_new ("NULL"))));
			(__temp565 == NULL ? NULL : (__temp565 = (g_object_unref (__temp565), NULL)));
			vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION (cunrefcall));
			(cunrefcall == NULL ? NULL : (cunrefcall = (g_object_unref (cunrefcall), NULL)));
		}
		(type_args == NULL ? NULL : (type_args = (g_list_free (type_args), NULL)));
	} else {
		if (g_utf8_collate (unref_function, "g_string_free") == 0) {
			ValaCCodeConstant* __temp566 = NULL;
			vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp566 = vala_ccode_constant_new ("TRUE"))));
			(__temp566 == NULL ? NULL : (__temp566 = (g_object_unref (__temp566), NULL)));
		}
	}
	vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION (ccall));
	ValaCCodeConstant* __temp567 = NULL;
	vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION ((__temp567 = vala_ccode_constant_new ("NULL"))));
	(__temp567 == NULL ? NULL : (__temp567 = (g_object_unref (__temp567), NULL)));
	ValaCCodeAssignment* cassign = vala_ccode_assignment_new (cvar, VALA_CCODE_EXPRESSION (ccomma), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE);
	ValaCCodeConditionalExpression* __temp568 = NULL;
	return VALA_CCODE_EXPRESSION ((__temp568 = vala_ccode_conditional_expression_new (VALA_CCODE_EXPRESSION (cisnull), VALA_CCODE_EXPRESSION (vala_ccode_constant_new ("NULL")), VALA_CCODE_EXPRESSION (vala_ccode_parenthesized_expression_new (VALA_CCODE_EXPRESSION (cassign)))), (cisnull == NULL ? NULL : (cisnull = (g_object_unref (cisnull), NULL))), (ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL))), (ccomma == NULL ? NULL : (ccomma = (g_object_unref (ccomma), NULL))), (cassign == NULL ? NULL : (cassign = (g_object_unref (cassign), NULL))), __temp568));
	(cisnull == NULL ? NULL : (cisnull = (g_object_unref (cisnull), NULL)));
	(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
	(ccomma == NULL ? NULL : (ccomma = (g_object_unref (ccomma), NULL)));
	(cassign == NULL ? NULL : (cassign = (g_object_unref (cassign), NULL)));
}


static void vala_code_generator_real_visit_end_full_expression (ValaCodeVisitor* base, ValaExpression* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_EXPRESSION (expr));
	if (!vala_code_generator_get_memory_management (self)) {
		GList* __temp569 = NULL;
		self->priv->temp_vars = (__temp569 = NULL, (self->priv->temp_vars == NULL ? NULL : (self->priv->temp_vars = (g_list_foreach (self->priv->temp_vars, (GFunc) g_object_unref, NULL), g_list_free (self->priv->temp_vars), NULL))), __temp569);
		GList* __temp570 = NULL;
		self->priv->temp_ref_vars = (__temp570 = NULL, (self->priv->temp_ref_vars == NULL ? NULL : (self->priv->temp_ref_vars = (g_list_foreach (self->priv->temp_ref_vars, (GFunc) g_object_unref, NULL), g_list_free (self->priv->temp_ref_vars), NULL))), __temp570);
		return;
	}
	/* expr is a full expression, i.e. an initializer, the
	 * expression in an expression statement, the controlling
	 * expression in if, while, for, or foreach statements
	 *
	 * we unref temporary variables at the end of a full
	 * expression
	 
	 can't automatically deep copy lists yet, so do it
	 * manually for now
	 * replace with
	 * expr.temp_vars = temp_vars;
	 * when deep list copying works
	 */
	GList* __temp571 = NULL;
	expr->temp_vars = (__temp571 = NULL, (expr->temp_vars == NULL ? NULL : (expr->temp_vars = (g_list_foreach (expr->temp_vars, (GFunc) g_object_unref, NULL), g_list_free (expr->temp_vars), NULL))), __temp571);
	{
		GList* __temp573 = NULL;
		__temp573 = self->priv->temp_vars;
		GList* decl1_it;
		for (decl1_it = __temp573; decl1_it != NULL; decl1_it = decl1_it->next) {
			ValaVariableDeclarator* decl1 = decl1_it->data;
			{
				ValaVariableDeclarator* __temp572 = NULL;
				expr->temp_vars = g_list_append (expr->temp_vars, (__temp572 = decl1, (__temp572 == NULL ? NULL : g_object_ref (__temp572))));
			}
		}
	}
	GList* __temp574 = NULL;
	self->priv->temp_vars = (__temp574 = NULL, (self->priv->temp_vars == NULL ? NULL : (self->priv->temp_vars = (g_list_foreach (self->priv->temp_vars, (GFunc) g_object_unref, NULL), g_list_free (self->priv->temp_vars), NULL))), __temp574);
	if (self->priv->temp_ref_vars == NULL) {
		return;
	}
	/* nothing to do without temporary variables */
	ValaVariableDeclarator* full_expr_decl = vala_code_generator_get_temp_variable_declarator (self, vala_expression_get_static_type (expr), TRUE);
	ValaVariableDeclarator* __temp575 = NULL;
	expr->temp_vars = g_list_append (expr->temp_vars, (__temp575 = full_expr_decl, (__temp575 == NULL ? NULL : g_object_ref (__temp575))));
	ValaCCodeCommaExpression* expr_list = g_object_new (VALA_TYPE_CCODE_COMMA_EXPRESSION, NULL);
	ValaCCodeAssignment* __temp576 = NULL;
	vala_ccode_comma_expression_append_expression (expr_list, VALA_CCODE_EXPRESSION ((__temp576 = vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_variable_declarator_get_name (full_expr_decl))), VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (expr))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))));
	(__temp576 == NULL ? NULL : (__temp576 = (g_object_unref (__temp576), NULL)));
	{
		GList* __temp579 = NULL;
		__temp579 = self->priv->temp_ref_vars;
		GList* decl_it;
		for (decl_it = __temp579; decl_it != NULL; decl_it = decl_it->next) {
			ValaVariableDeclarator* decl = decl_it->data;
			{
				ValaCCodeExpression* __temp578 = NULL;
				ValaCCodeIdentifier* __temp577 = NULL;
				vala_ccode_comma_expression_append_expression (expr_list, (__temp578 = vala_code_generator_get_unref_expression (self, VALA_CCODE_EXPRESSION ((__temp577 = vala_ccode_identifier_new (vala_variable_declarator_get_name (decl)))), vala_variable_declarator_get_type_reference (decl))));
				(__temp578 == NULL ? NULL : (__temp578 = (g_object_unref (__temp578), NULL)));
				(__temp577 == NULL ? NULL : (__temp577 = (g_object_unref (__temp577), NULL)));
			}
		}
	}
	ValaCCodeIdentifier* __temp580 = NULL;
	vala_ccode_comma_expression_append_expression (expr_list, VALA_CCODE_EXPRESSION ((__temp580 = vala_ccode_identifier_new (vala_variable_declarator_get_name (full_expr_decl)))));
	(__temp580 == NULL ? NULL : (__temp580 = (g_object_unref (__temp580), NULL)));
	vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE (expr_list));
	GList* __temp581 = NULL;
	self->priv->temp_ref_vars = (__temp581 = NULL, (self->priv->temp_ref_vars == NULL ? NULL : (self->priv->temp_ref_vars = (g_list_foreach (self->priv->temp_ref_vars, (GFunc) g_object_unref, NULL), g_list_free (self->priv->temp_ref_vars), NULL))), __temp581);
	(full_expr_decl == NULL ? NULL : (full_expr_decl = (g_object_unref (full_expr_decl), NULL)));
	(expr_list == NULL ? NULL : (expr_list = (g_object_unref (expr_list), NULL)));
}


static void vala_code_generator_append_temp_decl (ValaCodeGenerator* self, ValaCCodeFragment* cfrag, GList* temp_vars)
{
	g_return_if_fail (VALA_IS_CODE_GENERATOR (self));
	g_return_if_fail (VALA_IS_CCODE_FRAGMENT (cfrag));
	{
		GList* __temp583 = NULL;
		__temp583 = temp_vars;
		GList* decl_it;
		for (decl_it = __temp583; decl_it != NULL; decl_it = decl_it->next) {
			ValaVariableDeclarator* decl = decl_it->data;
			{
				ValaCCodeDeclaration* cdecl = vala_ccode_declaration_new (vala_type_reference_get_cname (vala_variable_declarator_get_type_reference (decl), TRUE, !vala_type_reference_get_takes_ownership (vala_variable_declarator_get_type_reference (decl))));
				ValaCCodeVariableDeclarator* vardecl = vala_ccode_variable_declarator_new (vala_variable_declarator_get_name (decl));
				vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR (vardecl));
				if (vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl)) != NULL && vala_data_type_is_reference_type (vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl)))) {
					ValaCCodeConstant* __temp582 = NULL;
					vala_ccode_variable_declarator_set_initializer (vardecl, VALA_CCODE_EXPRESSION ((__temp582 = vala_ccode_constant_new ("NULL"))));
					(__temp582 == NULL ? NULL : (__temp582 = (g_object_unref (__temp582), NULL)));
				}
				vala_ccode_fragment_append (cfrag, VALA_CCODE_NODE (cdecl));
				(cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL)));
				(vardecl == NULL ? NULL : (vardecl = (g_object_unref (vardecl), NULL)));
			}
		}
	}
}


static void vala_code_generator_real_visit_expression_statement (ValaCodeVisitor* base, ValaExpressionStatement* stmt)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_EXPRESSION_STATEMENT (stmt));
	ValaCCodeExpressionStatement* __temp584 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (stmt), VALA_CCODE_NODE ((__temp584 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_expression_statement_get_expression (stmt))))))));
	(__temp584 == NULL ? NULL : (__temp584 = (g_object_unref (__temp584), NULL)));
	/* free temporary objects */
	if (!vala_code_generator_get_memory_management (self)) {
		GList* __temp585 = NULL;
		self->priv->temp_vars = (__temp585 = NULL, (self->priv->temp_vars == NULL ? NULL : (self->priv->temp_vars = (g_list_foreach (self->priv->temp_vars, (GFunc) g_object_unref, NULL), g_list_free (self->priv->temp_vars), NULL))), __temp585);
		GList* __temp586 = NULL;
		self->priv->temp_ref_vars = (__temp586 = NULL, (self->priv->temp_ref_vars == NULL ? NULL : (self->priv->temp_ref_vars = (g_list_foreach (self->priv->temp_ref_vars, (GFunc) g_object_unref, NULL), g_list_free (self->priv->temp_ref_vars), NULL))), __temp586);
		return;
	}
	if (self->priv->temp_vars == NULL) {
		return;
	}
	/* nothing to do without temporary variables */
	ValaCCodeFragment* cfrag = g_object_new (VALA_TYPE_CCODE_FRAGMENT, NULL);
	vala_code_generator_append_temp_decl (self, cfrag, self->priv->temp_vars);
	vala_ccode_fragment_append (cfrag, vala_code_node_get_ccodenode (VALA_CODE_NODE (stmt)));
	{
		GList* __temp589 = NULL;
		__temp589 = self->priv->temp_ref_vars;
		GList* decl_it;
		for (decl_it = __temp589; decl_it != NULL; decl_it = decl_it->next) {
			ValaVariableDeclarator* decl = decl_it->data;
			{
				ValaCCodeExpressionStatement* __temp588 = NULL;
				ValaCCodeIdentifier* __temp587 = NULL;
				vala_ccode_fragment_append (cfrag, VALA_CCODE_NODE ((__temp588 = vala_ccode_expression_statement_new (vala_code_generator_get_unref_expression (self, VALA_CCODE_EXPRESSION ((__temp587 = vala_ccode_identifier_new (vala_variable_declarator_get_name (decl)))), vala_variable_declarator_get_type_reference (decl))))));
				(__temp588 == NULL ? NULL : (__temp588 = (g_object_unref (__temp588), NULL)));
				(__temp587 == NULL ? NULL : (__temp587 = (g_object_unref (__temp587), NULL)));
			}
		}
	}
	vala_code_node_set_ccodenode (VALA_CODE_NODE (stmt), VALA_CCODE_NODE (cfrag));
	GList* __temp590 = NULL;
	self->priv->temp_vars = (__temp590 = NULL, (self->priv->temp_vars == NULL ? NULL : (self->priv->temp_vars = (g_list_foreach (self->priv->temp_vars, (GFunc) g_object_unref, NULL), g_list_free (self->priv->temp_vars), NULL))), __temp590);
	GList* __temp591 = NULL;
	self->priv->temp_ref_vars = (__temp591 = NULL, (self->priv->temp_ref_vars == NULL ? NULL : (self->priv->temp_ref_vars = (g_list_foreach (self->priv->temp_ref_vars, (GFunc) g_object_unref, NULL), g_list_free (self->priv->temp_ref_vars), NULL))), __temp591);
	(cfrag == NULL ? NULL : (cfrag = (g_object_unref (cfrag), NULL)));
}


static void vala_code_generator_create_temp_decl (ValaCodeGenerator* self, ValaStatement* stmt, GList* temp_vars)
{
	g_return_if_fail (VALA_IS_CODE_GENERATOR (self));
	g_return_if_fail (VALA_IS_STATEMENT (stmt));
	/* declare temporary variables */
	if (temp_vars == NULL) {
		return;
	}
	/* nothing to do without temporary variables */
	ValaCCodeFragment* cfrag = g_object_new (VALA_TYPE_CCODE_FRAGMENT, NULL);
	vala_code_generator_append_temp_decl (self, cfrag, temp_vars);
	vala_ccode_fragment_append (cfrag, vala_code_node_get_ccodenode (VALA_CODE_NODE (stmt)));
	vala_code_node_set_ccodenode (VALA_CODE_NODE (stmt), VALA_CCODE_NODE (cfrag));
	(cfrag == NULL ? NULL : (cfrag = (g_object_unref (cfrag), NULL)));
}


static void vala_code_generator_real_visit_if_statement (ValaCodeVisitor* base, ValaIfStatement* stmt)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_IF_STATEMENT (stmt));
	if (vala_if_statement_get_false_statement (stmt) != NULL) {
		ValaCCodeIfStatement* __temp592 = NULL;
		vala_code_node_set_ccodenode (VALA_CODE_NODE (stmt), VALA_CCODE_NODE ((__temp592 = vala_ccode_if_statement_new (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_if_statement_get_condition (stmt)))), VALA_CCODE_STATEMENT (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_if_statement_get_true_statement (stmt)))), VALA_CCODE_STATEMENT (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_if_statement_get_false_statement (stmt))))))));
		(__temp592 == NULL ? NULL : (__temp592 = (g_object_unref (__temp592), NULL)));
	} else {
		ValaCCodeIfStatement* __temp593 = NULL;
		vala_code_node_set_ccodenode (VALA_CODE_NODE (stmt), VALA_CCODE_NODE ((__temp593 = vala_ccode_if_statement_new (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_if_statement_get_condition (stmt)))), VALA_CCODE_STATEMENT (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_if_statement_get_true_statement (stmt)))), NULL))));
		(__temp593 == NULL ? NULL : (__temp593 = (g_object_unref (__temp593), NULL)));
	}
	vala_code_generator_create_temp_decl (self, VALA_STATEMENT (stmt), vala_if_statement_get_condition (stmt)->temp_vars);
}


static void vala_code_generator_real_visit_switch_statement (ValaCodeVisitor* base, ValaSwitchStatement* stmt)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_SWITCH_STATEMENT (stmt));
	/* we need a temporary variable to save the property value*/
	ValaVariableDeclarator* temp_decl = vala_code_generator_get_temp_variable_declarator (self, vala_expression_get_static_type (vala_switch_statement_get_expression (stmt)), TRUE);
	ValaVariableDeclarator* __temp594 = NULL;
	vala_switch_statement_get_expression (stmt)->temp_vars = g_list_prepend (vala_switch_statement_get_expression (stmt)->temp_vars, (__temp594 = temp_decl, (__temp594 == NULL ? NULL : g_object_ref (__temp594))));
	ValaCCodeIdentifier* ctemp = vala_ccode_identifier_new (vala_variable_declarator_get_name (temp_decl));
	ValaCCodeAssignment* cinit = vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (ctemp), VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_switch_statement_get_expression (stmt)))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE);
	ValaCCodeFragment* cswitchblock = g_object_new (VALA_TYPE_CCODE_FRAGMENT, NULL);
	ValaCCodeExpressionStatement* __temp595 = NULL;
	vala_ccode_fragment_append (cswitchblock, VALA_CCODE_NODE ((__temp595 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (cinit)))));
	(__temp595 == NULL ? NULL : (__temp595 = (g_object_unref (__temp595), NULL)));
	vala_code_node_set_ccodenode (VALA_CODE_NODE (stmt), VALA_CCODE_NODE (cswitchblock));
	vala_code_generator_create_temp_decl (self, VALA_STATEMENT (stmt), vala_switch_statement_get_expression (stmt)->temp_vars);
	GList* default_statements = NULL;
	/* generate nested if statements*/
	ValaCCodeStatement* ctopstmt = NULL;
	ValaCCodeIfStatement* coldif = NULL;
	{
		GList* __temp604 = NULL;
		__temp604 = vala_switch_statement_get_sections (stmt);
		GList* section_it;
		for (section_it = __temp604; section_it != NULL; section_it = section_it->next) {
			ValaSwitchSection* section = section_it->data;
			{
				if (vala_switch_section_has_default_label (section)) {
					GList* __temp596 = NULL;
					default_statements = (__temp596 = vala_switch_section_get_statements (section), (default_statements == NULL ? NULL : (default_statements = (g_list_free (default_statements), NULL))), __temp596);
				} else {
					ValaCCodeBinaryExpression* cor = NULL;
					{
						GList* __temp600 = NULL;
						__temp600 = vala_switch_section_get_labels (section);
						GList* label_it;
						for (label_it = __temp600; label_it != NULL; label_it = label_it->next) {
							ValaSwitchLabel* label = label_it->data;
							{
								ValaCCodeBinaryExpression* ccmp = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_EQUALITY, VALA_CCODE_EXPRESSION (ctemp), VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_switch_label_get_expression (label)))));
								if (cor == NULL) {
									ValaCCodeBinaryExpression* __temp598 = NULL;
									ValaCCodeBinaryExpression* __temp597 = NULL;
									cor = (__temp598 = (__temp597 = ccmp, (__temp597 == NULL ? NULL : g_object_ref (__temp597))), (cor == NULL ? NULL : (cor = (g_object_unref (cor), NULL))), __temp598);
								} else {
									ValaCCodeBinaryExpression* __temp599 = NULL;
									cor = (__temp599 = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_OR, VALA_CCODE_EXPRESSION (cor), VALA_CCODE_EXPRESSION (ccmp)), (cor == NULL ? NULL : (cor = (g_object_unref (cor), NULL))), __temp599);
								}
								(ccmp == NULL ? NULL : (ccmp = (g_object_unref (ccmp), NULL)));
							}
						}
						(__temp600 == NULL ? NULL : (__temp600 = (g_list_free (__temp600), NULL)));
					}
					ValaCCodeBlock* cblock = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
					{
						GList* __temp601 = NULL;
						__temp601 = vala_switch_section_get_statements (section);
						GList* body_stmt_it;
						for (body_stmt_it = __temp601; body_stmt_it != NULL; body_stmt_it = body_stmt_it->next) {
							ValaStatement* body_stmt = body_stmt_it->data;
							{
								vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE (VALA_CCODE_STATEMENT (vala_code_node_get_ccodenode (VALA_CODE_NODE (body_stmt)))));
							}
						}
						(__temp601 == NULL ? NULL : (__temp601 = (g_list_free (__temp601), NULL)));
					}
					ValaCCodeDoStatement* cdo = vala_ccode_do_statement_new (VALA_CCODE_STATEMENT (cblock), VALA_CCODE_EXPRESSION (vala_ccode_constant_new ("0")));
					ValaCCodeIfStatement* cif = vala_ccode_if_statement_new (VALA_CCODE_EXPRESSION (cor), VALA_CCODE_STATEMENT (cdo), NULL);
					if (coldif != NULL) {
						vala_ccode_if_statement_set_false_statement (coldif, VALA_CCODE_STATEMENT (cif));
					} else {
						ValaCCodeStatement* __temp603 = NULL;
						ValaCCodeIfStatement* __temp602 = NULL;
						ctopstmt = (__temp603 = VALA_CCODE_STATEMENT ((__temp602 = cif, (__temp602 == NULL ? NULL : g_object_ref (__temp602)))), (ctopstmt == NULL ? NULL : (ctopstmt = (g_object_unref (ctopstmt), NULL))), __temp603);
					}
					coldif = cif;
					(cor == NULL ? NULL : (cor = (g_object_unref (cor), NULL)));
					(cblock == NULL ? NULL : (cblock = (g_object_unref (cblock), NULL)));
					(cdo == NULL ? NULL : (cdo = (g_object_unref (cdo), NULL)));
					(cif == NULL ? NULL : (cif = (g_object_unref (cif), NULL)));
				}
			}
		}
		(__temp604 == NULL ? NULL : (__temp604 = (g_list_free (__temp604), NULL)));
	}
	if (default_statements != NULL) {
		ValaCCodeBlock* cblock = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
		{
			GList* __temp605 = NULL;
			__temp605 = default_statements;
			GList* body_stmt_it;
			for (body_stmt_it = __temp605; body_stmt_it != NULL; body_stmt_it = body_stmt_it->next) {
				ValaStatement* body_stmt = body_stmt_it->data;
				{
					vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE (VALA_CCODE_STATEMENT (vala_code_node_get_ccodenode (VALA_CODE_NODE (body_stmt)))));
				}
			}
		}
		ValaCCodeDoStatement* cdo = vala_ccode_do_statement_new (VALA_CCODE_STATEMENT (cblock), VALA_CCODE_EXPRESSION (vala_ccode_constant_new ("0")));
		if (coldif == NULL) {
			/* there is only one section and that section
			 contains a default label*/
			ValaCCodeStatement* __temp607 = NULL;
			ValaCCodeDoStatement* __temp606 = NULL;
			ctopstmt = (__temp607 = VALA_CCODE_STATEMENT ((__temp606 = cdo, (__temp606 == NULL ? NULL : g_object_ref (__temp606)))), (ctopstmt == NULL ? NULL : (ctopstmt = (g_object_unref (ctopstmt), NULL))), __temp607);
		} else {
			vala_ccode_if_statement_set_false_statement (coldif, VALA_CCODE_STATEMENT (cdo));
		}
		(cblock == NULL ? NULL : (cblock = (g_object_unref (cblock), NULL)));
		(cdo == NULL ? NULL : (cdo = (g_object_unref (cdo), NULL)));
	}
	vala_ccode_fragment_append (cswitchblock, VALA_CCODE_NODE (ctopstmt));
	(temp_decl == NULL ? NULL : (temp_decl = (g_object_unref (temp_decl), NULL)));
	(ctemp == NULL ? NULL : (ctemp = (g_object_unref (ctemp), NULL)));
	(cinit == NULL ? NULL : (cinit = (g_object_unref (cinit), NULL)));
	(cswitchblock == NULL ? NULL : (cswitchblock = (g_object_unref (cswitchblock), NULL)));
	(default_statements == NULL ? NULL : (default_statements = (g_list_free (default_statements), NULL)));
	(ctopstmt == NULL ? NULL : (ctopstmt = (g_object_unref (ctopstmt), NULL)));
}


static void vala_code_generator_real_visit_while_statement (ValaCodeVisitor* base, ValaWhileStatement* stmt)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_WHILE_STATEMENT (stmt));
	ValaCCodeWhileStatement* __temp608 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (stmt), VALA_CCODE_NODE ((__temp608 = vala_ccode_while_statement_new (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_while_statement_get_condition (stmt)))), VALA_CCODE_STATEMENT (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_while_statement_get_body (stmt))))))));
	(__temp608 == NULL ? NULL : (__temp608 = (g_object_unref (__temp608), NULL)));
	vala_code_generator_create_temp_decl (self, VALA_STATEMENT (stmt), vala_while_statement_get_condition (stmt)->temp_vars);
}


static void vala_code_generator_real_visit_for_statement (ValaCodeVisitor* base, ValaForStatement* stmt)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_FOR_STATEMENT (stmt));
	ValaCCodeForStatement* cfor = vala_ccode_for_statement_new (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_for_statement_get_condition (stmt)))), VALA_CCODE_STATEMENT (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_for_statement_get_body (stmt)))));
	{
		GList* __temp609 = NULL;
		__temp609 = vala_for_statement_get_initializer (stmt);
		GList* init_expr_it;
		for (init_expr_it = __temp609; init_expr_it != NULL; init_expr_it = init_expr_it->next) {
			ValaExpression* init_expr = init_expr_it->data;
			{
				vala_ccode_for_statement_add_initializer (cfor, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (init_expr))));
			}
		}
		(__temp609 == NULL ? NULL : (__temp609 = (g_list_free (__temp609), NULL)));
	}
	{
		GList* __temp610 = NULL;
		__temp610 = vala_for_statement_get_iterator (stmt);
		GList* it_expr_it;
		for (it_expr_it = __temp610; it_expr_it != NULL; it_expr_it = it_expr_it->next) {
			ValaExpression* it_expr = it_expr_it->data;
			{
				vala_ccode_for_statement_add_iterator (cfor, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (it_expr))));
			}
		}
		(__temp610 == NULL ? NULL : (__temp610 = (g_list_free (__temp610), NULL)));
	}
	vala_code_node_set_ccodenode (VALA_CODE_NODE (stmt), VALA_CCODE_NODE (cfor));
	vala_code_generator_create_temp_decl (self, VALA_STATEMENT (stmt), vala_for_statement_get_condition (stmt)->temp_vars);
	(cfor == NULL ? NULL : (cfor = (g_object_unref (cfor), NULL)));
}


static void vala_code_generator_real_visit_end_foreach_statement (ValaCodeVisitor* base, ValaForeachStatement* stmt)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_FOREACH_STATEMENT (stmt));
	ValaCCodeBlock* cblock = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
	ValaCCodeForStatement* cfor = NULL;
	ValaVariableDeclarator* collection_backup = vala_code_generator_get_temp_variable_declarator (self, vala_expression_get_static_type (vala_foreach_statement_get_collection (stmt)), TRUE);
	ValaVariableDeclarator* __temp611 = NULL;
	vala_foreach_statement_get_collection (stmt)->temp_vars = g_list_prepend (vala_foreach_statement_get_collection (stmt)->temp_vars, (__temp611 = collection_backup, (__temp611 == NULL ? NULL : g_object_ref (__temp611))));
	ValaCCodeFragment* cfrag = g_object_new (VALA_TYPE_CCODE_FRAGMENT, NULL);
	vala_code_generator_append_temp_decl (self, cfrag, vala_foreach_statement_get_collection (stmt)->temp_vars);
	vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE (cfrag));
	ValaCCodeExpressionStatement* __temp612 = NULL;
	vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE ((__temp612 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_variable_declarator_get_name (collection_backup))), VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_foreach_statement_get_collection (stmt)))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))))));
	(__temp612 == NULL ? NULL : (__temp612 = (g_object_unref (__temp612), NULL)));
	vala_code_node_set_ccodenode (VALA_CODE_NODE (stmt), VALA_CCODE_NODE (cblock));
	if (VALA_IS_ARRAY (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_foreach_statement_get_collection (stmt))))) {
		ValaArray* arr = VALA_ARRAY (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_foreach_statement_get_collection (stmt))));
		ValaCCodeExpression* array_len = vala_code_generator_get_array_length_cexpression (self, vala_foreach_statement_get_collection (stmt), 1);
		if (VALA_IS_CCODE_CONSTANT (array_len)) {
			char* it_name = g_strdup_printf ("%s_it", vala_foreach_statement_get_variable_name (stmt));
			ValaCCodeDeclaration* citdecl = vala_ccode_declaration_new (vala_type_reference_get_cname (vala_expression_get_static_type (vala_foreach_statement_get_collection (stmt)), FALSE, FALSE));
			ValaCCodeVariableDeclarator* __temp613 = NULL;
			vala_ccode_declaration_add_declarator (citdecl, VALA_CCODE_DECLARATOR ((__temp613 = vala_ccode_variable_declarator_new (it_name))));
			(__temp613 == NULL ? NULL : (__temp613 = (g_object_unref (__temp613), NULL)));
			vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE (citdecl));
			ValaCCodeBlock* cbody = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
			ValaCCodeDeclaration* cdecl = vala_ccode_declaration_new (vala_type_reference_get_cname (vala_foreach_statement_get_type_reference (stmt), FALSE, FALSE));
			ValaCCodeVariableDeclarator* __temp614 = NULL;
			vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR ((__temp614 = vala_ccode_variable_declarator_new_with_initializer (vala_foreach_statement_get_variable_name (stmt), VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (g_strdup_printf ("*%s", it_name)))))));
			(__temp614 == NULL ? NULL : (__temp614 = (g_object_unref (__temp614), NULL)));
			vala_ccode_block_add_statement (cbody, VALA_CCODE_NODE (cdecl));
			vala_ccode_block_add_statement (cbody, vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_foreach_statement_get_body (stmt))));
			ValaCCodeBinaryExpression* ccond = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_INEQUALITY, VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (g_strdup_printf ("*%s", it_name))), VALA_CCODE_EXPRESSION (vala_ccode_constant_new ("NULL")));
			ValaCCodeForStatement* cfor = vala_ccode_for_statement_new (VALA_CCODE_EXPRESSION (ccond), VALA_CCODE_STATEMENT (cbody));
			ValaCCodeAssignment* __temp615 = NULL;
			vala_ccode_for_statement_add_initializer (cfor, VALA_CCODE_EXPRESSION ((__temp615 = vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (it_name)), VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_variable_declarator_get_name (collection_backup))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))));
			(__temp615 == NULL ? NULL : (__temp615 = (g_object_unref (__temp615), NULL)));
			ValaCCodeAssignment* __temp616 = NULL;
			vala_ccode_for_statement_add_iterator (cfor, VALA_CCODE_EXPRESSION ((__temp616 = vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (it_name)), VALA_CCODE_EXPRESSION (vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_PLUS, VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (it_name)), VALA_CCODE_EXPRESSION (vala_ccode_constant_new ("1")))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))));
			(__temp616 == NULL ? NULL : (__temp616 = (g_object_unref (__temp616), NULL)));
			vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE (cfor));
			(it_name == NULL ? NULL : (it_name = (g_free (it_name), NULL)));
			(citdecl == NULL ? NULL : (citdecl = (g_object_unref (citdecl), NULL)));
			(cbody == NULL ? NULL : (cbody = (g_object_unref (cbody), NULL)));
			(cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL)));
			(ccond == NULL ? NULL : (ccond = (g_object_unref (ccond), NULL)));
			(cfor == NULL ? NULL : (cfor = (g_object_unref (cfor), NULL)));
		} else {
			char* it_name = (g_strconcat (vala_foreach_statement_get_variable_name (stmt), "_it", NULL));
			ValaCCodeDeclaration* citdecl = vala_ccode_declaration_new ("int");
			ValaCCodeVariableDeclarator* __temp617 = NULL;
			vala_ccode_declaration_add_declarator (citdecl, VALA_CCODE_DECLARATOR ((__temp617 = vala_ccode_variable_declarator_new (it_name))));
			(__temp617 == NULL ? NULL : (__temp617 = (g_object_unref (__temp617), NULL)));
			vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE (citdecl));
			ValaCCodeBlock* cbody = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
			ValaCCodeDeclaration* cdecl = vala_ccode_declaration_new (vala_type_reference_get_cname (vala_foreach_statement_get_type_reference (stmt), FALSE, FALSE));
			ValaCCodeVariableDeclarator* __temp618 = NULL;
			vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR ((__temp618 = vala_ccode_variable_declarator_new_with_initializer (vala_foreach_statement_get_variable_name (stmt), VALA_CCODE_EXPRESSION (vala_ccode_element_access_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_variable_declarator_get_name (collection_backup))), VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (it_name))))))));
			(__temp618 == NULL ? NULL : (__temp618 = (g_object_unref (__temp618), NULL)));
			vala_ccode_block_add_statement (cbody, VALA_CCODE_NODE (cdecl));
			vala_ccode_block_add_statement (cbody, vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_foreach_statement_get_body (stmt))));
			ValaCCodeBinaryExpression* ccond_ind1 = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_INEQUALITY, array_len, VALA_CCODE_EXPRESSION (vala_ccode_constant_new ("-1")));
			ValaCCodeBinaryExpression* ccond_ind2 = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_LESS_THAN, VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (it_name)), array_len);
			ValaCCodeBinaryExpression* ccond_ind = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_AND, VALA_CCODE_EXPRESSION (ccond_ind1), VALA_CCODE_EXPRESSION (ccond_ind2));
			ValaCCodeBinaryExpression* ccond_term1 = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_EQUALITY, array_len, VALA_CCODE_EXPRESSION (vala_ccode_constant_new ("-1")));
			ValaCCodeBinaryExpression* ccond_term2 = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_INEQUALITY, VALA_CCODE_EXPRESSION (vala_ccode_element_access_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_variable_declarator_get_name (collection_backup))), VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (it_name)))), VALA_CCODE_EXPRESSION (vala_ccode_constant_new ("NULL")));
			ValaCCodeBinaryExpression* ccond_term = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_AND, VALA_CCODE_EXPRESSION (ccond_term1), VALA_CCODE_EXPRESSION (ccond_term2));
			ValaCCodeBinaryExpression* ccond = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_OR, VALA_CCODE_EXPRESSION (vala_ccode_parenthesized_expression_new (VALA_CCODE_EXPRESSION (ccond_ind))), VALA_CCODE_EXPRESSION (vala_ccode_parenthesized_expression_new (VALA_CCODE_EXPRESSION (ccond_term))));
			ValaCCodeForStatement* cfor = vala_ccode_for_statement_new (VALA_CCODE_EXPRESSION (ccond), VALA_CCODE_STATEMENT (cbody));
			ValaCCodeAssignment* __temp619 = NULL;
			vala_ccode_for_statement_add_initializer (cfor, VALA_CCODE_EXPRESSION ((__temp619 = vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (it_name)), VALA_CCODE_EXPRESSION (vala_ccode_constant_new ("0")), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))));
			(__temp619 == NULL ? NULL : (__temp619 = (g_object_unref (__temp619), NULL)));
			ValaCCodeAssignment* __temp620 = NULL;
			vala_ccode_for_statement_add_iterator (cfor, VALA_CCODE_EXPRESSION ((__temp620 = vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (it_name)), VALA_CCODE_EXPRESSION (vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_PLUS, VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (it_name)), VALA_CCODE_EXPRESSION (vala_ccode_constant_new ("1")))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))));
			(__temp620 == NULL ? NULL : (__temp620 = (g_object_unref (__temp620), NULL)));
			vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE (cfor));
			(it_name == NULL ? NULL : (it_name = (g_free (it_name), NULL)));
			(citdecl == NULL ? NULL : (citdecl = (g_object_unref (citdecl), NULL)));
			(cbody == NULL ? NULL : (cbody = (g_object_unref (cbody), NULL)));
			(cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL)));
			(ccond_ind1 == NULL ? NULL : (ccond_ind1 = (g_object_unref (ccond_ind1), NULL)));
			(ccond_ind2 == NULL ? NULL : (ccond_ind2 = (g_object_unref (ccond_ind2), NULL)));
			(ccond_ind == NULL ? NULL : (ccond_ind = (g_object_unref (ccond_ind), NULL)));
			(ccond_term1 == NULL ? NULL : (ccond_term1 = (g_object_unref (ccond_term1), NULL)));
			(ccond_term2 == NULL ? NULL : (ccond_term2 = (g_object_unref (ccond_term2), NULL)));
			(ccond_term == NULL ? NULL : (ccond_term = (g_object_unref (ccond_term), NULL)));
			(ccond == NULL ? NULL : (ccond = (g_object_unref (ccond), NULL)));
			(cfor == NULL ? NULL : (cfor = (g_object_unref (cfor), NULL)));
		}
		(array_len == NULL ? NULL : (array_len = (g_object_unref (array_len), NULL)));
	} else {
		if (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_foreach_statement_get_collection (stmt))) == self->priv->list_type || vala_type_reference_get_data_type (vala_expression_get_static_type (vala_foreach_statement_get_collection (stmt))) == self->priv->slist_type) {
			char* it_name = g_strdup_printf ("%s_it", vala_foreach_statement_get_variable_name (stmt));
			ValaCCodeDeclaration* citdecl = vala_ccode_declaration_new (vala_type_reference_get_cname (vala_expression_get_static_type (vala_foreach_statement_get_collection (stmt)), FALSE, FALSE));
			ValaCCodeVariableDeclarator* __temp621 = NULL;
			vala_ccode_declaration_add_declarator (citdecl, VALA_CCODE_DECLARATOR ((__temp621 = vala_ccode_variable_declarator_new (it_name))));
			(__temp621 == NULL ? NULL : (__temp621 = (g_object_unref (__temp621), NULL)));
			vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE (citdecl));
			ValaCCodeBlock* cbody = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
			ValaCCodeDeclaration* cdecl = vala_ccode_declaration_new (vala_type_reference_get_cname (vala_foreach_statement_get_type_reference (stmt), FALSE, FALSE));
			ValaCCodeVariableDeclarator* __temp622 = NULL;
			vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR ((__temp622 = vala_ccode_variable_declarator_new_with_initializer (vala_foreach_statement_get_variable_name (stmt), VALA_CCODE_EXPRESSION (vala_ccode_member_access_new_pointer (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (it_name)), "data"))))));
			(__temp622 == NULL ? NULL : (__temp622 = (g_object_unref (__temp622), NULL)));
			vala_ccode_block_add_statement (cbody, VALA_CCODE_NODE (cdecl));
			vala_ccode_block_add_statement (cbody, vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_foreach_statement_get_body (stmt))));
			ValaCCodeBinaryExpression* ccond = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_INEQUALITY, VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (it_name)), VALA_CCODE_EXPRESSION (vala_ccode_constant_new ("NULL")));
			ValaCCodeForStatement* cfor = vala_ccode_for_statement_new (VALA_CCODE_EXPRESSION (ccond), VALA_CCODE_STATEMENT (cbody));
			ValaCCodeAssignment* __temp623 = NULL;
			vala_ccode_for_statement_add_initializer (cfor, VALA_CCODE_EXPRESSION ((__temp623 = vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (it_name)), VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_variable_declarator_get_name (collection_backup))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))));
			(__temp623 == NULL ? NULL : (__temp623 = (g_object_unref (__temp623), NULL)));
			ValaCCodeAssignment* __temp624 = NULL;
			vala_ccode_for_statement_add_iterator (cfor, VALA_CCODE_EXPRESSION ((__temp624 = vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (it_name)), VALA_CCODE_EXPRESSION (vala_ccode_member_access_new_pointer (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (it_name)), "next")), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))));
			(__temp624 == NULL ? NULL : (__temp624 = (g_object_unref (__temp624), NULL)));
			vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE (cfor));
			(it_name == NULL ? NULL : (it_name = (g_free (it_name), NULL)));
			(citdecl == NULL ? NULL : (citdecl = (g_object_unref (citdecl), NULL)));
			(cbody == NULL ? NULL : (cbody = (g_object_unref (cbody), NULL)));
			(cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL)));
			(ccond == NULL ? NULL : (ccond = (g_object_unref (ccond), NULL)));
			(cfor == NULL ? NULL : (cfor = (g_object_unref (cfor), NULL)));
		}
	}
	if (vala_code_generator_get_memory_management (self) && vala_type_reference_get_transfers_ownership (vala_expression_get_static_type (vala_foreach_statement_get_collection (stmt)))) {
		ValaCCodeExpressionStatement* __temp626 = NULL;
		ValaCCodeIdentifier* __temp625 = NULL;
		vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE ((__temp626 = vala_ccode_expression_statement_new (vala_code_generator_get_unref_expression (self, VALA_CCODE_EXPRESSION ((__temp625 = vala_ccode_identifier_new (vala_variable_declarator_get_name (collection_backup)))), vala_expression_get_static_type (vala_foreach_statement_get_collection (stmt)))))));
		(__temp626 == NULL ? NULL : (__temp626 = (g_object_unref (__temp626), NULL)));
		(__temp625 == NULL ? NULL : (__temp625 = (g_object_unref (__temp625), NULL)));
	}
	(cblock == NULL ? NULL : (cblock = (g_object_unref (cblock), NULL)));
	(collection_backup == NULL ? NULL : (collection_backup = (g_object_unref (collection_backup), NULL)));
	(cfrag == NULL ? NULL : (cfrag = (g_object_unref (cfrag), NULL)));
}


static void vala_code_generator_real_visit_break_statement (ValaCodeVisitor* base, ValaBreakStatement* stmt)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_BREAK_STATEMENT (stmt));
	ValaCCodeBreakStatement* __temp627 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (stmt), VALA_CCODE_NODE ((__temp627 = g_object_new (VALA_TYPE_CCODE_BREAK_STATEMENT, NULL))));
	(__temp627 == NULL ? NULL : (__temp627 = (g_object_unref (__temp627), NULL)));
}


static void vala_code_generator_real_visit_continue_statement (ValaCodeVisitor* base, ValaContinueStatement* stmt)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_CONTINUE_STATEMENT (stmt));
	ValaCCodeContinueStatement* __temp628 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (stmt), VALA_CCODE_NODE ((__temp628 = g_object_new (VALA_TYPE_CCODE_CONTINUE_STATEMENT, NULL))));
	(__temp628 == NULL ? NULL : (__temp628 = (g_object_unref (__temp628), NULL)));
}


static void vala_code_generator_append_local_free (ValaCodeGenerator* self, ValaSymbol* sym, ValaCCodeFragment* cfrag, gboolean stop_at_loop)
{
	g_return_if_fail (VALA_IS_CODE_GENERATOR (self));
	g_return_if_fail (sym == NULL || VALA_IS_SYMBOL (sym));
	g_return_if_fail (cfrag == NULL || VALA_IS_CCODE_FRAGMENT (cfrag));
	ValaBlock* b = VALA_BLOCK (vala_symbol_get_node (sym));
	GList* local_vars = vala_block_get_local_variables (b);
	{
		GList* __temp631 = NULL;
		__temp631 = local_vars;
		GList* decl_it;
		for (decl_it = __temp631; decl_it != NULL; decl_it = decl_it->next) {
			ValaVariableDeclarator* decl = decl_it->data;
			{
				if (vala_symbol_get_active (vala_code_node_get_symbol (VALA_CODE_NODE (decl))) && vala_data_type_is_reference_type (vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl))) && vala_type_reference_get_takes_ownership (vala_variable_declarator_get_type_reference (decl))) {
					ValaCCodeExpressionStatement* __temp630 = NULL;
					ValaCCodeIdentifier* __temp629 = NULL;
					vala_ccode_fragment_append (cfrag, VALA_CCODE_NODE ((__temp630 = vala_ccode_expression_statement_new (vala_code_generator_get_unref_expression (self, VALA_CCODE_EXPRESSION ((__temp629 = vala_ccode_identifier_new (vala_variable_declarator_get_name (decl)))), vala_variable_declarator_get_type_reference (decl))))));
					(__temp630 == NULL ? NULL : (__temp630 = (g_object_unref (__temp630), NULL)));
					(__temp629 == NULL ? NULL : (__temp629 = (g_object_unref (__temp629), NULL)));
				}
			}
		}
	}
	if (VALA_IS_BLOCK (vala_symbol_get_node (vala_symbol_get_parent_symbol (sym)))) {
		vala_code_generator_append_local_free (self, vala_symbol_get_parent_symbol (sym), cfrag, stop_at_loop);
	}
	(local_vars == NULL ? NULL : (local_vars = (g_list_free (local_vars), NULL)));
}


static void vala_code_generator_create_local_free (ValaCodeGenerator* self, ValaStatement* stmt)
{
	g_return_if_fail (VALA_IS_CODE_GENERATOR (self));
	g_return_if_fail (stmt == NULL || VALA_IS_STATEMENT (stmt));
	if (!vala_code_generator_get_memory_management (self)) {
		return;
	}
	ValaCCodeFragment* cfrag = g_object_new (VALA_TYPE_CCODE_FRAGMENT, NULL);
	vala_code_generator_append_local_free (self, self->priv->current_symbol, cfrag, FALSE);
	vala_ccode_fragment_append (cfrag, vala_code_node_get_ccodenode (VALA_CODE_NODE (stmt)));
	vala_code_node_set_ccodenode (VALA_CODE_NODE (stmt), VALA_CCODE_NODE (cfrag));
	(cfrag == NULL ? NULL : (cfrag = (g_object_unref (cfrag), NULL)));
}


static gboolean vala_code_generator_append_local_free_expr (ValaCodeGenerator* self, ValaSymbol* sym, ValaCCodeCommaExpression* ccomma, gboolean stop_at_loop)
{
	g_return_val_if_fail (VALA_IS_CODE_GENERATOR (self), FALSE);
	g_return_val_if_fail (sym == NULL || VALA_IS_SYMBOL (sym), FALSE);
	g_return_val_if_fail (ccomma == NULL || VALA_IS_CCODE_COMMA_EXPRESSION (ccomma), FALSE);
	gboolean found = FALSE;
	ValaBlock* b = VALA_BLOCK (vala_symbol_get_node (sym));
	GList* local_vars = vala_block_get_local_variables (b);
	{
		GList* __temp634 = NULL;
		__temp634 = local_vars;
		GList* decl_it;
		for (decl_it = __temp634; decl_it != NULL; decl_it = decl_it->next) {
			ValaVariableDeclarator* decl = decl_it->data;
			{
				if (vala_symbol_get_active (vala_code_node_get_symbol (VALA_CODE_NODE (decl))) && vala_data_type_is_reference_type (vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl))) && vala_type_reference_get_takes_ownership (vala_variable_declarator_get_type_reference (decl))) {
					found = TRUE;
					ValaCCodeExpression* __temp633 = NULL;
					ValaCCodeIdentifier* __temp632 = NULL;
					vala_ccode_comma_expression_append_expression (ccomma, (__temp633 = vala_code_generator_get_unref_expression (self, VALA_CCODE_EXPRESSION ((__temp632 = vala_ccode_identifier_new (vala_variable_declarator_get_name (decl)))), vala_variable_declarator_get_type_reference (decl))));
					(__temp633 == NULL ? NULL : (__temp633 = (g_object_unref (__temp633), NULL)));
					(__temp632 == NULL ? NULL : (__temp632 = (g_object_unref (__temp632), NULL)));
				}
			}
		}
	}
	if (VALA_IS_BLOCK (vala_symbol_get_node (vala_symbol_get_parent_symbol (sym)))) {
		found = found || vala_code_generator_append_local_free_expr (self, vala_symbol_get_parent_symbol (sym), ccomma, stop_at_loop);
	}
	gboolean __temp635;
	return (__temp635 = found, (local_vars == NULL ? NULL : (local_vars = (g_list_free (local_vars), NULL))), __temp635);
	(local_vars == NULL ? NULL : (local_vars = (g_list_free (local_vars), NULL)));
}


static void vala_code_generator_create_local_free_expr (ValaCodeGenerator* self, ValaExpression* expr)
{
	g_return_if_fail (VALA_IS_CODE_GENERATOR (self));
	g_return_if_fail (expr == NULL || VALA_IS_EXPRESSION (expr));
	if (!vala_code_generator_get_memory_management (self)) {
		return;
	}
	ValaVariableDeclarator* return_expr_decl = vala_code_generator_get_temp_variable_declarator (self, vala_expression_get_static_type (expr), TRUE);
	ValaCCodeCommaExpression* ccomma = g_object_new (VALA_TYPE_CCODE_COMMA_EXPRESSION, NULL);
	ValaCCodeAssignment* __temp636 = NULL;
	vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION ((__temp636 = vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_variable_declarator_get_name (return_expr_decl))), VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (expr))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))));
	(__temp636 == NULL ? NULL : (__temp636 = (g_object_unref (__temp636), NULL)));
	if (!vala_code_generator_append_local_free_expr (self, self->priv->current_symbol, ccomma, FALSE)) {
		(return_expr_decl == NULL ? NULL : (return_expr_decl = (g_object_unref (return_expr_decl), NULL)));
		(ccomma == NULL ? NULL : (ccomma = (g_object_unref (ccomma), NULL)));
		return;
	}
	/* no local variables need to be freed */
	ValaCCodeIdentifier* __temp637 = NULL;
	vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION ((__temp637 = vala_ccode_identifier_new (vala_variable_declarator_get_name (return_expr_decl)))));
	(__temp637 == NULL ? NULL : (__temp637 = (g_object_unref (__temp637), NULL)));
	vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE (ccomma));
	ValaVariableDeclarator* __temp638 = NULL;
	expr->temp_vars = g_list_append (expr->temp_vars, (__temp638 = return_expr_decl, (__temp638 == NULL ? NULL : g_object_ref (__temp638))));
	(return_expr_decl == NULL ? NULL : (return_expr_decl = (g_object_unref (return_expr_decl), NULL)));
	(ccomma == NULL ? NULL : (ccomma = (g_object_unref (ccomma), NULL)));
}


static void vala_code_generator_real_visit_begin_return_statement (ValaCodeVisitor* base, ValaReturnStatement* stmt)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_RETURN_STATEMENT (stmt));
	if (vala_return_statement_get_return_expression (stmt) != NULL) {
		/* avoid unnecessary ref/unref pair*/
		if (vala_expression_get_ref_missing (vala_return_statement_get_return_expression (stmt)) && vala_expression_get_symbol_reference (vala_return_statement_get_return_expression (stmt)) != NULL && VALA_IS_VARIABLE_DECLARATOR (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_return_statement_get_return_expression (stmt))))) {
			ValaVariableDeclarator* decl = VALA_VARIABLE_DECLARATOR (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_return_statement_get_return_expression (stmt))));
			if (vala_type_reference_get_takes_ownership (vala_variable_declarator_get_type_reference (decl))) {
				/* return expression is local variable taking ownership and
				 * current method is transferring ownership */
				vala_expression_set_ref_sink (vala_return_statement_get_return_expression (stmt), TRUE);
				/* don't ref expression*/
				vala_expression_set_ref_missing (vala_return_statement_get_return_expression (stmt), FALSE);
			}
		}
	}
}


static void vala_code_generator_real_visit_end_return_statement (ValaCodeVisitor* base, ValaReturnStatement* stmt)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_RETURN_STATEMENT (stmt));
	if (vala_return_statement_get_return_expression (stmt) == NULL) {
		ValaCCodeReturnStatement* __temp639 = NULL;
		vala_code_node_set_ccodenode (VALA_CODE_NODE (stmt), VALA_CCODE_NODE ((__temp639 = vala_ccode_return_statement_new (NULL))));
		(__temp639 == NULL ? NULL : (__temp639 = (g_object_unref (__temp639), NULL)));
		vala_code_generator_create_local_free (self, VALA_STATEMENT (stmt));
	} else {
		ValaSymbol* return_expression_symbol = NULL;
		/* avoid unnecessary ref/unref pair*/
		if (vala_expression_get_ref_sink (vala_return_statement_get_return_expression (stmt)) && vala_expression_get_symbol_reference (vala_return_statement_get_return_expression (stmt)) != NULL && VALA_IS_VARIABLE_DECLARATOR (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_return_statement_get_return_expression (stmt))))) {
			ValaVariableDeclarator* decl = VALA_VARIABLE_DECLARATOR (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_return_statement_get_return_expression (stmt))));
			if (vala_type_reference_get_takes_ownership (vala_variable_declarator_get_type_reference (decl))) {
				/* return expression is local variable taking ownership and
				 * current method is transferring ownership 
				 don't unref expression*/
				return_expression_symbol = vala_code_node_get_symbol (VALA_CODE_NODE (decl));
				vala_symbol_set_active (return_expression_symbol, FALSE);
			}
		}
		vala_code_generator_create_local_free_expr (self, vala_return_statement_get_return_expression (stmt));
		if (vala_expression_get_static_type (vala_return_statement_get_return_expression (stmt)) != NULL && vala_type_reference_get_data_type (vala_expression_get_static_type (vala_return_statement_get_return_expression (stmt))) != vala_type_reference_get_data_type (self->priv->current_return_type)) {
			/* cast required */
			if (VALA_IS_CLASS (vala_type_reference_get_data_type (self->priv->current_return_type))) {
				ValaInstanceCast* __temp640 = NULL;
				vala_code_node_set_ccodenode (VALA_CODE_NODE (vala_return_statement_get_return_expression (stmt)), VALA_CCODE_NODE ((__temp640 = vala_instance_cast_new (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_return_statement_get_return_expression (stmt)))), vala_type_reference_get_data_type (self->priv->current_return_type)))));
				(__temp640 == NULL ? NULL : (__temp640 = (g_object_unref (__temp640), NULL)));
			}
		}
		ValaCCodeReturnStatement* __temp641 = NULL;
		vala_code_node_set_ccodenode (VALA_CODE_NODE (stmt), VALA_CCODE_NODE ((__temp641 = vala_ccode_return_statement_new (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_return_statement_get_return_expression (stmt))))))));
		(__temp641 == NULL ? NULL : (__temp641 = (g_object_unref (__temp641), NULL)));
		vala_code_generator_create_temp_decl (self, VALA_STATEMENT (stmt), vala_return_statement_get_return_expression (stmt)->temp_vars);
		if (return_expression_symbol != NULL) {
			vala_symbol_set_active (return_expression_symbol, TRUE);
		}
	}
}


static char* vala_code_generator_get_symbol_lock_name (ValaCodeGenerator* self, ValaSymbol* sym)
{
	g_return_val_if_fail (VALA_IS_CODE_GENERATOR (self), NULL);
	g_return_val_if_fail (VALA_IS_SYMBOL (sym), NULL);
	return g_strdup_printf ("__lock_%s", vala_symbol_get_name (sym));
}


/**
 * Visit operation called for lock statements.
 *
 * @param stmt a lock statement
 */
static void vala_code_generator_real_visit_lock_statement (ValaCodeVisitor* base, ValaLockStatement* stmt)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_LOCK_STATEMENT (stmt));
	ValaCCodeFragment* cn = g_object_new (VALA_TYPE_CCODE_FRAGMENT, NULL);
	ValaCCodeExpression* l = NULL;
	ValaCCodeFunctionCall* fc = NULL;
	ValaExpression* inner_node = vala_member_access_get_inner ((VALA_MEMBER_ACCESS (vala_lock_statement_get_resource (stmt))));
	if (inner_node == NULL) {
		ValaCCodeExpression* __temp643 = NULL;
		l = (__temp643 = VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("self")), (l == NULL ? NULL : (l = (g_object_unref (l), NULL))), __temp643);
	} else {
		if (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_expression_get_symbol_reference (vala_lock_statement_get_resource (stmt)))) != VALA_CODE_NODE (self->priv->current_class)) {
			ValaCCodeExpression* __temp644 = NULL;
			l = (__temp644 = VALA_CCODE_EXPRESSION (vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_data_type_get_upper_case_cname ((VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_expression_get_symbol_reference (vala_lock_statement_get_resource (stmt)))))), NULL))))), (l == NULL ? NULL : (l = (g_object_unref (l), NULL))), __temp644);
			vala_ccode_function_call_add_argument ((VALA_CCODE_FUNCTION_CALL (l)), VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (inner_node))));
		} else {
			ValaCCodeExpression* __temp646 = NULL;
			ValaCCodeExpression* __temp645 = NULL;
			l = (__temp646 = (__temp645 = VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (inner_node))), (__temp645 == NULL ? NULL : g_object_ref (__temp645))), (l == NULL ? NULL : (l = (g_object_unref (l), NULL))), __temp646);
		}
	}
	ValaCCodeExpression* __temp647 = NULL;
	l = (__temp647 = VALA_CCODE_EXPRESSION (vala_ccode_member_access_new_pointer (VALA_CCODE_EXPRESSION (vala_ccode_member_access_new_pointer (l, "priv")), vala_code_generator_get_symbol_lock_name (self, vala_expression_get_symbol_reference (vala_lock_statement_get_resource (stmt))))), (l == NULL ? NULL : (l = (g_object_unref (l), NULL))), __temp647);
	ValaCCodeFunctionCall* __temp648 = NULL;
	fc = (__temp648 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_method_get_cname ((VALA_METHOD (vala_symbol_get_node (vala_symbol_lookup (vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (self->priv->mutex_type))), "lock")))))))), (fc == NULL ? NULL : (fc = (g_object_unref (fc), NULL))), __temp648);
	vala_ccode_function_call_add_argument (fc, l);
	ValaCCodeExpressionStatement* __temp649 = NULL;
	vala_ccode_fragment_append (cn, VALA_CCODE_NODE ((__temp649 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (fc)))));
	(__temp649 == NULL ? NULL : (__temp649 = (g_object_unref (__temp649), NULL)));
	vala_ccode_fragment_append (cn, vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_lock_statement_get_body (stmt))));
	ValaCCodeFunctionCall* __temp650 = NULL;
	fc = (__temp650 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_method_get_cname ((VALA_METHOD (vala_symbol_get_node (vala_symbol_lookup (vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (self->priv->mutex_type))), "unlock")))))))), (fc == NULL ? NULL : (fc = (g_object_unref (fc), NULL))), __temp650);
	vala_ccode_function_call_add_argument (fc, l);
	ValaCCodeExpressionStatement* __temp651 = NULL;
	vala_ccode_fragment_append (cn, VALA_CCODE_NODE ((__temp651 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (fc)))));
	(__temp651 == NULL ? NULL : (__temp651 = (g_object_unref (__temp651), NULL)));
	vala_code_node_set_ccodenode (VALA_CODE_NODE (stmt), VALA_CCODE_NODE (cn));
	(cn == NULL ? NULL : (cn = (g_object_unref (cn), NULL)));
	(l == NULL ? NULL : (l = (g_object_unref (l), NULL)));
	(fc == NULL ? NULL : (fc = (g_object_unref (fc), NULL)));
}


/**
 * Visit operations called for array creation expresions.
 *
 * @param expr an array creation expression
 */
static void vala_code_generator_real_visit_end_array_creation_expression (ValaCodeVisitor* base, ValaArrayCreationExpression* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_ARRAY_CREATION_EXPRESSION (expr));
	/* FIXME: rank > 1 not supported yet */
	if (vala_array_creation_expression_get_rank (expr) > 1) {
		vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), "Creating arrays with rank greater than 1 is not supported yet");
	}
	GList* sizes = vala_array_creation_expression_get_sizes (expr);
	ValaCCodeFunctionCall* gnew = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_constant_new ("g_new0")));
	ValaCCodeIdentifier* __temp652 = NULL;
	vala_ccode_function_call_add_argument (gnew, VALA_CCODE_EXPRESSION ((__temp652 = vala_ccode_identifier_new (vala_data_type_get_cname (vala_type_reference_get_data_type (vala_array_creation_expression_get_element_type (expr)), FALSE)))));
	(__temp652 == NULL ? NULL : (__temp652 = (g_object_unref (__temp652), NULL)));
	/* FIXME: had to add Expression cast due to possible compiler bug */
	vala_ccode_function_call_add_argument (gnew, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE ((VALA_EXPRESSION (g_list_first (sizes)->data))))));
	if (vala_array_creation_expression_get_initializer_list (expr) != NULL) {
		ValaCCodeCommaExpression* ce = g_object_new (VALA_TYPE_CCODE_COMMA_EXPRESSION, NULL);
		ValaVariableDeclarator* temp_var = vala_code_generator_get_temp_variable_declarator (self, vala_expression_get_static_type (VALA_EXPRESSION (expr)), TRUE);
		ValaCCodeIdentifier* name_cnode = vala_ccode_identifier_new (vala_variable_declarator_get_name (temp_var));
		int i = 0;
		ValaVariableDeclarator* __temp653 = NULL;
		self->priv->temp_vars = g_list_prepend (self->priv->temp_vars, (__temp653 = temp_var, (__temp653 == NULL ? NULL : g_object_ref (__temp653))));
		/* FIXME: had to add Expression cast due to possible compiler bug */
		ValaCCodeAssignment* __temp654 = NULL;
		vala_ccode_comma_expression_append_expression (ce, VALA_CCODE_EXPRESSION ((__temp654 = vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (name_cnode), VALA_CCODE_EXPRESSION (gnew), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))));
		(__temp654 == NULL ? NULL : (__temp654 = (g_object_unref (__temp654), NULL)));
		{
			GList* __temp656 = NULL;
			__temp656 = vala_initializer_list_get_initializers (vala_array_creation_expression_get_initializer_list (expr));
			GList* e_it;
			for (e_it = __temp656; e_it != NULL; e_it = e_it->next) {
				ValaExpression* e = e_it->data;
				{
					ValaCCodeAssignment* __temp655 = NULL;
					vala_ccode_comma_expression_append_expression (ce, VALA_CCODE_EXPRESSION ((__temp655 = vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_element_access_new (VALA_CCODE_EXPRESSION (name_cnode), VALA_CCODE_EXPRESSION (vala_ccode_constant_new (g_strdup_printf ("%i", i))))), VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (e))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))));
					(__temp655 == NULL ? NULL : (__temp655 = (g_object_unref (__temp655), NULL)));
					i++;
				}
			}
			(__temp656 == NULL ? NULL : (__temp656 = (g_list_free (__temp656), NULL)));
		}
		vala_ccode_comma_expression_append_expression (ce, VALA_CCODE_EXPRESSION (name_cnode));
		vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE (ce));
		(ce == NULL ? NULL : (ce = (g_object_unref (ce), NULL)));
		(temp_var == NULL ? NULL : (temp_var = (g_object_unref (temp_var), NULL)));
		(name_cnode == NULL ? NULL : (name_cnode = (g_object_unref (name_cnode), NULL)));
	} else {
		vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE (gnew));
	}
	(sizes == NULL ? NULL : (sizes = (g_list_free (sizes), NULL)));
	(gnew == NULL ? NULL : (gnew = (g_object_unref (gnew), NULL)));
}


static void vala_code_generator_real_visit_boolean_literal (ValaCodeVisitor* base, ValaBooleanLiteral* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_BOOLEAN_LITERAL (expr));
	ValaCCodeConstant* __temp657 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp657 = vala_ccode_constant_new ((vala_boolean_literal_get_value (expr) ? "TRUE" : "FALSE")))));
	(__temp657 == NULL ? NULL : (__temp657 = (g_object_unref (__temp657), NULL)));
}


static void vala_code_generator_real_visit_character_literal (ValaCodeVisitor* base, ValaCharacterLiteral* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_CHARACTER_LITERAL (expr));
	if (vala_character_literal_get_char (expr) >= 0x20 && vala_character_literal_get_char (expr) < 0x80) {
		ValaCCodeConstant* __temp658 = NULL;
		vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp658 = vala_ccode_constant_new (vala_character_literal_get_value (expr)))));
		(__temp658 == NULL ? NULL : (__temp658 = (g_object_unref (__temp658), NULL)));
	} else {
		ValaCCodeConstant* __temp659 = NULL;
		vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp659 = vala_ccode_constant_new (g_strdup_printf ("%uU", vala_character_literal_get_char (expr))))));
		(__temp659 == NULL ? NULL : (__temp659 = (g_object_unref (__temp659), NULL)));
	}
}


static void vala_code_generator_real_visit_integer_literal (ValaCodeVisitor* base, ValaIntegerLiteral* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_INTEGER_LITERAL (expr));
	ValaCCodeConstant* __temp660 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp660 = vala_ccode_constant_new (vala_integer_literal_get_value (expr)))));
	(__temp660 == NULL ? NULL : (__temp660 = (g_object_unref (__temp660), NULL)));
}


static void vala_code_generator_real_visit_real_literal (ValaCodeVisitor* base, ValaRealLiteral* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_REAL_LITERAL (expr));
	ValaCCodeConstant* __temp661 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp661 = vala_ccode_constant_new (vala_real_literal_get_value (expr)))));
	(__temp661 == NULL ? NULL : (__temp661 = (g_object_unref (__temp661), NULL)));
}


static void vala_code_generator_real_visit_string_literal (ValaCodeVisitor* base, ValaStringLiteral* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_STRING_LITERAL (expr));
	ValaCCodeConstant* __temp662 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp662 = vala_ccode_constant_new (vala_string_literal_get_value (expr)))));
	(__temp662 == NULL ? NULL : (__temp662 = (g_object_unref (__temp662), NULL)));
}


static void vala_code_generator_real_visit_null_literal (ValaCodeVisitor* base, ValaNullLiteral* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_NULL_LITERAL (expr));
	ValaCCodeConstant* __temp663 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp663 = vala_ccode_constant_new ("NULL"))));
	(__temp663 == NULL ? NULL : (__temp663 = (g_object_unref (__temp663), NULL)));
}


static void vala_code_generator_real_visit_literal_expression (ValaCodeVisitor* base, ValaLiteralExpression* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_LITERAL_EXPRESSION (expr));
	vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_literal_expression_get_literal (expr))));
	vala_code_generator_visit_expression (self, VALA_EXPRESSION (expr));
}


static void vala_code_generator_process_cmember (ValaCodeGenerator* self, ValaMemberAccess* expr, ValaCCodeExpression* pub_inst, ValaDataType* base_type)
{
	g_return_if_fail (VALA_IS_CODE_GENERATOR (self));
	g_return_if_fail (VALA_IS_MEMBER_ACCESS (expr));
	g_return_if_fail (pub_inst == NULL || VALA_IS_CCODE_EXPRESSION (pub_inst));
	g_return_if_fail (base_type == NULL || VALA_IS_DATA_TYPE (base_type));
	if (VALA_IS_METHOD (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))))) {
		ValaMethod* m = VALA_METHOD (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))));
		if (VALA_IS_BASE_ACCESS (vala_member_access_get_inner (expr)) && (vala_method_get_is_virtual (m) || vala_method_get_overrides (m))) {
			ValaClass* base_class = VALA_CLASS (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (vala_method_get_base_method (m))))));
			char* __temp664 = NULL;
			ValaCCodeFunctionCall* __temp665 = NULL;
			ValaCCodeFunctionCall* vcast = (__temp665 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (g_strdup_printf ("%s_CLASS", (__temp664 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (base_class), NULL)))))), (__temp664 == NULL ? NULL : (__temp664 = (g_free (__temp664), NULL))), __temp665);
			ValaCCodeIdentifier* __temp667 = NULL;
			char* __temp666 = NULL;
			vala_ccode_function_call_add_argument (vcast, VALA_CCODE_EXPRESSION ((__temp667 = vala_ccode_identifier_new (g_strdup_printf ("%s_parent_class", (__temp666 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (self->priv->current_class), NULL)))))));
			(__temp667 == NULL ? NULL : (__temp667 = (g_object_unref (__temp667), NULL)));
			(__temp666 == NULL ? NULL : (__temp666 = (g_free (__temp666), NULL)));
			ValaCCodeMemberAccess* __temp668 = NULL;
			vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp668 = vala_ccode_member_access_new_pointer (VALA_CCODE_EXPRESSION (vcast), vala_method_get_name (m)))));
			(__temp668 == NULL ? NULL : (__temp668 = (g_object_unref (__temp668), NULL)));
			(vcast == NULL ? NULL : (vcast = (g_object_unref (vcast), NULL)));
			return;
			(vcast == NULL ? NULL : (vcast = (g_object_unref (vcast), NULL)));
		}
		if (!vala_method_get_overrides (m)) {
			ValaCCodeIdentifier* __temp669 = NULL;
			vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp669 = vala_ccode_identifier_new (vala_method_get_cname (m)))));
			(__temp669 == NULL ? NULL : (__temp669 = (g_object_unref (__temp669), NULL)));
		} else {
			ValaCCodeIdentifier* __temp670 = NULL;
			vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp670 = vala_ccode_identifier_new (vala_method_get_cname (vala_method_get_base_method (m))))));
			(__temp670 == NULL ? NULL : (__temp670 = (g_object_unref (__temp670), NULL)));
		}
	} else {
		if (VALA_IS_FIELD (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))))) {
			ValaField* f = VALA_FIELD (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))));
			if (vala_field_get_instance (f)) {
				ValaCCodeExpression* typed_inst = NULL;
				if (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (f)))) != VALA_CODE_NODE (base_type)) {
					/* FIXME: use C cast if debugging disabled*/
					ValaCCodeExpression* __temp671 = NULL;
					typed_inst = (__temp671 = VALA_CCODE_EXPRESSION (vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_data_type_get_upper_case_cname ((VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (f)))))), NULL))))), (typed_inst == NULL ? NULL : (typed_inst = (g_object_unref (typed_inst), NULL))), __temp671);
					vala_ccode_function_call_add_argument ((VALA_CCODE_FUNCTION_CALL (typed_inst)), pub_inst);
				} else {
					ValaCCodeExpression* __temp673 = NULL;
					ValaCCodeExpression* __temp672 = NULL;
					typed_inst = (__temp673 = (__temp672 = pub_inst, (__temp672 == NULL ? NULL : g_object_ref (__temp672))), (typed_inst == NULL ? NULL : (typed_inst = (g_object_unref (typed_inst), NULL))), __temp673);
				}
				ValaCCodeExpression* inst = NULL;
				if (f->access == VALA_MEMBER_ACCESSIBILITY_PRIVATE) {
					ValaCCodeExpression* __temp674 = NULL;
					inst = (__temp674 = VALA_CCODE_EXPRESSION (vala_ccode_member_access_new_pointer (typed_inst, "priv")), (inst == NULL ? NULL : (inst = (g_object_unref (inst), NULL))), __temp674);
				} else {
					ValaCCodeExpression* __temp676 = NULL;
					ValaCCodeExpression* __temp675 = NULL;
					inst = (__temp676 = (__temp675 = typed_inst, (__temp675 == NULL ? NULL : g_object_ref (__temp675))), (inst == NULL ? NULL : (inst = (g_object_unref (inst), NULL))), __temp676);
				}
				if (vala_data_type_is_reference_type ((VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (f)))))))) {
					ValaCCodeMemberAccess* __temp677 = NULL;
					vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp677 = vala_ccode_member_access_new_pointer (inst, vala_field_get_cname (f)))));
					(__temp677 == NULL ? NULL : (__temp677 = (g_object_unref (__temp677), NULL)));
				} else {
					ValaCCodeMemberAccess* __temp678 = NULL;
					vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp678 = vala_ccode_member_access_new (inst, vala_field_get_cname (f), FALSE))));
					(__temp678 == NULL ? NULL : (__temp678 = (g_object_unref (__temp678), NULL)));
				}
				(typed_inst == NULL ? NULL : (typed_inst = (g_object_unref (typed_inst), NULL)));
				(inst == NULL ? NULL : (inst = (g_object_unref (inst), NULL)));
			} else {
				ValaCCodeIdentifier* __temp679 = NULL;
				vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp679 = vala_ccode_identifier_new (vala_field_get_cname (f)))));
				(__temp679 == NULL ? NULL : (__temp679 = (g_object_unref (__temp679), NULL)));
			}
		} else {
			if (VALA_IS_CONSTANT (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))))) {
				ValaConstant* c = VALA_CONSTANT (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))));
				ValaCCodeIdentifier* __temp680 = NULL;
				vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp680 = vala_ccode_identifier_new (vala_constant_get_cname (c)))));
				(__temp680 == NULL ? NULL : (__temp680 = (g_object_unref (__temp680), NULL)));
			} else {
				if (VALA_IS_PROPERTY (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))))) {
					ValaProperty* prop = VALA_PROPERTY (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))));
					ValaClass* cl = VALA_CLASS (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (prop)))));
					if (!vala_property_get_no_accessor_method (prop)) {
						char* __temp681 = NULL;
						ValaCCodeFunctionCall* __temp682 = NULL;
						ValaCCodeFunctionCall* ccall = (__temp682 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (g_strdup_printf ("%s_get_%s", (__temp681 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL)), vala_property_get_name (prop))))), (__temp681 == NULL ? NULL : (__temp681 = (g_free (__temp681), NULL))), __temp682);
						/* explicitly use strong reference as ccast
						 * gets unrefed at the end of the inner block
						 */
						ValaCCodeExpression* __temp683 = NULL;
						ValaCCodeExpression* typed_pub_inst = (__temp683 = pub_inst, (__temp683 == NULL ? NULL : g_object_ref (__temp683)));
						/* cast if necessary */
						if (VALA_DATA_TYPE (cl) != base_type) {
							/* FIXME: use C cast if debugging disabled*/
							ValaCCodeFunctionCall* ccast = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), NULL))));
							vala_ccode_function_call_add_argument (ccast, pub_inst);
							ValaCCodeExpression* __temp685 = NULL;
							ValaCCodeFunctionCall* __temp684 = NULL;
							typed_pub_inst = (__temp685 = VALA_CCODE_EXPRESSION ((__temp684 = ccast, (__temp684 == NULL ? NULL : g_object_ref (__temp684)))), (typed_pub_inst == NULL ? NULL : (typed_pub_inst = (g_object_unref (typed_pub_inst), NULL))), __temp685);
							(ccast == NULL ? NULL : (ccast = (g_object_unref (ccast), NULL)));
						}
						vala_ccode_function_call_add_argument (ccall, typed_pub_inst);
						vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE (ccall));
						(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
						(typed_pub_inst == NULL ? NULL : (typed_pub_inst = (g_object_unref (typed_pub_inst), NULL)));
					} else {
						ValaCCodeFunctionCall* ccall = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("g_object_get")));
						ValaCCodeFunctionCall* ccast = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("G_OBJECT")));
						vala_ccode_function_call_add_argument (ccast, pub_inst);
						vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION (ccast));
						/* property name is second argument of g_object_get*/
						ValaCCodeConstant* __temp686 = NULL;
						vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp686 = vala_property_get_canonical_cconstant (prop))));
						(__temp686 == NULL ? NULL : (__temp686 = (g_object_unref (__temp686), NULL)));
						/* we need a temporary variable to save the property value*/
						ValaVariableDeclarator* temp_decl = vala_code_generator_get_temp_variable_declarator (self, vala_expression_get_static_type (VALA_EXPRESSION (expr)), TRUE);
						ValaVariableDeclarator* __temp687 = NULL;
						self->priv->temp_vars = g_list_prepend (self->priv->temp_vars, (__temp687 = temp_decl, (__temp687 == NULL ? NULL : g_object_ref (__temp687))));
						ValaCCodeIdentifier* ctemp = vala_ccode_identifier_new (vala_variable_declarator_get_name (temp_decl));
						ValaCCodeUnaryExpression* __temp688 = NULL;
						vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp688 = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, VALA_CCODE_EXPRESSION (ctemp)))));
						(__temp688 == NULL ? NULL : (__temp688 = (g_object_unref (__temp688), NULL)));
						ValaCCodeConstant* __temp689 = NULL;
						vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp689 = vala_ccode_constant_new ("NULL"))));
						(__temp689 == NULL ? NULL : (__temp689 = (g_object_unref (__temp689), NULL)));
						ValaCCodeCommaExpression* ccomma = g_object_new (VALA_TYPE_CCODE_COMMA_EXPRESSION, NULL);
						vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION (ccall));
						vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION (ctemp));
						vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE (ccomma));
						(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
						(ccast == NULL ? NULL : (ccast = (g_object_unref (ccast), NULL)));
						(temp_decl == NULL ? NULL : (temp_decl = (g_object_unref (temp_decl), NULL)));
						(ctemp == NULL ? NULL : (ctemp = (g_object_unref (ctemp), NULL)));
						(ccomma == NULL ? NULL : (ccomma = (g_object_unref (ccomma), NULL)));
					}
				} else {
					if (VALA_IS_ENUM_VALUE (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))))) {
						ValaEnumValue* ev = VALA_ENUM_VALUE (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))));
						ValaCCodeConstant* __temp690 = NULL;
						vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp690 = vala_ccode_constant_new (vala_enum_value_get_cname (ev)))));
						(__temp690 == NULL ? NULL : (__temp690 = (g_object_unref (__temp690), NULL)));
					} else {
						if (VALA_IS_VARIABLE_DECLARATOR (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))))) {
							ValaVariableDeclarator* decl = VALA_VARIABLE_DECLARATOR (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))));
							ValaCCodeIdentifier* __temp691 = NULL;
							vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp691 = vala_ccode_identifier_new (vala_variable_declarator_get_name (decl)))));
							(__temp691 == NULL ? NULL : (__temp691 = (g_object_unref (__temp691), NULL)));
						} else {
							if (VALA_IS_FORMAL_PARAMETER (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))))) {
								ValaFormalParameter* p = VALA_FORMAL_PARAMETER (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))));
								if (g_utf8_collate (vala_formal_parameter_get_name (p), "this") == 0) {
									vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE (pub_inst));
								} else {
									if (vala_type_reference_get_is_out (vala_formal_parameter_get_type_reference (p)) || vala_type_reference_get_reference_to_value_type (vala_formal_parameter_get_type_reference (p))) {
										ValaCCodeIdentifier* __temp692 = NULL;
										vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp692 = vala_ccode_identifier_new (g_strdup_printf ("*%s", vala_formal_parameter_get_name (p))))));
										(__temp692 == NULL ? NULL : (__temp692 = (g_object_unref (__temp692), NULL)));
									} else {
										ValaCCodeIdentifier* __temp693 = NULL;
										vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp693 = vala_ccode_identifier_new (vala_formal_parameter_get_name (p)))));
										(__temp693 == NULL ? NULL : (__temp693 = (g_object_unref (__temp693), NULL)));
									}
								}
							} else {
								if (VALA_IS_SIGNAL (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))))) {
									ValaSignal* sig = VALA_SIGNAL (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))));
									ValaDataType* cl = VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (sig)))));
									if (vala_signal_get_has_emitter (sig)) {
										char* __temp694 = NULL;
										ValaCCodeFunctionCall* __temp695 = NULL;
										ValaCCodeFunctionCall* ccall = (__temp695 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (g_strdup_printf ("%s_%s", (__temp694 = vala_data_type_get_lower_case_cname (cl, NULL)), vala_signal_get_name (sig))))), (__temp694 == NULL ? NULL : (__temp694 = (g_free (__temp694), NULL))), __temp695);
										/* explicitly use strong reference as ccast
										 * gets unrefed at the end of the inner block
										 */
										ValaCCodeExpression* __temp696 = NULL;
										ValaCCodeExpression* typed_pub_inst = (__temp696 = pub_inst, (__temp696 == NULL ? NULL : g_object_ref (__temp696)));
										/* cast if necessary */
										if (cl != base_type) {
											/* FIXME: use C cast if debugging disabled*/
											ValaCCodeFunctionCall* ccast = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_data_type_get_upper_case_cname (cl, NULL))));
											vala_ccode_function_call_add_argument (ccast, pub_inst);
											ValaCCodeExpression* __temp698 = NULL;
											ValaCCodeFunctionCall* __temp697 = NULL;
											typed_pub_inst = (__temp698 = VALA_CCODE_EXPRESSION ((__temp697 = ccast, (__temp697 == NULL ? NULL : g_object_ref (__temp697)))), (typed_pub_inst == NULL ? NULL : (typed_pub_inst = (g_object_unref (typed_pub_inst), NULL))), __temp698);
											(ccast == NULL ? NULL : (ccast = (g_object_unref (ccast), NULL)));
										}
										vala_ccode_function_call_add_argument (ccall, typed_pub_inst);
										vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE (ccall));
										(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
										(typed_pub_inst == NULL ? NULL : (typed_pub_inst = (g_object_unref (typed_pub_inst), NULL)));
									} else {
										ValaCCodeFunctionCall* ccall = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("g_signal_emit_by_name")));
										/* FIXME: use C cast if debugging disabled*/
										ValaCCodeFunctionCall* ccast = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("G_OBJECT")));
										vala_ccode_function_call_add_argument (ccast, pub_inst);
										vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION (ccast));
										ValaCCodeConstant* __temp699 = NULL;
										vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp699 = vala_signal_get_canonical_cconstant (sig))));
										(__temp699 == NULL ? NULL : (__temp699 = (g_object_unref (__temp699), NULL)));
										vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE (ccall));
										(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
										(ccast == NULL ? NULL : (ccast = (g_object_unref (ccast), NULL)));
									}
								}
							}
						}
					}
				}
			}
		}
	}
}


static void vala_code_generator_real_visit_parenthesized_expression (ValaCodeVisitor* base, ValaParenthesizedExpression* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_PARENTHESIZED_EXPRESSION (expr));
	ValaCCodeParenthesizedExpression* __temp700 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp700 = vala_ccode_parenthesized_expression_new (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_parenthesized_expression_get_inner (expr))))))));
	(__temp700 == NULL ? NULL : (__temp700 = (g_object_unref (__temp700), NULL)));
	vala_code_generator_visit_expression (self, VALA_EXPRESSION (expr));
}


static void vala_code_generator_real_visit_member_access (ValaCodeVisitor* base, ValaMemberAccess* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_MEMBER_ACCESS (expr));
	ValaCCodeExpression* pub_inst = NULL;
	ValaDataType* base_type = NULL;
	if (vala_member_access_get_inner (expr) == NULL) {
		ValaCCodeExpression* __temp701 = NULL;
		pub_inst = (__temp701 = VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("self")), (pub_inst == NULL ? NULL : (pub_inst = (g_object_unref (pub_inst), NULL))), __temp701);
		if (self->priv->current_type_symbol != NULL) {
			/* base type is available if this is a type method */
			base_type = VALA_DATA_TYPE (vala_symbol_get_node (self->priv->current_type_symbol));
		}
	} else {
		ValaCCodeExpression* __temp703 = NULL;
		ValaCCodeExpression* __temp702 = NULL;
		pub_inst = (__temp703 = (__temp702 = VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_member_access_get_inner (expr)))), (__temp702 == NULL ? NULL : g_object_ref (__temp702))), (pub_inst == NULL ? NULL : (pub_inst = (g_object_unref (pub_inst), NULL))), __temp703);
		if (vala_expression_get_static_type (vala_member_access_get_inner (expr)) != NULL) {
			base_type = vala_type_reference_get_data_type (vala_expression_get_static_type (vala_member_access_get_inner (expr)));
		}
	}
	vala_code_generator_process_cmember (self, expr, pub_inst, base_type);
	vala_code_generator_visit_expression (self, VALA_EXPRESSION (expr));
	(pub_inst == NULL ? NULL : (pub_inst = (g_object_unref (pub_inst), NULL)));
}


static ValaCCodeExpression* vala_code_generator_get_array_length_cexpression (ValaCodeGenerator* self, ValaExpression* array_expr, int dim)
{
	g_return_val_if_fail (VALA_IS_CODE_GENERATOR (self), NULL);
	g_return_val_if_fail (VALA_IS_EXPRESSION (array_expr), NULL);
	gboolean is_out = FALSE;
	if (VALA_IS_UNARY_EXPRESSION (array_expr)) {
		ValaUnaryExpression* unary_expr = VALA_UNARY_EXPRESSION (array_expr);
		if (vala_unary_expression_get_operator (unary_expr) == VALA_UNARY_OPERATOR_OUT) {
			array_expr = vala_unary_expression_get_inner (unary_expr);
			is_out = TRUE;
		}
	}
	if (VALA_IS_ARRAY_CREATION_EXPRESSION (array_expr)) {
		GList* size = vala_array_creation_expression_get_sizes ((VALA_ARRAY_CREATION_EXPRESSION (array_expr)));
		ValaExpression* length_expr = VALA_EXPRESSION (g_list_nth_data (size, dim - 1));
		ValaCCodeExpression* __temp704 = NULL;
		ValaCCodeExpression* __temp705 = NULL;
		return (__temp705 = (__temp704 = VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (length_expr))), (__temp704 == NULL ? NULL : g_object_ref (__temp704))), (size == NULL ? NULL : (size = (g_list_free (size), NULL))), __temp705);
		(size == NULL ? NULL : (size = (g_list_free (size), NULL)));
	} else {
		if (vala_expression_get_symbol_reference (array_expr) != NULL) {
			if (VALA_IS_FORMAL_PARAMETER (vala_symbol_get_node (vala_expression_get_symbol_reference (array_expr)))) {
				ValaFormalParameter* param = VALA_FORMAL_PARAMETER (vala_symbol_get_node (vala_expression_get_symbol_reference (array_expr)));
				if (!vala_formal_parameter_get_no_array_length (param)) {
					ValaCCodeIdentifier* length_expr = vala_ccode_identifier_new (vala_code_generator_get_array_length_cname (self, vala_formal_parameter_get_name (param), dim));
					if (is_out) {
						ValaCCodeUnaryExpression* __temp706 = NULL;
						return VALA_CCODE_EXPRESSION ((__temp706 = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, VALA_CCODE_EXPRESSION (length_expr)), (length_expr == NULL ? NULL : (length_expr = (g_object_unref (length_expr), NULL))), __temp706));
					} else {
						return VALA_CCODE_EXPRESSION (length_expr);
					}
					(length_expr == NULL ? NULL : (length_expr = (g_object_unref (length_expr), NULL)));
				}
			} else {
				if (VALA_IS_VARIABLE_DECLARATOR (vala_symbol_get_node (vala_expression_get_symbol_reference (array_expr)))) {
					ValaVariableDeclarator* decl = VALA_VARIABLE_DECLARATOR (vala_symbol_get_node (vala_expression_get_symbol_reference (array_expr)));
					ValaCCodeIdentifier* length_expr = vala_ccode_identifier_new (vala_code_generator_get_array_length_cname (self, vala_variable_declarator_get_name (decl), dim));
					if (is_out) {
						ValaCCodeUnaryExpression* __temp708 = NULL;
						return VALA_CCODE_EXPRESSION ((__temp708 = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, VALA_CCODE_EXPRESSION (length_expr)), (length_expr == NULL ? NULL : (length_expr = (g_object_unref (length_expr), NULL))), __temp708));
					} else {
						return VALA_CCODE_EXPRESSION (length_expr);
					}
					(length_expr == NULL ? NULL : (length_expr = (g_object_unref (length_expr), NULL)));
				} else {
					if (VALA_IS_FIELD (vala_symbol_get_node (vala_expression_get_symbol_reference (array_expr)))) {
						ValaField* field = VALA_FIELD (vala_symbol_get_node (vala_expression_get_symbol_reference (array_expr)));
						if (!vala_field_get_no_array_length (field)) {
							char* length_cname = vala_code_generator_get_array_length_cname (self, vala_field_get_name (field), dim);
							ValaMemberAccess* ma = VALA_MEMBER_ACCESS (array_expr);
							ValaCCodeExpression* pub_inst = NULL;
							ValaDataType* base_type = NULL;
							ValaCCodeExpression* length_expr = NULL;
							if (vala_member_access_get_inner (ma) == NULL) {
								ValaCCodeExpression* __temp710 = NULL;
								pub_inst = (__temp710 = VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("self")), (pub_inst == NULL ? NULL : (pub_inst = (g_object_unref (pub_inst), NULL))), __temp710);
								if (self->priv->current_type_symbol != NULL) {
									/* base type is available if this is a type method */
									base_type = VALA_DATA_TYPE (vala_symbol_get_node (self->priv->current_type_symbol));
								}
							} else {
								ValaCCodeExpression* __temp712 = NULL;
								ValaCCodeExpression* __temp711 = NULL;
								pub_inst = (__temp712 = (__temp711 = VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_member_access_get_inner (ma)))), (__temp711 == NULL ? NULL : g_object_ref (__temp711))), (pub_inst == NULL ? NULL : (pub_inst = (g_object_unref (pub_inst), NULL))), __temp712);
								if (vala_expression_get_static_type (vala_member_access_get_inner (ma)) != NULL) {
									base_type = vala_type_reference_get_data_type (vala_expression_get_static_type (vala_member_access_get_inner (ma)));
								}
							}
							if (vala_field_get_instance (field)) {
								ValaCCodeExpression* typed_inst = NULL;
								if (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (field)))) != VALA_CODE_NODE (base_type)) {
									/* FIXME: use C cast if debugging disabled*/
									ValaCCodeExpression* __temp713 = NULL;
									typed_inst = (__temp713 = VALA_CCODE_EXPRESSION (vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_data_type_get_upper_case_cname ((VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (field)))))), NULL))))), (typed_inst == NULL ? NULL : (typed_inst = (g_object_unref (typed_inst), NULL))), __temp713);
									vala_ccode_function_call_add_argument ((VALA_CCODE_FUNCTION_CALL (typed_inst)), pub_inst);
								} else {
									ValaCCodeExpression* __temp715 = NULL;
									ValaCCodeExpression* __temp714 = NULL;
									typed_inst = (__temp715 = (__temp714 = pub_inst, (__temp714 == NULL ? NULL : g_object_ref (__temp714))), (typed_inst == NULL ? NULL : (typed_inst = (g_object_unref (typed_inst), NULL))), __temp715);
								}
								ValaCCodeExpression* inst = NULL;
								if (field->access == VALA_MEMBER_ACCESSIBILITY_PRIVATE) {
									ValaCCodeExpression* __temp716 = NULL;
									inst = (__temp716 = VALA_CCODE_EXPRESSION (vala_ccode_member_access_new_pointer (typed_inst, "priv")), (inst == NULL ? NULL : (inst = (g_object_unref (inst), NULL))), __temp716);
								} else {
									ValaCCodeExpression* __temp718 = NULL;
									ValaCCodeExpression* __temp717 = NULL;
									inst = (__temp718 = (__temp717 = typed_inst, (__temp717 == NULL ? NULL : g_object_ref (__temp717))), (inst == NULL ? NULL : (inst = (g_object_unref (inst), NULL))), __temp718);
								}
								if (vala_data_type_is_reference_type ((VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (field)))))))) {
									ValaCCodeExpression* __temp719 = NULL;
									length_expr = (__temp719 = VALA_CCODE_EXPRESSION (vala_ccode_member_access_new_pointer (inst, length_cname)), (length_expr == NULL ? NULL : (length_expr = (g_object_unref (length_expr), NULL))), __temp719);
								} else {
									ValaCCodeExpression* __temp720 = NULL;
									length_expr = (__temp720 = VALA_CCODE_EXPRESSION (vala_ccode_member_access_new (inst, length_cname, FALSE)), (length_expr == NULL ? NULL : (length_expr = (g_object_unref (length_expr), NULL))), __temp720);
								}
								(typed_inst == NULL ? NULL : (typed_inst = (g_object_unref (typed_inst), NULL)));
								(inst == NULL ? NULL : (inst = (g_object_unref (inst), NULL)));
							} else {
								ValaCCodeExpression* __temp721 = NULL;
								length_expr = (__temp721 = VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (length_cname)), (length_expr == NULL ? NULL : (length_expr = (g_object_unref (length_expr), NULL))), __temp721);
							}
							if (is_out) {
								ValaCCodeUnaryExpression* __temp722 = NULL;
								return VALA_CCODE_EXPRESSION ((__temp722 = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, length_expr), (length_cname == NULL ? NULL : (length_cname = (g_free (length_cname), NULL))), (pub_inst == NULL ? NULL : (pub_inst = (g_object_unref (pub_inst), NULL))), (length_expr == NULL ? NULL : (length_expr = (g_object_unref (length_expr), NULL))), __temp722));
							} else {
								ValaCCodeExpression* __temp723 = NULL;
								return (__temp723 = length_expr, (length_cname == NULL ? NULL : (length_cname = (g_free (length_cname), NULL))), (pub_inst == NULL ? NULL : (pub_inst = (g_object_unref (pub_inst), NULL))), __temp723);
							}
							(length_cname == NULL ? NULL : (length_cname = (g_free (length_cname), NULL)));
							(pub_inst == NULL ? NULL : (pub_inst = (g_object_unref (pub_inst), NULL)));
							(length_expr == NULL ? NULL : (length_expr = (g_object_unref (length_expr), NULL)));
						}
					}
				}
			}
		}
	}
	if (!is_out) {
		return VALA_CCODE_EXPRESSION (vala_ccode_constant_new ("-1"));
	} else {
		return VALA_CCODE_EXPRESSION (vala_ccode_constant_new ("NULL"));
	}
}


static void vala_code_generator_real_visit_end_invocation_expression (ValaCodeVisitor* base, ValaInvocationExpression* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_INVOCATION_EXPRESSION (expr));
	ValaCCodeFunctionCall* ccall = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_invocation_expression_get_call (expr)))));
	ValaMethod* m = NULL;
	GList* params = NULL;
	if (!(VALA_IS_MEMBER_ACCESS (vala_invocation_expression_get_call (expr)))) {
		vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), "unsupported method invocation");
		(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
		(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
		return;
	}
	ValaMemberAccess* ma = VALA_MEMBER_ACCESS (vala_invocation_expression_get_call (expr));
	if (VALA_IS_INVOKABLE (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_invocation_expression_get_call (expr))))) {
		ValaInvokable* i = VALA_INVOKABLE (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_invocation_expression_get_call (expr))));
		GList* __temp726 = NULL;
		params = (__temp726 = vala_invokable_get_parameters (i), (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp726);
		if (VALA_IS_METHOD (i)) {
			m = VALA_METHOD (i);
		} else {
			if (VALA_IS_SIGNAL (i)) {
				ValaCCodeFunctionCall* __temp728 = NULL;
				ValaCCodeFunctionCall* __temp727 = NULL;
				ccall = (__temp728 = (__temp727 = VALA_CCODE_FUNCTION_CALL (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_invocation_expression_get_call (expr)))), (__temp727 == NULL ? NULL : g_object_ref (__temp727))), (ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL))), __temp728);
			}
		}
	}
	/* explicitly use strong reference as ccall gets unrefed
	 * at end of inner block
	 */
	ValaCCodeExpression* instance = NULL;
	if (m != NULL && vala_method_get_instance (m)) {
		ValaMethod* base_method = m;
		if (vala_method_get_overrides (m)) {
			base_method = vala_method_get_base_method (m);
		}
		gboolean req_cast = FALSE;
		if (vala_member_access_get_inner (ma) == NULL) {
			ValaCCodeExpression* __temp729 = NULL;
			instance = (__temp729 = VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("self")), (instance == NULL ? NULL : (instance = (g_object_unref (instance), NULL))), __temp729);
			/* require casts for overriden and inherited methods */
			req_cast = vala_method_get_overrides (m) || (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (m))) != self->priv->current_type_symbol);
		} else {
			ValaCCodeExpression* __temp731 = NULL;
			ValaCCodeExpression* __temp730 = NULL;
			instance = (__temp731 = (__temp730 = VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_member_access_get_inner (ma)))), (__temp730 == NULL ? NULL : g_object_ref (__temp730))), (instance == NULL ? NULL : (instance = (g_object_unref (instance), NULL))), __temp731);
			/* reqiure casts if the type of the used instance is
			 * different than the type which declared the method */
			req_cast = vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (base_method)))) != VALA_CODE_NODE (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_member_access_get_inner (ma))));
		}
		if (vala_method_get_instance_by_reference (m)) {
			ValaCCodeExpression* __temp732 = NULL;
			instance = (__temp732 = VALA_CCODE_EXPRESSION (vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, instance)), (instance == NULL ? NULL : (instance = (g_object_unref (instance), NULL))), __temp732);
		}
		if (req_cast && vala_data_type_is_reference_type ((VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (m)))))))) {
			/* FIXME: use C cast if debugging disabled*/
			ValaCCodeFunctionCall* ccall = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_data_type_get_upper_case_cname ((VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (base_method)))))), NULL))));
			vala_ccode_function_call_add_argument (ccall, instance);
			ValaCCodeExpression* __temp734 = NULL;
			ValaCCodeFunctionCall* __temp733 = NULL;
			instance = (__temp734 = VALA_CCODE_EXPRESSION ((__temp733 = ccall, (__temp733 == NULL ? NULL : g_object_ref (__temp733)))), (instance == NULL ? NULL : (instance = (g_object_unref (instance), NULL))), __temp734);
			(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
		}
		if (!vala_method_get_instance_last (m)) {
			vala_ccode_function_call_add_argument (ccall, instance);
		}
	}
	gboolean ellipsis = FALSE;
	int i = 1;
	{
		GList* __temp744 = NULL;
		__temp744 = vala_invocation_expression_get_argument_list (expr);
		GList* arg_it;
		for (arg_it = __temp744; arg_it != NULL; arg_it = arg_it->next) {
			ValaExpression* arg = arg_it->data;
			{
				/* explicitly use strong reference as ccall gets
				 * unrefed at end of inner block
				 */
				ValaCCodeExpression* __temp735 = NULL;
				ValaCCodeExpression* cexpr = (__temp735 = VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (arg))), (__temp735 == NULL ? NULL : g_object_ref (__temp735)));
				if (params != NULL) {
					ValaFormalParameter* param = VALA_FORMAL_PARAMETER (params->data);
					ellipsis = vala_formal_parameter_get_ellipsis (param);
					if (!ellipsis) {
						if (vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (param)) != NULL && vala_data_type_is_reference_type (vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (param))) && vala_type_reference_get_data_type (vala_expression_get_static_type (arg)) != NULL) {
							if (!vala_formal_parameter_get_no_array_length (param) && VALA_IS_ARRAY (vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (param)))) {
								ValaArray* arr = VALA_ARRAY (vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (param)));
								{
									int dim;
									for (dim = 1; dim <= vala_array_get_rank (arr); dim++) {
										ValaCCodeExpression* __temp736 = NULL;
										vala_ccode_function_call_add_argument (ccall, (__temp736 = vala_code_generator_get_array_length_cexpression (self, arg, dim)));
										(__temp736 == NULL ? NULL : (__temp736 = (g_object_unref (__temp736), NULL)));
									}
								}
							}
							if (vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (param)) != vala_type_reference_get_data_type (vala_expression_get_static_type (arg))) {
								/* FIXME: use C cast if debugging disabled*/
								ValaCCodeFunctionCall* ccall = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_data_type_get_upper_case_cname (vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (param)), NULL))));
								vala_ccode_function_call_add_argument (ccall, cexpr);
								ValaCCodeExpression* __temp738 = NULL;
								ValaCCodeFunctionCall* __temp737 = NULL;
								cexpr = (__temp738 = VALA_CCODE_EXPRESSION ((__temp737 = ccall, (__temp737 == NULL ? NULL : g_object_ref (__temp737)))), (cexpr == NULL ? NULL : (cexpr = (g_object_unref (cexpr), NULL))), __temp738);
								(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
							}
						} else {
							if (VALA_IS_CALLBACK (vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (param)))) {
								ValaCCodeExpression* __temp739 = NULL;
								cexpr = (__temp739 = VALA_CCODE_EXPRESSION (vala_ccode_cast_expression_new (cexpr, vala_data_type_get_cname (vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (param)), FALSE))), (cexpr == NULL ? NULL : (cexpr = (g_object_unref (cexpr), NULL))), __temp739);
							} else {
								if (vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (param)) == NULL && VALA_IS_STRUCT (vala_type_reference_get_data_type (vala_expression_get_static_type (arg)))) {
									/* convert integer to pointer if this is a generic method parameter */
									ValaStruct* st = VALA_STRUCT (vala_type_reference_get_data_type (vala_expression_get_static_type (arg)));
									if (VALA_DATA_TYPE (st) == vala_type_reference_get_data_type (self->priv->bool_type) || vala_struct_is_integer_type (st)) {
										ValaCCodeFunctionCall* cconv = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("GINT_TO_POINTER")));
										vala_ccode_function_call_add_argument (cconv, cexpr);
										ValaCCodeExpression* __temp741 = NULL;
										ValaCCodeFunctionCall* __temp740 = NULL;
										cexpr = (__temp741 = VALA_CCODE_EXPRESSION ((__temp740 = cconv, (__temp740 == NULL ? NULL : g_object_ref (__temp740)))), (cexpr == NULL ? NULL : (cexpr = (g_object_unref (cexpr), NULL))), __temp741);
										(cconv == NULL ? NULL : (cconv = (g_object_unref (cconv), NULL)));
									}
								}
							}
						}
					}
				}
				vala_ccode_function_call_add_argument (ccall, cexpr);
				i++;
				if (params != NULL) {
					GList* __temp743 = NULL;
					GList* __temp742 = NULL;
					params = (__temp743 = (__temp742 = params->next, (__temp742 == NULL ? NULL : g_list_copy (__temp742))), (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp743);
				}
				(cexpr == NULL ? NULL : (cexpr = (g_object_unref (cexpr), NULL)));
			}
		}
		(__temp744 == NULL ? NULL : (__temp744 = (g_list_free (__temp744), NULL)));
	}
	while (params != NULL) {
		ValaFormalParameter* param = VALA_FORMAL_PARAMETER (params->data);
		if (vala_formal_parameter_get_ellipsis (param)) {
			ellipsis = TRUE;
			break;
		}
		if (vala_formal_parameter_get_default_expression (param) == NULL) {
			char* __temp745 = NULL;
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp745 = g_strdup_printf ("no default expression for argument %d", i)));
			(__temp745 == NULL ? NULL : (__temp745 = (g_free (__temp745), NULL)));
			(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
			(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
			(instance == NULL ? NULL : (instance = (g_object_unref (instance), NULL)));
			return;
		}
		/* evaluate default expression here as the code
		 * generator might not have visited the formal
		 * parameter yet */
		vala_code_node_accept (VALA_CODE_NODE (vala_formal_parameter_get_default_expression (param)), VALA_CODE_VISITOR (self));
		if (!vala_formal_parameter_get_no_array_length (param) && vala_formal_parameter_get_type_reference (param) != NULL && VALA_IS_ARRAY (vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (param)))) {
			ValaArray* arr = VALA_ARRAY (vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (param)));
			{
				int dim;
				for (dim = 1; dim <= vala_array_get_rank (arr); dim++) {
					ValaCCodeExpression* __temp746 = NULL;
					vala_ccode_function_call_add_argument (ccall, (__temp746 = vala_code_generator_get_array_length_cexpression (self, vala_formal_parameter_get_default_expression (param), dim)));
					(__temp746 == NULL ? NULL : (__temp746 = (g_object_unref (__temp746), NULL)));
				}
			}
		}
		vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_formal_parameter_get_default_expression (param)))));
		i++;
		GList* __temp748 = NULL;
		GList* __temp747 = NULL;
		params = (__temp748 = (__temp747 = params->next, (__temp747 == NULL ? NULL : g_list_copy (__temp747))), (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp748);
	}
	if (m != NULL && vala_method_get_instance (m) && vala_method_get_instance_last (m)) {
		vala_ccode_function_call_add_argument (ccall, instance);
	} else {
		if (ellipsis) {
			/* ensure variable argument list ends with NULL
			 * except when using printf-style arguments */
			if (m == NULL || !vala_method_get_printf_format (m)) {
				ValaCCodeConstant* __temp749 = NULL;
				vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp749 = vala_ccode_constant_new ("NULL"))));
				(__temp749 == NULL ? NULL : (__temp749 = (g_object_unref (__temp749), NULL)));
			}
		}
	}
	if (m != NULL && vala_method_get_instance (m) && vala_method_get_returns_modified_pointer (m)) {
		ValaCCodeAssignment* __temp750 = NULL;
		vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp750 = vala_ccode_assignment_new (instance, VALA_CCODE_EXPRESSION (ccall), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))));
		(__temp750 == NULL ? NULL : (__temp750 = (g_object_unref (__temp750), NULL)));
	} else {
		vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE (ccall));
		vala_code_generator_visit_expression (self, VALA_EXPRESSION (expr));
	}
	(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
	(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
	(instance == NULL ? NULL : (instance = (g_object_unref (instance), NULL)));
}


static void vala_code_generator_real_visit_element_access (ValaCodeVisitor* base, ValaElementAccess* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_ELEMENT_ACCESS (expr));
	GList* indices = vala_element_access_get_indices (expr);
	int rank = g_list_length (indices);
	if (rank == 1) {
		/* FIXME: had to add Expression cast due to possible compiler bug */
		ValaCCodeElementAccess* __temp751 = NULL;
		vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp751 = vala_ccode_element_access_new (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_element_access_get_container (expr)))), VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE ((VALA_EXPRESSION (g_list_first (indices)->data)))))))));
		(__temp751 == NULL ? NULL : (__temp751 = (g_object_unref (__temp751), NULL)));
	} else {
		vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), "Arrays with more then one dimension are not supported yet");
		(indices == NULL ? NULL : (indices = (g_list_free (indices), NULL)));
		return;
	}
	vala_code_generator_visit_expression (self, VALA_EXPRESSION (expr));
	(indices == NULL ? NULL : (indices = (g_list_free (indices), NULL)));
}


static void vala_code_generator_real_visit_base_access (ValaCodeVisitor* base, ValaBaseAccess* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_BASE_ACCESS (expr));
	ValaInstanceCast* __temp752 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp752 = vala_instance_cast_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("self")), vala_type_reference_get_data_type (vala_expression_get_static_type (VALA_EXPRESSION (expr)))))));
	(__temp752 == NULL ? NULL : (__temp752 = (g_object_unref (__temp752), NULL)));
}


static void vala_code_generator_real_visit_postfix_expression (ValaCodeVisitor* base, ValaPostfixExpression* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_POSTFIX_EXPRESSION (expr));
	ValaCCodeUnaryOperator op = (vala_postfix_expression_get_increment (expr) ? VALA_CCODE_UNARY_OPERATOR_POSTFIX_INCREMENT : VALA_CCODE_UNARY_OPERATOR_POSTFIX_DECREMENT);
	ValaCCodeUnaryExpression* __temp753 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp753 = vala_ccode_unary_expression_new (op, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_postfix_expression_get_inner (expr))))))));
	(__temp753 == NULL ? NULL : (__temp753 = (g_object_unref (__temp753), NULL)));
	vala_code_generator_visit_expression (self, VALA_EXPRESSION (expr));
}


static ValaCCodeExpression* vala_code_generator_get_ref_expression (ValaCodeGenerator* self, ValaExpression* expr)
{
	g_return_val_if_fail (VALA_IS_CODE_GENERATOR (self), NULL);
	g_return_val_if_fail (VALA_IS_EXPRESSION (expr), NULL);
	/* (temp = expr, temp == NULL ? NULL : ref (temp))
	 *
	 * can be simplified to
	 * ref (expr)
	 * if static type of expr is non-null
	 */
	if (vala_type_reference_get_data_type (vala_expression_get_static_type (expr)) == NULL && vala_type_reference_get_type_parameter (vala_expression_get_static_type (expr)) != NULL) {
		vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), "Missing generics support for memory management");
		return VALA_CCODE_EXPRESSION (NULL);
	}
	char* ref_function = NULL;
	if (vala_data_type_is_reference_counting (vala_type_reference_get_data_type (vala_expression_get_static_type (expr)))) {
		ref_function = vala_data_type_get_ref_function (vala_type_reference_get_data_type (vala_expression_get_static_type (expr)));
	} else {
		ref_function = vala_data_type_get_dup_function (vala_type_reference_get_data_type (vala_expression_get_static_type (expr)));
	}
	ValaCCodeFunctionCall* ccall = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (ref_function)));
	if (vala_type_reference_get_non_null (vala_expression_get_static_type (expr))) {
		vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (expr))));
		return VALA_CCODE_EXPRESSION (ccall);
	} else {
		ValaVariableDeclarator* decl = vala_code_generator_get_temp_variable_declarator (self, vala_expression_get_static_type (expr), FALSE);
		ValaVariableDeclarator* __temp756 = NULL;
		self->priv->temp_vars = g_list_prepend (self->priv->temp_vars, (__temp756 = decl, (__temp756 == NULL ? NULL : g_object_ref (__temp756))));
		ValaCCodeIdentifier* ctemp = vala_ccode_identifier_new (vala_variable_declarator_get_name (decl));
		ValaCCodeBinaryExpression* cisnull = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_EQUALITY, VALA_CCODE_EXPRESSION (ctemp), VALA_CCODE_EXPRESSION (vala_ccode_constant_new ("NULL")));
		vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION (ctemp));
		ValaCCodeCommaExpression* ccomma = g_object_new (VALA_TYPE_CCODE_COMMA_EXPRESSION, NULL);
		ValaCCodeAssignment* __temp757 = NULL;
		vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION ((__temp757 = vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (ctemp), VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (expr))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))));
		(__temp757 == NULL ? NULL : (__temp757 = (g_object_unref (__temp757), NULL)));
		ValaCCodeConditionalExpression* __temp758 = NULL;
		vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION ((__temp758 = vala_ccode_conditional_expression_new (VALA_CCODE_EXPRESSION (cisnull), VALA_CCODE_EXPRESSION (vala_ccode_constant_new ("NULL")), VALA_CCODE_EXPRESSION (ccall)))));
		(__temp758 == NULL ? NULL : (__temp758 = (g_object_unref (__temp758), NULL)));
		ValaCCodeCommaExpression* __temp759 = NULL;
		return VALA_CCODE_EXPRESSION ((__temp759 = ccomma, (decl == NULL ? NULL : (decl = (g_object_unref (decl), NULL))), (ctemp == NULL ? NULL : (ctemp = (g_object_unref (ctemp), NULL))), (cisnull == NULL ? NULL : (cisnull = (g_object_unref (cisnull), NULL))), __temp759));
		(decl == NULL ? NULL : (decl = (g_object_unref (decl), NULL)));
		(ctemp == NULL ? NULL : (ctemp = (g_object_unref (ctemp), NULL)));
		(cisnull == NULL ? NULL : (cisnull = (g_object_unref (cisnull), NULL)));
		(ccomma == NULL ? NULL : (ccomma = (g_object_unref (ccomma), NULL)));
	}
	(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
}


static void vala_code_generator_visit_expression (ValaCodeGenerator* self, ValaExpression* expr)
{
	g_return_if_fail (VALA_IS_CODE_GENERATOR (self));
	g_return_if_fail (VALA_IS_EXPRESSION (expr));
	if (vala_expression_get_static_type (expr) != NULL && vala_type_reference_get_transfers_ownership (vala_expression_get_static_type (expr)) && vala_type_reference_get_floating_reference (vala_expression_get_static_type (expr))) {
		/* constructor of GInitiallyUnowned subtype
		 * returns floating reference, sink it
		 */
		ValaCCodeFunctionCall* csink = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("g_object_ref_sink")));
		vala_ccode_function_call_add_argument (csink, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (expr))));
		vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE (csink));
		(csink == NULL ? NULL : (csink = (g_object_unref (csink), NULL)));
	}
	if (vala_expression_get_ref_leaked (expr)) {
		ValaVariableDeclarator* decl = vala_code_generator_get_temp_variable_declarator (self, vala_expression_get_static_type (expr), TRUE);
		ValaVariableDeclarator* __temp760 = NULL;
		self->priv->temp_vars = g_list_prepend (self->priv->temp_vars, (__temp760 = decl, (__temp760 == NULL ? NULL : g_object_ref (__temp760))));
		ValaVariableDeclarator* __temp761 = NULL;
		self->priv->temp_ref_vars = g_list_prepend (self->priv->temp_ref_vars, (__temp761 = decl, (__temp761 == NULL ? NULL : g_object_ref (__temp761))));
		ValaCCodeParenthesizedExpression* __temp762 = NULL;
		vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp762 = vala_ccode_parenthesized_expression_new (VALA_CCODE_EXPRESSION (vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_variable_declarator_get_name (decl))), VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (expr))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))))));
		(__temp762 == NULL ? NULL : (__temp762 = (g_object_unref (__temp762), NULL)));
		(decl == NULL ? NULL : (decl = (g_object_unref (decl), NULL)));
	} else {
		if (vala_expression_get_ref_missing (expr)) {
			ValaCCodeExpression* __temp763 = NULL;
			vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp763 = vala_code_generator_get_ref_expression (self, expr))));
			(__temp763 == NULL ? NULL : (__temp763 = (g_object_unref (__temp763), NULL)));
		}
	}
}


static void vala_code_generator_real_visit_end_object_creation_expression (ValaCodeVisitor* base, ValaObjectCreationExpression* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_OBJECT_CREATION_EXPRESSION (expr));
	if (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)) == NULL) {
		/* no construction method*/
		if (VALA_IS_CLASS (vala_type_reference_get_data_type (vala_object_creation_expression_get_type_reference (expr)))) {
			ValaCCodeFunctionCall* ccall = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("g_object_new")));
			ValaCCodeConstant* __temp764 = NULL;
			vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp764 = vala_ccode_constant_new (vala_data_type_get_type_id (vala_type_reference_get_data_type (vala_object_creation_expression_get_type_reference (expr)))))));
			(__temp764 == NULL ? NULL : (__temp764 = (g_object_unref (__temp764), NULL)));
			ValaCCodeConstant* __temp765 = NULL;
			vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp765 = vala_ccode_constant_new ("NULL"))));
			(__temp765 == NULL ? NULL : (__temp765 = (g_object_unref (__temp765), NULL)));
			vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE (ccall));
			(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
		} else {
			if (vala_type_reference_get_data_type (vala_object_creation_expression_get_type_reference (expr)) == self->priv->list_type || vala_type_reference_get_data_type (vala_object_creation_expression_get_type_reference (expr)) == self->priv->slist_type) {
				/* NULL is an empty list*/
				ValaCCodeConstant* __temp766 = NULL;
				vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp766 = vala_ccode_constant_new ("NULL"))));
				(__temp766 == NULL ? NULL : (__temp766 = (g_object_unref (__temp766), NULL)));
			} else {
				ValaCCodeFunctionCall* ccall = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("g_new0")));
				ValaCCodeConstant* __temp767 = NULL;
				vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp767 = vala_ccode_constant_new (vala_data_type_get_cname (vala_type_reference_get_data_type (vala_object_creation_expression_get_type_reference (expr)), FALSE)))));
				(__temp767 == NULL ? NULL : (__temp767 = (g_object_unref (__temp767), NULL)));
				ValaCCodeConstant* __temp768 = NULL;
				vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp768 = vala_ccode_constant_new ("1"))));
				(__temp768 == NULL ? NULL : (__temp768 = (g_object_unref (__temp768), NULL)));
				vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE (ccall));
				(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
			}
		}
	} else {
		/* use construction method*/
		ValaMethod* m = VALA_METHOD (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))));
		GList* params = vala_invokable_get_parameters (VALA_INVOKABLE (m));
		ValaCCodeFunctionCall* ccall = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_method_get_cname (m))));
		gboolean ellipsis = FALSE;
		int i = 1;
		{
			GList* __temp774 = NULL;
			__temp774 = vala_object_creation_expression_get_argument_list (expr);
			GList* arg_it;
			for (arg_it = __temp774; arg_it != NULL; arg_it = arg_it->next) {
				ValaExpression* arg = arg_it->data;
				{
					/* explicitly use strong reference as ccall gets
					 * unrefed at end of inner block
					 */
					ValaCCodeExpression* __temp769 = NULL;
					ValaCCodeExpression* cexpr = (__temp769 = VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (arg))), (__temp769 == NULL ? NULL : g_object_ref (__temp769)));
					if (params != NULL) {
						ValaFormalParameter* param = VALA_FORMAL_PARAMETER (params->data);
						ellipsis = vala_formal_parameter_get_ellipsis (param);
						if (!vala_formal_parameter_get_ellipsis (param) && vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (param)) != NULL && vala_data_type_is_reference_type (vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (param))) && vala_type_reference_get_data_type (vala_expression_get_static_type (arg)) != NULL && vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (param)) != vala_type_reference_get_data_type (vala_expression_get_static_type (arg))) {
							/* FIXME: use C cast if debugging disabled*/
							ValaCCodeFunctionCall* ccall = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_data_type_get_upper_case_cname (vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (param)), NULL))));
							vala_ccode_function_call_add_argument (ccall, cexpr);
							ValaCCodeExpression* __temp771 = NULL;
							ValaCCodeFunctionCall* __temp770 = NULL;
							cexpr = (__temp771 = VALA_CCODE_EXPRESSION ((__temp770 = ccall, (__temp770 == NULL ? NULL : g_object_ref (__temp770)))), (cexpr == NULL ? NULL : (cexpr = (g_object_unref (cexpr), NULL))), __temp771);
							(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
						}
					}
					vala_ccode_function_call_add_argument (ccall, cexpr);
					i++;
					if (params != NULL) {
						GList* __temp773 = NULL;
						GList* __temp772 = NULL;
						params = (__temp773 = (__temp772 = params->next, (__temp772 == NULL ? NULL : g_list_copy (__temp772))), (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp773);
					}
					(cexpr == NULL ? NULL : (cexpr = (g_object_unref (cexpr), NULL)));
				}
			}
			(__temp774 == NULL ? NULL : (__temp774 = (g_list_free (__temp774), NULL)));
		}
		while (params != NULL) {
			ValaFormalParameter* param = VALA_FORMAL_PARAMETER (params->data);
			if (vala_formal_parameter_get_ellipsis (param)) {
				ellipsis = TRUE;
				break;
			}
			if (vala_formal_parameter_get_default_expression (param) == NULL) {
				char* __temp775 = NULL;
				vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp775 = g_strdup_printf ("no default expression for argument %d", i)));
				(__temp775 == NULL ? NULL : (__temp775 = (g_free (__temp775), NULL)));
				(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
				(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
				return;
			}
			/* evaluate default expression here as the code
			 * generator might not have visited the formal
			 * parameter yet */
			vala_code_node_accept (VALA_CODE_NODE (vala_formal_parameter_get_default_expression (param)), VALA_CODE_VISITOR (self));
			vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_formal_parameter_get_default_expression (param)))));
			i++;
			GList* __temp777 = NULL;
			GList* __temp776 = NULL;
			params = (__temp777 = (__temp776 = params->next, (__temp776 == NULL ? NULL : g_list_copy (__temp776))), (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp777);
		}
		if (ellipsis) {
			/* ensure variable argument list ends with NULL*/
			ValaCCodeConstant* __temp778 = NULL;
			vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp778 = vala_ccode_constant_new ("NULL"))));
			(__temp778 == NULL ? NULL : (__temp778 = (g_object_unref (__temp778), NULL)));
		}
		vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE (ccall));
		(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
		(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
	}
	vala_code_generator_visit_expression (self, VALA_EXPRESSION (expr));
}


static void vala_code_generator_real_visit_typeof_expression (ValaCodeVisitor* base, ValaTypeofExpression* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_TYPEOF_EXPRESSION (expr));
	ValaCCodeIdentifier* __temp779 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp779 = vala_ccode_identifier_new (vala_data_type_get_type_id (vala_type_reference_get_data_type (vala_typeof_expression_get_type_reference (expr)))))));
	(__temp779 == NULL ? NULL : (__temp779 = (g_object_unref (__temp779), NULL)));
}


static void vala_code_generator_real_visit_unary_expression (ValaCodeVisitor* base, ValaUnaryExpression* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_UNARY_EXPRESSION (expr));
	ValaCCodeUnaryOperator op;
	if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_PLUS) {
		op = VALA_CCODE_UNARY_OPERATOR_PLUS;
	} else {
		if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_MINUS) {
			op = VALA_CCODE_UNARY_OPERATOR_MINUS;
		} else {
			if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_LOGICAL_NEGATION) {
				op = VALA_CCODE_UNARY_OPERATOR_LOGICAL_NEGATION;
			} else {
				if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_BITWISE_COMPLEMENT) {
					op = VALA_CCODE_UNARY_OPERATOR_BITWISE_COMPLEMENT;
				} else {
					if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_INCREMENT) {
						op = VALA_CCODE_UNARY_OPERATOR_PREFIX_INCREMENT;
					} else {
						if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_DECREMENT) {
							op = VALA_CCODE_UNARY_OPERATOR_PREFIX_DECREMENT;
						} else {
							if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_REF) {
								op = VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF;
							} else {
								if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_OUT) {
									op = VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF;
								}
							}
						}
					}
				}
			}
		}
	}
	ValaCCodeUnaryExpression* __temp780 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp780 = vala_ccode_unary_expression_new (op, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_unary_expression_get_inner (expr))))))));
	(__temp780 == NULL ? NULL : (__temp780 = (g_object_unref (__temp780), NULL)));
	vala_code_generator_visit_expression (self, VALA_EXPRESSION (expr));
}


static void vala_code_generator_real_visit_cast_expression (ValaCodeVisitor* base, ValaCastExpression* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_CAST_EXPRESSION (expr));
	if (VALA_IS_CLASS (vala_type_reference_get_data_type (vala_cast_expression_get_type_reference (expr))) || VALA_IS_INTERFACE (vala_type_reference_get_data_type (vala_cast_expression_get_type_reference (expr)))) {
		/* GObject cast*/
		ValaInstanceCast* __temp781 = NULL;
		vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp781 = vala_instance_cast_new (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_cast_expression_get_inner (expr)))), vala_type_reference_get_data_type (vala_cast_expression_get_type_reference (expr))))));
		(__temp781 == NULL ? NULL : (__temp781 = (g_object_unref (__temp781), NULL)));
	} else {
		ValaCCodeCastExpression* __temp782 = NULL;
		vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp782 = vala_ccode_cast_expression_new (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_cast_expression_get_inner (expr)))), vala_type_reference_get_cname (vala_cast_expression_get_type_reference (expr), FALSE, FALSE)))));
		(__temp782 == NULL ? NULL : (__temp782 = (g_object_unref (__temp782), NULL)));
	}
	vala_code_generator_visit_expression (self, VALA_EXPRESSION (expr));
}


static void vala_code_generator_real_visit_binary_expression (ValaCodeVisitor* base, ValaBinaryExpression* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_BINARY_EXPRESSION (expr));
	ValaCCodeBinaryOperator op;
	if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_PLUS) {
		op = VALA_CCODE_BINARY_OPERATOR_PLUS;
	} else {
		if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_MINUS) {
			op = VALA_CCODE_BINARY_OPERATOR_MINUS;
		} else {
			if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_MUL) {
				op = VALA_CCODE_BINARY_OPERATOR_MUL;
			} else {
				if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_DIV) {
					op = VALA_CCODE_BINARY_OPERATOR_DIV;
				} else {
					if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_MOD) {
						op = VALA_CCODE_BINARY_OPERATOR_MOD;
					} else {
						if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_SHIFT_LEFT) {
							op = VALA_CCODE_BINARY_OPERATOR_SHIFT_LEFT;
						} else {
							if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_SHIFT_RIGHT) {
								op = VALA_CCODE_BINARY_OPERATOR_SHIFT_RIGHT;
							} else {
								if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_LESS_THAN) {
									op = VALA_CCODE_BINARY_OPERATOR_LESS_THAN;
								} else {
									if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_GREATER_THAN) {
										op = VALA_CCODE_BINARY_OPERATOR_GREATER_THAN;
									} else {
										if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_LESS_THAN_OR_EQUAL) {
											op = VALA_CCODE_BINARY_OPERATOR_LESS_THAN_OR_EQUAL;
										} else {
											if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_GREATER_THAN_OR_EQUAL) {
												op = VALA_CCODE_BINARY_OPERATOR_GREATER_THAN_OR_EQUAL;
											} else {
												if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_EQUALITY) {
													op = VALA_CCODE_BINARY_OPERATOR_EQUALITY;
												} else {
													if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_INEQUALITY) {
														op = VALA_CCODE_BINARY_OPERATOR_INEQUALITY;
													} else {
														if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_BITWISE_AND) {
															op = VALA_CCODE_BINARY_OPERATOR_BITWISE_AND;
														} else {
															if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_BITWISE_OR) {
																op = VALA_CCODE_BINARY_OPERATOR_BITWISE_OR;
															} else {
																if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_BITWISE_XOR) {
																	op = VALA_CCODE_BINARY_OPERATOR_BITWISE_XOR;
																} else {
																	if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_AND) {
																		op = VALA_CCODE_BINARY_OPERATOR_AND;
																	} else {
																		if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_OR) {
																			op = VALA_CCODE_BINARY_OPERATOR_OR;
																		}
																	}
																}
															}
														}
													}
												}
											}
										}
									}
								}
							}
						}
					}
				}
			}
		}
	}
	ValaCCodeExpression* __temp783 = NULL;
	ValaCCodeExpression* cleft = (__temp783 = VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_binary_expression_get_left (expr)))), (__temp783 == NULL ? NULL : g_object_ref (__temp783)));
	ValaCCodeExpression* __temp784 = NULL;
	ValaCCodeExpression* cright = (__temp784 = VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_binary_expression_get_right (expr)))), (__temp784 == NULL ? NULL : g_object_ref (__temp784)));
	if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_EQUALITY || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_INEQUALITY) {
		if (vala_expression_get_static_type (vala_binary_expression_get_left (expr)) != NULL && vala_expression_get_static_type (vala_binary_expression_get_right (expr)) != NULL && VALA_IS_CLASS (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_binary_expression_get_left (expr)))) && VALA_IS_CLASS (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_binary_expression_get_right (expr))))) {
			ValaClass* left_cl = VALA_CLASS (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_binary_expression_get_left (expr))));
			ValaClass* right_cl = VALA_CLASS (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_binary_expression_get_right (expr))));
			if (left_cl != right_cl) {
				if (vala_data_type_is_subtype_of (VALA_DATA_TYPE (left_cl), VALA_DATA_TYPE (right_cl))) {
					ValaCCodeExpression* __temp785 = NULL;
					cleft = (__temp785 = VALA_CCODE_EXPRESSION (vala_instance_cast_new (cleft, VALA_DATA_TYPE (right_cl))), (cleft == NULL ? NULL : (cleft = (g_object_unref (cleft), NULL))), __temp785);
				} else {
					if (vala_data_type_is_subtype_of (VALA_DATA_TYPE (right_cl), VALA_DATA_TYPE (left_cl))) {
						ValaCCodeExpression* __temp786 = NULL;
						cright = (__temp786 = VALA_CCODE_EXPRESSION (vala_instance_cast_new (cright, VALA_DATA_TYPE (left_cl))), (cright == NULL ? NULL : (cright = (g_object_unref (cright), NULL))), __temp786);
					}
				}
			}
		}
	}
	ValaCCodeBinaryExpression* __temp787 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp787 = vala_ccode_binary_expression_new (op, cleft, cright))));
	(__temp787 == NULL ? NULL : (__temp787 = (g_object_unref (__temp787), NULL)));
	vala_code_generator_visit_expression (self, VALA_EXPRESSION (expr));
	(cleft == NULL ? NULL : (cleft = (g_object_unref (cleft), NULL)));
	(cright == NULL ? NULL : (cright = (g_object_unref (cright), NULL)));
}


static void vala_code_generator_real_visit_type_check (ValaCodeVisitor* base, ValaTypeCheck* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_TYPE_CHECK (expr));
	ValaCCodeFunctionCall* ccheck = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_data_type_get_upper_case_cname (vala_type_reference_get_data_type (vala_type_check_get_type_reference (expr)), "IS_"))));
	vala_ccode_function_call_add_argument (ccheck, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_type_check_get_expression (expr)))));
	vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE (ccheck));
	(ccheck == NULL ? NULL : (ccheck = (g_object_unref (ccheck), NULL)));
}


static void vala_code_generator_real_visit_conditional_expression (ValaCodeVisitor* base, ValaConditionalExpression* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_CONDITIONAL_EXPRESSION (expr));
	ValaCCodeConditionalExpression* __temp788 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp788 = vala_ccode_conditional_expression_new (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_conditional_expression_get_condition (expr)))), VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_conditional_expression_get_true_expression (expr)))), VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_conditional_expression_get_false_expression (expr))))))));
	(__temp788 == NULL ? NULL : (__temp788 = (g_object_unref (__temp788), NULL)));
}


static void vala_code_generator_real_visit_end_lambda_expression (ValaCodeVisitor* base, ValaLambdaExpression* l)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_LAMBDA_EXPRESSION (l));
	ValaCCodeIdentifier* __temp789 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (l), VALA_CCODE_NODE ((__temp789 = vala_ccode_identifier_new (vala_method_get_cname (vala_lambda_expression_get_method (l))))));
	(__temp789 == NULL ? NULL : (__temp789 = (g_object_unref (__temp789), NULL)));
}


static void vala_code_generator_real_visit_end_assignment (ValaCodeVisitor* base, ValaAssignment* a)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_ASSIGNMENT (a));
	ValaMemberAccess* ma = NULL;
	if (VALA_IS_MEMBER_ACCESS (vala_assignment_get_left (a))) {
		ma = VALA_MEMBER_ACCESS (vala_assignment_get_left (a));
	}
	if (vala_expression_get_symbol_reference (vala_assignment_get_left (a)) != NULL && VALA_IS_PROPERTY (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_assignment_get_left (a))))) {
		ValaProperty* prop = VALA_PROPERTY (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_assignment_get_left (a))));
		ValaClass* cl = VALA_CLASS (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (prop)))));
		if (vala_member_access_get_inner (ma) == NULL && VALA_IS_STATEMENT (vala_code_node_get_parent_node (VALA_CODE_NODE (a))) && vala_statement_get_construction ((VALA_STATEMENT (vala_code_node_get_parent_node (VALA_CODE_NODE (a)))))) {
			/* this property is used as a construction parameter*/
			ValaCCodeIdentifier* cpointer = vala_ccode_identifier_new ("__params_it");
			ValaCCodeCommaExpression* ccomma = g_object_new (VALA_TYPE_CCODE_COMMA_EXPRESSION, NULL);
			/* set name in array for current parameter*/
			ValaCCodeMemberAccess* cnamemember = vala_ccode_member_access_new_pointer (VALA_CCODE_EXPRESSION (cpointer), "name");
			ValaCCodeAssignment* cnameassign = vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (cnamemember), VALA_CCODE_EXPRESSION (vala_property_get_canonical_cconstant (prop)), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE);
			vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION (cnameassign));
			ValaCCodeUnaryExpression* gvaluearg = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, VALA_CCODE_EXPRESSION (vala_ccode_member_access_new_pointer (VALA_CCODE_EXPRESSION (cpointer), "value")));
			/* initialize GValue in array for current parameter*/
			ValaCCodeFunctionCall* cvalueinit = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("g_value_init")));
			vala_ccode_function_call_add_argument (cvalueinit, VALA_CCODE_EXPRESSION (gvaluearg));
			ValaCCodeIdentifier* __temp790 = NULL;
			vala_ccode_function_call_add_argument (cvalueinit, VALA_CCODE_EXPRESSION ((__temp790 = vala_ccode_identifier_new (vala_data_type_get_type_id (vala_type_reference_get_data_type (vala_property_get_type_reference (prop)))))));
			(__temp790 == NULL ? NULL : (__temp790 = (g_object_unref (__temp790), NULL)));
			vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION (cvalueinit));
			/* set GValue for current parameter*/
			ValaCCodeFunctionCall* cvalueset = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_code_generator_get_value_setter_function (self, vala_property_get_type_reference (prop))));
			vala_ccode_function_call_add_argument (cvalueset, VALA_CCODE_EXPRESSION (gvaluearg));
			vala_ccode_function_call_add_argument (cvalueset, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_assignment_get_right (a)))));
			vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION (cvalueset));
			/* move pointer to next parameter in array*/
			ValaCCodeUnaryExpression* __temp791 = NULL;
			vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION ((__temp791 = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_POSTFIX_INCREMENT, VALA_CCODE_EXPRESSION (cpointer)))));
			(__temp791 == NULL ? NULL : (__temp791 = (g_object_unref (__temp791), NULL)));
			vala_code_node_set_ccodenode (VALA_CODE_NODE (a), VALA_CCODE_NODE (ccomma));
			(cpointer == NULL ? NULL : (cpointer = (g_object_unref (cpointer), NULL)));
			(ccomma == NULL ? NULL : (ccomma = (g_object_unref (ccomma), NULL)));
			(cnamemember == NULL ? NULL : (cnamemember = (g_object_unref (cnamemember), NULL)));
			(cnameassign == NULL ? NULL : (cnameassign = (g_object_unref (cnameassign), NULL)));
			(gvaluearg == NULL ? NULL : (gvaluearg = (g_object_unref (gvaluearg), NULL)));
			(cvalueinit == NULL ? NULL : (cvalueinit = (g_object_unref (cvalueinit), NULL)));
			(cvalueset == NULL ? NULL : (cvalueset = (g_object_unref (cvalueset), NULL)));
		} else {
			char* set_func = g_strdup ("g_object_set");
			if (!vala_property_get_no_accessor_method (prop)) {
				char* __temp793 = NULL;
				char* __temp792 = NULL;
				set_func = (__temp793 = g_strdup_printf ("%s_set_%s", (__temp792 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL)), vala_property_get_name (prop)), (set_func == NULL ? NULL : (set_func = (g_free (set_func), NULL))), __temp793);
				(__temp792 == NULL ? NULL : (__temp792 = (g_free (__temp792), NULL)));
			}
			ValaCCodeFunctionCall* ccall = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (set_func)));
			/* target instance is first argument */
			ValaCCodeExpression* instance = NULL;
			gboolean req_cast = FALSE;
			if (vala_member_access_get_inner (ma) == NULL) {
				ValaCCodeExpression* __temp794 = NULL;
				instance = (__temp794 = VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("self")), (instance == NULL ? NULL : (instance = (g_object_unref (instance), NULL))), __temp794);
				/* require casts for inherited properties */
				req_cast = (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (prop))) != self->priv->current_type_symbol);
			} else {
				ValaCCodeExpression* __temp796 = NULL;
				ValaCCodeExpression* __temp795 = NULL;
				instance = (__temp796 = (__temp795 = VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_member_access_get_inner (ma)))), (__temp795 == NULL ? NULL : g_object_ref (__temp795))), (instance == NULL ? NULL : (instance = (g_object_unref (instance), NULL))), __temp796);
				/* require casts if the type of the used instance is
				 * different than the type which declared the property */
				req_cast = vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (prop)))) != VALA_CODE_NODE (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_member_access_get_inner (ma))));
			}
			if (req_cast && vala_data_type_is_reference_type ((VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (prop)))))))) {
				ValaCCodeFunctionCall* ccast = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_data_type_get_upper_case_cname ((VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (prop)))))), NULL))));
				vala_ccode_function_call_add_argument (ccast, instance);
				ValaCCodeExpression* __temp798 = NULL;
				ValaCCodeFunctionCall* __temp797 = NULL;
				instance = (__temp798 = VALA_CCODE_EXPRESSION ((__temp797 = ccast, (__temp797 == NULL ? NULL : g_object_ref (__temp797)))), (instance == NULL ? NULL : (instance = (g_object_unref (instance), NULL))), __temp798);
				(ccast == NULL ? NULL : (ccast = (g_object_unref (ccast), NULL)));
			}
			vala_ccode_function_call_add_argument (ccall, instance);
			ValaCCodeExpression* __temp799 = NULL;
			ValaCCodeExpression* cexpr = (__temp799 = VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_assignment_get_right (a)))), (__temp799 == NULL ? NULL : g_object_ref (__temp799)));
			if (vala_property_get_no_accessor_method (prop)) {
				/* property name is second argument of g_object_set */
				ValaCCodeConstant* __temp800 = NULL;
				vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp800 = vala_property_get_canonical_cconstant (prop))));
				(__temp800 == NULL ? NULL : (__temp800 = (g_object_unref (__temp800), NULL)));
			} else {
				if (vala_type_reference_get_data_type (vala_property_get_type_reference (prop)) != NULL && vala_data_type_is_reference_type (vala_type_reference_get_data_type (vala_property_get_type_reference (prop))) && vala_type_reference_get_data_type (vala_expression_get_static_type (vala_assignment_get_right (a))) != NULL && vala_type_reference_get_data_type (vala_property_get_type_reference (prop)) != vala_type_reference_get_data_type (vala_expression_get_static_type (vala_assignment_get_right (a)))) {
					/* cast is necessary */
					ValaCCodeFunctionCall* ccast = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_data_type_get_upper_case_cname (vala_type_reference_get_data_type (vala_property_get_type_reference (prop)), NULL))));
					vala_ccode_function_call_add_argument (ccast, cexpr);
					ValaCCodeExpression* __temp802 = NULL;
					ValaCCodeFunctionCall* __temp801 = NULL;
					cexpr = (__temp802 = VALA_CCODE_EXPRESSION ((__temp801 = ccast, (__temp801 == NULL ? NULL : g_object_ref (__temp801)))), (cexpr == NULL ? NULL : (cexpr = (g_object_unref (cexpr), NULL))), __temp802);
					(ccast == NULL ? NULL : (ccast = (g_object_unref (ccast), NULL)));
				}
			}
			if (vala_assignment_get_operator (a) != VALA_ASSIGNMENT_OPERATOR_SIMPLE) {
				ValaCCodeBinaryOperator cop;
				if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_BITWISE_OR) {
					cop = VALA_CCODE_BINARY_OPERATOR_BITWISE_OR;
				} else {
					if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_BITWISE_AND) {
						cop = VALA_CCODE_BINARY_OPERATOR_BITWISE_AND;
					} else {
						if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_BITWISE_XOR) {
							cop = VALA_CCODE_BINARY_OPERATOR_BITWISE_XOR;
						} else {
							if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_ADD) {
								cop = VALA_CCODE_BINARY_OPERATOR_PLUS;
							} else {
								if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_SUB) {
									cop = VALA_CCODE_BINARY_OPERATOR_MINUS;
								} else {
									if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_MUL) {
										cop = VALA_CCODE_BINARY_OPERATOR_MUL;
									} else {
										if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_DIV) {
											cop = VALA_CCODE_BINARY_OPERATOR_DIV;
										} else {
											if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_PERCENT) {
												cop = VALA_CCODE_BINARY_OPERATOR_MOD;
											} else {
												if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_SHIFT_LEFT) {
													cop = VALA_CCODE_BINARY_OPERATOR_SHIFT_LEFT;
												} else {
													if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_SHIFT_RIGHT) {
														cop = VALA_CCODE_BINARY_OPERATOR_SHIFT_RIGHT;
													}
												}
											}
										}
									}
								}
							}
						}
					}
				}
				ValaCCodeExpression* __temp803 = NULL;
				cexpr = (__temp803 = VALA_CCODE_EXPRESSION (vala_ccode_binary_expression_new (cop, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_assignment_get_left (a)))), VALA_CCODE_EXPRESSION (vala_ccode_parenthesized_expression_new (cexpr)))), (cexpr == NULL ? NULL : (cexpr = (g_object_unref (cexpr), NULL))), __temp803);
			}
			vala_ccode_function_call_add_argument (ccall, cexpr);
			if (vala_property_get_no_accessor_method (prop)) {
				ValaCCodeConstant* __temp804 = NULL;
				vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp804 = vala_ccode_constant_new ("NULL"))));
				(__temp804 == NULL ? NULL : (__temp804 = (g_object_unref (__temp804), NULL)));
			}
			vala_code_node_set_ccodenode (VALA_CODE_NODE (a), VALA_CCODE_NODE (ccall));
			(set_func == NULL ? NULL : (set_func = (g_free (set_func), NULL)));
			(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
			(instance == NULL ? NULL : (instance = (g_object_unref (instance), NULL)));
			(cexpr == NULL ? NULL : (cexpr = (g_object_unref (cexpr), NULL)));
		}
	} else {
		if (vala_expression_get_symbol_reference (vala_assignment_get_left (a)) != NULL && VALA_IS_SIGNAL (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_assignment_get_left (a))))) {
			ValaSignal* sig = VALA_SIGNAL (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_assignment_get_left (a))));
			ValaMethod* m = VALA_METHOD (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_assignment_get_right (a))));
			char* connect_func = "g_signal_connect_object";
			if (!vala_method_get_instance (m)) {
				connect_func = "g_signal_connect";
			}
			ValaCCodeFunctionCall* ccall = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (connect_func)));
			if (vala_member_access_get_inner (ma) != NULL) {
				vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_member_access_get_inner (ma)))));
			} else {
				ValaCCodeIdentifier* __temp805 = NULL;
				vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp805 = vala_ccode_identifier_new ("self"))));
				(__temp805 == NULL ? NULL : (__temp805 = (g_object_unref (__temp805), NULL)));
			}
			ValaCCodeConstant* __temp806 = NULL;
			vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp806 = vala_signal_get_canonical_cconstant (sig))));
			(__temp806 == NULL ? NULL : (__temp806 = (g_object_unref (__temp806), NULL)));
			ValaCCodeCastExpression* __temp807 = NULL;
			vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp807 = vala_ccode_cast_expression_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_method_get_cname (m))), "GCallback"))));
			(__temp807 == NULL ? NULL : (__temp807 = (g_object_unref (__temp807), NULL)));
			if (vala_method_get_instance (m)) {
				if (VALA_IS_MEMBER_ACCESS (vala_assignment_get_right (a))) {
					ValaMemberAccess* right_ma = VALA_MEMBER_ACCESS (vala_assignment_get_right (a));
					if (vala_member_access_get_inner (right_ma) != NULL) {
						vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_member_access_get_inner (right_ma)))));
					} else {
						ValaCCodeIdentifier* __temp808 = NULL;
						vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp808 = vala_ccode_identifier_new ("self"))));
						(__temp808 == NULL ? NULL : (__temp808 = (g_object_unref (__temp808), NULL)));
					}
				} else {
					if (VALA_IS_LAMBDA_EXPRESSION (vala_assignment_get_right (a))) {
						ValaCCodeIdentifier* __temp809 = NULL;
						vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp809 = vala_ccode_identifier_new ("self"))));
						(__temp809 == NULL ? NULL : (__temp809 = (g_object_unref (__temp809), NULL)));
					}
				}
				ValaCCodeConstant* __temp810 = NULL;
				vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp810 = vala_ccode_constant_new ("0"))));
				(__temp810 == NULL ? NULL : (__temp810 = (g_object_unref (__temp810), NULL)));
			} else {
				ValaCCodeConstant* __temp811 = NULL;
				vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp811 = vala_ccode_constant_new ("NULL"))));
				(__temp811 == NULL ? NULL : (__temp811 = (g_object_unref (__temp811), NULL)));
			}
			vala_code_node_set_ccodenode (VALA_CODE_NODE (a), VALA_CCODE_NODE (ccall));
			(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
		} else {
			/* explicitly use strong reference as ccast gets
			 * unrefed at end of inner block
			 */
			ValaCCodeExpression* __temp812 = NULL;
			ValaCCodeExpression* rhs = (__temp812 = VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_assignment_get_right (a)))), (__temp812 == NULL ? NULL : g_object_ref (__temp812)));
			if (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_assignment_get_left (a))) != NULL && vala_type_reference_get_data_type (vala_expression_get_static_type (vala_assignment_get_right (a))) != NULL && vala_data_type_is_reference_type (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_assignment_get_left (a)))) && vala_type_reference_get_data_type (vala_expression_get_static_type (vala_assignment_get_right (a))) != vala_type_reference_get_data_type (vala_expression_get_static_type (vala_assignment_get_left (a)))) {
				ValaCCodeFunctionCall* ccast = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_data_type_get_upper_case_cname (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_assignment_get_left (a))), NULL))));
				vala_ccode_function_call_add_argument (ccast, rhs);
				ValaCCodeExpression* __temp814 = NULL;
				ValaCCodeFunctionCall* __temp813 = NULL;
				rhs = (__temp814 = VALA_CCODE_EXPRESSION ((__temp813 = ccast, (__temp813 == NULL ? NULL : g_object_ref (__temp813)))), (rhs == NULL ? NULL : (rhs = (g_object_unref (rhs), NULL))), __temp814);
				(ccast == NULL ? NULL : (ccast = (g_object_unref (ccast), NULL)));
			}
			gboolean unref_old = (vala_code_generator_get_memory_management (self) && vala_type_reference_get_takes_ownership (vala_expression_get_static_type (vala_assignment_get_left (a))));
			gboolean array = FALSE;
			if (VALA_IS_ARRAY (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_assignment_get_left (a))))) {
				array = !(VALA_IS_CCODE_CONSTANT (vala_code_generator_get_array_length_cexpression (self, vala_assignment_get_left (a), 1)));
			}
			if (unref_old || array) {
				ValaCCodeCommaExpression* ccomma = g_object_new (VALA_TYPE_CCODE_COMMA_EXPRESSION, NULL);
				ValaVariableDeclarator* temp_decl = vala_code_generator_get_temp_variable_declarator (self, vala_expression_get_static_type (vala_assignment_get_left (a)), TRUE);
				ValaVariableDeclarator* __temp815 = NULL;
				self->priv->temp_vars = g_list_prepend (self->priv->temp_vars, (__temp815 = temp_decl, (__temp815 == NULL ? NULL : g_object_ref (__temp815))));
				ValaCCodeAssignment* __temp816 = NULL;
				vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION ((__temp816 = vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_variable_declarator_get_name (temp_decl))), rhs, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))));
				(__temp816 == NULL ? NULL : (__temp816 = (g_object_unref (__temp816), NULL)));
				if (unref_old) {
					/* unref old value */
					ValaCCodeExpression* __temp817 = NULL;
					vala_ccode_comma_expression_append_expression (ccomma, (__temp817 = vala_code_generator_get_unref_expression (self, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_assignment_get_left (a)))), vala_expression_get_static_type (vala_assignment_get_left (a)))));
					(__temp817 == NULL ? NULL : (__temp817 = (g_object_unref (__temp817), NULL)));
				}
				if (array) {
					ValaCCodeExpression* lhs_array_len = vala_code_generator_get_array_length_cexpression (self, vala_assignment_get_left (a), 1);
					ValaCCodeExpression* rhs_array_len = vala_code_generator_get_array_length_cexpression (self, vala_assignment_get_right (a), 1);
					ValaCCodeAssignment* __temp818 = NULL;
					vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION ((__temp818 = vala_ccode_assignment_new (lhs_array_len, rhs_array_len, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))));
					(__temp818 == NULL ? NULL : (__temp818 = (g_object_unref (__temp818), NULL)));
					(lhs_array_len == NULL ? NULL : (lhs_array_len = (g_object_unref (lhs_array_len), NULL)));
					(rhs_array_len == NULL ? NULL : (rhs_array_len = (g_object_unref (rhs_array_len), NULL)));
				}
				ValaCCodeIdentifier* __temp819 = NULL;
				vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION ((__temp819 = vala_ccode_identifier_new (vala_variable_declarator_get_name (temp_decl)))));
				(__temp819 == NULL ? NULL : (__temp819 = (g_object_unref (__temp819), NULL)));
				ValaCCodeExpression* __temp821 = NULL;
				ValaCCodeCommaExpression* __temp820 = NULL;
				rhs = (__temp821 = VALA_CCODE_EXPRESSION ((__temp820 = ccomma, (__temp820 == NULL ? NULL : g_object_ref (__temp820)))), (rhs == NULL ? NULL : (rhs = (g_object_unref (rhs), NULL))), __temp821);
				(ccomma == NULL ? NULL : (ccomma = (g_object_unref (ccomma), NULL)));
				(temp_decl == NULL ? NULL : (temp_decl = (g_object_unref (temp_decl), NULL)));
			}
			ValaCCodeAssignmentOperator cop = VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE;
			if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_BITWISE_OR) {
				cop = VALA_CCODE_ASSIGNMENT_OPERATOR_BITWISE_OR;
			} else {
				if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_BITWISE_AND) {
					cop = VALA_CCODE_ASSIGNMENT_OPERATOR_BITWISE_AND;
				} else {
					if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_BITWISE_XOR) {
						cop = VALA_CCODE_ASSIGNMENT_OPERATOR_BITWISE_XOR;
					} else {
						if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_ADD) {
							cop = VALA_CCODE_ASSIGNMENT_OPERATOR_ADD;
						} else {
							if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_SUB) {
								cop = VALA_CCODE_ASSIGNMENT_OPERATOR_SUB;
							} else {
								if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_MUL) {
									cop = VALA_CCODE_ASSIGNMENT_OPERATOR_MUL;
								} else {
									if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_DIV) {
										cop = VALA_CCODE_ASSIGNMENT_OPERATOR_DIV;
									} else {
										if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_PERCENT) {
											cop = VALA_CCODE_ASSIGNMENT_OPERATOR_PERCENT;
										} else {
											if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_SHIFT_LEFT) {
												cop = VALA_CCODE_ASSIGNMENT_OPERATOR_SHIFT_LEFT;
											} else {
												if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_SHIFT_RIGHT) {
													cop = VALA_CCODE_ASSIGNMENT_OPERATOR_SHIFT_RIGHT;
												}
											}
										}
									}
								}
							}
						}
					}
				}
			}
			ValaCCodeAssignment* __temp822 = NULL;
			vala_code_node_set_ccodenode (VALA_CODE_NODE (a), VALA_CCODE_NODE ((__temp822 = vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_assignment_get_left (a)))), rhs, cop))));
			(__temp822 == NULL ? NULL : (__temp822 = (g_object_unref (__temp822), NULL)));
			(rhs == NULL ? NULL : (rhs = (g_object_unref (rhs), NULL)));
		}
	}
}


gboolean vala_code_generator_get_memory_management (ValaCodeGenerator* self)
{
	g_return_val_if_fail (VALA_IS_CODE_GENERATOR (self), FALSE);
	return self->priv->_memory_management;
}


void vala_code_generator_set_memory_management (ValaCodeGenerator* self, gboolean value)
{
	g_return_if_fail (VALA_IS_CODE_GENERATOR (self));
	self->priv->_memory_management = value;
}


static void vala_code_generator_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (object);
	switch (property_id) {
		case VALA_CODE_GENERATOR_MEMORY_MANAGEMENT:
		g_value_set_boolean (value, vala_code_generator_get_memory_management (self));
		break;
	}
}


static void vala_code_generator_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (object);
	switch (property_id) {
		case VALA_CODE_GENERATOR_MEMORY_MANAGEMENT:
		vala_code_generator_set_memory_management (self, g_value_get_boolean (value));
		break;
	}
}


static void vala_code_generator_class_init (ValaCodeGeneratorClass * klass)
{
	vala_code_generator_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValaCodeGeneratorPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_code_generator_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_code_generator_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_code_generator_dispose;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_source_file = vala_code_generator_real_visit_begin_source_file;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_source_file = vala_code_generator_real_visit_end_source_file;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_class = vala_code_generator_real_visit_begin_class;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_class = vala_code_generator_real_visit_end_class;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_struct = vala_code_generator_real_visit_begin_struct;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_interface = vala_code_generator_real_visit_begin_interface;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_interface = vala_code_generator_real_visit_end_interface;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_enum = vala_code_generator_real_visit_begin_enum;
	VALA_CODE_VISITOR_CLASS (klass)->visit_enum_value = vala_code_generator_real_visit_enum_value;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_callback = vala_code_generator_real_visit_end_callback;
	VALA_CODE_VISITOR_CLASS (klass)->visit_member = vala_code_generator_real_visit_member;
	VALA_CODE_VISITOR_CLASS (klass)->visit_constant = vala_code_generator_real_visit_constant;
	VALA_CODE_VISITOR_CLASS (klass)->visit_field = vala_code_generator_real_visit_field;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_method = vala_code_generator_real_visit_begin_method;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_method = vala_code_generator_real_visit_end_method;
	VALA_CODE_VISITOR_CLASS (klass)->visit_formal_parameter = vala_code_generator_real_visit_formal_parameter;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_property = vala_code_generator_real_visit_end_property;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_property_accessor = vala_code_generator_real_visit_begin_property_accessor;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_property_accessor = vala_code_generator_real_visit_end_property_accessor;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_signal = vala_code_generator_real_visit_end_signal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_constructor = vala_code_generator_real_visit_end_constructor;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_block = vala_code_generator_real_visit_begin_block;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_block = vala_code_generator_real_visit_end_block;
	VALA_CODE_VISITOR_CLASS (klass)->visit_empty_statement = vala_code_generator_real_visit_empty_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_declaration_statement = vala_code_generator_real_visit_declaration_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_variable_declarator = vala_code_generator_real_visit_variable_declarator;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_initializer_list = vala_code_generator_real_visit_end_initializer_list;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_full_expression = vala_code_generator_real_visit_end_full_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_expression_statement = vala_code_generator_real_visit_expression_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_if_statement = vala_code_generator_real_visit_if_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_switch_statement = vala_code_generator_real_visit_switch_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_while_statement = vala_code_generator_real_visit_while_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_for_statement = vala_code_generator_real_visit_for_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_foreach_statement = vala_code_generator_real_visit_end_foreach_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_break_statement = vala_code_generator_real_visit_break_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_continue_statement = vala_code_generator_real_visit_continue_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_return_statement = vala_code_generator_real_visit_begin_return_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_return_statement = vala_code_generator_real_visit_end_return_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_lock_statement = vala_code_generator_real_visit_lock_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_array_creation_expression = vala_code_generator_real_visit_end_array_creation_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_boolean_literal = vala_code_generator_real_visit_boolean_literal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_character_literal = vala_code_generator_real_visit_character_literal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_integer_literal = vala_code_generator_real_visit_integer_literal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_real_literal = vala_code_generator_real_visit_real_literal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_string_literal = vala_code_generator_real_visit_string_literal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_null_literal = vala_code_generator_real_visit_null_literal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_literal_expression = vala_code_generator_real_visit_literal_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_parenthesized_expression = vala_code_generator_real_visit_parenthesized_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_member_access = vala_code_generator_real_visit_member_access;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_invocation_expression = vala_code_generator_real_visit_end_invocation_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_element_access = vala_code_generator_real_visit_element_access;
	VALA_CODE_VISITOR_CLASS (klass)->visit_base_access = vala_code_generator_real_visit_base_access;
	VALA_CODE_VISITOR_CLASS (klass)->visit_postfix_expression = vala_code_generator_real_visit_postfix_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_object_creation_expression = vala_code_generator_real_visit_end_object_creation_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_typeof_expression = vala_code_generator_real_visit_typeof_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_unary_expression = vala_code_generator_real_visit_unary_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_cast_expression = vala_code_generator_real_visit_cast_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_binary_expression = vala_code_generator_real_visit_binary_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_type_check = vala_code_generator_real_visit_type_check;
	VALA_CODE_VISITOR_CLASS (klass)->visit_conditional_expression = vala_code_generator_real_visit_conditional_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_lambda_expression = vala_code_generator_real_visit_end_lambda_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_assignment = vala_code_generator_real_visit_end_assignment;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CODE_GENERATOR_MEMORY_MANAGEMENT, g_param_spec_boolean ("memory-management", "foo", "bar", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void vala_code_generator_init (ValaCodeGenerator * self)
{
	self->priv = VALA_CODE_GENERATOR_GET_PRIVATE (self);
	self->priv->next_temp_var_id = 0;
}


static void vala_code_generator_dispose (GObject * obj)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (obj);
	(self->priv->context == NULL ? NULL : (self->priv->context = (g_object_unref (self->priv->context), NULL)));
	(self->priv->root_symbol == NULL ? NULL : (self->priv->root_symbol = (g_object_unref (self->priv->root_symbol), NULL)));
	(self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL)));
	(self->priv->current_type_symbol == NULL ? NULL : (self->priv->current_type_symbol = (g_object_unref (self->priv->current_type_symbol), NULL)));
	(self->priv->current_class == NULL ? NULL : (self->priv->current_class = (g_object_unref (self->priv->current_class), NULL)));
	(self->priv->current_return_type == NULL ? NULL : (self->priv->current_return_type = (g_object_unref (self->priv->current_return_type), NULL)));
	(self->priv->header_begin == NULL ? NULL : (self->priv->header_begin = (g_object_unref (self->priv->header_begin), NULL)));
	(self->priv->header_type_declaration == NULL ? NULL : (self->priv->header_type_declaration = (g_object_unref (self->priv->header_type_declaration), NULL)));
	(self->priv->header_type_definition == NULL ? NULL : (self->priv->header_type_definition = (g_object_unref (self->priv->header_type_definition), NULL)));
	(self->priv->header_type_member_declaration == NULL ? NULL : (self->priv->header_type_member_declaration = (g_object_unref (self->priv->header_type_member_declaration), NULL)));
	(self->priv->source_begin == NULL ? NULL : (self->priv->source_begin = (g_object_unref (self->priv->source_begin), NULL)));
	(self->priv->source_include_directives == NULL ? NULL : (self->priv->source_include_directives = (g_object_unref (self->priv->source_include_directives), NULL)));
	(self->priv->source_type_member_declaration == NULL ? NULL : (self->priv->source_type_member_declaration = (g_object_unref (self->priv->source_type_member_declaration), NULL)));
	(self->priv->source_signal_marshaller_declaration == NULL ? NULL : (self->priv->source_signal_marshaller_declaration = (g_object_unref (self->priv->source_signal_marshaller_declaration), NULL)));
	(self->priv->source_type_member_definition == NULL ? NULL : (self->priv->source_type_member_definition = (g_object_unref (self->priv->source_type_member_definition), NULL)));
	(self->priv->instance_init_fragment == NULL ? NULL : (self->priv->instance_init_fragment = (g_object_unref (self->priv->instance_init_fragment), NULL)));
	(self->priv->instance_dispose_fragment == NULL ? NULL : (self->priv->instance_dispose_fragment = (g_object_unref (self->priv->instance_dispose_fragment), NULL)));
	(self->priv->source_signal_marshaller_definition == NULL ? NULL : (self->priv->source_signal_marshaller_definition = (g_object_unref (self->priv->source_signal_marshaller_definition), NULL)));
	(self->priv->instance_struct == NULL ? NULL : (self->priv->instance_struct = (g_object_unref (self->priv->instance_struct), NULL)));
	(self->priv->type_struct == NULL ? NULL : (self->priv->type_struct = (g_object_unref (self->priv->type_struct), NULL)));
	(self->priv->instance_priv_struct == NULL ? NULL : (self->priv->instance_priv_struct = (g_object_unref (self->priv->instance_priv_struct), NULL)));
	(self->priv->prop_enum == NULL ? NULL : (self->priv->prop_enum = (g_object_unref (self->priv->prop_enum), NULL)));
	(self->priv->cenum == NULL ? NULL : (self->priv->cenum = (g_object_unref (self->priv->cenum), NULL)));
	(self->priv->function == NULL ? NULL : (self->priv->function = (g_object_unref (self->priv->function), NULL)));
	(self->priv->block == NULL ? NULL : (self->priv->block = (g_object_unref (self->priv->block), NULL)));
	(self->priv->temp_vars == NULL ? NULL : (self->priv->temp_vars = (g_list_foreach (self->priv->temp_vars, (GFunc) g_object_unref, NULL), g_list_free (self->priv->temp_vars), NULL)));
	(self->priv->temp_ref_vars == NULL ? NULL : (self->priv->temp_ref_vars = (g_list_foreach (self->priv->temp_ref_vars, (GFunc) g_object_unref, NULL), g_list_free (self->priv->temp_ref_vars), NULL)));
	(self->priv->user_marshal_list == NULL ? NULL : (self->priv->user_marshal_list = (g_hash_table_unref (self->priv->user_marshal_list), NULL)));
	(self->priv->predefined_marshal_list == NULL ? NULL : (self->priv->predefined_marshal_list = (g_hash_table_unref (self->priv->predefined_marshal_list), NULL)));
	(self->priv->bool_type == NULL ? NULL : (self->priv->bool_type = (g_object_unref (self->priv->bool_type), NULL)));
	(self->priv->char_type == NULL ? NULL : (self->priv->char_type = (g_object_unref (self->priv->char_type), NULL)));
	(self->priv->unichar_type == NULL ? NULL : (self->priv->unichar_type = (g_object_unref (self->priv->unichar_type), NULL)));
	(self->priv->short_type == NULL ? NULL : (self->priv->short_type = (g_object_unref (self->priv->short_type), NULL)));
	(self->priv->ushort_type == NULL ? NULL : (self->priv->ushort_type = (g_object_unref (self->priv->ushort_type), NULL)));
	(self->priv->int_type == NULL ? NULL : (self->priv->int_type = (g_object_unref (self->priv->int_type), NULL)));
	(self->priv->uint_type == NULL ? NULL : (self->priv->uint_type = (g_object_unref (self->priv->uint_type), NULL)));
	(self->priv->long_type == NULL ? NULL : (self->priv->long_type = (g_object_unref (self->priv->long_type), NULL)));
	(self->priv->ulong_type == NULL ? NULL : (self->priv->ulong_type = (g_object_unref (self->priv->ulong_type), NULL)));
	(self->priv->int64_type == NULL ? NULL : (self->priv->int64_type = (g_object_unref (self->priv->int64_type), NULL)));
	(self->priv->uint64_type == NULL ? NULL : (self->priv->uint64_type = (g_object_unref (self->priv->uint64_type), NULL)));
	(self->priv->string_type == NULL ? NULL : (self->priv->string_type = (g_object_unref (self->priv->string_type), NULL)));
	(self->priv->float_type == NULL ? NULL : (self->priv->float_type = (g_object_unref (self->priv->float_type), NULL)));
	(self->priv->double_type == NULL ? NULL : (self->priv->double_type = (g_object_unref (self->priv->double_type), NULL)));
	(self->priv->list_type == NULL ? NULL : (self->priv->list_type = (g_object_unref (self->priv->list_type), NULL)));
	(self->priv->slist_type == NULL ? NULL : (self->priv->slist_type = (g_object_unref (self->priv->slist_type), NULL)));
	(self->priv->mutex_type == NULL ? NULL : (self->priv->mutex_type = (g_object_unref (self->priv->mutex_type), NULL)));
	ValaCodeGeneratorClass * klass;
	GObjectClass * parent_class;
	klass = VALA_CODE_GENERATOR_CLASS (g_type_class_peek (VALA_TYPE_CODE_GENERATOR));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_code_generator_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaCodeGeneratorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_code_generator_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaCodeGenerator), 0, (GInstanceInitFunc) vala_code_generator_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_CODE_VISITOR, "ValaCodeGenerator", &g_define_type_info, 0);
	}
	return g_define_type_id;
}




